#pragma once

/*
 * media transport type
 */
enum YKStreamType {
	TYPE_UDP_RAW,
	TYPE_RTSP
};

class YKStreamSender
{
public:
	virtual ~YKStreamSender(void);
	/*
	 * create new Sender by stream type for sending stream.
	 */
	static YKStreamSender* createNew(YKStreamType type);

	/*
	 * Do some initial work before you send stream.
	 * addr: used by udp raw stream, indicate the dst host of the stream (ignored in rtsp);
	 * port: used by udp raw stream, indicate the port of the stream (ignored in rtsp);
	 */
	virtual int setup(char* addr, int port) = 0;

	/*
	 * Send data to the client.
	 * buf: the data need to be sent;
	 * len: size of the data need to be sent;
	 */
	virtual int sendData(unsigned char* buf, int len) = 0;

    /*
     * Stop stream sending .
     */
	virtual int stop() = 0;

	/*
	 * Realse resource when stop the stream.
	 */
	virtual int tearDown() = 0;

protected:
	YKStreamSender(YKStreamType);
	YKStreamType mType;
};

