#ifndef _EVENT_MANAGER_H
#define _EVENT_MANAGER_H

#include <hardware_legacy/uevent.h>
#include <hardware/sensors.h>

#include <utils/String8.h>

#include <utils/Thread.h>
#include <utils/Log.h>

#include <unistd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


#include "ConfigData.h"
#include "BackCarMonitor.h"
#include "CdrCamera.h"
#include <Gsensor.h>
#include "platform.h"

extern int detectPlugIn(int fd);

using namespace android;

// add for rf_module
#define RF_SHORTPRESEE_CODE 82
#define RF_LONGPRESEE_CODE  24
typedef enum RF_Event
{
   RF_PRESS_EERO_EVENT = 100,
   RF_SHORTPRESS_EVENT,
   RF_LONGPRESS_EVENT,

} RF_Event;
//rf_module end

enum event_msg {
	EVENT_UVC_PLUG_IN	= 1,
	EVENT_UVC_PLUG_OUT,
	EVENT_CONNECT2PC,
	EVENT_DISCONNECFROMPC,
	EVENT_IMPACT,
	EVENT_BATTERY,
	EVENT_DELAY_SHUTDOWN,
	EVENT_CANCEL_SHUTDOWN,
	EVENT_MOUNT,
	EVENT_REMOVESD,
	EVENT_INSERTSD,
	EVENT_HDMI_PLUGIN,
	EVENT_HDMI_PLUGOUT,
	EVENT_TVD_PLUG_IN,
	EVENT_TVD_PLUG_OUT,
#ifdef APP_TVOUT
	EVENT_TVOUT_PLUG_IN,
	EVENT_TVOUT_PLUG_OUT,
#endif
	EVENT_BACK_CAR_IN,
	EVENT_BACK_CAR_OUT,
	EVENT_DB_INITEND
};

enum {
	GSNESOR_HIGH = 0,
	GSNESOR_MIDDLE ,
	GSNESOR_LOW ,
};

enum{
	BATTERY_CHARGING = 1,
	BATTERY_DISCHARGING,
	BATTERY_NOTCHARGING,
	BATTERY_FULL
};

//#define THRESHOLD_VALUE 18
#define THRESHOLD_HIGH_VALUE   11
#define THRESHOLD_MIDDLE_VALUE 15
#define THRESHOLD_LOW_VALUE    19


#define HDMI_NODE "hdmi"
#define HDMI_ONLINE_FILE	"sys/devices/virtual/switch/hdmi/state"


#define UVC_NODE	"video"

#ifdef PLATFORM_0
	#define UVC_ADD		"add@/devices/platform/sw-ehci."
	#define UVC_REMOVE	"remove@/devices/platform/sw-ehci."
	#define USB_NODE				"usb"
	#define USB_ONLINE_FILE			"/sys/class/power_supply/usb/online"
#else
	#define UVC_ADD		"add@/devices/platform/sunxi-"
	#define UVC_REMOVE	"remove@/devices/platform/sunxi-"
	#define USB_NODE				"android0"
	#define USB_ONLINE_FILE			"/sys/class/android_usb/android0/state"
	#define USB_CONNECTED  "CONNECTED"
	#define USB_DISCONNECT "DISCONNECTED"
#endif
#define BATTERY_NODE			"battery"
#define POWER_BATTERY			"power_supply/battery"
#define UEVENT_CHANGE			"change"

#define BATTERY_ONLINE_FILE		"/sys/class/power_supply/battery/online"
#define BATTERY_CAPACITY			"/sys/class/power_supply/battery/capacity"
#define AC_ONLINE_FILE			"/sys/class/power_supply/ac/online"
#define USB_ONLINE				"/sys/class/power_supply/usb/online"
#define TVD_FILE_ONLINE			"/sys/devices/tvd/tvd"

#define CD_TFCARD_PATH "/dev/block/vold/179:0"
#define CD_TFCARD_LUN_PATH "/sys/class/android_usb/android0/f_mass_storage/lun/file"

#define BATTERY_STATUS_FILE			"/sys/class/power_supply/battery/status"

#define SUBTRACT(x1, x2)   ((x1-x2)>0?(x1-x2):(x2-x1))
#define GSDataFilePath		STORAGE_PATH".db"

class EventManager;

class EventListener
{
public:
		EventListener(){};
		virtual ~EventListener(){};
		virtual int notify(int message, int val) = 0;
};

class ImpactThread : public Thread
{
public:
	ImpactThread(EventManager *em);
	~ImpactThread();
	int init(void);
	int exit(void);
	void setImpactLevel(unsigned int val);
	void setAdasSwitch(bool flag);
	GFRAMEIN getGSensorData();
	void adasSetGSensorData(GSENSOROUT data);
	void startThread()
	{
		run("ImpactThread", PRIORITY_NORMAL);
	}
	void stopThread()
	{
		requestExitAndWait();
	}
	bool threadLoop()
	{
		impactLoop();
		return true;
	}
#ifdef APP_WIFI
	void keepGSensorData(float x, float y, float z);
#endif
private:
	int sensorCount;
	int mImpactLevel; 
	struct sensors_poll_device_t* device;
	struct sensor_t const *sensorList;
	
	EventManager *mEM;
	int impactInit(void);
	int impactLoop(void);
	CdrCamera *mCamCSI;
	bool isAdasOpen;
	GSENSORIN mGSensor;
	GFRAMEIN mGsensordata;
	GSENSOROUT mGoutdata;
#ifdef APP_WIFI
	float gx0, gy0, gz0;
	Mutex mLock;
	bool mIsWirteing;
#endif
};

class RecordPreview;
class EventManager
{
public:
	EventManager(bool tvdOnline, bool hasTVD);
	~EventManager();
	int init(bool hasTVD);
	int exit(void);
	void connect2PC();
	void disconnectFromPC();
	int impactEvent();
	int checkUSB(char *udata);
	bool isBatteryOnline();
	static int getBattery();
	static int getBatteryStatus();
	int checkHDMI(const char *str);
	void detectTVD();
	void startDetectTVD();
	void hdmiAudioOn(bool open);
#ifdef APP_TVOUT
	void detectTVout();
	int startDetectTVout();
#endif
	void checkDevMore(int idx);
	int usbConnnect();
	void extraDetectLoop();
	void setExtraDetect(bool enable);
	void setImpactLevel(unsigned int value);
	void setCdrCamerCSI(CdrCamera *mCSI);
	CdrCamera * getCdrCamerCSI();
	void setAdasIsOpen(bool flag);
	void rf_module_init();
	int initUinputEvent();
	class UinputEventThread : public Thread
	{
	public:
		UinputEventThread(EventManager *em)
			: Thread(false)
			, mEM(em)
		{
		}
		void startThread()
		{
			run("UinputEventThread", PRIORITY_NORMAL);
		}
		void stopThread()
		{
			requestExitAndWait();
		}
		bool threadLoop()
		{
			mEM->uinputEventLoop();
			return true;
		}
	private:
		EventManager *mEM;
	};

	class UeventThread : public Thread
	{
	public:
		UeventThread(EventManager *em)
			: Thread(false)
			, mEM(em)
		{
		}
		void startThread()
		{
			run("UeventThread", PRIORITY_NORMAL);
		}
		void stopThread()
		{
			requestExitAndWait();
		}
		bool threadLoop()
		{
			mEM->ueventLoop();
			return true;
		}
	private:
		EventManager *mEM;
	};
	class ExtraDetectThread : public Thread
	{
	public:
		ExtraDetectThread(EventManager *em)
			: Thread(false)
			, mEM(em)
		{
		}
		void startThread()
		{
			run("ExtraDetectThread", PRIORITY_NORMAL);
		}
		void stopThread()
		{
			requestExitAndWait();
		}
		bool threadLoop()
		{
			mEM->extraDetectLoop();
			return true;
		}
	private:
		EventManager *mEM;
	};

	class TVDDetectThread : public Thread
	{
	public:
		TVDDetectThread(EventManager *em)
			: Thread(false),
			  mEM(em)
		{}
		void startThread()
		{run("TVDDetectThread", PRIORITY_NORMAL);}
		void stopThread()
		{requestExitAndWait();}
		bool threadLoop()
		{
			mEM->detectTVD();
			return true;
		}
	private:
		EventManager *mEM;
		bool mNeedStop;
		int mFD;
	};
#ifdef APP_TVOUT
	class TVoutDetectThread : public Thread
	{
	public:
		TVoutDetectThread(EventManager *em)
			: Thread(false),
			  mEM(em)
		{}
		void startThread()
		{run("TVoutDetectThread", PRIORITY_NORMAL);}
		void stopThread()
		{requestExitAndWait();}
		bool threadLoop()
		{
			mEM->detectTVout();
			return true;
		}
	private:
		EventManager *mEM;
	};
#endif
	void setEventListener(EventListener *pListener);
	static void int2set(bool enable);
	void setGsensorSensitivity(int level);
	void detectUVCBackCarState();
	void startDetectUVCBackCarState();
	void stopDetectUVCBackCarState();
	class UVCBackCarDetectThread : public Thread
	{
	public:
		UVCBackCarDetectThread(EventManager *em)
			: Thread(false),
			  mEM(em)
		{}
		void startThread()
		{run("UVCBackCarDetectThread", PRIORITY_NORMAL);}
		void stopThread()
		{requestExitAndWait();}
		bool threadLoop()
		{
			mEM->detectUVCBackCarState();
			return true;
		}
	private:
		EventManager *mEM;
	};
#if defined(VIDEO_CONFIG_TEXT_TRACK) && defined(VIDEO_CONFIG_TEXT_GSR)
    void setRecordPreview(RecordPreview *mRP);
    RecordPreview *getRecordPreview();
#endif
private:
	sp<UeventThread> mUT;
	sp<UinputEventThread> mUET;
	sp<ImpactThread> mIT;
	sp<TVDDetectThread>mTVDDetectThread;
	sp<ExtraDetectThread>mEDT;
	EventListener *mListener;
	int ueventLoop(void);
	int uinputEventLoop(void);
	int checkBattery(int &level);
	int checkACPlug();
	bool mBatteryOnline;
	bool mUSBDisconnect;
	bool mTVDOnline;
	bool mNeedDelay;
	bool mACOnline;
	bool mPCCam;
	CdrCamera *mCamCSI;
#ifdef APP_TVOUT
	sp<TVoutDetectThread> mTVoutDetectThread;
#endif
	bool isUVCBackCarThreadRunning;
	sp<UVCBackCarDetectThread> mUVCBackCarDetectThread;
	BackCarMonitor * mBackCarMonitor;
    RecordPreview *mRecordPreview;
};

#endif
