#ifndef _BACKCAR_MONITOR_H
#define _BACKCAR_MONITOR_H

#include <utils/String8.h>

#include <utils/Log.h>

#include <unistd.h>
#include <linux/videodev2.h>


using namespace android;


/*
 * Dynamic controls
 */

#define UVC_CTRL_DATA_TYPE_RAW		0
#define UVC_CTRL_DATA_TYPE_SIGNED	1
#define UVC_CTRL_DATA_TYPE_UNSIGNED	2
#define UVC_CTRL_DATA_TYPE_BOOLEAN	3
#define UVC_CTRL_DATA_TYPE_ENUM		4
#define UVC_CTRL_DATA_TYPE_BITMASK	5

/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

#define V4L2_CID_BASE_EXTCTR_SONIX					0x0A0c4501
#define V4L2_CID_BASE_SONIX                       	V4L2_CID_BASE_EXTCTR_SONIX
#define V4L2_CID_ASIC_CTRL_SONIX                    V4L2_CID_BASE_SONIX+1
#define V4L2_CID_I2C_CTRL_SONIX						V4L2_CID_BASE_SONIX+2
#define V4L2_CID_SF_READ_SONIX                      V4L2_CID_BASE_SONIX+3
#define V4L2_CID_LAST_EXTCTR_SONIX					V4L2_CID_IMG_SETTING_SONIX

/* ---------------------------------------------------------------------------- */

#define UVC_GUID_SONIX_USER_HW_CONTROL       	{0x70, 0x33, 0xf0, 0x28, 0x11, 0x63, 0x2e, 0x4a, 0xba, 0x2c, 0x68, 0x90, 0xeb, 0x33, 0x40, 0x16}

// ----------------------------- XU Control Selector ------------------------------------
#define XU_SONIX_ASIC_CTRL 				0x01   // 与control 1 相对应
#define XU_SONIX_I2C_CTRL				0x02
#define XU_SONIX_SF_READ 				0x03

//#define XU_SONIX_H264_FMT  			0x06
//#define XU_SONIX_H264_QP   			0x07
//#define XU_SONIX_H264_BITRATE 		0x08
//#define XU_SONIX_FRAME_INFO  			0x06
//#define XU_SONIX_H264_CTRL   			0x07
//#define XU_SONIX_MJPG_CTRL		 		0x08
//#define XU_SONIX_OSD_CTRL	  			0x09
//#define XU_SONIX_MOTION_DETECTION		0x0A
//#define XU_SONIX_IMG_SETTING	 		0x0B

// ----------------------------- XU Control Selector ------------------------------------




#define UVC_CONTROL_SET_CUR	(1 << 0)
#define UVC_CONTROL_GET_CUR	(1 << 1)
#define UVC_CONTROL_GET_MIN	(1 << 2)
#define UVC_CONTROL_GET_MAX	(1 << 3)
#define UVC_CONTROL_GET_RES	(1 << 4)
#define UVC_CONTROL_GET_DEF	(1 << 5)
/* Control should be saved at suspend and restored at resume. */
#define UVC_CONTROL_RESTORE	(1 << 6)
/* Control can be updated by the camera. */
#define UVC_CONTROL_AUTO_UPDATE	(1 << 7)

#define UVC_CONTROL_GET_RANGE   (UVC_CONTROL_GET_CUR | UVC_CONTROL_GET_MIN | \
                                 UVC_CONTROL_GET_MAX | UVC_CONTROL_GET_RES | \
                                 UVC_CONTROL_GET_DEF)

struct uvc_xu_control_info {
    __u8 entity[16];
    __u8 index;
    __u8 selector;
    __u16 size;
    __u32 flags;
};

struct uvc_xu_control_mapping {
    __u32 id;
    __u8 name[32];
    __u8 entity[16];
    __u8 selector;

    __u8 size;
    __u8 offset;
    enum v4l2_ctrl_type v4l2_type;
    __u32 data_type;
};

struct uvc_xu_control {
    __u8 unit;
    __u8 selector;
    __u16 size;
    __u8 *data;
};

#define UVCIOC_CTRL_ADD		_IOW('U', 1, struct uvc_xu_control_info)
#define UVCIOC_CTRL_MAP		_IOWR('U', 2, struct uvc_xu_control_mapping)
#define UVCIOC_CTRL_GET		_IOWR('U', 3, struct uvc_xu_control)
#define UVCIOC_CTRL_SET		_IOW('U', 4, struct uvc_xu_control)

#define LENGTH_OF_SONIX_XU_CTR (3)
#define LENGTH_OF_SONIX_XU_MAP (3)



class BackCarMonitor
{
public:
	BackCarMonitor();
	~BackCarMonitor(){ if(handle > 0)close(handle);}
	
	int XU_ASIC_Get_Data(int fd, int Addr, __u8 *uData);
	int getBackCarState();
private:
	int handle;
	int g_unit;
	bool isXuCtrlInited;
};

#endif
