/*************************************************
Copyright (C), 2015, AllwinnerTech. Co., Ltd.
File name: softap_controller.h
Author: yinzh@allwinnertech.com
Version: 0.1
Date: 2015-11-23
Description:
    用于实现无线网卡作为softap的控制管理
History:
*************************************************/
#ifndef _SOFTAP_CONTROLLER_H
#define _SOFTAP_CONTROLLER_H

#include <utils/String8.h>
#include <utils/List.h>
#include <utils/Thread.h>
#include <hardware_legacy/softap.h>

#define DEFAULT_WIFI_PASSWORD      "12345678"

enum SOFTAP_STATUS {
    SOFTAP_DISABLED = 0,
    SOFTAP_ENABLED,
};

class SoftApController
{
    public:
        SoftApController();
        ~SoftApController();
        void EnableSoftAp();
        void DisableSoftAp(bool wait_exit=false);
        void SetSoftApIpAddr(const char *ip);
        void GetSoftApIpAddr(char *ip);
        int ResetSoftAp(const char *ssid, const char *pwd);
    private:
        class SoftApEnableThread : public android::Thread {
            public:
                SoftApEnableThread(SoftApController *sac)
                    : Thread(false)
                    , sac_(sac) {}
                android::status_t start() {
                    return run("SoftApEnableThread", android::PRIORITY_NORMAL);
                }
                bool threadLoop() {
                    sac_->softap_lock_.lock();
                    ::enableSoftAp();
                    sac_->softap_lock_.unlock();
                    return false;
                }

            private:
                SoftApController *sac_;
        };

        class SoftApDisableThread : public android::Thread {
            public:
                SoftApDisableThread(SoftApController *sac)
                    : Thread(false)
                    , sac_(sac) {}
                android::status_t start() {
                    return run("SoftApDisableThread", android::PRIORITY_NORMAL);
                }
                bool threadLoop() {
                    sac_->softap_lock_.lock();
                    ::disableSoftAp();
                    sac_->softap_lock_.unlock();
                    return false;
                }

            private:
                SoftApController *sac_;
        };

        int softap_status_;
        android::String8 ssid_;
        android::String8 pwd_;
        android::String8 ip_addr_;
        android::Mutex softap_lock_;
        android::sp<SoftApEnableThread> enable_thread_;
        android::sp<SoftApDisableThread> disable_thread_;

        void GenerateSSID();
}; // class SoftApController

#endif // softap_controller.h
