/*************************************************
Copyright (C), 2015, AllwinnerTech. Co., Ltd.
File name: wifi_manager.h
Author: yinzh@allwinnertech.com
Version: 0.1
Date: 2015-11-24
Description:
    wifi连接管理
History:
*************************************************/
#ifndef _WIFI_MANAGER_H
#define _WIFI_MANAGER_H

#include <utils/List.h>
#include <utils/Vector.h>
#include <utils/Thread.h>
#include <hardware_legacy/wifi_api.h>

#ifdef LOG_TAG
#undef LOG_TAG
#define LOG_TAG "wifi_manager.h"
#endif

enum WIFI_STATUS {
    WIFI_DISABLED = 0,
    WIFI_ENABLED,
    WIFI_SCANNING,
    WIFI_CONNECTING,
    WIFI_CONNECTED,
    WIFI_DISCONNECTED,
};

struct ApInfo {
    int8_t strength;
    char *ssid;
    const char *encryp_type;

    ~ApInfo() {
        free(ssid);
    }
};

class WifiManager
{
    public:
        WifiManager();
        ~WifiManager();
        void EnableWifi();
        void DisableWifi(bool wait_exit=false);
        android::Vector<ApInfo*> &GetScanResult();
        int GetWifiSignalStrength();
        int ConnectWifiAp(uint8_t scan_idx, const char *ssid, const char *pwd);
        void DisconnectWifiAp();

    private:
        class WifiEnableThread : public android::Thread {
            public:
                WifiEnableThread(WifiManager *wm)
                    : Thread(false)
                    , wm_(wm) {}
                android::status_t start() {
                    return run("WifiEnableThread", android::PRIORITY_NORMAL);
                }
                bool threadLoop();

            private:
                WifiManager *wm_;
        };
        class WifiDisableThread : public android::Thread {
            public:
                WifiDisableThread(WifiManager *wm)
                    : Thread(false)
                    , wm_(wm) {}
                android::status_t start() {
                    return run("WifiDisableThread", android::PRIORITY_NORMAL);
                }
                bool threadLoop();
            private:
                WifiManager *wm_;
        };
        class WifiConnectThread : public android::Thread {
            public:
                WifiConnectThread(WifiManager *wm)
                    : Thread(false)
                    , wm_(wm) {}
                android::status_t start() {
                    return run("WifiConnectThread", android::PRIORITY_NORMAL);
                }
                bool threadLoop();
            private:
                WifiManager *wm_;
        };
        int wifi_status_;
        android::Mutex wifi_lock_;
        android::sp<WifiEnableThread> enable_thread_;
        android::sp<WifiDisableThread> disable_thread_;
        android::sp<WifiConnectThread> connect_thread_;
        // 连接过的wifi ap列表
        wifi_connected_list configured_list_;
        android::Vector<ApInfo*> scan_list_;

        int RequesetIP();
        int ReleaseIP();
        void ClearScanList();
}; // class WifiManager

#endif // wifi_manager.h

