/*
   File description: These interfaces are invoked by web server
   */
#ifndef _YK_CMDS_CALLBACK_H_
#define _YK_CMDS_CALLBACK_H_

#include <youku/ykcamera_common.h>
#include "YoukuService.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <utils/Mutex.h>

extern android::Mutex gNewSpecialVideoMutex;

enum CameraMode {
	TAKE_PHOTO = 0,
	RECORD_VIDEO,
	MULTI_SHUT
};

class CdrMain;
class RecordPreview;
class ResourceManager;
class Menu;
class YoukuCmdsCallback
{
public:
	YoukuCmdsCallback(CdrMain *cdrMain);
	CdrMain *GetCdrMain();
	~YoukuCmdsCallback(){};
	/*
	 * 0: 1080P60FPS
	 * 1: 1080P30FPS
	 * 2: 720P120FPS
	 * 3: 720P60FPS
	 * 4: 720P30FPS
	 * 5: 2.7K30FPS
	 * 6: 4K15FPS
	 */
	int SetVideoResolutionFPS(int item, void *data);
	/*
	 * not using
	 */
	int SetVideoFPS(int item, void *data);
	/*
	 * 0: auto
	 * 1: daylight
	 * 2: cloudy-daylight
	 * 3: incandescent
	 * 4: fluorescent
	 */
	int SetVideoWriteBalance(int item, void *data);
	/*
	 * 0: close
	 * 1: 2min
	 * 2: 3min
	 * 3: 5min
	 */
	int SetVideoCyclicRecord(int item, void *data);
	// int SetAutoTakePicturePIV(int enable, void *data);
	/*
	 * 0: -3
	 * 1: -2
	 * 2: -1
	 * 3: 0
	 * 4: 1
	 * 5: 2
	 * 6: 3
	 */
	int SetVideoEV(int item, void *data);
	/*
	 * 0: disable
	 * 1: enable
	 */
	int SetVideoDateWaterPrint(int enable, void *data);
	/*
	 * 0: disable
	 * 1: enable
	 */
	int SetAudioRecord(int enable, void *data);
	/*
	 * 0: 2M
	 * 1: 5M
	 * 3: 8M
	 * 4: 12M
	 * 5: 16M
	 */
	int SetPhotoResolution(int item, void *data);
	/*
	 * 0: 3p/s
	 * 1: 5p/s
	 * 2: 10p/s
	 */
	int SetMultiCaptureRate(int item, void *data);
	/*
	 * same with SetVideoWriteBalance
	 */
	int SetPhotoWriteBalance(int item, void *data);
	/*
	 * same with SetVideoEV
	 */
	int SetPhotoEV(int item, void *data);
	/*
	 * 0: close
	 * 1: 3s
	 * 2: 5s
	 * 3: 10s
	 */
	int SetPhotoCountDownTake(int item, void *data);
	/*
	 * 0: disable
	 * 1: enable
	 */
	int SetPhotoDateWaterPrint(int enable, void *data);
	/*
	 * 0: 480x270
	 * 1: 640x360
	 */
	int SetStreamResolution(int item, void *data);
	/*
	 * 0: 800bps
	 * 1: 1Mbps
	 * 2: 2Mbps
	 */
	int SetStreamBitrate(int item, void *data);
	/*
	 * 0: 10s
	 * 1: 20s
	 * 2: 30s
	 * 3: close
	 */
	int SetLCDSleep(int item, void *data);
	/*
	 * 0: disable
	 * 1: enable
	 */
	int SetLEDBlink(int enable, void *data);
	/*
	 * 0: close
	 * 1: 50%
	 * 2: 100%
	 */
	int SetBeepVolume(int item, void *data);
	/*
	 * 0: close
	 * 1: 0.5s/f
	 * 2: 1s/f
	 * 3: 2s/f
	 * 4: 5s/f
	 * 5: 10s/f
	 * 6: 30s/f
	 * 7: 60s/f
	 */
	int SetTimeLapseRate(int item, void *data);
	/*
	 * photo mode:
	 *	  0/1: take picture
	 * video mode:
	 *    0: stoprecord
	 *    1: startrecord
	 */
	int DoShoot(int enable, void *data);
	/*
	 * 0: take photo
	 * 1: video mode
	 * 2: multi shut
	 */
	int ChangeCameraMode(int item, void *data);
	int SetAutoPowerOff(int item, void *data);
	int FactoryMode(int enable, void *data);
	int FormatSDCard(int enable, void *data);
	int SetSSID(int enable, void *data);
	int SetPowerOff(int enable, void *data);
	int DeleteMediaFile(int enable, void *data);
	int DeleteRecentlyMediaFile(int enable, void *data);
	int CleanSDCard(int enable, void *data);
	int SetSystemlang(int language);
	int SetLCDDispaly(bool on);
	int SetSystemDateTime(int item, void *data);
	/*
	 * 0: disable
	 * 1: enable
	 */
	int EnableStream(int enable, void *data);
	int GetDeviceInfo(int enable, void *data);
	int CleanNewSpecialVideoFlag(int enable, void *data);
	YoukuDeviceControl &GetYoukuDevCtrl();
	void Init(YKStreamSender *sender);
	int ReSetupYoukuSend();
private:
	YKStreamSender *mStreamSender;
	ResourceManager *mRM;
	CdrMain *mCdrMain;
	RecordPreview *mRP;
	Menu *mMenu;
	YoukuDeviceControl mDevCtrl;
	int mCameraMode;
};

#endif
