#ifndef _STORAGE_MANAGER_H
#define _STORAGE_MANAGER_H

#include <sys/vfs.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>

#include <fcntl.h>

#include <utils/String8.h>
#include <StorageMonitor.h>
#include <DBCtrl.h>
#include <private/media/VideoFrame.h>

#include <minigui/common.h>
#include <minigui/gdi.h>

#include "Process.h"
#include "misc.h"
#include "cdr_res.h"
#include "camera.h"

#include "Rtc.h"
#include "EventManager.h"

using namespace android;

long long totalSize(const char *path);
long long availSize(const char *path);

#define MOUNT_POINT STORAGE_POINT
#define MOUNT_PATH STORAGE_PATH
#define SZ_1M ((unsigned long long)(1<<20))
#define SZ_1G ((unsigned long long)(1<<30))
//#define RESERVED_SIZE (SZ_1M * 300)
#define RESERVED_SIZE ((unsigned long long)((SZ_1M*500)/SZ_1M))
#define LOOP_COVERAGE_SIZE (RESERVED_SIZE + ((SZ_1M*500)/SZ_1M))

#define FILE_SZ_1M ((uint32_t)(1*1024*1024))

#define ALIGN( num, to ) (((num) + (to-1)) & (~(to-1)))
#define ALIGN32M(num) (ALIGN(num, (1<<25)))

#define MINUTE(ms) (ms/1000)
#define CSI_PREALLOC_SIZE(ms) (ALIGN32M((MINUTE(ms)*FRONT_CAM_BITRATE/8)))
#define UVC_PREALLOC_SIZE(ms) (ALIGN32M((MINUTE(ms)*BACK_CAM_BITRATE/8)))
#define SMALL_VIDEO_SIZE(ms) (ALIGN32M((MINUTE(ms)*SMALL_VIDEO_BITRATE/8)))
#define AWMD_PREALLOC_SIZE(ms) CSI_PREALLOC_SIZE(ms)


#define CDR_TABLE "CdrFile"

#define FILE_EXSIST(PATH)	(!access(PATH, F_OK))

#define FLAG_NONE ""
#define FLAG_SOS "_SOS"
#define FLAG_WOND "_WON"

#define FILE_FRONT "A"
#define FILE_BACK "B"

#define EXT_VIDEO_MP4 ".mp4"
#define EXT_PIC_JPG ".jpg"
#define EXT_PIC_BMP ".bmp"
#define EXT_NET_PIC	"_net"

#define GENE_IMPACT_TIME_FILE  "/data/impcttime.db"
#define IMPACT_TIME_TEXT_LENGTH   20
#define IMPACT_TIME_TEXT  128

enum {
	RET_IO_OK = 0,
	RET_IO_NO_RECORD = -150,
	RET_IO_DB_READ = -151,
	RET_IO_DEL_FILE = -152,
	RET_IO_OPEN_FILE = -153,
	RET_NOT_MOUNT = -154,
	RET_DISK_FULL = -155,
	RET_NEED_LOOP_COVERAGE = -156
};

typedef struct elem
{
	char *file;
	long unsigned int time;
	char *type;
	char *info;
    int filesize;
}Elem;

typedef struct {
	String8 file;
	int time;
	String8 type;
	String8 info;
}dbRecord_t;

enum FOLDER_TYPE{
	ALL_FOLDER = 0,
	WONDERFUL_FOLDER ,
	SOS_FOLDER ,
	PHOTO_FOLDER ,
	VIDEO_FOLDER ,
};

class CommonListener
{
public:
	CommonListener(){};
	virtual ~CommonListener(){}
	virtual void finish(int what, int extra) = 0;
};

#ifdef __cplusplus
extern "C" {
#endif
extern int Rgb2Jpg(unsigned char *src_buffer, const char *jpeg_file, int width, int height, int depth);
#ifdef __cplusplus
} 
#endif


class StorageManager : public StorageMonitorListener
{
private:
	void init();
	StorageManager();
	~StorageManager();
	StorageManager& operator=(const StorageManager& o);
public:
	static StorageManager* getInstance();
	void exit();
	void notify(int what, int status, char *msg);
	bool isInsert();
	bool isMount();
	int doMount(const char *fsPath, const char *mountPoint,
	             bool ro, bool remount, bool executable,
	             int ownerUid, int ownerGid, int permMask, bool createLost);
	int format(CommonListener *listener);
	int reNameFile(const char *oldpath, const char *newpath);
	void dbInit();
	void dbExit();
	void dbReset();
	int dbCount(char *item);
	void dbGetFiles(char *item, Vector<String8>&file);
	void dbGetFile(char *item, String8 &file );
	int dbGetSortByTimeRecord(dbRecord_t &record, int index);
	void dbAddFile(Elem *elem);
	int dbDelFile(char const *file);
	int generateFile(time_t &now, String8 &file, int cam_type, fileType_t type, bool nextFd = false);
	int saveVideoFrameToBmpFile(VideoFrame* videoFrame, const char* bmpFile, char* pic_type = EXT_PIC_BMP);
	int savePicture(void* data, int size, String8 &file);
	int checkDirs();
	int dbUpdate();
	int dbUpdate(char *type);
	int deleteFile(const char *file);
	int deleteAllFiles();
	int deleteRecentlyFile();
	bool dbIsFileExist(String8 file);
	int shareVol();
	int unShareVol();
	bool isDiskFull();
	int fileSize(String8 path);
	void dbUpdateFile(const char* oldname, const char *newname);
	String8 lockFile(const char *path);
	int getCapacity(unsigned long long *as, unsigned long long *ts);
	int getRemainingPictures(void);
	int getRemainingVideoTime(bool uvc_used);
	void stopUpdate();
    void DdInitnotify(int msg);
	class InitDbThread : public Thread
	{
	public:
		InitDbThread(StorageManager *storage): Thread(false),sm(storage){	}
		status_t start() {
			return run("InitDbThread", PRIORITY_NORMAL);
		}
		virtual bool threadLoop() {
			sm->DBInitLock(true);
			sm->dbInit();
			sm->DBInitLock(false);
            //delete LOST.DIR save tfcard space
            system("rm -rf /mnt/extsd/LOST.DIR");
            sm->DdInitnotify(EVENT_DB_INITEND);
			return false;
		}

	private:
		StorageManager *sm;
	};

	class FormatThread : public Thread
	{
	public:
		virtual bool threadLoop() {
			mSM->format();
			mSM->dbReset();
			mListener->finish(0, 0);
			return false;
		}
		status_t start() {
			return run("FormatThread", PRIORITY_NORMAL);
		}
		FormatThread(StorageManager *sm, CommonListener *listener) {mSM = sm; mListener = listener;};
	private:
		StorageManager *mSM;
		CommonListener *mListener;
	};
	void setEventListener(EventListener *pListener);
	int  readCpuInfo(char ret[]);
	int geneImpactTimeFile();
	int readImpactTimeFile(char ret[]);


	class StorageSyncThread : public Thread
	{
	public:
		virtual bool threadLoop() {
			storageSyncLoop();
			return false;
		}
		status_t start() {
			return run("StorageSyncThread", PRIORITY_NORMAL);
		}
		void stopThread()
		{
			mLoop = false;
		}
		void storageSyncLoop();
		StorageSyncThread(StorageManager *sm, bool loop=false);
	private:
		StorageManager *mSM;
		bool mLoop;
	};
	void startSyncThread(bool loop=false);
	void stopSyncThread();
	void storageSync();
	void DBInitLock(bool flag);
	char* getFileType(const char* file);
	int delNetBmp(const char* file);
	int judgeFileFolder(const char* file);
	int geneNetPicName(String8 file, char netbmp[], char* image_type);
	int geneNetPicName(String8 file, String8 &netFileName, char* image_type);
	void generateSsid(char ssid[]);
private:
	int isInsert(const char *checkPath);
	int isMount(const char *checkPath);
	int deleteFiles(const char *type, const char *info, bool isDeleteAll);
	int deleteFiles();
	int format();
	StorageMonitor *mStorageMonitor;
	DBCtrl *mDBC;
	Mutex mDBLock;
	sp <FormatThread> mFT;
	sp <StorageSyncThread> mSST;
	bool mInsert;
	bool mMounted;
	EventListener *mListener;
	bool mDBInited;
	Mutex mDBInitL;
	sp<InitDbThread> mInitDbTh;
	bool mStopDBUpdate;
    bool mDbInitStat;
};
#endif
