#ifndef _WINDOWS_H
#define _WINDOWS_H
#include <platform.h>
#include <CedarMediaPlayer.h>
#include <utils/KeyedVector.h>
#include "debug.h"

#include <minigui/common.h>
#include <minigui/minigui.h>
#include <minigui/gdi.h>
#include <minigui/window.h>


#define DE2


#include "RecordPreview.h"
#include "CdrCamera.h"
#include "CdrMediaRecorder.h"
#include "PlayBack.h"

#include "PowerManager.h"
#include "StorageManager.h"
#include "ResourceManager.h"
#include "cdr_widgets.h"
#include "EventManager.h"
#include "PicObj.h"
#include "CdrServer.h"
#include "softap_controller.h"

#ifdef YOUKU_PLATFORM
#include "YoukuService.h"
#include "YoukuCmdsCallback.h"
#endif


class CdrServer;
class MainWindow;

typedef struct 
{
	int idx;
	MainWindow *win;
	timer_t id;
	time_t time;
}timer_set;


enum playerState {
	STATE_ERROR					= -1,
	STATE_IDLE					= 0,
	STATE_PREPARING				= 1,
	STATE_PREPARED				= 2,
	STATE_STARTED				= 3,
	STATE_PAUSED				= 4,
	STATE_PLAYBACK_COMPLETED	= 5,
	STATE_STOPPED				= 6,
	STATE_END					= 7
};

enum {
	FORBID_NORMAL = 0,
	FORBID_FORMAT,
};

#define CameraIdRecordNum 2

#define WINDOW_STATUSBAR		"CDRStatusBar"
#define WINDOW_RECORDPREVIEW	"RecordPreview"
#define WINDOW_MENU				"CDRMenu"
#define WINDOW_PLAYBACKPREVIEW	"PlayBkPreview"
#define WINDOW_PLAYBACK			"PlayBack"
#define DOSBOOTBLOCKSIZE 		512
#define UPDATE_FILE_PATH		STORAGE_PATH"full_img.fex"


extern int RegisterCDRWindows(void);
extern void UnRegisterCDRWindows(void);

typedef enum {
	STATUS_RECORDPREVIEW = 1,
	STATUS_PHOTOGRAPH = 2,
	STATUS_AWMD	= 3
}RPWindowState_t;

typedef enum {
	PARK_MON_VTL_10S = 10,
	PARK_MON_VTL_15S = 15,
	PARK_MON_VTL_20S = 20,
	PARK_MON_VTL_30S = 30,
}PARK_MON_VTL_t;


typedef struct {
	unsigned int curWindowID;
	RPWindowState_t RPWindowState;
}windowState_t;

#ifdef YOUKU_PLATFORM
class YoukuService;
#endif
class CdrMain;
class PowerManager;
class MainWindow
{
public:
	MainWindow(){mHwnd = 0;}
	~MainWindow(){};
	HWND getHwnd() {
		return mHwnd;
	}

	void updateWindowFont(void)
	{
		HWND hChild;

		hChild = GetNextChild(mHwnd, 0);
		while( hChild && (hChild != HWND_INVALID) )
		{
			SetWindowFont(hChild, GetWindowFont(mHwnd));
			hChild = GetNextChild(mHwnd, hChild);
		}
	}
	void showWindow(int cmd) {
		ShowWindow(mHwnd, cmd);
	}
	void keyProc(int keyCode, int isLongPress);
protected:
	HWND mHwnd;
};

class StatusBar : public MainWindow
{
public:
	StatusBar(CdrMain* cdrMain);
	~StatusBar();

	void msgCallback(int msgType, unsigned int data);
	void handleWindowPicEvent(windowState_t *windowState);
	int createSTBWidgets(HWND hParent);
	void updateTime(HWND hStatusBar);
	int updateRecordTime(bool setLabelBlank=false);
	int increaseRecordTime();
	int initPic(PicObj *po);
	int handleRecordTimerMsg(unsigned int message, unsigned int recordType);
	void setRecordVTL(unsigned int vtl);
	unsigned int getCurrenWindowId();
	void setCountOnce(bool val);
	HWND getLabelHwnd();
	void setLabelHwnd(HWND hwnd);
    HWND getTimeHwnd();
    void setTimeHwnd(HWND hwnd);
    HWND getFileNameHwnd();
	void setLabelDisp(bool val);
	bool getLabelDisp();
	void stopRecordTimer();
    void startSystemTimeTimer();
	void stopSystemTimeTimer();
	HWND getHwnd();
	class Timer
	{
		public:
			Timer(MainWindow *win);
			~Timer();
			void init(void(*callback)(sigval_t), int timer_idx);
			void start(unsigned int time_sec, bool isOneShot=false);
			void stop();
			bool timerStoped();
		private:
			MainWindow *mWin;
			timer_set mTimerdata;
			bool mTimerStarted;
	};
	void needStop();
    bool mScreenShotText;
private:
	Vector <PicObj*> mPic;
	CdrMain* mCdrMain;
	SWORD  mRecordTime;
	DWORD  mRecordVTL;
	DWORD  mCurRecordVTL;
	unsigned int mCurWindowID;
	bool mCountOnce;
	Timer *mRecordTimer;
	Timer *mSystemTimeTimer;
	Mutex mTimerLock;
	HWND mLabelHwnd;
    HWND mTimeHwnd;
    HWND mFileNameHwnd;
	bool mLabelDisp;
};

class MediaRecorderObj	//this class contains cdrmediarecord, easy to control record for each object
{
public:
	MediaRecorderObj(CdrMediaRecorder *cr, bool session):mMediaRecord(cr),hasSession(session){};
	bool getSession() {  return hasSession; };
	CdrMediaRecorder * getCdrMediaRecorder() { return mMediaRecord; };
	void setSession(bool flag) { hasSession = flag; };
	void setCdrMediaRecord(CdrMediaRecorder *cr){ mMediaRecord = cr;  };
	void setUseCam(int id) {  mUseCam = id; };
	int getUseCam() {  return mUseCam; } ;
private:
	CdrMediaRecorder *mMediaRecord;
	bool hasSession;	// values: true net_record, value:false cam_record
	int mUseCam;  //which camera was used(values :  CAM_CSI: 0; CAM_UVC: 1)
};

class RecordPreview : public MainWindow, public RecordListener, public ErrorListener
{
public:
	RecordPreview(CdrMain* cdrMain);
	~RecordPreview();
	void closecarBackCSI();
	void init();
	int createWindow();
	virtual void recordListener(CdrMediaRecorder* cmr, int what, int extra);
	virtual void errorListener(CdrMediaRecorder* cmr, int what, int extra);
	int startRecord();
	int startRecord(int cam_type);
	int startRecord(int cam_type, CdrRecordParam_t param);
	int stopRecord(int cam_type);
	int stopRecord();
	int stopRecordSync();
	void startPreview();
	void startPreview(int cam_type);
	void stopPreview(int cam_type);
	void stopPreview();

	bool isRecording(int cam_type);
	bool isRecording();
	bool isPreviewing(int cam_type);
	bool isPreviewing();
	CdrDisplay *getCommonDisp(int disp_id);

	int keyProc(int keyCode, int isLongPress);
	int msgCallback(int msgType, int idx);

	int setCamExist(int cam_type, bool bExist);
	bool isCamExist(int cam_type);
	void feedAwmdTimer(int seconds);
	int queryAwmdExpirationTime(time_t* sec, long* nsec);
	void updateAWMDWindowPic();
	void savePicture(void* data, int size, int id);
	int impactOccur();
    void prepareCamera4Playback(bool closeLayer=true);
	void restorePreviewOutSide(unsigned int windowID);

	void switchVoice();
	void setSdcardState(bool bExist);
	void otherScreen(int screen);
	void storagePlug(bool plugin);
	bool hasTVD(bool &hasTVD);
	bool needDelayStartAWMD(void);	/* set flag, delay some time after stopRecord, when awmd is open */
	void setDelayStartAWMD(bool value);
	void restoreZOrder();
	unsigned int mAwmdDelayCount;
	bool getUvcState();
	void takePicFinish(int cam_id);
	void setRecordTime(PARK_MON_VTL_t idx);
	void releaseRecorder(int cam_id);
	bool isPhotoWindow();
	
	void sessionInit(void *rtspSession, int camId);
	void setSessionSize(int idx);
	void setSessionBitrate(int idx);
	int camChange(int cam_idx, bool bStop=true);
	bool camExisted(int idx);
#ifdef PLATFORM_0
	int getHdl(int disp_id);
	void createVirtualCam();
	void destroyVirtualCam();
	void setVirtualCam(CdrMediaRecorder *cmr);
#endif
	void changeResolution(int level);
	unsigned int  getWindowsID();
	bool getAlignLine();
	void setAlignLine(bool flag);
	void lockFile(int camId);
	void adasScreenSwitch(bool flag);
	bool getIsAdasScreen();
	void getAdasCallback();
	int getDuration();
	void needCache(bool val);
	void startPCCam();
	void stopPCCam();
	CdrCamera *getCdrCamera(int camId);
	RPWindowState_t getRPWindowState();
	void setRPWindowState(RPWindowState_t rWindowStat);
	bool checkSDIsFAT();
	void errorOccur(int error);
	void resume();
	void suspend();
	void createMRObjList();
	MediaRecorderObj* getMediaRecordObj(enum CameraIdRecord id);
	int startSmallVideoRecord();
	int CSIScreenShot();
	int handSmallVideo(Elem elem);
	int generateNetPic(String8 filename, char* file_type, String8 &picFile);
	void camTakePicture(int cam);
	bool IsImpactFileStart();
	void handWakeUpRecord(StorageManager* sm);
	void setADAS(bool val, int tips = 1);
	int getCurSessionCam();
	void setCurSessionCam(int camId);
	void screenShot();
	CdrMain * getCdrMain();
	void carBackTransPIP(bool flag);
	void setUvcCarModem(bool val);
	bool getUvcCarModem();
	void showUvcCarBackLines(bool val);
	void switchAWMD(bool enable);
    void setRtspSession(bool flag);
#if defined(VIDEO_CONFIG_TEXT_TRACK) && defined(VIDEO_CONFIG_TEXT_GSR)
    void addGsensorData(float xa, float ya, float za);
#endif
#if defined(VIDEO_CONFIG_TEXT_TRACK) && defined(VIDEO_CONFIG_TEXT_GPS)
    class SendGpsThread: public Thread{
        public:
            SendGpsThread(RecordPreview *mp)
                : Thread(false)
            {
                gps_fd = open("/dev/ttyS2", O_RDONLY);  // gps module send data to AP by serial port
                if (gps_fd < 0) {
                    db_error("can not open gps dev!!!");
                    gps_fd = 0;
                }
                isRun = false;
            }
            ~SendGpsThread(){}
            virtual bool threadLoop() {
                if(isRun == true) {
                    /* software simulate GPS NEMA-0183-$GPRMC */
//                    static int i;
//                    const char *raw_data[] = {"$GPRMC,092750.000,A,2223.0005,N,11333.0001,E,14.62,31.66,280511,,,A*43$",
//                                    "$GPRMC,092750.000,A,2223.0015,N,11333.0021,E,08.21,31.66,280511,,,A*43$",
//                                    "$GPRMC,092750.000,A,2223.0025,N,11333.0041,E,21.67,31.66,280511,,,A*43$",
//                                    "$GPRMC,092750.000,A,2223.0035,N,11333.0061,E,16.62,31.66,280511,,,A*43$",
//                                    "$GPRMC,092750.000,A,2223.0045,N,11333.0081,E,14.62,31.66,280511,,,A*43$",
//                                    "$GPRMC,092750.000,A,2223.0055,N,11333.0101,E,19.62,31.66,280511,,,A*43$",
//                                    "$GPRMC,092750.000,A,2223.0065,N,11333.0121,E,14.62,31.66,280511,,,A*43$",
//                                    "*sd$GPRMC,092750.000,A,2223.0075,N,11333.0141,E,09.62,31.66,280511,,,A*43$",
//                                    "$GPRMC,092750.000,A,2223.0085,N,11333.0081,E,24.62,31.66,280511,,,A*43$",
//                                    "$GPRMC,092750.000,A,2223.0095,N,11333.0061,E,16.62,31.66,280511,,,A*43$"};
//                    String8 data(raw_data[i++%10]);
//                    pcmr->sendGpsData(data);
//                    usleep(1000*1000);
//                    return true;

                    /* use real gps module */
                    char buf[1024];
                    if (gps_fd > 0) {
                        read(gps_fd, buf, 1024);
                        String8 data(buf);
                        if ((strncmp(data.string(), "$GPRMC", 6) == 0) && (pcmr->getRecorderState() == RECORDER_RECORDING)) {
                            pcmr->sendGpsData(data);
                        }
                        return true;
                    } else {
                        return false;
                    }
                } else {
//                    if (gps_fd > 0) {
//                        close(gps_fd);
//                        gps_fd = 0;
//                    }
                    return false;
                }
            }
            status_t startThread() {
                isRun = true;
                return run("SendGpsThread", PRIORITY_NORMAL);
            }
            void stopThread() {
                if (gps_fd != 0) {
                    return;
                }
                isRun = false;
            }
            bool isRunning() {
                return isRun;
            }
            void setPCMR(CdrMediaRecorder *ptr) {
                pcmr = ptr;
            }

        private:
            int gps_fd;
            bool isRun;
            CdrMediaRecorder *pcmr;
    };
#endif
private:
	int initCamera(int cam_type, bool needNewCamera = true);
	int prepareRecord(int cam_type, CdrRecordParam_t param, Size *resolution=NULL);


	void recordRelease(int cam_type);
	void recordRelease();

	void previewRelease(int cam_type);
	void previewRelease();

	void setSilent(bool mode);
	void setVideoQuality(int idx);
	void setPicQuality(int idx);
	void setRecordTime(int idx);
	void setAWMD(bool val);	
	void setWatermark(int flag);
	void switchAWMD();
	int transformPIP(pipMode_t newMode);
	int takePicture(int flag, int mode = 1); //flag: 1 take picture, 2 screen shot ; mode: 1 CSI and UVC take pic or screen shot, 2 only csi take pic or screen shot 
	void setDeleteFlag(int cam_type, String8 file);
	void clrDeleteFlag(int cam_type, String8 file);
	String8 getDeleteFlag(int cam_type);
	CdrMain *mCdrMain;
	CdrCamera *mCamera[CAM_CNT];
//	CdrMediaRecorder *mRecorder[CAM_CNT+2];
#if defined(VIDEO_CONFIG_TEXT_TRACK) && defined(VIDEO_CONFIG_TEXT_GPS)
    sp<SendGpsThread> mSGpsThread;
#endif

	Mutex mLock;
	Mutex mLockPic;
	Size mVideoSize;
	RPWindowState_t mRPWindowState, mOldRPWindowState;
	pipMode_t mPIPMode, oldPIPMode;
	recordDBInfo_t mRecordDBInfo[CAM_CNT];
	recordDBInfo_t mExtraDBInfo[CAM_CNT];
	int mDuration;
	bool mCamExist[CAM_CNT];
	String8 mNeedDelete[CAM_CNT];
	CDR_RECT rectBig, rectSmall;
	bool mUvcPlugFlag;
	Mutex mImpactLock;
	Mutex mRecordLock;
	Mutex mDeleteLock;
	bool mLockFlag[CAM_CNT];
	int mImpactFlag[CAM_CNT];
	String8 mBakFileString[CAM_CNT];
	time_t mBakNow[CAM_CNT];
	bool mHasTVD;
	int mUVCNode;
	bool mAwmdDelayStart;
	Mutex mAwmdLock;
	bool mSilent;
	bool mAllowTakePic[CAM_CNT];
	
	bool mStartRtspSession;
	bool mAlignLine;
	bool mIsAdasScreen;
#ifdef PLATFORM_0
	VirtualCamera *mVirHC;
#endif
	Size mSessionSize;
	uint32_t mSessionBitrate;
	void *mSession;
	int mCurSessionCam;
	bool mNeedCache;
	String8 mNeedDelFile[CAM_CNT];
	bool mKeyStop;
	KeyedVector <enum CameraIdRecord, class MediaRecorderObj*> mMRObjList;  //recordlist 
	smallVideoDBInfo_t mSmallRecordDBInfo;
	bool mSmallRecordTakePic;
	String8 mTakePicName[CAM_CNT];
	int mImpactFileStart[CAM_CNT];
	BITMAP backCarBmp;
	bool mUVCCarModem;
};

class PlayBackPreview : public MainWindow
{
public:
	PlayBackPreview(CdrMain* pCdrMain);
	~PlayBackPreview();

	int keyProc(int keyCode, int isLongPress);
	int createSubWidgets(HWND hWnd);

	int getFileInfo(void);
	int updatePreviewFilelist(void);
	void showThumnail();
	int getPicFileName(String8 &picFile);
	int getNetPicFileName(String8 &bmpFile, String8 file_name, char* file_type);
	int getBmpPicFromVideo(const char* videoFile, const char* picFile, bool netbmp);
	bool isCurrentVideoFile(void);
	void nextPreviewFile();
	void prevPreviewFile();
	void getCurrentFileName(String8 &file);
	void getCurrentFileInfo(PLBPreviewFileInfo_t &info);
	void showPlaybackPreview();
	void deleteFileDialog();
	void showBlank();
	void fixIdx();
#ifdef PLATFORM_0
	void otherScreen(int screen);
	int beforeOtherScreen();
	int afterOtherScreen(int layer, int screen);
#endif
	int getFileList(char **file_list, int len, enum FOLDER_TYPE folder_type = ALL_FOLDER);
	int msgCallback(int message, int data);
	int mCurID;
	int generateNetPic();
#ifdef USE_IPS_SCREEN
    HWND bottomHwnd;
#endif
	class GeneNetPicTh : public Thread			//generate bmp for translate app  use in turn on cdr
	{
	public:
		GeneNetPicTh(PlayBackPreview *pLB): Thread(false),mPlb(pLB){	}
		status_t start() {
			return run("GeneNetPicTh", PRIORITY_NORMAL);
		}
		virtual bool threadLoop() {
			mPlb->generateNetPic();
			return false;
		}

	private:
		PlayBackPreview *mPlb;
	};

private:
	CdrMain* mCdrMain;
	BITMAP bmpIcon;
	BITMAP bmpImage;
	PLBPreviewFileInfo_t fileInfo;
	int playBackVideo();
	sp<GeneNetPicTh> mGeneNetTh;
#ifdef PLATFORM_0
	int mDispScreen;
	CDR_RECT mDispRect;
#endif
};

class PlayBack : public MainWindow
{
public:
	PlayBack(CdrMain *cdrMain);
	~PlayBack();
	int createSubWidgets(HWND hWnd);
	int keyProc(int keyCode, int isLongPress);
	void setDisplay(int hlay);
	int preparePlay(String8 filePath);
	int PreparePlayBack(String8 filePath);
	int startPlayBack();
	void start();
	void pause();
	void seek(int msec);
	void stop();
	void release();
	void reset();
	playerState getCurrentState();
	void initProgressBar(void);
	void resetProgressBar(void);
	void updateProgressBar(void);
	bool isStarted();
	void noWork(bool idle);
	void prepareCamera4Playback();
	int stopPlayback(bool bRestoreAlpha=true);
	void sendToPBP(int mId);
	int mCurId;
	void setPlayerVol(int val);
#ifdef PLATFORM_0
	int restoreAlpha();
#endif
	class PlayBackListener : public CedarMediaPlayer::OnPreparedListener
							, public CedarMediaPlayer::OnCompletionListener
							, public CedarMediaPlayer::OnErrorListener
							, public CedarMediaPlayer::OnVideoSizeChangedListener
							, public CedarMediaPlayer::OnInfoListener
							, public CedarMediaPlayer::OnSeekCompleteListener
	{
	public:
		PlayBackListener(PlayBack *pb){mPB = pb;};
		void onPrepared(CedarMediaPlayer *pMp);
		void onCompletion(CedarMediaPlayer *pMp);
		bool onError(CedarMediaPlayer *pMp, int what, int extra);
		void onVideoSizeChanged(CedarMediaPlayer *pMp, int width, int height);
		bool onInfo(CedarMediaPlayer *pMp, int what, int extra);
		void onSeekComplete(CedarMediaPlayer *pMp);
	private:
		PlayBack *mPB;
	};
public:
	playerState mCurrentState;
	playerState mTargetState;
	/* if user send the msg MSG_PLB_COMPLETE with wParam 0, 
	 * then mStopFlag will be set to 1, indicate onCompletion not send MSG_PLB_COMPLETE again*/
	unsigned int mStopFlag;	
private:
	CdrMain *mCdrMain;
	CdrDisplay *mCdrDisp;
	CedarMediaPlayer *mCMP;
	PlayBackListener *mPlayBackListener;
	BITMAP bmpIcon;
	BITMAP bmpProgressBar;
	int mDuration;
};

class Menu : public MainWindow , public CommonListener
{
public:
	Menu(CdrMain* pCdrMain);
	~Menu();

	int createSubWidgets(HWND hWnd);
	int createMenuListContents(HWND hWnd);

	int keyProc(int keyCode, int isLongPress);
	int updateNewSelected(int oldSel, int newSel);

	int ShowSubMenu(unsigned int menuIndex, bool &isModified);

	int HandleSubMenuChange(unsigned int menuIndex, int newSel);
	int HandleSubMenuChange(unsigned int menuIndex, bool newValue);

	int updateMenuTexts(void);
	int updateSwitchIcons(void);

	int ShowMessageBox(unsigned int menuIndex,int val=0);
	int ShowDateDialog();
	int ShowFormattingTip(int val=0);

	void doFormatTFCard(void);

	void doFactoryReset(void);
	void finish(int what, int extra);
	void cancelAlphaEffect(void);
	CdrMain * getCdrMain();
  	void setSubMenuFlag(bool flag);
    bool getSubMenuFlag();
	int updateCheckBoxImage(int newSel);
	int showLicensePlateWM(void);
private:
	int getMenuListAttr(menuListAttr_t &attr);
	int getItemImages(MENULISTITEMINFO*  mlii);
	int getItemStrings(MENULISTITEMINFO* mlii);
	int getFirstValueStrings(MENULISTITEMINFO *mlii, int menuIndex);
	int getFirstValueImages(MENULISTITEMINFO* mlii, int menuIndex);
	int getSecondValueImages(MENULISTITEMINFO* mlii, MLFlags* mlFlags);

	int getSubMenuData(unsigned subMenuIndex, subMenuData_t &subMenuData);
	int getCheckBoxStatus(unsigned int menuIndex, bool &isChecked);
	int getMessageBoxData(unsigned int menuIndex, MessageBox_t &messageBoxData);

	void showAlphaEffect(void);
	CdrMain* mCdrMain;
	BITMAP *itemImageArray;
	BITMAP *value1ImageArray;
	BITMAP unfoldImageLight;
	BITMAP unfoldImageDark;
	bool mSubMenu;
};

typedef enum {
	TIMER_LOWPOWER_IDX,
	TIMER_NOWORK_IDX,
	TIMER_NOWORK_IDX2,
	TIMER_RECORD_SHUTDOWN_IDX,
	TIMER_RECORD_IDX,
	TIMER_SYSTEM_TIME_IDX,
}timer_idx_enum;

class CdrMain : public MainWindow, EventListener
{
public:
	CdrMain();
	~CdrMain();
	void initPreview(void* none);
	void initStage2(void* none);
	int msgCallback(int msgType, unsigned int data);
	void exit();
	int createMainWindows(void);
	int createOtherWindows(void);
	void msgLoop();
	HWND getWindowHandle(unsigned int windowID);
	MainWindow* windowID2Window(unsigned int windowID);
	void setCurrentWindowState(windowState_t windowState);
	void setCurrentWindowState(RPWindowState_t status);
	void changeWindow(unsigned int windowID);
	void changeWindow(unsigned int fromID, unsigned int toID);
	void DispatchKeyEvent(MainWindow *mWin, int keyCode, int isLongPress);
	void keyProc(int keyCode, int isLongPress);
	CdrDisplay *getCommonDisp(int disp_id);
	void configInit();
	int notify(int message, int data);
	void forbidRecord(int val);
	int isEnableRecord();
	unsigned int getCurWindowID();
	void prepareCamera4Playback(bool closeLayer=true);
	bool isScreenOn();
	void setOffTime();
	void setUSBStorageMode(bool enable);
	bool getUSBStorageMode();
	bool getWakeupState();
	void setWakeupState(bool flag);
	void shutdown();
	void noWork(bool idle);
	bool isRecording();
	bool isPlaying();
	bool isBatteryOnline();
	void showShutdownLogo();
	bool isShuttingdown();
	bool mShutdowning;
	void setSTBRecordVTL(unsigned int vtl);
	bool isPhotoWindow();
#ifdef PLATFORM_0
	int getHdl(int disp_id);
#endif
	bool getUvcState();
/* NAT */
#ifdef APP_WIFI
    int networkInit();
    SoftApController *getSoftApController();
#endif
#ifdef YOUKU_PLATFORM
	static int ykCamCmdsHandler(int cmd, handler_parameter *param, void *data);
	YoukuService *getYoukuService();
	void initYoukuService();
	friend class YoukuCmdsCallback;
	static YoukuCmdsCallback *mYoukuCmdsCallback;
#endif // YOUKU_PLATFORM
	void sessionChange(bool connect);
	int sessionControl(int sid, int cmd, char* buf);
	int sessionServerInit();
	int lockFile(const char *file);
	int sessionNotify(int cmd, char* param_in, int param_len, int sid = -1);
	int camChange(bool session, int param_in);
	bool isWifiConnecting();
	void otaUpdate();
	bool getFormatTFAdasS();
	void setFormatTFAdasS(bool flag);
	bool getNeedCheckSD();
	void setNeedCheckSD(bool flag);
	void cdrOtaUpdate();
	bool getPC2Connected();
	void startPCCam();
	void stopPCCam();
	bool getPCCamMode();
	bool getOTAAdasScreen();
	void checkShutdown();
	void playSounds(CDR_SOUNDS sounds);
	class InitPreviewThread : public Thread
	{
	public:
		InitPreviewThread(CdrMain* pContext) 
			: Thread(false),
			  mWindowContext(pContext) {
		}
		~InitPreviewThread(){}
		virtual bool threadLoop() {
			mWindowContext->initPreview();	
			return false;
		}

		status_t start() {
			return run("InitPreview", PRIORITY_NORMAL);
		}

	private:
		CdrMain* mWindowContext;
	};

	class InitStage2Thread : public Thread
	{
	public:
		InitStage2Thread(CdrMain* pContext) 
			: Thread(false),
			  mWindowContext(pContext) {
		}
		~InitStage2Thread(){}
		virtual bool threadLoop() {
			mWindowContext->initStage2();	
			return false;
		}

		status_t start() {
			return run("InitStage2", PRIORITY_NORMAL);
		}

	private:
		CdrMain* mWindowContext;
	};
	void initCdrTime();
	bool checkSDFs();
	bool getIsLowPower();
	void keepAdasScreenFlag();
	void recoverAdasScreenFlag();

	void TinyPlayAudio(char *path);
	void setNoUSBCharge(bool val);
	void setFirstACConnect(bool val);
	bool getIfDeleFisrtFile();
	void setm2pcAdasScreen(bool flag);
	void setUsbCharge(bool flag);
	bool getNoUsbCharge();
	bool getUsbcharge();
	EventManager * getEventManager();
	RecordPreview * getRecordPreview();
	void * getObjPreview(unsigned int windowID);
	int cdrReplyToApp(int cmd, char *buf, int buf_len);
	bool getUVCCarBackAdasScreen();
	class TinyParkAudioThread: public Thread
	{
		public:
			TinyParkAudioThread(CdrMain *mre,char *fpath)
				: Thread(false), mCdr(mre),filepath(fpath)
			{
			
			}
			~TinyParkAudioThread(){}
			virtual bool threadLoop() {
				mCdr->TinyPlayAudio(filepath);
				return false;
			}
			status_t start() {
				return run("TinyParkAudioThread", PRIORITY_NORMAL);
			}
		private:
			CdrMain* mCdr;
			char * filepath;
	};
	void handBeginDownloadBuf(char *buf);
	bool getIsShowUpdateTips();
	bool isToRecordPreview();
private:
	void initTimerData();
	void modifyNoworkTimerInterval(time_t  time, time_t  time2);
	void startShutdownTimer(int idx);
	void cancelShutdownTimer(int idx);
	void stopShutdownTimer(int idx);
	int initPreview();
	int initStage2();
	StatusBar *mSTB;
	RecordPreview *mRecordPreview;
	PlayBackPreview* mPlayBackPreview;
	PlayBack *mPlayBack;
	Menu* mMenu;
	windowState_t curWindowState;
	sp <InitPreviewThread> mInitPT;
	sp <InitStage2Thread> mInitStage2T;
	PowerManager *mPM;
	EventManager *mEM;
	bool mEnableRecord;
	bool usbStorageConnected;
	bool isWakenUpOnParking;
	unsigned int mTime;
	Condition  mPOR;
	Mutex mLock;
	bool mOtherWindowInited;
	CdrServer *mCdrServer;
	bool mSessionConnect,mSessionSubMenu,mIsShowSubMenu,mCarBack,mAWMD;
	int mLowPower;
	bool mOtherDialogOpened;
	bool mOriginAdasScreen,m2pcAdasScreen,mFormatTFAdasS,mNeedCheckSD,mOTAAdasScreen;
	TinyParkAudioThread *mTinyParkAudioTh;
	int mGSensorVal;
	bool m2pcConnected;
	bool mPCCam;
	bool isAutoShutdownEnabled;
	bool mUSBcharge;
	bool mFiACConnect,mNoUSBCharge;
	bool mShowUpdateTips;
	bool  mUVCCarBackAdasScreen;
    bool mKeyShutDown;
    bool mShutdownTimer;
#ifdef APP_WIFI
    SoftApController *mSoftApController;
#endif
#ifdef YOUKU_PLATFORM
	YoukuService *mYoukuService;
#endif
public:
	
	WPARAM downKey;
	bool isKeyUp;
	bool isLongPress;
	bool mFinishConfig;
	bool mShowLogoInTime;
};

class PowerManager
{
public:
	PowerManager(CdrMain *cm);
	~PowerManager();

	void setOffTime(unsigned int time);
	void pulse();
	void powerOff();
	int screenOff();
	int screenOn();

	int readyToOffScreen();
	void screenSwitch();
	int setBatteryLevel();
	int getBatteryLevel(void);
	void reboot();
	bool isScreenOn();
	void adjustCpuFreq();

	void connect2StandbyService();
	void enterStandby();
	void watchdogRun();
	void watchdogThread();
	void enterStandby(int pid=-1);
	void setEnterOTAService(bool flag);
		
	class ScreenSwitchThread : public Thread
	{
	public:
		ScreenSwitchThread(PowerManager *pm)
			: Thread(false)
			, mPM(pm)
		{
		}
		void startThread()
		{
			run("ScreenSwitchThread", PRIORITY_NORMAL);
		}
		void stopThread()
		{
			requestExitAndWait();
		}
		bool threadLoop()
		{
			mPM->readyToOffScreen();
			return true;
		}
	private:
		PowerManager *mPM;
	};

	class poweroffThread : public Thread
	{
	public:
		poweroffThread(PowerManager* pm) : mPM(pm){}
		~poweroffThread(){}
		virtual bool threadLoop() {
			mPM->poweroff();	
			return false;
		}

		status_t start() {
			return run("poweroffThread", PRIORITY_NORMAL);
		}

	private:
		PowerManager* mPM;
	};

	class WatchDogThread : public Thread
	{
	public:
		WatchDogThread(PowerManager *pm)
			: Thread(false)
			, mPM(pm)
		{
		}
		void startThread()
		{
			run("WatchDogThread", PRIORITY_NORMAL);
		}
		void stopThread()
		{
			requestExitAndWait();
		}
		bool threadLoop()
		{
			mPM->watchdogThread();
			return false;
		}
	private:
		PowerManager *mPM;
	};


private:
	CdrMain *mCdrMain;
	sp<ScreenSwitchThread> mSS;
	sp<poweroffThread> mPO;
	sp<WatchDogThread> mWD;
	int mDispFd;
	unsigned int mOffTime;
	Mutex mLock;
	Condition mCon;
	int mState;
	int mBatteryLevel;
	int poweroff();
	bool isEnterStandby;
	sp<IStandbyService> mStandbyService;
};


#endif //_WINDOW_H
