#include "CdrCamera.h"
#include "posixTimer.h"
#undef LOG_TAG
#define LOG_TAG "CdrCamera.cpp"
#include "debug.h"
//#define SENSOR_DEBUG
CdrCamera::CdrCamera(int cam_type, CDR_RECT &rect, int node)
	: mHC(NULL),
	mCD(NULL),
	mId(cam_type),
	mRect(rect),
	mJpegCallback(NULL),
	mAwmdCallback(NULL),
	mAwmdTimerCallback(NULL),
	mAwmdTimerID(0),
	mLWM(0),
	mTWM(0),
	mNode(0)
{
	ViewInfo sur;
	sur.x = mRect.x;
	sur.y = mRect.y;
	sur.w = mRect.w;
	sur.h = mRect.h;
//	db_error("open camera%d\n",  node);
	mHC = HerbCamera::open(node);
	mNode = node;
	if(mHC == NULL) {
		db_error("fail to open HerbCamera");
		return;
	}
	mCD = new CdrDisplay(cam_type, &sur);
	if(mCD == NULL) {
		db_error("fail to new CedarDisplay");
		return;
	}

	mHlay = mCD->getHandle();
//	db_msg("camera %d, hlay is %d\n", cam_type, mHlay);

	mNeedAWMD = false;
	mAWMDing = false;
	mPreviewing = false;
	mWaterMarking = false;
	mADASing = false;
#ifdef APP_ADAS	
	mNeedADAS = false;
#endif // APP_ADAS
	mPreviewFlip = HerbCamera::Parameters::PREVIEW_FLIP::NoFlip;
	memset(strLWM, 0, sizeof(strLWM));
	mWMPos.x = 0;
	mWMPos.y = 0;

	if(cam_type == CAM_CSI || cam_type == CAM_UVC){
		HerbCamera::Parameters params;
		mHC->getParameters(&params);
		// Change 9 to 4 by Harry
		// In v3s,it only have 64M memory !
		params.setVideoCaptureBufferNum(4);
		mHC->setParameters(&params);
	}
}

CdrCamera::~CdrCamera()
{
	db_msg("CdrCamera Destructor\n");
	if (mCD) {
		delete mCD;
		mCD = NULL;
	}
	if (mHC) {
		mHC->stopPreview();
		release();
		delete mHC;
		mHC = NULL;
	}
	if(mAwmdTimerID) {
		stopTimer(mAwmdTimerID);
		deleteTimer(mAwmdTimerID);
		mAwmdTimerID = 0;
	}
}

HerbCamera* CdrCamera::getCamera(void)
{
	return mHC;
}

void CdrCamera::startPreview(void)
{
	Mutex::Autolock _l(mLock);

	db_msg("startPreview %d\n", mId);
	if(mPreviewing == true) {
		db_msg("preview is started\n");
		return;
	}
	mHC->startPreview();
	mPreviewing = true;
	setLWaterMark();
#ifdef APP_ADAS
	if (mNeedAWMD) {
		startAWMD();
	}
	if (mNeedADAS) {
		startAdas();
	}
#endif // APP_ADAS	
}

void CdrCamera::stopPreview(void)
{
	Mutex::Autolock _l(mLock);

	db_msg("stopPreview %d\n", mId);
	if(mPreviewing == false) {
		db_msg("preview is already stoped\n");
		return;
	}
	mHC->stopPreview();
	mPreviewing = false;
	db_msg("stopPreview\n");

	stopAWMD();
	disableWaterMark();
#ifdef APP_ADAS	
	stopAdas();
#endif
}

void CdrCamera::setWaterMarkPos(int h)
{
	mWMPos.x = 64;//(h > 1280) ? 1400 : 1400;
	mWMPos.y = 150;//(h > 720) ? 970 : 970;
}

void CdrCamera::enableWaterMark()
{
	db_msg("mWaterMarking:%d mPreviewing:%d", mWaterMarking, mPreviewing);
	char waterMark[100]= {0};
	if (mHC && (mWaterMarking == false)) {
		sprintf(waterMark ,"%d,%d,1",mWMPos.x ,mWMPos.y);
		db_msg("%s", waterMark);
#ifdef USE_IPS_SCREEN
        if(mId == CAM_CSI){
            mHC->setWaterMark(CAMERA_WATERMARK_ONLY_MAIN_CH, waterMark);  //csi show watermark in preview and record video
        }else if(mId == CAM_UVC){
            mHC->setWaterMark(CAMERA_WATERMARK_ONLY_MAIN_CH, waterMark);  //uvc only show watermark in record video
        }
#else
		if(mId == CAM_CSI){
			mHC->setWaterMark(CAMERA_WATERMARK_ALL, waterMark);  //csi show watermark in preview and record video
		}else if(mId == CAM_UVC){
			mHC->setWaterMark(CAMERA_WATERMARK_ONLY_MAIN_CH, waterMark);  //uvc only show watermark in record video
		}
#endif
		mWaterMarking = true;
	}
}

void CdrCamera::enableLicenseWaterMark(bool IsTimeWM ,bool IsLicenseWM ,const char* licenseWM)
{
	char waterMark[100]= {0};
	int licenseWM_x = mWMPos.x + 180;
	int licenseWM_y = mWMPos.y + 50;
	db_msg("mWaterMarking:%d mPreviewing:%d", mWaterMarking, mPreviewing);
	sprintf(waterMark ,"%d,%d,%d,%d,%d,%s",mWMPos.x , mWMPos.y , IsTimeWM , licenseWM_x , licenseWM_y ,licenseWM);
	db_msg("waterMark %s", waterMark);
	if (mHC && (mWaterMarking == false)) {
		mHC->setWaterMark(1, waterMark);
		mWaterMarking = true;
	}
}

void CdrCamera::disableWaterMark()
{
	//if (mId != CAM_CSI) {
	//	return ;
	//}
	if (mHC && (mWaterMarking)) {
		mHC->setWaterMark(0, NULL);
		mWaterMarking = false;
	}
}

void CdrCamera::setWhiteBalance(const char *mode)
{
	if (mId != CAM_CSI) {
		return ;
	}
	db_error("setWhiteBalance %s", mode);
	HerbCamera::Parameters params;

	mHC->getParameters(&params);
	//params.dump();

	params.setWhiteBalance(mode);
#ifndef SENSOR_DEBUG
	mHC->setParameters(&params);
#endif
}

void CdrCamera::setContrast(int mode)
{
	db_error("setContrast");
	HerbCamera::Parameters params;

	mHC->getParameters(&params);
	//params.dump();
	mode = 25*mode;
	params.setContrastValue(mode);
#ifndef SENSOR_DEBUG
	mHC->setParameters(&params);
#endif
}

void CdrCamera::setExposure(int mode)
{
	db_error("setExposure");
	HerbCamera::Parameters params;

	mHC->getParameters(&params);
	//params.dump();

	params.setExposureCompensation(mode);
#ifndef SENSOR_DEBUG
	mHC->setParameters(&params);
#endif
}


CdrDisplay* CdrCamera::getDisp(void)
{
	return mCD;
}

void CdrCamera::onPictureTaken(void *data, int size, HerbCamera* pCamera)
{
	if(mJpegCallback) {
		(*mJpegCallback)(data, size, mCaller, mId);
	}
}

void CdrCamera::setJpegCallback(void(*callback)(void* data, int size, void* caller, int id))
{
	mJpegCallback = callback;
}

void CdrCamera::takePicture(void)
{
	if(mHC) {
		mHC->takePicture(NULL, NULL, NULL, this);
	}
}

#ifdef APP_ADAS
void CdrCamera::setAdasCallback(void(*callback)(void* data,void * caller))
{
	mAdasCallback = callback;
}
#endif

void CdrCamera::onError(int error, HerbCamera *pCamera)
{
       if(mErrorCallback) {
               (*mErrorCallback)(error, mCaller);
       }
}

void CdrCamera::setErrorCallback(void(*callback)(int error, void* caller))
{
       mErrorCallback = callback;
}

void CdrCamera::setImageQuality(ImageQuality_t quality)
{
	HerbCamera::Parameters params;
	unsigned int w = 0;
	unsigned int h = 0;
	if(mHC == NULL) {
		db_error("mHC is NULL\n");
		return;
	}
	mHC->getParameters(&params);
	//params.dump();

	switch(quality) {
	case ImageQuality1M:
		{
			w = 1280;
			h = 720;
		}
		break;
	case ImageQuality3M:
		{
			w = 1920;
			h = 1080;
		}
		break;
	case ImageQuality5M:
		{
			w = 3200;
			h = 1800;
		}
		break;
	case ImageQuality8M:
		{
			w = 3840;
			h = 2160;
		}
		break;
	case CSIImageQualityNet:
		{
			w = 480;
			h = 270;
		}
		break;
	case UVCImageQualityNet:
		{
			w = 320;
			h = 192;
		}
		break;
	default:
		{
			db_msg("not support image_quality:%d\n", quality);
			return ;
		}
		break;
	}
	params.setPictureSize(w, h);
	mHC->setParameters(&params);
}

void CdrCamera::setPictureSizeMode(int mode)
{ 
	HerbCamera::Parameters params;
	mHC->getParameters(&params);	
	params.setPictureSizeMode(mode);
	mHC->setParameters(&params);
}

void CdrCamera::setCameraFlip(bool flip)
{
	db_msg(" flip=%d",flip);
	if (flip) {
		mPreviewFlip = HerbCamera::Parameters::PREVIEW_FLIP::LeftRightFlip;
	} else {
		mPreviewFlip = HerbCamera::Parameters::PREVIEW_FLIP::NoFlip;
	}
	if (mHC) {
		HerbCamera::Parameters params;
		mHC->getParameters(&params);
		params.setPreviewFlip(mPreviewFlip);
		mHC->setParameters(&params);
	}
}

int CdrCamera::initCamera(unsigned int width, unsigned int height, unsigned int preview_sz_w, unsigned int preview_sz_h)
{
	HerbCamera::Parameters params;
	v4l2_queryctrl ctrl;
	if (!mHC) {
		return -1;
	}
	mHC->getParameters(&params);
	//params.dump();
	params.setPreviewSize(preview_sz_w, preview_sz_h); //isp small size
	params.setVideoSize(width, height);	//real sensor size
	ALOGD("(f:%s, l:%d) setPreviewSize[%dx%d], VideoSize[%dx%d]", __FUNCTION__, __LINE__, preview_sz_w, preview_sz_h, width, height);
	params.setPreviewFrameRate(FRONT_CAM_FRAMERATE);
	params.setPreviewFlip(mPreviewFlip);
    params.setPictureMode(CameraParameters::AWEXTEND_PICTURE_MODE_FAST);
    params.setPictureSizeMode(CameraParameters::AWEXTEND_PICTURE_SIZE_MODE::UseParameterPictureSize);
#ifndef SENSOR_DEBUG
	if(mId == CAM_CSI) {
		params.setContrastValue(ctrl.default_value);
		params.setBrightnessValue(ctrl.default_value);
		params.setSaturationValue(ctrl.default_value);
		params.setHueValue(ctrl.default_value);
	}
#endif
#ifdef USE_IPS_SCREEN
    params.setPreviewRotation(90);
#endif
	mHC->setParameters(&params);
	mHC->setPreviewDisplay(mHlay);
	mCD->setBottom();
	mHC->setErrorCallback(this);	
	return 0;
}

void CdrCamera::startAWMD()
{
	if(!mAwmdTimerID) {
		if(mAwmdTimerCallback == NULL) {
			db_error("%s %d, awmd timer call back not set\n", __FUNCTION__, __LINE__);
			return;
		}
		if(createTimer(mCaller, &mAwmdTimerID, mAwmdTimerCallback) < 0) {
			db_error("%s %d, create timer failed\n", __FUNCTION__, __LINE__);
			return;	
		}
	}

	db_msg("%s %d\n", __FUNCTION__, __LINE__);
	if (mHC && (mAWMDing == false)) {
		db_msg("%s %d\n", __FUNCTION__, __LINE__);
		mHC->setAWMoveDetectionListener(this);
		mHC->startAWMoveDetection();
		mHC->setAWMDSensitivityLevel(1);
		
		mAWMDing = true;
	}
	
}

void CdrCamera::stopAWMD()
{
	db_msg("%s %d\n", __FUNCTION__, __LINE__);
	if (mHC && (mAWMDing)) {
		db_msg("%s %d\n", __FUNCTION__, __LINE__);
		mHC->stopAWMoveDetection();
		if(mAwmdTimerID) {
			stopTimer(mAwmdTimerID);
			deleteTimer(mAwmdTimerID);
			mAwmdTimerID = 0;
		}
		mAWMDing = false;
	}
}

#ifdef APP_ADAS
void CdrCamera::startAdas()
{
	db_msg(" mADASing %d ", mADASing);
	if (mHC&& (mADASing == false)) {
		mHC->setAdasDetectionListener(this);
		mHC->adasStartDetection();
		mADASing = true;
	}
}


void CdrCamera::stopAdas()
{
#if 0 // Del by Harry
	db_msg(" mADASing %d ", mADASing);
	if (mHC && mADASing) {
		mHC->adasStopDetection();
		mADASing = false;
	}
#endif 
}

void CdrCamera::setAdas(bool val)
{
	if (mId != CAM_CSI) {
		return ;
	}
	mNeedADAS = val;
	db_msg("%s %d, val is %d\n", __FUNCTION__, __LINE__, val);
	if (mHC && (mPreviewing == true)) {
		db_msg("%s %d\n", __FUNCTION__, __LINE__);
		if (val) {
			startAdas();
		} else {
			stopAdas();
		}
	}
}

void CdrCamera::setAdasAudioVol(int val)
{
	if (mHC) {
		mHC->adasSetAduioVol(val);
	}
}

void CdrCamera::setAdasFCWSensity(int val)
{
	if (mHC) {
		mHC->adasSetDistanceDetectLevel(val);
	}
}

void CdrCamera::setAdasMode(int mode)
{
	if(mHC){
		mHC->adasSetMode(mode);
	}
}

void CdrCamera::setAdasTipsMode(int val)
{
	if (mHC) {
		mHC->setAdasTipsMode(val);
	}
}

void CdrCamera::adasSetGSensorData(int val0, float val1, float val2)
{
   if(mHC){
        mHC->adasSetGsensorData(val0, val1, val2);
   }
}
#endif 

void CdrCamera::setAWMD(bool val)
{
	Mutex::Autolock _l(mLock);
	if (mId != CAM_CSI) {
		return ;
	}
	mNeedAWMD = val;
	db_msg("%s %d, val is %d\n", __FUNCTION__, __LINE__, val);
	if (mHC && (mPreviewing == true)) {
		db_msg("%s %d\n", __FUNCTION__, __LINE__);
		if (val) {
			startAWMD();
		} else {
			stopAWMD();
		}
	}
}

#ifdef APP_ADAS
// for testing adas
void CdrCamera::onAdasDetection(void *data, int size, HerbCamera *pCamera){
	//for testing adas

	
	if(mAdasCallback){
	   (*mAdasCallback)(data,mAdasCaller);
	}
	
	
}

void CdrCamera::setAdasCaller(void *caller)
{
	mAdasCaller = caller;
}

#endif

void CdrCamera::setLWaterMark(int flag)
{
	if (mHC && (mPreviewing == true)){
		if (mLWM) {
			enableLicenseWaterMark(mTWM ,mLWM ,strLWM);
		} else if (mTWM) { //T
			enableWaterMark();
		}else if( flag == 1){
			//disableWaterMark();
			mHC->setWaterMark(0, NULL);
			if(mWaterMarking)
			 mWaterMarking = false;
		}
	}
}

void CdrCamera::setLWaterMark(bool IsTimeWM ,bool IsLicenseWM , const char *licenseWM )
{
	db_msg("IsTimeWM:%d IsLicenseWM:%d, licenseWM:%s", IsTimeWM, IsLicenseWM, licenseWM);
	int cnt = 0;
	if (IsTimeWM != mTWM) {
		cnt++;
		mTWM = IsTimeWM;
	}
	if (IsLicenseWM != mLWM) {
		cnt++;
		mLWM = IsLicenseWM;
	}
	if (strcmp(strLWM, licenseWM)) {
		cnt++;
		strcpy(strLWM, licenseWM);
	}
	if (cnt == 0) {
		return;
	}

	Mutex::Autolock _l(mLock);
	setLWaterMark(1);
}

void CdrCamera::release()
{
	mHC->release();
}

void CdrCamera::setAwmdCallBack(void (*callback)(int value, void *caller))
{
	mAwmdCallback = callback;
}

void CdrCamera::onAWMoveDetection(int value, HerbCamera *pCamera)
{
	//	db_msg("%s %d\n", __FUNCTION__, __LINE__);
	if(mAwmdCallback) {
		(*mAwmdCallback)(value, mCaller);
	}
}

bool CdrCamera::isPreviewing(void)
{
	Mutex::Autolock _l(mLock);
	return mPreviewing;
}

void CdrCamera::setAwmdTimerCallBack(void (*callback)(sigval_t val))
{
	mAwmdTimerCallback = callback;
}

void CdrCamera::feedAwmdTimer(int seconds)
{
	if(mAwmdTimerID)	
		setOneShotTimer(seconds, 0, mAwmdTimerID);
}

int CdrCamera::queryAwmdExpirationTime(time_t* sec, long* nsec)
{
	if(mAwmdTimerID) {
		if(getExpirationTime(sec, nsec, mAwmdTimerID) < 0) {
			db_error("%s %d, get awmd timer expiration time failed\n", __FUNCTION__, __LINE__);
			return -1;
		}
	} else
		return -1;

	return 0;
}

void CdrCamera::setCaller(void* caller)
{
	mCaller = caller;
}

int CdrCamera::getInputSource()
{
	int val;
	mHC->getInputSource(&val);
	return val;
}
int CdrCamera::getTVinSystem()
{
	int val;
	mHC->getTVinSystem(&val);
	return val;
}

void CdrCamera::setUvcMode(int mode)
{
	mHC->setUvcGadgetMode(mode);
}

void CdrCamera::suspend()
{
	release();
}

void CdrCamera::resume()
{
	mHC = HerbCamera::open(mNode);
}

void CdrCamera::setJpegQuality(int val)
{
	HerbCamera::Parameters params;
	mHC->getParameters(&params);
	mJpegQuality = params.getJpegQuality();
	params.setJpegQuality(val);
	mHC->setParameters(&params);
}

void CdrCamera::resetJpegQuality()
{
	HerbCamera::Parameters params;
	mHC->getParameters(&params);
	params.setJpegQuality(mJpegQuality);
	mHC->setParameters(&params);
}

void CdrCamera::setTakepicType(TakePicType type)
{
	mTakePicType = type;
}

TakePicType CdrCamera::getTakepicType()
{
	return mTakePicType;
}

