#include "BackCarMonitor.h"
#include "debug.h"
#include <errno.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include <unistd.h>
#include <fcntl.h>


static struct uvc_xu_control_info sonix_xu_ctrls[] =
{
    {
        UVC_GUID_SONIX_USER_HW_CONTROL,
        XU_SONIX_ASIC_CTRL,
        0,
        4,
        UVC_CONTROL_SET_CUR | UVC_CONTROL_GET_MIN | UVC_CONTROL_GET_MAX |
                    UVC_CONTROL_GET_DEF | UVC_CONTROL_AUTO_UPDATE | UVC_CONTROL_GET_CUR
    },
    {
        UVC_GUID_SONIX_USER_HW_CONTROL,
        XU_SONIX_I2C_CTRL,
        1,
        8,
        UVC_CONTROL_SET_CUR | UVC_CONTROL_GET_CUR
    },
    {
        UVC_GUID_SONIX_USER_HW_CONTROL,
        XU_SONIX_SF_READ,
        2,
        11,
        UVC_CONTROL_SET_CUR | UVC_CONTROL_GET_CUR
    },

};

//  SONiX XU Ctrls Mapping
static struct uvc_xu_control_mapping sonix_xu_mappings[] =
{
    {
        V4L2_CID_ASIC_CTRL_SONIX,
        "SONiX: Asic Control",
        UVC_GUID_SONIX_USER_HW_CONTROL,
        XU_SONIX_ASIC_CTRL,
        4,
        0,
        V4L2_CTRL_TYPE_INTEGER,
        UVC_CTRL_DATA_TYPE_SIGNED
    },
    {
        V4L2_CID_I2C_CTRL_SONIX,
        "SONiX: I2C Control",
        UVC_GUID_SONIX_USER_HW_CONTROL,
        XU_SONIX_I2C_CTRL,
        8,
        0,
        V4L2_CTRL_TYPE_INTEGER,
        UVC_CTRL_DATA_TYPE_UNSIGNED
    },
    {
        V4L2_CID_SF_READ_SONIX,
        "SONiX: Serial Flash Read",
        UVC_GUID_SONIX_USER_HW_CONTROL,
        XU_SONIX_SF_READ,
        11,
        0,
        V4L2_CTRL_TYPE_INTEGER,
        UVC_CTRL_DATA_TYPE_UNSIGNED
    },


};

int BackCarMonitor::XU_ASIC_Get_Data(int fd, int Addr, __u8 *uData)
{
//    db_info("XU_ASIC_Get_Data ==>\n");
    int ret = 0;
    int err = 0;
    __u8 ctrldata[4]={0};

    struct uvc_xu_control xctrl = {
            3,										/* bUnitID 				*/
//            g_unit,
            XU_SONIX_ASIC_CTRL,					   /* function selector	*/
            4,										/* data size			*/
            (__u8*)&ctrldata								/* *data				*/
        };

//	g_unit =  4;
	xctrl.unit   = g_unit;

    // Switch command
    xctrl.data[0] = Addr & 0xFF;				// Tag
    xctrl.data[1] = (Addr & 0xFF00)>>8;
	xctrl.data[2] = 0x0;
    xctrl.data[3] = 0xff;                       // Dummy

    if ((err=ioctl(fd, UVCIOC_CTRL_SET, &xctrl)) < 0)
    {
        db_msg("XU_ASIC_Get_Data ==> Switch cmd : ioctl(UVCIOC_CTRL_SET) FAILED (%i)11 == \n",err);
        if(err==EINVAL)
            db_msg("Invalid arguments\n");
        return err;
    }

	// Asic Read
	xctrl.data[3] = 0x00;

    if ((err=ioctl(fd, UVCIOC_CTRL_GET, &xctrl)) < 0)
    {
        db_msg("XU_ASIC_Get_Data ==> ioctl(UVCIOC_CTRL_GET) FAILED (%i) == \n",err);
        if(err==EINVAL)
            db_msg("Invalid arguments\n");
        return err;
    }

//    db_info("   == XU_ASIC_Get_Data Success == \n");
//    for(i=0; i<4; i++)
//        db_info("      Get data[%d] : 0x%x\n", i, xctrl.data[i]);
    *uData = xctrl.data[2];

    return ret;

}
int XU_Init_Ctrl(int fd)
{
    int i=0;
    int err=0;
    /* try to add all controls listed above */
    for ( i=0; i<LENGTH_OF_SONIX_XU_CTR; i++ )
    {
        db_msg("Adding XU Ctrls - %s\n", sonix_xu_mappings[i].name);
        if ((err=ioctl(fd, UVCIOC_CTRL_ADD, &sonix_xu_ctrls[i])) < 0 )
        {
            if ((errno == EEXIST) || (errno != EACCES))
            {
                db_msg("UVCIOC_CTRL_ADD - Ignored, uvc driver had already defined\n");
               // return (-EEXIST);
               continue;
            }
            else if (errno == EACCES)
            {
                db_msg("Need admin previledges for adding extension unit(XU) controls\n");
                db_msg("please run 'SONiX_UVC_TestAP --add_ctrls' as root (or with sudo)\n");
            //    return  (-1);
				continue;
            }
            else db_msg("Control exists");
        }
    }
    /* after adding the controls, add the mapping now */
    for ( i=0; i<LENGTH_OF_SONIX_XU_MAP; i++ )
    {
        db_msg("Mapping XU Ctrls - %s\n", sonix_xu_mappings[i].name);
        if ((err=ioctl(fd, UVCIOC_CTRL_MAP, &sonix_xu_mappings[i])) < 0)
        {
            if ((errno!=EEXIST) || (errno != EACCES))
            {
                db_msg("UVCIOC_CTRL_MAP - Error");
             //   return (-2);
				continue;
            }
            else if (errno == EACCES)
            {
                db_msg("Need admin previledges for adding extension unit(XU) controls\n");
                db_msg("please run 'SONiX_UVC_TestAP --add_ctrls' as root (or with sudo)\n");
             //   return  (-1);
				continue;
            }
            else db_msg("Mapping exists");
        }
    }
    return 0;
}

int BackCarMonitor::getBackCarState()
{
	__u8 uData;
	int i;

	static __u8 songhan_chipid = 0;
	
	char uvc_path[32]={0};
	if(isXuCtrlInited == false) {
		if(handle < 0) {
			for(i=1; i<10; i++){
				sprintf(uvc_path, "/dev/video%d", i);
				//handle = open("/dev/video1", O_RDWR );
				handle = open(uvc_path, O_RDWR );
				if(handle > 0) {
					break;	
				}else {
					memset(uvc_path, 0 ,sizeof(uvc_path));
					continue;
				}
			}
			if(i>=10)
				return -1;
		}
	//	int ret = XU_Init_Ctrl(handle);
	//	if( ret == 0)
			isXuCtrlInited = true;
		int ret = XU_ASIC_Get_Data(handle, 0x101f, &uData);
		if(ret < 0){
			db_msg("change uint read again	\r\n");
			g_unit = 4;
		    ret = XU_ASIC_Get_Data(handle, 0x101f, &uData);
		}
		db_msg("UVC chip ID is %x\r\n", uData);
		if(uData == 0x25) {
			songhan_chipid =  0x25;
			g_unit = 4 ;
		} 
	}
	int ret = XU_ASIC_Get_Data(handle, 0x1005, &uData);
	if(ret < 0){
		db_msg("change uint read again	\r\n");
		if(g_unit == 3)
			g_unit = 4;
		else g_unit = 3;
	    ret = XU_ASIC_Get_Data(handle, 0x1005, &uData);
		if(ret < 0 )
			return -1;
	}
//	db_info("udata0 = %x\r\n", uData);
	return (uData & 0x01);
}

BackCarMonitor::BackCarMonitor()
				:handle(-1)
				,g_unit(3)
				,isXuCtrlInited(false)
{
	db_msg("----Constructor");
}
