
#include "ResourceManager.h"

#include <string.h>
#include <minigui/minigui.h>
#include <minigui/window.h>

#include <utils/Mutex.h>

#include "cdr.h"
#include "resource_impl.h"
#include "Rtc.h"

#include "AWNetInterface.h"
#include "platform.h"

#include "CdrServer.h"

#ifdef YOUKU_PLATFORM
#include <youku/ykcamera_common.h>
#endif

#ifdef LOG_TAG
#undef LOG_TAG
#endif
#define LOG_TAG "ResourceManager.cpp"
#include "debug.h"

typedef struct {
	const char* mainkey;
	void* addr;
}configTable2;

typedef struct {
	const char* mainkey;
	const char* subkey;
	void* addr;
}configTable3;

typedef struct{
	int vol;
	float voice;
}volumeTable;

volumeTable VolNum[]={{0,0.9},{1,0.6},{2,0.3}};

using namespace android;

static ResourceManager *gInstance;
static Mutex gInstanceMutex;

void ResourceManager::setHwnd(unsigned int win_id, HWND hwnd)
{
	mHwnd[win_id] = hwnd;
}

ResourceManager::ResourceManager()
	: configMenu("/data/menu.cfg")
	, defaultConfigMenu("/system/res/cfg/menu.cfg")
	, mCfgFileHandle(0)
	, mCfgMenuHandle(0)
	, mRecording(false)
{
	int big_w, big_h;
	getScreenInfo(&big_w, &big_h);
	sprintf(configFile, "/system/res/cfg/%dX%d.cfg", big_w, big_h);
	defaultConfigFile = configFile;

	memset(mHwnd, 0, sizeof(HWND)*WIN_CNT);
//	db_msg("ResourceManager Constructor\n");
	if(getCfgMenuHandle() < 0) {
		db_info("restore menu.cfg\n");
		if(copyFile(defaultConfigMenu, configMenu) < 0) {
			db_error("copyFile config file failed\n");	
		}
	} else {
		if (verifyConfig() == false){
			db_info("restore menu.cfg\n");
			if(copyFile(defaultConfigMenu, configMenu) < 0) {
				db_error("copyFile config file failed\n");
			}
		}
		releaseCfgMenuHandle();
	}
	menuDataAWMDEnable = false;
	int retx = LoadBitmapFromFile(HDC_SCREEN,&mNoVideoImage,"/system/res/others/no_video_image.png");
	if(retx!= ERR_BMP_OK){
		ALOGD("load the mNoVideoImage bitmap error"); 
		mNoVideoImage.bmWidth= -1;
	}

}

ResourceManager::~ResourceManager()
{
	UnloadBitmap(&mNoVideoImage);
	db_msg("ResourceManager Destructor\n");
}

int ResourceManager::initLangAndFont(void)
{
	db_msg("%s:%d\n",__func__,__LINE__);
	lang = getLangFromConfigFile();
	db_msg("%s:%d\n",__func__,__LINE__);
	if(lang == LANG_ERR) {
		db_error("get language failed\n");
		return -1;
	}
	db_msg("%s:%d\n",__func__,__LINE__);
#ifdef USE_IPS_SCREEN
		mLogFont = CreateLogFont("sxf", "arialuni", "UTF-8",
		FONT_WEIGHT_REGULAR, FONT_SLANT_ROMAN, FONT_SETWIDTH_NORMAL,
		FONT_OTHER_AUTOSCALE, FONT_UNDERLINE_NONE, FONT_STRUCKOUT_NONE, 30, 0);

		mLogFont_MSYH = CreateLogFont("sxf", "msyh", "UTF-8",
		FONT_WEIGHT_REGULAR, FONT_SLANT_ROMAN, FONT_SETWIDTH_NORMAL,
		FONT_OTHER_AUTOSCALE, FONT_UNDERLINE_NONE, FONT_STRUCKOUT_NONE, 30, 0);

		mLogFont_Malgun_Gothic = CreateLogFont("sxf", "gothic", "UTF-8",
		FONT_WEIGHT_REGULAR, FONT_SLANT_ROMAN, FONT_SETWIDTH_NORMAL,
		FONT_OTHER_AUTOSCALE, FONT_UNDERLINE_NONE, FONT_STRUCKOUT_NONE, 30, 0);
		if (lang == LANG_KR) {
			mCurLogFont = mLogFont_Malgun_Gothic;
		} else {
			mCurLogFont = mLogFont_MSYH;
		}
#else
		db_msg("%s:%d\n",__func__,__LINE__);
		//mLogFont = CreateLogFont("sxf", "arialuni", "UTF-8",
		//FONT_WEIGHT_REGULAR, FONT_SLANT_ROMAN, FONT_SETWIDTH_NORMAL,
		//FONT_OTHER_AUTOSCALE, FONT_UNDERLINE_NONE, FONT_STRUCKOUT_NONE, 16, 0);
		db_msg("%s:%d\n",__func__,__LINE__);
		mCurLogFont = mLogFont = NULL;
		db_msg("%s:%d\n",__func__,__LINE__);
#endif
	initLabel(); /* init lang labels */
	db_msg("%s:%d\n",__func__,__LINE__);
	SendMessage(mHwnd[WINDOWID_MAIN], MSG_RM_LANG_CHANGED, (WPARAM)&lang, 0);
	db_msg("%s:%d\n",__func__,__LINE__);
	SendMessage(mHwnd[WINDOWID_STATUSBAR], MSG_RM_LANG_CHANGED, (WPARAM)&lang, 0);
	db_msg("%s:%d\n",__func__,__LINE__);
	return 0;
}

ResourceManager* ResourceManager::getInstance()
{
	//	db_msg("getInstance\n");
	Mutex::Autolock _l(gInstanceMutex);
	if(gInstance != NULL) {
		return gInstance;
	}

	gInstance = new ResourceManager();
	return gInstance;
}

int ResourceManager::initStage1(void)
{
	int err_code;
	int retval = 0;

	db_msg("initStage1\n");


	if(getCfgFileHandle() < 0) {
		db_error("get file handle failed\n");
		return -1;
	}
	err_code = getRectFromFileHandle("rect_screen", &rScreenRect);
	if(err_code != ETC_OK) {
		db_error("get screen rect failed: %s\n", pEtcError(err_code));
		retval = -1;
		goto out;
	}

	if(initStatusBarResource() < 0) {
		db_error("init status bar resource failed\n");	
		retval = -1;
		goto out;
	}

out:
	releaseCfgFileHandle();
	return retval;
}

int ResourceManager::notifyAll()
{
	dump();
	SendMessage(mHwnd[WINDOWID_RECORDPREVIEW], MSG_REFRESH, 0, 0);
	SendMessage(mHwnd[WINDOWID_STATUSBAR], STBM_VOICE, rStatusBarData.audioIcon.current, 0);
	SendMessage(mHwnd[WINDOWID_MAIN], MSG_RM_SCREENSWITCH, menuDataSS.current, 0);
	SendMessage(mHwnd[WINDOWID_MAIN], MSG_GSENSOR_SENSITIVITY, menuDataGsensor.current, 0);
	SendMessage(mHwnd[WINDOWID_MAIN], MSG_IMPACTLEVEL_VALUE, menuDataImpactLevel.current, 0);
	SendMessage(mHwnd[WINDOWID_PLAYBACK],MSG_SET_VOLUME,(WPARAM)(mCdrVolume*100),0);
	SendMessage(mHwnd[WINDOWID_RECORDPREVIEW],MSG_ADAS_AUDIO_VOL,(WPARAM)(mCdrVolume*10),0);
	notifyWaterMark();
	return 0;
}
int ResourceManager::initStage2(void)
{
	int retval = 0;

	db_msg("initStage2\n");
	db_msg("%s:%d\n",__func__,__LINE__);

	initLangAndFont();

	db_msg("%s:%d\n",__func__,__LINE__);

	if(getCfgFileHandle() < 0) {
		db_error("get file handle failed\n");
		return -1;
	}
	db_msg("%s:%d\n",__func__,__LINE__);
	if(getCfgMenuHandle() < 0) {
		db_error("get menu handle failed\n");
		retval = -1;
		goto out;
	}
	db_msg("%s:%d\n",__func__,__LINE__);
	if(initPlayBackPreviewResource() < 0) {
		db_error("init PlayBackPreview resource failed\n");
		retval = -1;
		goto out;
	}
	db_msg("%s:%d\n",__func__,__LINE__);
	if(initPlayBackResource() < 0) {
		db_error("init PlayBack resource failed\n");
		retval = -1;
		goto out;
	}
	db_msg("%s:%d\n",__func__,__LINE__);
	if(initMenuListResource() < 0) {
		db_error("init Menu List resource failed\n");
		retval = -1;
		goto out;
	}
	/* ----------------------- */
#if 0 // Add by Harry
	if(initMenuResource() < 0) {
		db_error("init menu resource failed\n");
		retval = -1;
		goto out;
	}
#endif 
	db_msg("%s:%d\n",__func__,__LINE__);
	if(initOtherResource() < 0) {
		db_error("init other resource failed\n");
		retval = -1;
		goto out;	
	}
	db_msg("%s:%d\n",__func__,__LINE__);
	if(menuDataVolumeEnable == true)
		rStatusBarData.audioIcon.current = 1;
	else
		rStatusBarData.audioIcon.current = 0;

	if(menuDataAWMDEnable == true)
		rStatusBarData.awmdIcon.current = 1;
	else
		rStatusBarData.awmdIcon.current = 0;

	//notifyAll();
	db_msg("%s:%d\n",__func__,__LINE__);
out:
	releaseCfgFileHandle();
	db_msg("%s:%d\n",__func__,__LINE__);
	releaseCfgMenuHandle();
	db_msg("%s:%d\n",__func__,__LINE__);
	return retval;
}

int ResourceManager::initStatusBarResource(void)
{
	int err_code;
	int retval = 0;

	/*++++ Rect ++++*/
	configTable2 configTableRect[] = {
		{CFG_FILE_STB,			(void*)&rStatusBarData.STBRect },
		{CFG_FILE_STB_WNDPIC,	(void*)&rStatusBarData.windowPicRect },
		{CFG_FILE_STB_LABEL1,	(void*)&rStatusBarData.label1Rect },
		// Del by Harry
//		{CFG_FILE_STB_LABEL2,	(void*)&rStatusBarData.label2Rect },
		{CFG_FILE_STB_RESERVELABLE,	(void*)&rStatusBarData.reserveLabelRect },
		{CFG_FILE_STB_WIFI,			(void*)&rStatusBarData.wifiRect },
		{CFG_FILE_STB_PARK, 		(void*)&rStatusBarData.parkRect },
		{CFG_FILE_STB_UVC,			(void*)&rStatusBarData.uvcRect },
		{CFG_FILE_STB_AWMD,			(void*)&rStatusBarData.awmdRect },
		{CFG_FILE_STB_LOCK,			(void*)&rStatusBarData.lockRect },
		{CFG_FILE_STB_TFCARD,		(void*)&rStatusBarData.tfCardRect },
		{CFG_FILE_STB_AUDIO,		(void*)&rStatusBarData.audioRect },
		{CFG_FILE_STB_BAT,			(void*)&rStatusBarData.batteryRect },
		// Del by Harry
//		{CFG_FILE_STB_WOND,         (void*)&rStatusBarData.wondRect},
	};

	/*++++ get color ++++*/
	configTable3 configTableColor[] = {
		{CFG_FILE_STB, FGC_WIDGET, (void*)&rStatusBarData.fgc},
		{CFG_FILE_STB, BGC_WIDGET, (void*)&rStatusBarData.bgc},
	};

	/*++++ get the status_bar window picture name ++++*/
	configTable3 configTableValue[] = {
		{CFG_FILE_STB_WNDPIC,  "record_preview",	(void*)&rStatusBarData.recordPreviewPic },
		{CFG_FILE_STB_WNDPIC,  "screen_shot",		(void*)&rStatusBarData.screenShotPic },
		{CFG_FILE_STB_WNDPIC,  "playback_preview",	(void*)&rStatusBarData.playBackPreviewPic },
		{CFG_FILE_STB_WNDPIC,  "playback",			(void*)&rStatusBarData.playBackPic },
		{CFG_FILE_STB_WNDPIC,  "menu",				(void*)&rStatusBarData.menuPic },
	};

	/*++++ get the current icon info ++++*/
	configTable2 configTableIcon[] = {
		{CFG_FILE_STB_WIFI,			(void*)&rStatusBarData.wifiIcon},
		{CFG_FILE_STB_PARK, 		(void*)&rStatusBarData.parkIcon},
		{CFG_FILE_STB_UVC,			(void*)&rStatusBarData.uvcIcon},
		{CFG_FILE_STB_AWMD,			(void*)&rStatusBarData.awmdIcon},
		{CFG_FILE_STB_LOCK,			(void*)&rStatusBarData.lockIcon},
		{CFG_FILE_STB_TFCARD,		(void*)&rStatusBarData.tfCardIcon },
		{CFG_FILE_STB_AUDIO,		(void*)&rStatusBarData.audioIcon },
		{CFG_FILE_STB_BAT,			(void*)&rStatusBarData.batteryIcon },
		// Del by Harry
 //       {CFG_FILE_STB_WOND,         (void*)&rStatusBarData.wondIcon},
	};

	/*---- Rect ----*/
	for(unsigned int i = 0; i < TABLESIZE(configTableRect); i++) {
		err_code = getRectFromFileHandle(configTableRect[i].mainkey, (CDR_RECT*)configTableRect[i].addr );
		if(err_code != ETC_OK) {
			db_error("get %s rect failed: %s\n", configTableRect[i].mainkey,  pEtcError(err_code));
			retval = -1;
			goto out;
		}
	}

	/*---- get color ----*/
	for(unsigned int i = 0; i < TABLESIZE(configTableColor); i++) {
		err_code = getABGRColorFromFileHandle(configTableColor[i].mainkey, configTableColor[i].subkey, *(gal_pixel*)configTableColor[i].addr);
		if(err_code != ETC_OK) {
			db_error("get %s %s failed: %s\n", configTableColor[i].mainkey, configTableColor[i].subkey, pEtcError(err_code));
			retval = -1;
			goto out;
		}
		//		db_msg("%s %s color is 0x%lx\n", configTableColor[i].mainkey, configTableColor[i].subkey, 
		//				Pixel2DWORD(HDC_SCREEN, *(gal_pixel*)configTableColor[i].addr) );
	}


	char buf[50];
	/*---- get the status_bar window picture name ----*/
	for(unsigned int i = 0; i < TABLESIZE(configTableValue); i++) {
		err_code = getValueFromFileHandle(configTableValue[i].mainkey, configTableValue[i].subkey, buf, sizeof(buf));
		if(err_code != ETC_OK) {
			db_error("get %s %s pic failed: %s\n", configTableValue[i].mainkey, configTableValue[i].subkey, pEtcError(err_code));
			retval = -1;
			goto out;
		}
		*(String8*)configTableValue[i].addr = buf;
//		db_msg("get %s %s pic is %s\n", configTableValue[i].mainkey, configTableValue[i].subkey, (*(String8*)configTableValue[i].addr).string() );
	}


	/*---- get the current icon info ----*/
	unsigned int current;
	for(unsigned int i = 0; i < TABLESIZE(configTableIcon); i++) {
		err_code = fillCurrentIconInfoFromFileHandle(configTableIcon[i].mainkey, *(currentIcon_t*)configTableIcon[i].addr);
		if(err_code != ETC_OK) {
			db_error("get %s icon failed: %s\n", configTableIcon[i].mainkey, pEtcError(err_code) );
			retval = -1;
			goto out;
		}
		current = (*(currentIcon_t*)configTableIcon[i].addr).current;
		//		db_msg("%s current icon is %s\n", configTableIcon[i].mainkey, (*(currentIcon_t*)configTableIcon[i].addr).icon.itemAt(current).string() );
	}

	/************* end of init status bar resource ******************/

out:
	return retval;
}

void ResourceManager::setCurrentIconValue(enum ResourceID resID, int cur_val)
{
	switch(resID) {
	case ID_STATUSBAR_ICON_BAT:
		rStatusBarData.batteryIcon.current = (cur_val - 1);
		break;
	case ID_STATUSBAR_ICON_TFCARD:
		rStatusBarData.tfCardIcon.current = cur_val;
		break;
	case ID_STATUSBAR_ICON_AUDIO: 
		rStatusBarData.audioIcon.current = cur_val;
		break;
	case ID_STATUSBAR_ICON_LOCK: 
		rStatusBarData.lockIcon.current = cur_val;
		break;
	case ID_STATUSBAR_ICON_AWMD:
		rStatusBarData.awmdIcon.current = cur_val;
		break;
	case ID_STATUSBAR_ICON_UVC:
		rStatusBarData.uvcIcon.current = cur_val;
		break;
	case ID_STATUSBAR_ICON_WIFI:
		rStatusBarData.wifiIcon.current = cur_val;
		break;
	case ID_STATUSBAR_ICON_PARK:
		rStatusBarData.parkIcon.current= cur_val;
		break;
 // Del by Harry
 //   case ID_STATUSBAR_ICON_WOND:
 //       rStatusBarData.wondIcon.current= cur_val;
  //      break;
	default:
		db_error("invalide resID: %d\n", resID);
		break;
	}
}

int ResourceManager::initPlayBackPreviewResource(void)
{
	int err_code;
	int retval = 0;

	/*++++ Rect ++++*/
	configTable2 configTableRect[] = {
		/*  session name in the configFile			addr to store the value */
		{CFG_FILE_PLBPREVIEW_IMAGE,			(void*)&rPlayBackPreviewData.imageRect},
		{CFG_FILE_PLBPREVIEW_ICON,			(void*)&rPlayBackPreviewData.iconRect},
		{CFG_FILE_PLBPREVIEW_MENU,			(void*)&rPlayBackPreviewData.popupMenuRect},
		{CFG_FILE_PLBPREVIEW_MESSAGEBOX,	(void*)&rPlayBackPreviewData.messageBoxRect},
	};

	/*++++ item_width and item_height ++++*/
	configTable3 configTableIntValue[] = {
		{CFG_FILE_PLBPREVIEW_MENU,			"item_width",  (void*)&rPlayBackPreviewData.popupMenuItemWidth  },
		{CFG_FILE_PLBPREVIEW_MENU,			"item_height", (void*)&rPlayBackPreviewData.popupMenuItemHeight },
		{CFG_FILE_PLBPREVIEW_MESSAGEBOX,	"item_width",  (void*)&rPlayBackPreviewData.messageBoxItemWidth },
		{CFG_FILE_PLBPREVIEW_MESSAGEBOX,	"item_height", (void*)&rPlayBackPreviewData.messageBoxItemHeight}
	};

	/*++++ get color ++++*/
	configTable3 configTableColor[] = {
		{CFG_FILE_PLBPREVIEW_MENU, BGC_WIDGET,			(void*)&rPlayBackPreviewData.popupMenuBgcWidget },
		{CFG_FILE_PLBPREVIEW_MENU, BGC_ITEM_NORMAL,		(void*)&rPlayBackPreviewData.popupMenuBgcItemNormal },
		{CFG_FILE_PLBPREVIEW_MENU, BGC_ITEM_FOCUS,		(void*)&rPlayBackPreviewData.popUpMenuBgcItemFocus },
		{CFG_FILE_PLBPREVIEW_MENU, MAINC_THREED_BOX,	(void*)&rPlayBackPreviewData.popUpMenuMain3dBox },

		{CFG_FILE_PLBPREVIEW_MESSAGEBOX, BGC_WIDGET,			(void*)&rPlayBackPreviewData.messageBoxBgcWidget },
		{CFG_FILE_PLBPREVIEW_MESSAGEBOX, FGC_WIDGET,			(void*)&rPlayBackPreviewData.messageBoxFgcWidget },
		{CFG_FILE_PLBPREVIEW_MESSAGEBOX, BGC_ITEM_NORMAL,		(void*)&rPlayBackPreviewData.messageBoxBgcItemNormal },
		{CFG_FILE_PLBPREVIEW_MESSAGEBOX, BGC_ITEM_FOCUS,		(void*)&rPlayBackPreviewData.messageBoxBgcItemFous },
		{CFG_FILE_PLBPREVIEW_MESSAGEBOX, MAINC_THREED_BOX,		(void*)&rPlayBackPreviewData.messageBoxMain3dBox },
	};


	/*---- Rect ----*/
	for(unsigned int i = 0; i < TABLESIZE(configTableRect); i++) {
		err_code = getRectFromFileHandle(configTableRect[i].mainkey, (CDR_RECT*)configTableRect[i].addr);
		if(err_code != ETC_OK) {
			db_error("get %s rect failed: %s\n", configTableRect[i].mainkey, pEtcError(err_code));
			retval = -1;
			goto out;
		}
	}

	/* currentIcon_t */
	err_code = fillCurrentIconInfoFromFileHandle(CFG_FILE_PLBPREVIEW_ICON, rPlayBackPreviewData.icon);
	if(err_code != ETC_OK) {
		db_error("get playback preview current icon info failed: %s\n", pEtcError(err_code));
		retval = -1;
		goto out;
	}


	/*---- item_width and item_height ----*/
	for(unsigned int i = 0; i < TABLESIZE(configTableIntValue); i++) {
		err_code = getIntValueFromHandle(mCfgFileHandle, configTableIntValue[i].mainkey, configTableIntValue[i].subkey);
		if(err_code < 0) {
			db_error("get %s %s failed\n", configTableIntValue[i].mainkey, configTableIntValue[i].subkey);
			retval = -1;
			goto out;
		}
		*(unsigned int*)configTableIntValue[i].addr = err_code;
	}


	/*---- get color ----*/
	for(unsigned int i = 0; i < TABLESIZE(configTableColor); i++) {
		err_code = getABGRColorFromFileHandle(configTableColor[i].mainkey, configTableColor[i].subkey, *(gal_pixel*)configTableColor[i].addr);
		if(err_code != ETC_OK) {
			db_error("get %s %s failed: %s\n", configTableColor[i].mainkey, configTableColor[i].subkey, pEtcError(err_code));
			retval = -1;
			goto out;
		}
		//		db_msg("%s %s color is 0x%x\n", configTableColor[i].mainkey, configTableColor[i].subkey, 
		//				Pixel2DWORD(HDC_SCREEN, *(gal_pixel*)configTableColor[i].addr) );
	}

	/**** end of init the resource for PlayBackPreview window *****/

out:
	return retval;
}

int ResourceManager::initPlayBackResource(void)
{
	int err_code;
	int retval = 0;

	/* init the resource PlayBack window */
	err_code = getRectFromFileHandle(CFG_FILE_PLB_ICON, &rPlayBackData.iconRect);
	if(err_code != ETC_OK) {
		db_error("get playback icon rect failed: %s\n", pEtcError(err_code));
		retval = -1;
		goto out;
	}
	err_code = fillCurrentIconInfoFromFileHandle(CFG_FILE_PLB_ICON, rPlayBackData.icon);
	if(err_code != ETC_OK) {
		db_error("get playback current icon info failed: %s\n", pEtcError(err_code));
		retval = -1;
		goto out;
	}

	err_code = getRectFromFileHandle(CFG_FILE_PLB_PGB, &rPlayBackData.PGBRect);
	if(err_code != ETC_OK) {
		db_error("get playback ProgressBar rect failed: %s\n", pEtcError(err_code));
		retval = -1;
		goto out;
	}
	err_code = getABGRColorFromFileHandle(CFG_FILE_PLB_PGB, BGC_WIDGET, rPlayBackData.PGBBgcWidget);
	if(err_code != ETC_OK) {
		db_error("get playback bgc_widget failed: %s\n", pEtcError(err_code));
		retval = -1;
		goto out;
	}
	err_code = getABGRColorFromFileHandle(CFG_FILE_PLB_PGB, FGC_WIDGET, rPlayBackData.PGBFgcWidget);
	if(err_code != ETC_OK) {
		db_error("get playback fgc_widget failed: %s\n", pEtcError(err_code));
		retval = -1;
		goto out;
	}

out:
	return retval;
}

int ResourceManager::initMenuListResource(void)
{
	int err_code;
	int retval = 0;

	/*++++ get the menulist picture name ++++*/
	configTable3 configTableValue[] = {
		{CFG_FILE_ML, "choice",						(void*)&rMenuList.choicePic },
		{CFG_FILE_ML, "checkbox_checked_normal",	(void*)&rMenuList.checkedNormalPic },
		{CFG_FILE_ML, "checkbox_checked_press",		(void*)&rMenuList.checkedPressPic },
		{CFG_FILE_ML, "checkbox_unchecked_normal",	(void*)&rMenuList.uncheckedNormalPic },
		{CFG_FILE_ML, "checkbox_unchecked_press",	(void*)&rMenuList.uncheckedPressPic },
		{CFG_FILE_ML, "unfold_normal",	(void*)&rMenuList.unfoldNormalPic },
		{CFG_FILE_ML, "unfold_press",	(void*)&rMenuList.unfoldPressPic },
	};

	/*++++ get color ++++*/
	configTable3 configTableColor[] = {
		{CFG_FILE_ML, FGC_WIDGET,		(void*)&rMenuList.fgcWidget },
		{CFG_FILE_ML, BGC_WIDGET,		(void*)&rMenuList.bgcWidget },
		{CFG_FILE_ML, LINEC_ITEM,		(void*)&rMenuList.linecItem },
		{CFG_FILE_ML, STRINGC_NORMAL,	(void*)&rMenuList.stringcNormal },
		{CFG_FILE_ML, STRINGC_SELECTED, (void*)&rMenuList.stringcSelected },
		{CFG_FILE_ML, VALUEC_NORMAL,	(void*)&rMenuList.valuecNormal },
		{CFG_FILE_ML, VALUEC_SELECTED,	(void*)&rMenuList.valuecSelected },
		{CFG_FILE_ML, SCROLLBARC,		(void*)&rMenuList.scrollbarc },
		{CFG_FILE_SUBMENU, LINEC_TITLE,		(void*)&rMenuList.subMenuLinecTitle },
		{CFG_FILE_ML_MB, FGC_WIDGET,	(void*)&rMenuList.messageBoxFgcWidget },
		{CFG_FILE_ML_MB, BGC_WIDGET,	(void*)&rMenuList.messageBoxBgcWidget },
		{CFG_FILE_ML_MB, LINEC_TITLE,	(void*)&rMenuList.messageBoxLinecTitle },
		{CFG_FILE_ML_MB, LINEC_ITEM,	(void*)&rMenuList.messageBoxLinecItem },
		{CFG_FILE_ML_DATE, BGC_WIDGET,	(void*)&rMenuList.dateBgcWidget },
		{CFG_FILE_ML_DATE, "fgc_label",	(void*)&rMenuList.dateFgc_label },
		{CFG_FILE_ML_DATE, "fgc_number",	(void*)&rMenuList.dateFgc_number },
		{CFG_FILE_ML_DATE, LINEC_TITLE,	(void*)&rMenuList.dateLinecTitle },
		{CFG_FILE_ML_DATE, "borderc_selected",	(void*)&rMenuList.dateBordercSelected },
		{CFG_FILE_ML_DATE, "borderc_normal",	(void*)&rMenuList.dateBordercNormal },
	};

	/*++++ Rect ++++*/
	configTable2 configTableRect[] = {
		{CFG_FILE_ML,		(void*)&rMenuList.menuListRect},
		{CFG_FILE_SUBMENU,	(void*)&rMenuList.subMenuRect},
		{CFG_FILE_ML_MB,	(void*)&rMenuList.messageBoxRect},
		{CFG_FILE_ML_DATE,	(void*)&rMenuList.dateRect},
	};

	/*++++ get the current icon info ++++*/
	configTable2 configTableIcon[] = {
		{CFG_FILE_ML_WIFI,	(void*)&rMenuList.wifiIcon},
		{CFG_FILE_ML_PARK,	(void*)&rMenuList.parkIcon},
		{CFG_FILE_ML_VQ,	(void*)&rMenuList.videoQualityIcon},
		{CFG_FILE_ML_PQ,	(void*)&rMenuList.photoQualityIcon},
		{CFG_FILE_ML_VTL,	(void*)&rMenuList.timeLengthIcon},
		{CFG_FILE_ML_AWMD,	(void*)&rMenuList.moveDetectIcon},
		{CFG_FILE_ML_WB,	(void*)&rMenuList.whiteBalanceIcon},
		{CFG_FILE_ML_CONTRAST,	(void*)&rMenuList.contrastIcon},
		{CFG_FILE_ML_EXPOSURE,	(void*)&rMenuList.exposureIcon},
		{CFG_FILE_ML_POR,		(void*)&rMenuList.PORIcon},
		{CFG_FILE_ML_SS,		(void*)&rMenuList.screenSwitchIcon},
		{CFG_FILE_ML_VOLUME,	(void*)&rMenuList.volumeIcon},
		{CFG_FILE_ML_DATE,		(void*)&rMenuList.dateIcon},
		{CFG_FILE_ML_LANG,		(void*)&rMenuList.languageIcon},
		{CFG_FILE_ML_TWM,		(void*)&rMenuList.TWMIcon},
		{CFG_FILE_ML_FORMAT,	(void*)&rMenuList.formatIcon},
		{CFG_FILE_ML_FACTORYRESET,	(void*)&rMenuList.factoryResetIcon},
		{CFG_FILE_ML_FIRMWARE,		(void*)&rMenuList.firmwareIcon},
		{CFG_FILE_ML_ALIGNLINE,                (void*)&rMenuList.alignlineIcon},
		{CFG_FILE_ML_GSENSOR,	(void*)&rMenuList.gsensorIcon},
		{CFG_FILE_ML_VOICEVOL,	(void*)&rMenuList.voicevolIcon},
		{CFG_FILE_ML_SHUTDOWN,	(void*)&rMenuList.shutdownIcon},
		{CFG_FILE_ML_IMPACTLEVEL,	(void*)&rMenuList.impactlevelIcon},
		{CFG_FILE_ML_FCWSENSITY,	(void*)&rMenuList.fcwsensityIcon},
		{CFG_FILE_ML_LICENSE_PLATE_WM,	(void*)&rMenuList.licensePlateWMIcon},
#ifdef APP_ADAS
		{CFG_FILE_ML_SMARTALGORITHM,                (void*)&rMenuList.smartalgorithm}
#endif
	};

	configTable3 configTableIntValue[] = {
		{CFG_FILE_ML_DATE,		"title_height",	(void*)&rMenuList.dateTileRectH  },
		{CFG_FILE_ML_DATE,		"hBorder",		(void*)&rMenuList.dateHBorder  },
		{CFG_FILE_ML_DATE,		"yearW",		(void*)&rMenuList.dateYearW  },
		{CFG_FILE_ML_DATE,		"dateLabelW",	(void*)&rMenuList.dateLabelW  },
		{CFG_FILE_ML_DATE,		"numberW",		(void*)&rMenuList.dateNumberW  },
		{CFG_FILE_ML_DATE,		"boxH",			(void*)&rMenuList.dateBoxH  }
	};

	/*---- get the menulist picture name ----*/
	char buf[50];
	for(unsigned int i = 0; i < TABLESIZE(configTableValue); i++) {
		err_code = getValueFromFileHandle(configTableValue[i].mainkey, configTableValue[i].subkey, buf, sizeof(buf));
		if(err_code != ETC_OK) {
			db_error("get %s %s pic failed: %s\n", configTableValue[i].mainkey, configTableValue[i].subkey, pEtcError(err_code));
			retval = -1;
			goto out;
		}
		*(String8*)configTableValue[i].addr = buf;
		//		db_error("get %s %s pic is %s\n", configTableValue[i].mainkey, configTableValue[i].subkey, (*(String8*)configTableValue[i].addr).string() );
	}

	/*---- get color ----*/
	for(unsigned int i = 0; i < TABLESIZE(configTableColor); i++) {
		err_code = getABGRColorFromFileHandle(configTableColor[i].mainkey, configTableColor[i].subkey, *(gal_pixel*)configTableColor[i].addr);
		if(err_code != ETC_OK) {
			db_error("get %s %s failed: %s\n", configTableColor[i].mainkey, configTableColor[i].subkey, pEtcError(err_code));
			retval = -1;
			goto out;
		}
		//		db_error("get %s %s color is 0x%x\n", configTableColor[i].mainkey, configTableColor[i].subkey, 
		//				Pixel2DWORD(HDC_SCREEN, *(gal_pixel*)configTableColor[i].addr) );
	}

	/*---- Rect ----*/
	for(unsigned int i = 0; i < TABLESIZE(configTableRect); i++) {
		err_code = getRectFromFileHandle(configTableRect[i].mainkey, (CDR_RECT*)configTableRect[i].addr);
		if(err_code != ETC_OK) {
			db_error("get %s rect failed: %s\n", configTableRect[i].mainkey, pEtcError(err_code));
			retval = -1;
			goto out;
		}
	}

	/*---- get the current icon info ----*/
	unsigned int current;
	for(unsigned int i = 0; i < TABLESIZE(configTableIcon); i++) {
		err_code = fillCurrentIconInfoFromFileHandle(configTableIcon[i].mainkey, *(currentIcon_t*)configTableIcon[i].addr);
		if(err_code != ETC_OK) {
			db_error("get %s icon failed: %s\n", configTableIcon[i].mainkey, pEtcError(err_code) );
			retval = -1;
			goto out;
		}
		current = (*(currentIcon_t*)configTableIcon[i].addr).current;
		//		db_msg("%s current icon is %s\n", configTableIcon[i].mainkey, (*(currentIcon_t*)configTableIcon[i].addr).icon.itemAt(current).string() );
	}

	/*  init inval */
	for(unsigned int i = 0; i < TABLESIZE(configTableIntValue); i++) {
		err_code = getIntValueFromHandle(mCfgFileHandle, configTableIntValue[i].mainkey, configTableIntValue[i].subkey);
		if(err_code < 0) {
			db_error("get %s %s failed\n", configTableIntValue[i].mainkey, configTableIntValue[i].subkey);
			retval = -1;
			goto out;
		}
		*(unsigned int*)configTableIntValue[i].addr = err_code;
	}

out:
	return retval;
}

void ResourceManager::dump()
{

	db_msg("menuDataLang.count %d  menuDataLang.current %d", menuDataLang.count, menuDataLang.current);
	db_msg("menuDataVQ.count %d	menuDataVQ.current %d",menuDataVQ.count, menuDataVQ.current);
	db_msg("menuDataPQ.count %d menuDataPQ.current %d",menuDataPQ.count,menuDataPQ.current);
	db_msg("menuDataVTL.count %d menuDataVTL.current %d",menuDataVTL.count,menuDataVTL.current);
	db_msg("menuDataWB.count %d menuDataWB.current %d",menuDataWB.count,menuDataWB.current);
	db_msg("menuDataContrast.count %d menuDataContrast.current %d",menuDataContrast.count,menuDataContrast.current);
	db_msg("menuDataExposure.count %d menuDataExposure.current %d",menuDataExposure.count,menuDataExposure.current);
	db_msg("menuDataSS.count %d menuDataSS.current %d",menuDataSS.count,menuDataSS.current);
	db_msg("menuDataPOREnable %d",menuDataPOREnable);
	db_msg("menuDataVolumeEnable %d",menuDataVolumeEnable);
	db_msg("menuDataTWMEnable %d",menuDataTWMEnable);
	db_msg("menuDataAWMDEnable %d",menuDataAWMDEnable);
	db_msg("menuDatafcwsensity %d",menuDatafcwsensity);

}

int ResourceManager::initMenuResource(void)
{

	configTable2 configTableMenuItem[] = {
		{CFG_MENU_LANG, (void*)&menuDataLang},
		{CFG_MENU_VQ,	(void*)&menuDataVQ},
		{CFG_MENU_PQ,	(void*)&menuDataPQ},
		{CFG_MENU_VTL,	(void*)&menuDataVTL},
		{CFG_MENU_WB,	(void*)&menuDataWB},
		{CFG_MENU_CONTRAST, (void*)&menuDataContrast},
		{CFG_MENU_EXPOSURE, (void*)&menuDataExposure},
		{CFG_MENU_SS,		(void*)&menuDataSS},
		{CFG_MENU_GSENSOR,		(void*)&menuDataGsensor},
		{CFG_MENU_VOICEVOL,		(void*)&menuDataVoiceVol},
		{CFG_MENU_SHUTDOWN,		(void*)&menuDataShutDown},
		{CFG_MENU_IMPACTLEVEL,		(void*)&menuDataImpactLevel},
		{CFG_MENU_FCWSENSITY,		(void*)&menuDatafcwsensity},
#ifdef APP_ADAS
		{CFG_MENU_SMARTALGORITHM,	(void*)&menuDataSmartAlgorithmEnable}
#endif
	};

	configTable2 configTableSwitch[] = {
		{CFG_MENU_WIFI, (void*)&menuDataWIFIEnable},
		{CFG_MENU_PARK, (void*)&menuDataPARKEnable},
		{CFG_MENU_POR, (void*)&menuDataPOREnable},
		{CFG_MENU_VOLUME,	(void*)&menuDataVolumeEnable},
		{CFG_MENU_TWM,		(void*)&menuDataTWMEnable},
		{CFG_MENU_AWMD,		(void*)&menuDataAWMDEnable},
		{CFG_MENU_ALIGNLINE, 	(void*)&menuDataAlignlineEnable},
	};

	int value;
	for(unsigned int i = 0; i < TABLESIZE(configTableMenuItem); i++) {
		value = getIntValueFromHandle(mCfgMenuHandle, configTableMenuItem[i].mainkey, "current");	
		if(value < 0) {
			db_error("get current %s failed\n", configTableMenuItem[i].mainkey);
			return -1;
		}
		((menuItem_t*)(configTableMenuItem[i].addr))->current = value;

		value = getIntValueFromHandle(mCfgMenuHandle, configTableMenuItem[i].mainkey, "count");	
		if(value < 0) {
			db_error("get count %s failed\n", configTableMenuItem[i].mainkey);
			return -1;
		}
		((menuItem_t*)(configTableMenuItem[i].addr))->count = value;
		//		db_msg("%s current is %d, count is %d\n", configTableMenuItem[i].mainkey, ((menuItem_t*)(configTableMenuItem[i].addr))->current, value);	
	}

	mCdrVolume = VolNum[menuDataVoiceVol.current].voice;
	setKeySoundVol(mCdrVolume);

	value = getIntValueFromHandle(mCfgMenuHandle, "defaultlicense", "current");
	if(value < 0) {
		db_error("get switch defaultlicense %s failed\n");
		return -1;
	}
	if(value == 0) {
		menuDataLWM.LWMEnable = false;
	} else {
		menuDataLWM.LWMEnable = true;
	}
	if(GetValueFromEtcFile(configMenu, "defaultlicense", "position", menuDataLWM.lwaterMark ,16) < 0) {
		db_error("get defaultlicense failed\n");
		return -1;

	}else{
		db_msg(">>>>>>>>>>>>>>defaultlicense:position:%s",menuDataLWM.lwaterMark);
		db_msg(">>>>>>>>>>>>>>defaultlicense:enable?:%d",menuDataLWM.LWMEnable );
	}
	
	for(unsigned int i = 0; i < TABLESIZE(configTableSwitch); i++) {
		value = getIntValueFromHandle(mCfgMenuHandle, "switch", configTableSwitch[i].mainkey);
		if(value < 0) {
			db_error("get switch %s failed\n", configTableSwitch[i].mainkey);
			return -1;
		}
		if(value == 0) {
			*((bool*)configTableSwitch[i].addr) = false;
		} else {
			*((bool*)configTableSwitch[i].addr) = true;
		}
		db_error("switch %s is %d\n", configTableSwitch[i].mainkey, value);
	}

	return 0;
}

int ResourceManager::refreshLWMResource(void)
{
	int value;
	int retval = 0 ;
	getCfgMenuHandle();
		value = getIntValueFromHandle(mCfgMenuHandle, "defaultlicense", "current");
	if(value < 0) {
		db_error("get switch defaultlicense %s failed\n");
		return -1;
	}
	if(value == 0) {
		menuDataLWM.LWMEnable = false;
	} else {
		menuDataLWM.LWMEnable = true;
	}
	if(GetValueFromEtcFile(configMenu, "defaultlicense", "position", menuDataLWM.lwaterMark ,16) < 0) {
		db_error("get defaultlicense failed\n");
		return -1;
	}

	releaseCfgMenuHandle();
	return 1 ;
}

int ResourceManager::initOtherResource(void)
{
	int err_code;
	int retval = 0;
	configTable2 configTableRect[] = {
		{CFG_FILE_WARNING_MB,		(void*)&rWarningMB.rect},
		{CFG_FILE_CONNECT2PC,		(void*)&rConnectToPC.rect},
		{CFG_FILE_TIPLABEL,			(void*)&rTipLabel.rect}
	};

	configTable3 configTableColor[] = {
		{CFG_FILE_WARNING_MB, FGC_WIDGET,	(void*)&rWarningMB.fgcWidget },
		{CFG_FILE_WARNING_MB, BGC_WIDGET,	(void*)&rWarningMB.bgcWidget },
		{CFG_FILE_WARNING_MB, LINEC_TITLE,	(void*)&rWarningMB.linecTitle },
		{CFG_FILE_WARNING_MB, LINEC_ITEM,	(void*)&rWarningMB.linecItem },
		{CFG_FILE_CONNECT2PC, BGC_WIDGET,	(void*)&rConnectToPC.bgcWidget },
		{CFG_FILE_CONNECT2PC, BGC_ITEM_FOCUS,	(void*)&rConnectToPC.bgcItemFocus},
		{CFG_FILE_CONNECT2PC, BGC_ITEM_NORMAL,	(void*)&rConnectToPC.bgcItemNormal},
		{CFG_FILE_CONNECT2PC, MAINC_THREED_BOX,	(void*)&rConnectToPC.mainc_3dbox},
		{CFG_FILE_TIPLABEL,	  FGC_WIDGET,	(void*)&rTipLabel.fgcWidget},
		{CFG_FILE_TIPLABEL,	  BGC_WIDGET,	(void*)&rTipLabel.bgcWidget},
		{CFG_FILE_TIPLABEL,	  LINEC_TITLE,	(void*)&rTipLabel.linecTitle},
	};

	configTable3 configTableIntValue[] = {
		{CFG_FILE_CONNECT2PC, "item_width",		(void*)&rConnectToPC.itemWidth  },
		{CFG_FILE_CONNECT2PC, "item_height",	(void*)&rConnectToPC.itemHeight  },
		{CFG_FILE_TIPLABEL,	  "title_height",	(void*)&rTipLabel.titleHeight }
	};

	configTable3 configTableValue[] = {
		{CFG_FILE_OTHER_PIC, "poweroff",		(void*)&rPoweroffPic }
	};

	/*---- Rect ----*/
	for(unsigned int i = 0; i < TABLESIZE(configTableRect); i++) {
		err_code = getRectFromFileHandle(configTableRect[i].mainkey, (CDR_RECT*)configTableRect[i].addr);
		if(err_code != ETC_OK) {
			db_error("get %s rect failed: %s\n", configTableRect[i].mainkey, pEtcError(err_code));
			retval = -1;
			goto out;
		}
	}

	/*---- get color ----*/
	for(unsigned int i = 0; i < TABLESIZE(configTableColor); i++) {
		err_code = getABGRColorFromFileHandle(configTableColor[i].mainkey, configTableColor[i].subkey, *(gal_pixel*)configTableColor[i].addr);
		if(err_code != ETC_OK) {
			db_error("get %s %s failed: %s\n", configTableColor[i].mainkey, configTableColor[i].subkey, pEtcError(err_code));
			retval = -1;
			goto out;
		}
	}

	/*  init inval */
	for(unsigned int i = 0; i < TABLESIZE(configTableIntValue); i++) {
		err_code = getIntValueFromHandle(mCfgFileHandle, configTableIntValue[i].mainkey, configTableIntValue[i].subkey);
		if(err_code < 0) {
			db_error("get %s %s failed\n", configTableIntValue[i].mainkey, configTableIntValue[i].subkey);
			retval = -1;
			goto out;
		}
		*(unsigned int*)configTableIntValue[i].addr = err_code;
	}

	/*---- get the menulist picture name ----*/
	char buf[50];
	for(unsigned int i = 0; i < TABLESIZE(configTableValue); i++) {
		err_code = getValueFromFileHandle(configTableValue[i].mainkey, configTableValue[i].subkey, buf, sizeof(buf));
		if(err_code != ETC_OK) {
			db_error("get %s %s pic failed: %s\n", configTableValue[i].mainkey, configTableValue[i].subkey, pEtcError(err_code));
			retval = -1;
			goto out;
		}
		*(String8*)configTableValue[i].addr = buf;
		//db_error("get %s %s pic is %s\n", configTableValue[i].mainkey, configTableValue[i].subkey, (*(String8*)configTableValue[i].addr).string() );
	}

out:
	return retval;
}

gal_pixel ResourceManager::getStatusBarColor(enum ResourceID resID, enum ColorType type)
{
	gal_pixel color = 0;
	if(resID == ID_STATUSBAR) {
		switch(type) {
		case COLOR_FGC:
			color = rStatusBarData.fgc;
			break;
		case COLOR_BGC:
			color = rStatusBarData.bgc;
			break;
		default:
			db_error("invalid ColorType: %d\n", type);
			break;
		}
	} else {
		db_error("invalid resID: %d\n", resID);
	}

	return color;
}

gal_pixel ResourceManager::getPlaybackPreviewColor(enum ResourceID resID, enum ColorType type)
{
	gal_pixel color = 0;
	if(resID == ID_PLAYBACKPREVIEW_MENU) {
		switch(type) {
		case COLOR_BGC:
			color = rPlayBackPreviewData.popupMenuBgcWidget;
			break;
		case COLOR_BGC_ITEMFOCUS:
			color = rPlayBackPreviewData.popUpMenuBgcItemFocus;
			break;
		case COLOR_BGC_ITEMNORMAL:
			color = rPlayBackPreviewData.popupMenuBgcItemNormal;
			break;
		case COLOR_MAIN3DBOX:
			color = rPlayBackPreviewData.popUpMenuMain3dBox;
			break;
		default:
			db_error("invalid ColorType: %d\n", type);
			break;
		}
	} else if(resID == ID_PLAYBACKPREVIEW_MB) {
		switch(type) {
		case COLOR_BGC:
			color = rPlayBackPreviewData.messageBoxBgcWidget;
			break;
		case COLOR_FGC:
			color = rPlayBackPreviewData.messageBoxFgcWidget;
			break;
		case COLOR_BGC_ITEMFOCUS:
			color = rPlayBackPreviewData.messageBoxBgcItemFous;
			break;
		case COLOR_BGC_ITEMNORMAL:
			color = rPlayBackPreviewData.messageBoxBgcItemNormal;
			break;
		case COLOR_MAIN3DBOX:
			color = rPlayBackPreviewData.messageBoxMain3dBox;
			break;
		default:
			db_error("invalid ColorType: %d\n", type);
			break;
		}
	} else {
		db_error("invalid resID: %d\n", resID);
	}

	return color;
}

gal_pixel ResourceManager::getPlaybackColor(enum ResourceID resID, enum ColorType type)
{
	gal_pixel color = 0;
	if(resID == ID_PLAYBACK_PGB) {
		switch(type) {
		case COLOR_BGC:
			color = rPlayBackData.PGBBgcWidget;
			break;
		case COLOR_FGC:
			color = rPlayBackData.PGBFgcWidget;
			break;
		default:
			db_error("invalid ColorType: %d\n", type);
			break;
		}
	} else {
		db_error("invalid resID: %d\n", resID);
	}

	return color;
}


gal_pixel ResourceManager::getMenuListColor(enum ResourceID resID, enum ColorType type)
{
	gal_pixel color = 0;
	if(resID == ID_MENU_LIST) {
		switch(type) {
		case COLOR_BGC:
			color = rMenuList.bgcWidget;
			break;
		case COLOR_FGC:
			color = rMenuList.fgcWidget;
			break;
		case COLOR_LINEC_ITEM:
			color = rMenuList.linecItem;
			break;
		case COLOR_STRINGC_NORMAL:
			color = rMenuList.stringcNormal;
			break;
		case COLOR_STRINGC_SELECTED:
			color = rMenuList.stringcSelected;
			break;
		case COLOR_VALUEC_NORMAL:
			color = rMenuList.valuecNormal;
			break;
		case COLOR_VALUEC_SELECTED:
			color = rMenuList.valuecSelected;
			break;
		case COLOR_SCROLLBARC:
			color = rMenuList.scrollbarc;
			break;
		default:
			db_error("invalid ColorType: %d\n", type);
			break;
		}
	} else if(resID == ID_MENU_LIST_MB) {
		switch(type) {
		case COLOR_BGC:
			color = rMenuList.messageBoxBgcWidget;
			break;
		case COLOR_FGC:
			color = rMenuList.messageBoxFgcWidget;
			break;
		case COLOR_LINEC_TITLE:
			color = rMenuList.messageBoxLinecTitle;
			break;
		case COLOR_LINEC_ITEM:
			color = rMenuList.messageBoxLinecItem;
			break;
		default:
			db_error("invalid ColorType: %d\n", type);
			break;
		}
	} else if(resID == ID_MENU_LIST_DATE) {
		switch(type) {
		case COLOR_BGC:
			color = rMenuList.dateBgcWidget;
			break;
		case COLOR_FGC_LABEL:
			color = rMenuList.dateFgc_label;
			break;
		case COLOR_FGC_NUMBER:
			color = rMenuList.dateFgc_number;
			break;
		case COLOR_LINEC_TITLE:
			color = rMenuList.dateLinecTitle;
			break;
		case COLOR_BORDERC_NORMAL:
			color = rMenuList.dateBordercNormal;
			break;
		case COLOR_BORDERC_SELECTED:
			color = rMenuList.dateBordercSelected;
			break;
		default:
			db_error("invalid ColorType: %d\n", type);
			break;
		}

	} else {
		db_error("invalid resID: %d\n", resID);
	}

	return color;
}

gal_pixel ResourceManager::getSubMenuColor(enum ResourceID resID, enum ColorType type)
{
	gal_pixel color = 0;
	if(resID == ID_SUBMENU) {
		switch(type) {
		case COLOR_BGC:
			color = rMenuList.subMenuBgcWidget;
			break;
		case COLOR_FGC:
			color = rMenuList.subMenuFgcWidget;
			break;
		case COLOR_LINEC_TITLE:
			color = rMenuList.subMenuLinecTitle;
			break;
		default:
			db_error("invalid ColorType: %d\n", type);
			break;
		}
	} else {
		db_error("invalid resID: %d\n", resID);
	}

	return color;
}

gal_pixel ResourceManager::getWarningMBColor(enum ResourceID resID, enum ColorType type)
{
	gal_pixel color = 0;
	if(resID == ID_WARNNING_MB) {
		switch(type) {
		case COLOR_BGC:
			color = rWarningMB.bgcWidget;
			break;
		case COLOR_FGC:
			color = rWarningMB.fgcWidget;
			break;
		case COLOR_LINEC_TITLE:
			color = rWarningMB.linecTitle;
			break;
		case COLOR_LINEC_ITEM:
			color = rWarningMB.linecItem;
			break;
		default:
			db_error("invalid ColorType: %d\n", type);
			break;
		}
	} else {
		db_error("invalid resID: %d\n", resID);
	}

	return color;
}

gal_pixel ResourceManager::getConnect2PCColor(enum ResourceID resID, enum ColorType type)
{
	gal_pixel color = 0;
	if(resID == ID_CONNECT2PC) {
		switch(type) {
		case COLOR_BGC:
			color = rConnectToPC.bgcWidget;
			break;
		case COLOR_BGC_ITEMFOCUS:
			color = rConnectToPC.bgcItemFocus;
			break;
		case COLOR_BGC_ITEMNORMAL:
			color = rConnectToPC.bgcItemNormal;
			break;
		case COLOR_MAIN3DBOX:
			color = rConnectToPC.mainc_3dbox;
			break;
		default:
			db_error("invalid ColorType: %d\n", type);
			break;
		}
	} else {
		db_error("invalid resID: %d\n", resID);
	}

	return color;
}

gal_pixel ResourceManager::getResColor(enum ResourceID resID, enum ColorType type)
{
	gal_pixel color = 0;

	switch(resID) {
	case ID_STATUSBAR:
		color = getStatusBarColor(resID, type);
		break;
	case ID_PLAYBACKPREVIEW_MENU:
	case ID_PLAYBACKPREVIEW_MB:
		color = getPlaybackPreviewColor(resID, type);
		break;
	case ID_PLAYBACK_PGB:
		color = getPlaybackColor(resID, type);
		break;
	case ID_MENU_LIST:
	case ID_MENU_LIST_MB:
	case ID_MENU_LIST_DATE:
		color = getMenuListColor(resID, type);
		break;
	case ID_SUBMENU:
		color = getSubMenuColor(resID, type);
		break;
	case ID_WARNNING_MB:
		color = getWarningMBColor(resID, type);
		break;
	case ID_CONNECT2PC:
		color = getConnect2PCColor(resID, type);
		break;
	case ID_TIPLABEL:
		if(type == COLOR_BGC)
			color = rTipLabel.bgcWidget;
		else if(type == COLOR_FGC)
			color = rTipLabel.fgcWidget;
		else if(type == COLOR_LINEC_TITLE)
			color = rTipLabel.linecTitle;
		break;
	default:
		db_error("invalid resIndex for the color\n");
		break;
	}

	return color;
}

int ResourceManager::getResRect(enum ResourceID resID, CDR_RECT &rect)
{
	switch(resID) {
	case ID_SCREEN:
	case ID_POWEROFF:
		rect = rScreenRect;
		break;
	case ID_STATUSBAR:
		rect = rStatusBarData.STBRect;
		break;
	case ID_STATUSBAR_ICON_WINDOWPIC:
		rect = rStatusBarData.windowPicRect;
		break;
	case ID_STATUSBAR_LABEL1:
		rect = rStatusBarData.label1Rect;
		break;
	case ID_STATUSBAR_LABEL2:
		rect = rStatusBarData.label2Rect;
		break;
	case ID_STATUSBAR_LABEL_RESERVE:
		rect = rStatusBarData.reserveLabelRect;
		break;
	case ID_STATUSBAR_ICON_WIFI:
		rect = rStatusBarData.wifiRect;
		break;
	case ID_STATUSBAR_ICON_PARK:
		rect = rStatusBarData.parkRect;
		break;
	case ID_STATUSBAR_ICON_AWMD:
		rect = rStatusBarData.awmdRect;
		break;
	case ID_STATUSBAR_ICON_LOCK:
		rect = rStatusBarData.lockRect;
		break;
	case ID_STATUSBAR_ICON_UVC:
		rect = rStatusBarData.uvcRect;
		break;
	case ID_STATUSBAR_ICON_TFCARD:
		rect = rStatusBarData.tfCardRect;
		break;
	case ID_STATUSBAR_ICON_AUDIO:
		rect = rStatusBarData.audioRect;
		break;
	case ID_STATUSBAR_ICON_BAT:
		rect = rStatusBarData.batteryRect;
		break;
// Del by Harry
//	case ID_STATUSBAR_ICON_WOND:
//		rect = rStatusBarData.wondRect;
//		break;
	case ID_PLAYBACKPREVIEW_IMAGE:
		rect = rPlayBackPreviewData.imageRect;
		break;
	case ID_PLAYBACKPREVIEW_ICON:
		rect = rPlayBackPreviewData.iconRect;
		break;
	case ID_PLAYBACKPREVIEW_MENU:
		rect = rPlayBackPreviewData.popupMenuRect;
		break;
	case ID_PLAYBACKPREVIEW_MB:
		rect = rPlayBackPreviewData.messageBoxRect;
		break;
	case ID_PLAYBACK_ICON:
		rect = rPlayBackData.iconRect;
		break;
	case ID_PLAYBACK_PGB:
		rect = rPlayBackData.PGBRect;
		break;
	case ID_MENU_LIST:
		rect  = rMenuList.menuListRect;
		break;
	case ID_SUBMENU:
		rect = rMenuList.subMenuRect;
		break;
	case ID_MENU_LIST_MB:
		rect = rMenuList.messageBoxRect;
		break;
	case ID_WARNNING_MB:
		rect = rWarningMB.rect;
		break;
	case ID_MENU_LIST_DATE:
		rect = rMenuList.dateRect;
		break;
	case ID_CONNECT2PC:
		rect = rConnectToPC.rect;
		break;
	case ID_TIPLABEL:
		rect = rTipLabel.rect;
		break;
	default:
		db_error("invalid resID index: %d\n", resID);
		return -1;
	}

	return 0;
}

int ResourceManager::getResBmp(enum ResourceID resID, enum BmpType type, BITMAP &bmp)
{
	int err_code;
	String8 file;

	if(resID == ID_STATUSBAR_ICON_WINDOWPIC) {
		if(getWindPicFileName(type, file) < 0 ) {
			db_error("get window pic bmp failed\n");	
			return -1;
		}
	} else if(resID >= ID_STATUSBAR_ICON_AWMD && resID <= ID_STATUSBAR_ICON_BAT) {
		if(getCurrentIconFileName(resID, file) < 0) {
			db_error("get current icon pic bmp failed\n");	
			return -1;
		}
	} else if(resID >= ID_MENU_LIST_VQ && resID <= ID_MENU_LIST_UNCHECKBOX_PIC) {
		err_code = getMLPICFileName(resID, type, file);	
		if(err_code < 0) {
			db_error("get menu_list pic bmp failed\n");	
			return -1;
		}
	} else if(resID == ID_SUBMENU_CHOICE_PIC){
		file = rMenuList.choicePic;
	} else if(resID == ID_PLAYBACKPREVIEW_ICON || resID == ID_PLAYBACK_ICON) {
		if(getCurrentIconFileName(resID, file) < 0) {
			db_error("get current icon pic bmp failed\n");	
			return -1;
		}
	} else if(resID == ID_POWEROFF) {
		file = rPoweroffPic;
	} else {
		db_error("invalid resID: %d\n", resID);
		return -1;
	}

	db_msg("resID: %d, bmp file is %s\n", resID, file.string());	

	err_code = LoadBitmapFromFile(HDC_SCREEN, &bmp, file.string());	
	if(err_code != ERR_BMP_OK) {
		db_error("load %s bitmap failed\n", file.string());
	}
	db_msg("LoadBitmapFromFile %s finished\n", file.string());

	return 0;
}

int ResourceManager::getResBmpSubMenuCheckbox(enum ResourceID resID, bool isHilight, BITMAP &bmp)
{

	bool isChecked = false;
	switch(resID) {
	case ID_MENU_LIST_WIFI:
		if(menuDataWIFIEnable == true)
			isChecked = true;
		break;
	case ID_MENU_LIST_PARK:
		if(menuDataPARKEnable == true)
			isChecked = true;
		break;
	case ID_MENU_LIST_ALIGNLINE:
		if(menuDataAlignlineEnable == true)
			isChecked = true;
		break;
	case ID_MENU_LIST_POR:
		if(menuDataPOREnable == true)
			isChecked = true;
		break;
	case ID_MENU_LIST_SILENTMODE:
		if(menuDataVolumeEnable == true)
			isChecked = true;
		break;
	case ID_MENU_LIST_TWM:
		if(menuDataTWMEnable == true)
			isChecked = true;
		break;
	case ID_MENU_LIST_AWMD:
		if(menuDataAWMDEnable == true)
			isChecked = true;
		break;
	default:
		db_error("invalid resID: %d\n", resID);
		return -1;
	}

	if(isChecked == true) {
		if(isHilight == true)
			return getResBmp(ID_MENU_LIST_CHECKBOX_PIC, BMPTYPE_SELECTED, bmp);
		else
			return getResBmp(ID_MENU_LIST_CHECKBOX_PIC, BMPTYPE_UNSELECTED, bmp);
	} else {
		if(isHilight == true)
			return getResBmp(ID_MENU_LIST_UNCHECKBOX_PIC, BMPTYPE_SELECTED, bmp);
		else	
			return getResBmp(ID_MENU_LIST_UNCHECKBOX_PIC, BMPTYPE_UNSELECTED, bmp);
	}

}

int ResourceManager::getResIntValue(enum ResourceID resID, enum IntValueType type)
{
	unsigned int intValue = -1;

	if(resID >= ID_MENU_LIST_VQ && resID <= ID_MENU_LIST_FIRMWARE) {
		if(type == INTVAL_SUBMENU_INDEX) {
			return getSubMenuCurrentIndex(resID);
		} else if(type == INTVAL_SUBMENU_COUNT) {
			return getSubMenuCounts(resID);
		} else {
			return getSubMenuIntAttr(resID, type);
		}
	} else {
		switch(resID) {
		case ID_PLAYBACKPREVIEW_MENU:
			if(type == INTVAL_ITEMWIDTH)
				intValue = rPlayBackPreviewData.popupMenuItemWidth;
			else if(type == INTVAL_ITEMHEIGHT) 
				intValue = rPlayBackPreviewData.popupMenuItemHeight;
			else
				db_error("invalid IntValueType: %d\n", type);
			break;
		case ID_PLAYBACKPREVIEW_MB:
			if(type == INTVAL_ITEMWIDTH)
				intValue = rPlayBackPreviewData.messageBoxItemWidth;
			else if(type == INTVAL_ITEMHEIGHT)
				intValue = rPlayBackPreviewData.messageBoxItemHeight;
			else
				db_error("invalid IntValueType: %d\n", type);
			break;
		case ID_CONNECT2PC:
			if(type == INTVAL_ITEMWIDTH)
				intValue = rConnectToPC.itemWidth;
			else if(type == INTVAL_ITEMHEIGHT)
				intValue = rConnectToPC.itemHeight;
			else
				db_error("invalid IntValueType: %d\n", type);
			break;
		case ID_TIPLABEL:
			if(type == INTVAL_TITLEHEIGHT)
				intValue = rTipLabel.titleHeight;
			else
				db_error("invalid IntValueType: %d\n", type);
			break;
		default:
			db_error("invalid resID: %d\n", resID);
			break;
		}
	}

	return intValue;
}

int ResourceManager::setResIntValue(enum ResourceID resID, enum IntValueType type, int value)
{
	if(resID >= ID_MENU_LIST_VQ && resID <= ID_MENU_LIST_FIRMWARE) {
		if(type == INTVAL_SUBMENU_INDEX)
			return setSubMenuCurrentIndex(resID, value);
		else {
			db_error("invalide IntValueType: %d\n", type);
		}
	} else {
		db_error("invalid resID %d\n", resID);
	}

	return -1;
}

bool ResourceManager::getResBoolValue(ResourceID resID)
{
	db_msg("getResBoolValue: resID: %d\n", resID);
	bool value = false;
	switch(resID) {
	case ID_MENU_LIST_WIFI:
		// In this app,wifi alway open
		value = true; // menuDataWIFIEnable;
		break;
	case ID_MENU_LIST_PARK:
		value = menuDataPARKEnable;
		break;
	case ID_MENU_LIST_ALIGNLINE:
		value = menuDataAlignlineEnable;
		break;
	case ID_MENU_LIST_POR:
		value = menuDataPOREnable;
		db_info("POR: value is %d\n", value);
		break;
	case ID_MENU_LIST_SILENTMODE:
		value = menuDataVolumeEnable;
		break;
	case ID_MENU_LIST_TWM:
		value = menuDataTWMEnable;
		break;
	case ID_MENU_LIST_LWM:
		value = menuDataLWM.LWMEnable;
		break;
	case ID_MENU_LIST_AWMD:
		value = menuDataAWMDEnable;
		db_info("AWMD: value is %d\n", value);
		break;
	default:
		db_error("invalid resID %d\n", resID);
	}

	return value;
}

int ResourceManager::notifyWaterMark()
{
	db_msg(" ");
	int wmFlag = 0;
	if (menuDataTWMEnable) {
		wmFlag |= WATERMARK_TWM;
	}
	if (menuDataLWM.LWMEnable) {
		wmFlag |= WATERMARK_LWM;
	}
	SendMessage(mHwnd[WINDOWID_RECORDPREVIEW], MSG_WATER_MARK, (WPARAM)wmFlag, 0);
	return 0;
}

int ResourceManager::setResBoolValue(enum ResourceID resID, bool value)
{
	db_msg("setResBoolValue: resID: %d, value: %d\n", resID, value);
	switch(resID) {
	case ID_MENU_LIST_WIFI:
		menuDataWIFIEnable = value;
		SendMessage(mHwnd[WINDOWID_STATUSBAR], STBM_WIFI, (WPARAM)value, 0);
		SendMessage(mHwnd[WINDOWID_MAIN], MSG_WIFI_SWITCH, (WPARAM)value, 0);
#ifdef YOUKU_PLATFORM
		SendMessage(mHwnd[WINDOWID_MAIN], MSG_YOUKU_SERVICE_SWITCH, (WPARAM)value, 0);
#endif
		break;
	case ID_MENU_LIST_PARK:
		menuDataPARKEnable = value;
		SendMessage(mHwnd[WINDOWID_STATUSBAR], STBM_PARK, (WPARAM)value, 0);
		break;
	case ID_MENU_LIST_ALIGNLINE:
		menuDataAlignlineEnable = value;
		SendMessage(mHwnd[WINDOWID_RECORDPREVIEW], MSG_DRAW_ALIGN_LINE, value, 0);
		break;
	case ID_MENU_LIST_POR:
		// Change force 1 by Harry
		// menuDataPOREnable = value;
		menuDataPOREnable = 1;
		break;
	case ID_STATUSBAR_ICON_AUDIO:
	case ID_MENU_LIST_SILENTMODE:
		menuDataVolumeEnable = value;
		if(value == true)
			rStatusBarData.audioIcon.current = 1;
		else
			rStatusBarData.audioIcon.current = 0;

		SendMessage(mHwnd[WINDOWID_RECORDPREVIEW], MSG_AUDIO_SILENT, (WPARAM)value, 0);
		SendMessage(mHwnd[WINDOWID_STATUSBAR], STBM_VOICE, (WPARAM)value, 0);
		break;
	case ID_MENU_LIST_TWM:
		menuDataTWMEnable = value;
		notifyWaterMark();
		break;
	case ID_MENU_LIST_LWM:
		menuDataLWM.LWMEnable = value;
		notifyWaterMark();
		break;
	case ID_MENU_LIST_AWMD:
		menuDataAWMDEnable = value;
		SendMessage(mHwnd[WINDOWID_RECORDPREVIEW], MSG_AWMD, (WPARAM)value, 0);
		SendMessage(mHwnd[WINDOWID_STATUSBAR], STBM_AWMD, (WPARAM)value, 0);
		break;
	default:
		db_error("invalide resID: %d\n", resID);
		return -1;
	}

	return 0;
}

const char* ResourceManager::getResMenuItemString(enum ResourceID resID)
{
	const char* ptr = NULL;
	switch(resID) {
	case ID_MENU_LIST_WIFI:
		{
			static char  wifiString[64]={0};
			char ssid[32]={0};
			cfgDataGetString("persist.sys.wifi.ssid", ssid, " ");
			if (strcmp(ssid, " ") == 0) {
				StorageManager *sm = StorageManager::getInstance();
				sm->generateSsid(ssid);
				cfgDataSetString("persist.sys.wifi.ssid", ssid);
			}
			sprintf(wifiString, "%s        ssid: %s", getLabel(LANG_LABEL_MENU_WIFI), ssid);
			db_msg("  wifiString:%s ", wifiString);
			ptr = wifiString;
		}
		break;
	case ID_MENU_LIST_PARK:
		ptr = getLabel(LANG_LABEL_MENU_PARK);
		break;
#ifdef APP_ADAS
	case ID_MENU_LIST_SMARTALGORITHM:
		ptr = getLabel(LANG_LABEL_MENU_SMARTALGORITHM);
		break;
#endif
	case ID_MENU_LIST_ALIGNLINE:
		ptr = getLabel(LANG_LABEL_MENU_ALIGNLINE);
		break;
	case ID_MENU_LIST_VQ:
		ptr = getLabel(LANG_LABEL_MENU_VQ);
		break;
	case ID_MENU_LIST_PQ:
		ptr = getLabel(LANG_LABEL_MENU_PQ);
		break;
	case ID_MENU_LIST_VTL:
		ptr = getLabel(LANG_LABEL_MENU_VTL);
		break;
	case ID_MENU_LIST_AWMD:
		ptr = getLabel(LANG_LABEL_MENU_AWMD);
		break;
	case ID_MENU_LIST_WB:
		ptr = getLabel(LANG_LABEL_MENU_WB);
		break;
	case ID_MENU_LIST_CONTRAST:
		ptr = getLabel(LANG_LABEL_MENU_CONTRAST);
		break;
	case ID_MENU_LIST_EXPOSURE:
		ptr = getLabel(LANG_LABEL_MENU_EXPOSURE);
		break;
	case ID_MENU_LIST_POR:
		ptr = getLabel(LANG_LABEL_MENU_POR);
		break;
	case ID_MENU_LIST_SS:
		ptr = getLabel(LANG_LABEL_MENU_SS);
		break;
	case ID_MENU_LIST_SILENTMODE:
		ptr = getLabel(LANG_LABEL_MENU_SILENTMODE);
		break;
	case ID_MENU_LIST_DATE:
		ptr = getLabel(LANG_LABEL_MENU_DATE);
		break;
	case ID_MENU_LIST_LANG:
		ptr = getLabel(LANG_LABEL_MENU_LANG);
		break;
	case ID_MENU_LIST_TWM:
		ptr = getLabel(LANG_LABEL_MENU_TWM);
		break;
	case ID_MENU_LIST_FORMAT:
		ptr = getLabel(LANG_LABEL_MENU_FORMAT);
		break;
	case ID_MENU_LIST_FRESET:
		ptr = getLabel(LANG_LABEL_MENU_FRESET);
		break;
	case ID_MENU_LIST_FIRMWARE:
		ptr = getLabel(LANG_LABEL_MENU_FIRMWARE);
		break;
	case ID_MENU_LIST_LICENSE_PLATE_WM:
		ptr = getLabel(LANG_LABEL_MENU_LICENSE_PLATE_WM);
		break;
	case ID_MENU_LIST_GSENSOR:
		ptr = getLabel(LANG_LABEL_MENU_GSENSOR);
		break;
	case ID_MENU_LIST_IMPACTLEVEL:
		ptr = getLabel(LANG_LABEL_MENU_IMPACTLEVEL);
		break;
	case ID_MENU_LIST_VOICEVOL:
		ptr = getLabel(LANG_LABEL_MENU_VOICEVOL);
		break;
	case ID_MENU_LIST_SHUTDOWN:
		ptr = getLabel(LANG_LABEL_MENU_SHUTDOWN);
		break;
	case ID_MENU_LIST_FCWSENSITY:
		ptr = getLabel(LANG_LABEL_MENU_FCWSENSITY);
		break;
	default:
		db_error("invalid resIndex %d\n", resID);
		return NULL;
	}

	return ptr;
}

#define CHECK_CURRENT_VALID(menuItem) do{ \
	if(menuItem.current >= menuItem.count) { \
		db_error("invalid current value\n"); break; } \
}while(0)
const char* ResourceManager::getResSubMenuCurString(enum ResourceID resID)
{
	const char* ptr = NULL;
	unsigned int current;
	switch(resID) {
	case ID_MENU_LIST_LANG:
		CHECK_CURRENT_VALID(menuDataLang);
		current = menuDataLang.current;
		ptr = getLabel(LANG_LABEL_SUBMENU_LANG_CONTENT0 + current);
		break;
	case ID_MENU_LIST_VQ:
		CHECK_CURRENT_VALID(menuDataVQ);
		current = menuDataVQ.current;
		ptr = getLabel(LANG_LABEL_SUBMENU_VQ_CONTENT0 + current);
		break;
	case ID_MENU_LIST_PQ:
		CHECK_CURRENT_VALID(menuDataPQ);
		current = menuDataPQ.current;
		ptr = getLabel(LANG_LABEL_SUBMENU_PQ_CONTENT0 + current);
		break;
	case ID_MENU_LIST_VTL:
		CHECK_CURRENT_VALID(menuDataVTL);
		current = menuDataVTL.current;
		ptr = getLabel(LANG_LABEL_SUBMENU_VTL_CONTENT0 + current);
		break;
	case ID_MENU_LIST_WB:
		CHECK_CURRENT_VALID(menuDataWB);
		current = menuDataWB.current;
		ptr = getLabel(LANG_LABEL_SUBMENU_WB_CONTENT0 + current);
		break;
	case ID_MENU_LIST_CONTRAST:
		CHECK_CURRENT_VALID(menuDataContrast);
		current = menuDataContrast.current;
		ptr = getLabel(LANG_LABEL_SUBMENU_CONTRAST_CONTENT0 + current);
		break;
	case ID_MENU_LIST_EXPOSURE:
		CHECK_CURRENT_VALID(menuDataExposure);
		current = menuDataExposure.current;
		ptr = getLabel(LANG_LABEL_SUBMENU_EXPOSURE_CONTENT0 + current);
		break;
	case ID_MENU_LIST_SS:
		CHECK_CURRENT_VALID(menuDataSS);
		current = menuDataSS.current;
		ptr = getLabel(LANG_LABEL_SUBMENU_SS_CONTENT0 + current);
		break;
	case ID_MENU_LIST_FIRMWARE:
		ptr = FIRMWARE_VERSION;
		break;
	case ID_MENU_LIST_GSENSOR:
		CHECK_CURRENT_VALID(menuDataGsensor);
		current = menuDataGsensor.current;
		ptr = getLabel(LANG_LABEL_SUBMENU_GSENSOR_CONTENT0 + current);
		break;
	case ID_MENU_LIST_VOICEVOL:
		CHECK_CURRENT_VALID(menuDataVoiceVol);
		current = menuDataVoiceVol.current;
		ptr = getLabel(LANG_LABEL_SUBMENU_VOICEVOL_CONTENT0 + current);
		break;
	case ID_MENU_LIST_SHUTDOWN:
		CHECK_CURRENT_VALID(menuDataShutDown);
		current = menuDataShutDown.current;
		ptr = getLabel(LANG_LABEL_SUBMENU_SHUTDOWN_CONTENT0 + current);
		break;
	case ID_MENU_LIST_IMPACTLEVEL:
		CHECK_CURRENT_VALID(menuDataImpactLevel);
		current = menuDataImpactLevel.current;
		ptr = getLabel(LANG_LABEL_SUBMENU_IMPACTLEVEL_CONTENT0 + current);
		break;
	case ID_MENU_LIST_FCWSENSITY:
		CHECK_CURRENT_VALID(menuDatafcwsensity);
		current = menuDatafcwsensity.current;
		ptr = getLabel(LANG_LABEL_SUBMENU_FCWSENSITY_CONTENT0 + current);
		break;
	case ID_MENU_LIST_SMARTALGORITHM:
		CHECK_CURRENT_VALID(menuDataSmartAlgorithmEnable);
		current = menuDataSmartAlgorithmEnable.current;
		ptr = getLabel(LANG_LABEL_SUBMENU_SMARTALGORITHM_CONTENT0 + current);
		break;
	default:
		db_error("invalid resIndex %d\n", resID);
		return NULL;
	}

	return ptr;
}

const char* ResourceManager::getResSubMenuTitle(enum ResourceID resID)
{
	const char* ptr = NULL;
	switch(resID) {
	case ID_MENU_LIST_LANG:
		ptr = getLabel(LANG_LABEL_SUBMENU_LANG_TITLE);
		break;
	case ID_MENU_LIST_VQ:
		ptr = getLabel(LANG_LABEL_SUBMENU_VQ_TITLE);
		break;
	case ID_MENU_LIST_PQ:
		ptr = getLabel(LANG_LABEL_SUBMENU_PQ_TITLE);
		break;
	case ID_MENU_LIST_VTL:
		ptr = getLabel(LANG_LABEL_SUBMENU_VTL_TITLE);
		break;
	case ID_MENU_LIST_WB:
		ptr = getLabel(LANG_LABEL_SUBMENU_WB_TITLE);
		break;
	case ID_MENU_LIST_CONTRAST:
		ptr = getLabel(LANG_LABEL_SUBMENU_CONTRAST_TITLE);
		break;
	case ID_MENU_LIST_EXPOSURE:
		ptr = getLabel(LANG_LABEL_SUBMENU_EXPOSURE_TITLE);
		break;
	case ID_MENU_LIST_SS:
		ptr = getLabel(LANG_LABEL_SUBMENU_SS_TITLE);
		break;		
	case ID_MENU_LIST_GSENSOR:
		ptr = getLabel(LANG_LABEL_SUBMENU_GSENSOR_TITLE);
		break;
	case ID_MENU_LIST_VOICEVOL:
		ptr = getLabel(LANG_LABEL_SUBMENU_VOICEVOL_TITLE);
		break;
	case ID_MENU_LIST_SHUTDOWN:
		ptr = getLabel(LANG_LABEL_SUBMENU_SHUTDOWN_TITLE);
		break;
	case ID_MENU_LIST_IMPACTLEVEL:
		ptr = getLabel(LANG_LABEL_SUBMENU_IMPACTLEVEL_TITLE);
		break;
	case ID_MENU_LIST_FCWSENSITY:
		ptr = getLabel(LANG_LABEL_SUBMENU_FCWSENSITY_TITLE);
		break;
	case ID_MENU_LIST_SMARTALGORITHM:
		ptr = getLabel(LANG_LABEL_SUBMENU_SMARTALGORITHM_TITLE);
		break;
	default:
		db_error("invalid resIndex %d\n", resID);
		return NULL;
	}

	return ptr;
}

LANGUAGE ResourceManager::getLanguage(void)
{

	return lang;
}

PLOGFONT ResourceManager::getLogFont(void)
{

	return mCurLogFont;
}

int ResourceManager::getResVTLms(void)
{
	int value;
	value = menuDataVTL.current;
	if(value == 0)
		return 60 * 1000;
	else if(value == 1)
		return 120 * 1000;
	else
		return -1;
}



int ResourceManager::getWindPicFileName(enum BmpType type, String8 &file)
{

	switch(type) {
	case BMPTYPE_WINDOWPIC_RECORDPREVIEW:
		file = rStatusBarData.recordPreviewPic;
		break;
	case BMPTYPE_WINDOWPIC_PHOTOGRAPH:
		file = rStatusBarData.screenShotPic;
		break;
	case BMPTYPE_WINDOWPIC_PLAYBACKPREVIEW:
		file = rStatusBarData.playBackPreviewPic;
		break;
	case BMPTYPE_WINDOWPIC_PLAYBACK:
		file = rStatusBarData.playBackPic;
		break;
	case BMPTYPE_WINDOWPIC_MENU:
		file = rStatusBarData.menuPic;
		break;
	default:
		db_error("invalid BmpType %d\n", type);
		return -1;
	}

	return 0;
}

#define CHECK_CURRENT_ICON_VALID(curIcon) do{ \
	if(curIcon.current >= curIcon.icon.size()) { \
		db_error("invalid current value\n"); break; } \
}while(0)
int ResourceManager::getCurrentIconFileName(enum ResourceID resID, String8 &file)
{

	int current;
	switch(resID) {
	case ID_STATUSBAR_ICON_WIFI:
		CHECK_CURRENT_ICON_VALID(rStatusBarData.wifiIcon);
		current = rStatusBarData.wifiIcon.current;
		file = rStatusBarData.wifiIcon.icon.itemAt(current);
		break;
	case ID_STATUSBAR_ICON_PARK:
		CHECK_CURRENT_ICON_VALID(rStatusBarData.parkIcon);
		current = rStatusBarData.parkIcon.current;
		file = rStatusBarData.parkIcon.icon.itemAt(current);
		break;
	case ID_STATUSBAR_ICON_AWMD:
		CHECK_CURRENT_ICON_VALID(rStatusBarData.awmdIcon);
		current = rStatusBarData.awmdIcon.current;
		file = rStatusBarData.awmdIcon.icon.itemAt(current);
		break;
	case ID_STATUSBAR_ICON_LOCK:
		CHECK_CURRENT_ICON_VALID(rStatusBarData.lockIcon);
		current = rStatusBarData.lockIcon.current;
		file = rStatusBarData.lockIcon.icon.itemAt(current);
		break;
	case ID_STATUSBAR_ICON_UVC:
		CHECK_CURRENT_ICON_VALID(rStatusBarData.uvcIcon);
		current = rStatusBarData.uvcIcon.current;
		file = rStatusBarData.uvcIcon.icon.itemAt(current);
		break;
	case ID_STATUSBAR_ICON_TFCARD:	
		CHECK_CURRENT_ICON_VALID(rStatusBarData.tfCardIcon);
		current = rStatusBarData.tfCardIcon.current;
		file = rStatusBarData.tfCardIcon.icon.itemAt(current);
		break;
	case ID_STATUSBAR_ICON_AUDIO:	
		CHECK_CURRENT_ICON_VALID(rStatusBarData.audioIcon);
		current = rStatusBarData.audioIcon.current;
		file = rStatusBarData.audioIcon.icon.itemAt(current);
		break;
	case ID_STATUSBAR_ICON_BAT :	
		CHECK_CURRENT_ICON_VALID(rStatusBarData.batteryIcon);
		current = rStatusBarData.batteryIcon.current;
		file = rStatusBarData.batteryIcon.icon.itemAt(current);
		break;
// Del by Harry
//    case ID_STATUSBAR_ICON_WOND :
//        CHECK_CURRENT_ICON_VALID(rStatusBarData.wondIcon);
//        current = rStatusBarData.wondIcon.current;
//        file = rStatusBarData.wondIcon.icon.itemAt(current);
//        break;
	case ID_PLAYBACKPREVIEW_ICON:
		CHECK_CURRENT_ICON_VALID(rPlayBackPreviewData.icon);
		current = rPlayBackPreviewData.icon.current;
		file = rPlayBackPreviewData.icon.icon.itemAt(current);
		break;
	case ID_PLAYBACK_ICON:
		CHECK_CURRENT_ICON_VALID(rPlayBackData.icon);
		current = rPlayBackData.icon.current;
		file = rPlayBackData.icon.icon.itemAt(current);
		break;
	default:
		db_error("invalid resID %d\n", resID);
		return -1;
	}

	return 0;
}

int ResourceManager::getMLPICFileName(enum ResourceID resID, enum BmpType type, String8 &file)
{
	if(resID >= ID_MENU_LIST_VQ && resID <= ID_MENU_LIST_UNCHECKBOX_PIC) {
		switch(resID) {
		case ID_MENU_LIST_WIFI:
			if (type == BMPTYPE_SELECTED)
				file = rMenuList.wifiIcon.icon.itemAt(1);
			else
				file = rMenuList.wifiIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_PARK:
				if (type == BMPTYPE_SELECTED)
					file = rMenuList.parkIcon.icon.itemAt(1);
				else
					file = rMenuList.parkIcon.icon.itemAt(0);
				break;
		case ID_MENU_LIST_ALIGNLINE:
				if (type == BMPTYPE_SELECTED)
					file = rMenuList.alignlineIcon.icon.itemAt(1);
				else
					file = rMenuList.alignlineIcon.icon.itemAt(0);
				break;
#ifdef APP_ADAS
		case ID_MENU_LIST_SMARTALGORITHM:
				if (type == BMPTYPE_SELECTED)
					file = rMenuList.smartalgorithm.icon.itemAt(1);
				else
					file = rMenuList.smartalgorithm.icon.itemAt(0);
				break;
#endif
		case ID_MENU_LIST_VQ:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.videoQualityIcon.icon.itemAt(1);
			else
				file = rMenuList.videoQualityIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_PQ:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.photoQualityIcon.icon.itemAt(1);
			else
				file = rMenuList.photoQualityIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_VTL:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.timeLengthIcon.icon.itemAt(1);
			else
				file = rMenuList.timeLengthIcon.icon.itemAt(0);
			break; 
		case ID_MENU_LIST_AWMD:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.moveDetectIcon.icon.itemAt(1);
			else
				file = rMenuList.moveDetectIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_WB:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.whiteBalanceIcon.icon.itemAt(1);
			else
				file = rMenuList.whiteBalanceIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_CONTRAST:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.contrastIcon.icon.itemAt(1);
			else
				file = rMenuList.contrastIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_EXPOSURE:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.exposureIcon.icon.itemAt(1);
			else
				file = rMenuList.exposureIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_POR:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.PORIcon.icon.itemAt(1);
			else
				file = rMenuList.PORIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_SS:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.screenSwitchIcon.icon.itemAt(1);
			else
				file = rMenuList.screenSwitchIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_SILENTMODE:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.volumeIcon.icon.itemAt(1);
			else
				file = rMenuList.volumeIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_DATE:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.dateIcon.icon.itemAt(1);
			else
				file = rMenuList.dateIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_LANG:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.languageIcon.icon.itemAt(1);
			else
				file = rMenuList.languageIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_TWM:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.TWMIcon.icon.itemAt(1);
			else
				file = rMenuList.TWMIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_FORMAT:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.formatIcon.icon.itemAt(1);
			else
				file = rMenuList.formatIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_FRESET:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.factoryResetIcon.icon.itemAt(1);
			else
				file = rMenuList.factoryResetIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_FIRMWARE:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.firmwareIcon.icon.itemAt(1);
			else
				file = rMenuList.firmwareIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_UNFOLD_PIC:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.unfoldPressPic;
			else
				file = rMenuList.unfoldNormalPic;
			break;
		case ID_MENU_LIST_CHECKBOX_PIC:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.checkedPressPic;
			else
				file = rMenuList.checkedNormalPic;
			break;
		case ID_MENU_LIST_UNCHECKBOX_PIC:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.uncheckedPressPic;
			else
				file = rMenuList.uncheckedNormalPic;
			break;
		case ID_MENU_LIST_GSENSOR:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.gsensorIcon.icon.itemAt(1);
			else
				file = rMenuList.gsensorIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_VOICEVOL:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.voicevolIcon.icon.itemAt(1);
			else
				file = rMenuList.voicevolIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_SHUTDOWN:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.shutdownIcon.icon.itemAt(1);
			else
				file = rMenuList.shutdownIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_IMPACTLEVEL:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.impactlevelIcon.icon.itemAt(1);
			else
				file = rMenuList.impactlevelIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_FCWSENSITY:
			if(type == BMPTYPE_SELECTED)
				file = rMenuList.fcwsensityIcon.icon.itemAt(1);
			else
				file = rMenuList.fcwsensityIcon.icon.itemAt(0);
			break;
		case ID_MENU_LIST_LICENSE_PLATE_WM:
			file = rMenuList.licensePlateWMIcon.icon.itemAt(0);
			break;
		default:
			break;
		}
	} else {
		db_error("invalid resID %d\n", resID);
		return -1;
	}

	return 0;
}

int ResourceManager::getSubMenuCurrentIndex(enum ResourceID resID)
{
	int index;

	switch(resID) {
	case ID_MENU_LIST_VQ:
		index = menuDataVQ.current;
		break;
	case ID_MENU_LIST_PQ:
		index = menuDataPQ.current;
		break;
	case ID_MENU_LIST_VTL:
		index = menuDataVTL.current;
		break;
	case ID_MENU_LIST_WB:
		index = menuDataWB.current;
		break;
	case ID_MENU_LIST_CONTRAST:
		index = menuDataContrast.current;
		break;
	case ID_MENU_LIST_EXPOSURE:
		index = menuDataExposure.current;
		break;
	case ID_MENU_LIST_SS:
		index = menuDataSS.current;
		break;
	case ID_MENU_LIST_LANG:
		index = menuDataLang.current;
		break;
	case ID_MENU_LIST_GSENSOR:
		index = menuDataGsensor.current;
		break;
	case ID_MENU_LIST_VOICEVOL:
		index = menuDataVoiceVol.current;
		break;
	case ID_MENU_LIST_SHUTDOWN:
		index = menuDataShutDown.current;
		break;
	case ID_MENU_LIST_IMPACTLEVEL:
		index = menuDataImpactLevel.current;
		break;	
	case ID_MENU_LIST_FCWSENSITY:
		index = menuDatafcwsensity.current;
		break;
	case ID_MENU_LIST_SMARTALGORITHM:
		index = menuDataSmartAlgorithmEnable.current;
		break;
	default:
		db_error("invalid resID: %d\n", resID);
		index = -1;
		break;
	}

	return index;
}

int ResourceManager::getSubMenuCounts(ResourceID resID)
{

	int counts;

	switch(resID) {
	case ID_MENU_LIST_VQ:
		counts = menuDataVQ.count;
		break;
	case ID_MENU_LIST_PQ:
		counts = menuDataPQ.count;
		break;
	case ID_MENU_LIST_VTL:
		counts = menuDataVTL.count;
		break;
	case ID_MENU_LIST_WB:
		counts = menuDataWB.count;
		break;
	case ID_MENU_LIST_CONTRAST:
		counts = menuDataContrast.count;
		break;
	case ID_MENU_LIST_EXPOSURE:
		counts = menuDataExposure.count;
		break;
	case ID_MENU_LIST_SS:
		counts = menuDataSS.count;
		break;
	case ID_MENU_LIST_LANG:
		counts = menuDataLang.count;
		break;
	case ID_MENU_LIST_GSENSOR:
		counts = menuDataGsensor.count;
		break;
	case ID_MENU_LIST_IMPACTLEVEL:
		counts = menuDataImpactLevel.count;
		break;
	case ID_MENU_LIST_VOICEVOL:
		counts = menuDataVoiceVol.count;
		break;
	case ID_MENU_LIST_SHUTDOWN:
		counts = menuDataShutDown.count;
		break;
	case ID_MENU_LIST_FCWSENSITY:
		counts = menuDatafcwsensity.count;
		break;
	case ID_MENU_LIST_SMARTALGORITHM:
		counts = menuDataSmartAlgorithmEnable.count;
		break;
	default:
		db_error("invalid resID: %d\n", resID);
		counts = -1;
		break;
	}

	return counts;
}

int ResourceManager::getSubMenuIntAttr(enum ResourceID resID, enum IntValueType type)
{
	int intValue = -1;
	if(resID == ID_MENU_LIST_DATE) {
		switch(type) {
		case INTVAL_TITLEHEIGHT:
			intValue = rMenuList.dateTileRectH;
			break;
		case INTVAL_HBORDER:
			intValue = rMenuList.dateHBorder;
			break;
		case INTVAL_YEARWIDTH:
			intValue = rMenuList.dateYearW;
			break;
		case INTVAL_LABELWIDTH:
			intValue = rMenuList.dateLabelW;
			break;
		case INTVAL_NUMBERWIDTH:
			intValue = rMenuList.dateNumberW;
			break;
		case INTVAL_BOXHEIGHT:
			intValue = rMenuList.dateBoxH;
			break;	
		default:
			db_error("invalid IntValueType: %d\n", type);
		}
	} else {
		db_error("invalid ResourceID: %d\n", resID);	
	}

	return intValue;
}

int ResourceManager::setSubMenuCurrentIndex(enum ResourceID resID, int newSel)
{

	int count;
	switch(resID) {
	case ID_MENU_LIST_LANG:
		count = menuDataLang.count;
		if(newSel >= count) {
			db_error("invalid value: %d, count is %d\n", newSel, count);
			return -1;
		}
		menuDataLang.current = newSel;
		updateLangandFont(newSel);
		break;
	case ID_MENU_LIST_VQ:
		count = menuDataVQ.count;
		if(newSel >= count) {
			db_error("invalid value: %d, count is %d\n", newSel, count);
			return -1;
		}
		menuDataVQ.current = newSel;
		SendMessage(mHwnd[WINDOWID_RECORDPREVIEW], MSG_RM_VIDEO_QUALITY, (WPARAM)newSel, 0);
		break;
	case ID_MENU_LIST_PQ:
		count = menuDataPQ.count;
		if(newSel >= count) {
			db_error("invalid value: %d, count is %d\n", newSel, count);
			return -1;
		}
		menuDataPQ.current = newSel;
		SendMessage(mHwnd[WINDOWID_RECORDPREVIEW], MSG_RM_PIC_QUALITY, (WPARAM)newSel, 0);
		break;
	case ID_MENU_LIST_VTL:
		count = menuDataVTL.count;
		if(newSel >= count) {
			db_error("invalid value: %d, count is %d\n", newSel, count);
			return -1;
		}
		menuDataVTL.current = newSel;
		SendMessage(mHwnd[WINDOWID_RECORDPREVIEW], MSG_RM_VIDEO_TIME_LENGTH, (WPARAM)newSel, 0);
		break;
	case ID_MENU_LIST_WB:
		count = menuDataWB.count;
		if(newSel >= count) {
			db_error("invalid value: %d, count is %d\n", newSel, count);
			return -1;
		}
		menuDataWB.current = newSel;
		SendMessage(mHwnd[WINDOWID_RECORDPREVIEW], MSG_RM_WHITEBALANCE, (WPARAM)newSel, 0);
		break;
	case ID_MENU_LIST_CONTRAST:
		count = menuDataContrast.count;
		if(newSel >= count) {
			db_error("invalid value: %d, count is %d\n", newSel, count);
			return -1;
		}
		menuDataContrast.current = newSel;
		SendMessage(mHwnd[WINDOWID_RECORDPREVIEW], MSG_RM_CONTRAST, (WPARAM)newSel, 0);
		break;
	case ID_MENU_LIST_EXPOSURE:
		count = menuDataExposure.count;
		if(newSel >= count) {
			db_error("invalid value: %d, count is %d\n", newSel, count);
			return -1;
		}
		menuDataExposure.current = newSel;
		SendMessage(mHwnd[WINDOWID_RECORDPREVIEW], MSG_RM_EXPOSURE, (WPARAM)newSel, 0);
		break;
	case ID_MENU_LIST_SS:
		count = menuDataSS.count;
		if(newSel >= count) {
			db_error("invalid value: %d, count is %d\n", newSel, count);
			return -1;
		}
		menuDataSS.current = newSel;
		SendMessage(mHwnd[WINDOWID_MAIN], MSG_RM_SCREENSWITCH, (WPARAM)newSel, 0);
		break;
	case ID_MENU_LIST_GSENSOR:
		count = menuDataGsensor.count;
		if(newSel >= count) {
			db_error("invalid value: %d, count is %d\n", newSel, count);
			return -1;
		}
		menuDataGsensor.current = newSel;
		SendMessage(mHwnd[WINDOWID_MAIN], MSG_GSENSOR_SENSITIVITY, (WPARAM)newSel, 0);		
		break;
	case ID_MENU_LIST_VOICEVOL:
		count = menuDataVoiceVol.count;
		if(newSel >= count) {
			db_error("invalid value: %d, count is %d\n", newSel, count);
			return -1;
		}
		menuDataVoiceVol.current = newSel;
		mCdrVolume = VolNum[newSel].voice;
		SendMessage(mHwnd[WINDOWID_PLAYBACK],MSG_SET_VOLUME,(WPARAM)(mCdrVolume*100),0);
		SendMessage(mHwnd[WINDOWID_RECORDPREVIEW],MSG_ADAS_AUDIO_VOL,(WPARAM)(mCdrVolume*10),0);
		setKeySoundVol(mCdrVolume);
		break;
	case ID_MENU_LIST_SHUTDOWN:
		count = menuDataShutDown.count;
		if(newSel >= count) {
			db_error("invalid value: %d, count is %d\n", newSel, count);
			return -1;
		}
		menuDataShutDown.current = newSel;
		SendMessage(mHwnd[WINDOWID_MAIN], MSG_AUTO_SHUTDOWN, (WPARAM)newSel, 0);
		break;
	case ID_MENU_LIST_IMPACTLEVEL:
		count = menuDataImpactLevel.count;
		if(newSel >= count) {
			db_error("invalid value: %d, count is %d\n", newSel, count);
			return -1;
		}
		menuDataImpactLevel.current = newSel;
		SendMessage(mHwnd[WINDOWID_MAIN], MSG_IMPACTLEVEL_VALUE, (WPARAM)newSel, 0);
		break;
	case ID_MENU_LIST_FCWSENSITY:
		count = menuDatafcwsensity.count;
		if(newSel >= count) {
			db_error("invalid value: %d, count is %d\n", newSel, count);
			return -1;
		}
		menuDatafcwsensity.current = newSel;
		SendMessage(mHwnd[WINDOWID_RECORDPREVIEW], MSG_ADAS_FCW_SENSITY, (WPARAM)newSel, 0);
		break;
	case ID_MENU_LIST_SMARTALGORITHM:
		db_msg("  **mADAS*newSel:%d***  ",newSel);
		count = menuDataSmartAlgorithmEnable.count;
		if(newSel >= count) {
			db_error("invalid value: %d, count is %d\n", newSel, count);
			return -1;
		}
		if(newSel == 3){
			SendMessage(mHwnd[WINDOWID_MAIN], MSG_SHOW_ADAS_FUNCTION, 0, 0);
			break;
		}
		menuDataSmartAlgorithmEnable.current = newSel;
		db_msg("  ******newSel:%d********  ",newSel);
		SendMessage(mHwnd[WINDOWID_RECORDPREVIEW], MSG_ADAS_SWITCH, newSel, 0);
		break;
	default:
		db_error("invalid resID: %d\n", resID);
		return -1;
	}

	return 0;
}

void ResourceManager::updateLangandFont(int langValue)
{
	LANGUAGE newLang;	

	if(langValue == LANG_TW)
		newLang = LANG_TW;
	else if(langValue == LANG_CN)
		newLang = LANG_CN;
	else if(langValue == LANG_EN)
		newLang = LANG_EN;
	else if(langValue == LANG_JPN)
		newLang = LANG_JPN;
	else if(langValue == LANG_KR)
		newLang = LANG_KR;
	else if(langValue == LANG_RS)
		newLang = LANG_RS;
	else return;

	if(newLang == lang)
		return;

	#if 0
	if( !(newLang == LANG_CN && lang == LANG_EN) && !(newLang == LANG_EN && lang == LANG_CN)&& 
		!(newLang == LANG_CN && lang == LANG_RS) && !(newLang == LANG_RS && lang == LANG_CN)&& 
		!(newLang == LANG_RS && lang == LANG_EN) && !(newLang == LANG_EN && lang == LANG_RS)) {
		db_msg("detroy logfont\n");
		//DestroyLogFont(mLogFont);
		//mLogFont = CreateLogFont("sxf", "arialuni", "UTF-8",
        //FONT_WEIGHT_REGULAR, FONT_SLANT_ROMAN, FONT_SETWIDTH_NORMAL,
		//FONT_OTHER_AUTOSCALE, FONT_UNDERLINE_NONE, FONT_STRUCKOUT_NONE, 16, 0);
	}
	#endif

	/* here to assign different fonts in different lanuages */
	mCurLogFont = mLogFont;

#ifdef USE_IPS_SCREEN
	if (newLang == LANG_KR) {
		mCurLogFont = mLogFont_Malgun_Gothic;
	} else {
		mCurLogFont = mLogFont_MSYH;
	}
#endif

	lang = newLang;
	db_msg("new lang is %d\n", lang);

	initLabel(); /* init lang labels */
	SendMessage(mHwnd[WINDOWID_MAIN], MSG_RM_LANG_CHANGED, (WPARAM)&lang, 0);
}

int ResourceManager::loadBmpFromConfig(const char* pSection, const char* pKey, PBITMAP bmp)
{

	int retval;
	char buf[CDR_FILE_LENGHT] = {0};

	if((retval = GetValueFromEtcFile(configFile, pSection, pKey, buf, sizeof(buf))) != ETC_OK) {
		db_error("get %s->%s failed: %s\n", pSection, pKey, pEtcError(retval));
		return -1;
	}

	if( (retval = LoadBitmapFromFile(HDC_SCREEN, bmp, buf)) != ERR_BMP_OK) {
		db_error("load %s failed: %s\n", buf, pBmpError(retval));
		return -1;
	}

	return 0;
}

void ResourceManager::unloadBitMap(BITMAP &bmp)
{
	if(bmp.bmBits != NULL) {
		UnloadBitmap(&bmp);
	}
}

static char err_buf[100];
char* ResourceManager::pEtcError(int err_code)
{
	bzero(err_buf, sizeof(err_buf));
	switch(err_code) 
	{
	case ETC_FILENOTFOUND:
		sprintf(err_buf, "The etc file not found");
		break;
	case ETC_SECTIONNOTFOUND:
		sprintf(err_buf, "The section not found");
		break;
	case ETC_KEYNOTFOUND:
		sprintf(err_buf, "The key not found");
		break;
	case ETC_TMPFILEFAILED:
		sprintf(err_buf, "Create tmpfile failed");
		break;
	case ETC_FILEIOFAILED:
		sprintf(err_buf, "IO operation failed to etc file");
		break;
	case ETC_INTCONV:
		sprintf(err_buf, "Convert the value string to an integer failed");
		break;
	case ETC_INVALIDOBJ:
		sprintf(err_buf, "Invalid object to etc file");
		break;
	case ETC_READONLYOBJ:
		sprintf(err_buf, "Read only to etc file");
		break;
	default:
		sprintf(err_buf, "Unkown error code %d", err_code);
		break;
	}

	return err_buf;
}

char* ResourceManager::pBmpError(int err_code)
{
	bzero(err_buf, sizeof(err_buf));
	switch(err_code)
	{
	case ERR_BMP_IMAGE_TYPE:
		sprintf(err_buf, "Not a valid bitmap");
		break;
	case ERR_BMP_UNKNOWN_TYPE:
		sprintf(err_buf, "Not recongnized bitmap type");
		break;
	case ERR_BMP_CANT_READ:
		sprintf(err_buf, "Read error");
		break;
	case ERR_BMP_CANT_SAVE:
		sprintf(err_buf, "Save error");
		break;
	case ERR_BMP_NOT_SUPPORTED:
		sprintf(err_buf, "Not supported bitmap type");
		break;
	case ERR_BMP_MEM:
		sprintf(err_buf, "Memory allocation error");
		break;
	case ERR_BMP_LOAD:
		sprintf(err_buf, "Loading error");
		break;
	case ERR_BMP_FILEIO:
		sprintf(err_buf, "I/O failed");
		break;
	case ERR_BMP_OTHER:
		sprintf(err_buf, "Other error");
		break;
	case ERR_BMP_ERROR_SOURCE:
		sprintf(err_buf, "A error data source");
		break;
	default:
		sprintf(err_buf, "Unkown error code %d", err_code);
		break;
	}

	return err_buf;
}



int ResourceManager::getCfgFileHandle(void)
{
	if (mCfgFileHandle != 0) {
		return 0;
	}

	db_error("getCfgFileHandle : configFile = %s\n",configFile);
	mCfgFileHandle = LoadEtcFile(configFile);
	if(mCfgFileHandle == 0) {
		db_error("getCfgFileHandle failed\n");
		return -1;
	}
	return 0;
}

int ResourceManager::getCfgMenuHandle(void)
{
	if (mCfgMenuHandle != 0) {
		return 0;
	}
	mCfgMenuHandle = LoadEtcFile(configMenu);
	if(mCfgMenuHandle == 0) {
		db_error("getCfgMenuHandle failed\n");
		return -1;
	}

	return 0;
}

void ResourceManager::releaseCfgFileHandle(void)
{
	if(mCfgFileHandle == 0) {
		return;
	}
	UnloadEtcFile(mCfgFileHandle);
	mCfgFileHandle = 0;
}

void ResourceManager::releaseCfgMenuHandle(void)
{
	if(mCfgMenuHandle == 0) {
		return;
	}
	UnloadEtcFile(mCfgMenuHandle);
	mCfgMenuHandle = 0;
}

/*
 * On success return the int value
 * On fail return -1
 * */
int ResourceManager::getIntValueFromHandle(GHANDLE cfgHandle, const char* mainkey, const char* subkey)
{
	int retval = 0;
	int err_code;
	if(mainkey == NULL || subkey == NULL) {
		db_error("NULL pointer\n");	
		return -1;
	}

	if(cfgHandle == 0) {
		db_error("cfgHandle not initialized\n");
		return -1;
	}

	if((err_code = GetIntValueFromEtc(cfgHandle, mainkey, subkey, &retval)) != ETC_OK) {
		db_error("get %s->%s failed:%s\n", mainkey, subkey, pEtcError(err_code));
		return -1;
	}

	return retval;	
}

int ResourceManager::getValueFromFileHandle(const char*mainkey, const char* subkey, char* value, unsigned int len)
{
	int err_code;
	if(mainkey == NULL || subkey == NULL) {
		db_error("NULL pointer\n");	
		return ETC_INVALIDOBJ;
	}

	if(mCfgFileHandle == 0) {
		db_error("cfgHandle not initialized\n");
		return ETC_INVALIDOBJ;
	}

	if((err_code = GetValueFromEtc(mCfgFileHandle, mainkey, subkey, value, len)) != ETC_OK) {
		db_error("get %s->%s failed:%s\n", mainkey, subkey, pEtcError(err_code));
		return err_code;
	}

	return ETC_OK;
}

/* get the rect from configFile
 * pWindow the the section name in the config file
 * */
int ResourceManager::getRectFromFileHandle(const char *pWindow, CDR_RECT *rect)
{
	int err_code;

	if(mCfgFileHandle == 0) {
		db_error("mCfgFileHandle not initialized\n");
		return ETC_INVALIDOBJ;
	}

	if((err_code = GetIntValueFromEtc(mCfgFileHandle, pWindow, "x", &rect->x)) != ETC_OK) {
		return err_code;
	}

	if((err_code = GetIntValueFromEtc(mCfgFileHandle, pWindow, "y", &rect->y)) != ETC_OK) {
		return err_code;
	}

	if((err_code = GetIntValueFromEtc(mCfgFileHandle, pWindow, "w", &rect->w)) != ETC_OK) {
		return err_code;
	}

	if((err_code = GetIntValueFromEtc(mCfgFileHandle, pWindow, "h", &rect->h)) != ETC_OK) {
		return err_code;
	}

	return ETC_OK;
}

/*
 * get the ABGR format color from the configFile
 * On success return ETC_OK
 * On fail return err code
 * */
int ResourceManager::getABGRColorFromFileHandle(const char* pWindow, const char* subkey, gal_pixel &pixel)
{
	char buf[20] = {0};
	int err_code;
	unsigned long int hex;
	unsigned char r, g, b, a;

	if(pWindow == NULL || subkey == NULL)
		return ETC_INVALIDOBJ;

	if(mCfgFileHandle == 0) {
		db_error("mCfgFileHandle not initialized\n");
		return ETC_INVALIDOBJ;
	}

	if((err_code = GetValueFromEtc(mCfgFileHandle, pWindow, subkey, buf, sizeof(buf))) != ETC_OK) {
		return err_code;
	}

	hex = strtoul(buf, NULL, 16);

	a = (hex >> 24) & 0xff;
	b = (hex >> 16) & 0xff;
	g = (hex >> 8) & 0xff;
	r = hex & 0xff;

	pixel = RGBA2Pixel(HDC_SCREEN, r, g, b, a);

	return ETC_OK;
}

/*
 * get current icon info from configFile, and fill the giving currentIcon
 * example : if current is 1, then the icon1 is used
 * [status_bar_battery]
 *	current=1
 *	icon0=/res/topbar/baterry1.png
 *	icon1=/res/topbar/baterry2.png
 *	icon2=/res/topbar/baterry3.png
 *	icon3=/res/topbar/baterry4.png
 * On success return ETC_OK
 * On fail return err code
 * */
int ResourceManager::fillCurrentIconInfoFromFileHandle(const char* pWindow, currentIcon_t &currentIcon)
{
	int current;
	char buf[10] = {0};
	char bufIcon[50] = {0};

	if(pWindow == NULL) {
		db_msg("invalid pWindow\n");
		return ETC_INVALIDOBJ;
	}

	current = getIntValueFromHandle(mCfgFileHandle, pWindow, "current");
	if(current < 0) {
		db_error("get current value from %s failed\n", pWindow);
		return ETC_KEYNOTFOUND;
	}
	currentIcon.current = current;
	//	db_msg("current is %d\n", currentIcon.current);

	current = 0;
	do
	{
		int err_code;
		sprintf(buf, "%s%d", "icon", current);	
		//		db_msg("buf is %s\n", buf);
		err_code = GetValueFromEtc(mCfgFileHandle, pWindow, buf, bufIcon, sizeof(bufIcon));
		if(err_code != ETC_OK) {
			if(current == 0) {
				return err_code;
			} else {
				break;
			}
		}
		currentIcon.icon.add(String8(bufIcon));
		current++;
	}while(1);

	//	db_msg("%s has %u icons\n", pWindow, currentIcon.icon.size());
	for(unsigned int i = 0;  i< currentIcon.icon.size(); i++) {
		//		db_msg("icon%d is %s\n", i, currentIcon.icon.itemAt(i).string());	
	}

	if(currentIcon.current >= currentIcon.icon.size()) {
		db_error("invalide current value, current is %d icon count is %u\n", currentIcon.current, currentIcon.icon.size());	
		return ETC_INVALIDOBJ;
	}

	return ETC_OK;
}


/*
 * get language from config menu
 * On success return LANGUAGE
 * On faile return LANG_ERR
 * */
LANGUAGE ResourceManager::getLangFromMenuHandle(void)
{
	int retval;

	retval = getIntValueFromHandle(mCfgMenuHandle, "language", "current");
	if(retval == -1) {
		db_error("get current language failed\n");
		return LANG_ERR;
	}

	if(retval == LANG_CN)
		return LANG_CN;
	else if(retval == LANG_TW)
		return LANG_TW;
	else if(retval == LANG_EN)
		return LANG_EN;
	else if(retval == LANG_JPN)
		return LANG_JPN;
	else if(retval == LANG_KR)
		return LANG_KR;
	else if(retval == LANG_RS)
		return LANG_RS;

	return LANG_ERR;
}

LANGUAGE ResourceManager::getLangFromConfigFile(void)
{
	int retval;
	if(GetIntValueFromEtcFile(configMenu, "language", "current", &retval) != ETC_OK) {
		db_error("get language failed\n");
		return LANG_ERR;
	}

	if(retval == LANG_CN)
		return LANG_CN;
	else if(retval == LANG_TW)
		return LANG_TW;
	else if(retval == LANG_EN)
		return LANG_EN;
	else if(retval == LANG_JPN)
		return LANG_JPN;
	else if(retval == LANG_KR)
		return LANG_KR;
	else if(retval == LANG_RS)
		return LANG_RS;

	return LANG_ERR;
}

void ResourceManager::getFirmwareInfo(FirmwareInfo *fwInfo)
{
	struct tm *tm;
	char updated[10];
	fwInfo->product_type = "X1";
	fwInfo->software_version = FIRMWARE_VERSION;
	getDateTime(&tm);
	sprintf(updated, "%d%02d%02d", tm->tm_year, tm->tm_hour, tm->tm_mday);
	fwInfo->updated = updated;
}

int ResourceManager::setRecordingStatus(bool recording)
{
	mRecording = recording;
	return 0;
}

BITMAP * ResourceManager::getNoVideoImage()
{
	if(	mNoVideoImage.bmWidth == -1){
		return NULL;
	}
	return &mNoVideoImage;
}

int ResourceManager::getConfig(__aw_cdr_net_config *config)
{	
	config->boot_record = (int)menuDataPOREnable;
	config->exposure = menuDataExposure.current;
	config->impact_sensitivity = menuDataImpactLevel.current;
	config->frontcar_crash_sensitivity = menuDatafcwsensity.current;
	config->language = menuDataLang.current;
	config->motion_detect = (int)menuDataAWMDEnable;
	config->mute = (int)menuDataVolumeEnable;
	config->photo_quality = menuDataPQ.current;
	config->rear_view_mirror = 0;
	config->record_duration = menuDataVTL.current;
	config->record_quality = menuDataVQ.current;
	config->smart_detect = menuDataSmartAlgorithmEnable.current;
	config->watermark = (int)menuDataTWMEnable;
	config->white_balance = menuDataWB.current;
	config->record_switch = mRecording;
	SendMessage(mHwnd[WINDOWID_MAIN] , MSG_GET_STATUSBAR_TIME, (WPARAM)config, 0);
	strcpy(config->firmware_version, FIRMWARE_VERSION);
	cfgDataGetString("ro.build.date.utc", config->buildTime, "1446178354");
	sprintf(config->manufacturer, "%s", MANUFACTURER);
	sprintf(config->otaVersion, "%s", OTA_VERSION);
	config->dev_type = PLATFORM_1;
	char val[4] = {0};
	cfgDataGetString("persist.sys.shotrecord.switch", val, "0");
	if(strcmp(val, "1") == 0){
		config->shotRecord = 1;
	}else{
		config->shotRecord = 0;
	}
	cfgDataGetString("persist.sys.wifi.pwd", config->wifiPsd, DEFAULT_WIFI_PASSWORD);
	return 0;
}

#ifdef YOUKU_PLATFORM
int ResourceManager::getConfig(CdrMain *cdrMain)
{
	int battery = EventManager::getBattery();
	device_status[DEVICE_INTERNAL_BATTERY_PRESENT].ivalue = (battery == -1)?0:battery;
	device_status[DEVICE_EXTERNAL_BATTERY_PRESENT].ivalue = (battery == -1)?1:0;  // extern power
	device_status[DEVICE_HOT].ivalue = -1;   // device hot
	device_status[DEVICE_BUSY].ivalue = -1;    // device busy
	device_status[DEVICE_QUICK_CAPTURE_ACTIVE].ivalue = -1;    // quick capture active

	struct tm *tm;
	getDateTime(&tm);
	sprintf(device_status[DEVICE_DATE_TIME].svalue, "%d-%02d-%02d, %02d:%02d:%02d",
			tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday, tm->tm_hour, tm->tm_min, tm->tm_sec);

	HWND hStatusBar;
	hStatusBar = cdrMain->getWindowHandle(WINDOWID_STATUSBAR);

	device_status[VIDEO_PROGRESS_COUNTER].ivalue =
		SendMessage(hStatusBar, STBM_GET_RECORD_TIME, 1, 0); // Ƶ
	device_status[SETTING_COMM_STREAM_SUPPORT].ivalue = 1;   // stream support

	StorageManager *sm = StorageManager::getInstance();
	device_status[STORAGE_SD_STATUS].ivalue = sm->isMount();

	device_status[STORAGE_REMAINING_PHOTOS].ivalue = sm->getRemainingPictures();// ʣƬ
	device_status[STORAGE_REMAINING_VIDEO_TIME].ivalue = sm->getRemainingVideoTime(cdrMain->getUvcState());// ʣƵʱ
	device_status[STORAGE_NUM_TOTAL_PHOTOS].ivalue = sm->dbCount((char*)TYPE_PIC);   // Ƭ
	device_status[STORAGE_NUM_TOTAL_VIDEOS].ivalue = sm->dbCount((char*)TYPE_VIDEO);;   // Ƶ

	uint64_t avail = 0;
	uint64_t total = 0;
	sm->getCapacity(&avail, &total);
	sprintf(device_status[STORAGE_REMAINING_SPACE].svalue, "%1.1fG", avail/(float)1024); // ʣռС
	device_status[OTA_STATUS].ivalue = -1;   // ״̬
	device_status[OTA_DOWNLOAD_CANCEL_REQUEST_PENDING].ivalue = -1;

	setting_status[SETTING_VIDEO_RESOLUTION].ivalue = menuDataVQ.current;
	setting_status[SETTING_VIDEO_RESOLUTION_FPS].ivalue = -1;

	setting_status[SETTING_VIDEO_TIMELAPSE_RATE].ivalue = -1;
	setting_status[SETTING_VIDEO_LOOPING].ivalue = menuDataVTL.current;

	setting_status[SETTING_VIDEO_WHITE_BALANCE].ivalue = menuDataWB.current;
	setting_status[SETTING_VIDEO_EV].ivalue = menuDataExposure.current;
	setting_status[SETTING_VIDEO_DATETIME_PR].ivalue = (int)menuDataTWMEnable;
	setting_status[SETTING_VIDEO_AUDIO_RECORD].ivalue = (int)(!menuDataVolumeEnable);

	setting_status[SETTING_PHOTO_RESOLUTION].ivalue = menuDataPQ.current;
	setting_status[SETTING_PHOTO_CONTINUOUS_RATE].ivalue = -1;
	setting_status[SETTING_PHOTO_WHITE_BALANCE].ivalue = menuDataWB.current;
	setting_status[SETTING_PHOTO_EV].ivalue = menuDataExposure.current;
	setting_status[SETTING_PHOTO_COUNTDOWN_ERVEL].ivalue = -1;
	setting_status[SETTING_PHOTO_DATE_PRINT].ivalue = -1;
	setting_status[SETTING_COMM_LCD_SLEEP].ivalue = -1;
	setting_status[SETTING_COMM_LED_BLINK].ivalue = -1;
	setting_status[SETTING_COMM_BEEP_VOLUME].ivalue = menuDataVoiceVol.current;
	setting_status[SETTING_COMM_AUTO_POWER_DOWN].ivalue = menuDataShutDown.current;

	return 0;
}
#endif
#define debug_print(...) \
	__android_log_print(ANDROID_LOG_ERROR, LOG_TAG, __VA_ARGS__)
int ResourceManager::sessionControl(int sid, int cmd, char* buf, void *srv)
{
	int ret = 0;
	debug_print("cmd:%x, param_in:%p", cmd, buf);
	CdrServer *cdrServer = (CdrServer*)srv;
	switch(cmd)
	{
		case NAT_CMD_SET_RECORDER_QUALITY://
			{
				int param_in = ((aw_cdr_set_recorder_quality*)buf)->index;
				aw_cdr_set_recorder_quality_resp resp;
				ret = setResIntValue(ID_MENU_LIST_VQ, INTVAL_SUBMENU_INDEX, param_in);
				debug_print("NAT_CMD_SET_RECORDER_QUALITY param_in:%d, ret:%d", param_in, ret);
				resp.result = ret;
				cdrServer->reply(sid, NAT_CMD_SET_RECORDER_QUALITY_RESP, 
					(char *)(&resp), sizeof(aw_cdr_set_recorder_quality_resp));
			}
			break;
		case NAT_CMD_SET_RECORDER_DURATION://
			{
				int param_in = ((aw_cdr_set_recorder_duration*)buf)->index;
				aw_cdr_set_recorder_duration_resp resp;
				ret = setResIntValue(ID_MENU_LIST_VTL, INTVAL_SUBMENU_INDEX, param_in);
				debug_print("NAT_CMD_SET_RECORDER_DURATION param_in:%d, ret:%d", param_in, ret);
				resp.result = ret;
				cdrServer->reply(sid, NAT_CMD_SET_RECORDER_DURATION_RESP, 
					(char *)(&resp), sizeof(aw_cdr_set_recorder_quality_resp));
			}
			break;
		case NAT_CMD_SET_PHOTO_QUALITY://
			{
				int param_in = ((aw_cdr_set_photo_quality*)buf)->index;
				ret = setResIntValue(ID_MENU_LIST_PQ, INTVAL_SUBMENU_INDEX, param_in);
				debug_print("NAT_CMD_SET_PHOTO_QUALITY_RESP param_in:%d, ret:%d", param_in, ret);
				aw_cdr_set_photo_quality_resp resp;
				resp.result = ret;
				cdrServer->reply(sid, NAT_CMD_SET_PHOTO_QUALITY_RESP, 
					(char *)(&resp), sizeof(aw_cdr_set_photo_quality_resp));
			}
			break;
		case NAT_CMD_SET_WHITE_BALANCE://
			{
				int param_in = ((aw_cdr_set_white_balance*)buf)->index;
				ret = setResIntValue(ID_MENU_LIST_WB, INTVAL_SUBMENU_INDEX, param_in);
				debug_print("NAT_CMD_SET_WHITE_BALANCE param_in:%d, ret:%d", param_in, ret);
				aw_cdr_set_white_balance_resp resp;
				resp.result = ret;
				cdrServer->reply(sid, NAT_CMD_SET_WHITE_BALANCE_RESP, 
					(char *)(&resp), sizeof(aw_cdr_set_photo_quality_resp));
			}
			break;
		case NAT_CMD_SET_EXPOSURE://
			{
				int param_in = ((aw_cdr_set_exposure*)buf)->index;
				ret = setResIntValue(ID_MENU_LIST_EXPOSURE, INTVAL_SUBMENU_INDEX, param_in);
				debug_print("NAT_CMD_SET_EXPOSURE param_in:%d, ret:%d", param_in, ret);
				aw_cdr_set_exposure_resp resp;
				resp.result = ret;
				cdrServer->reply(sid, NAT_CMD_SET_EXPOSURE_RESP, 
					(char *)(&resp), sizeof(aw_cdr_set_exposure_resp));

			}
			break;
		case NAT_CMD_SET_BOOT_RECORDER://
			{
				int param_in = ((aw_cdr_set_boot_recorder*)buf)->index;
				ret = setResBoolValue(ID_MENU_LIST_POR, (bool)param_in);
				aw_cdr_set_boot_recorder_resp resp;
				resp.result = ret;
				cdrServer->reply(sid, NAT_CMD_SET_BOOT_RECORDER_RESP, 
					(char *)(&resp), sizeof(aw_cdr_set_boot_recorder_resp));
			}
			break;
		case NAT_CMD_SET_MUTE://
			{
				int param_in = ((aw_cdr_set_mute*)buf)->index;
				ret = setResBoolValue(ID_MENU_LIST_SILENTMODE, (bool)param_in);
				aw_cdr_set_mute_resp resp;
				resp.result = ret;
				cdrServer->reply(sid, NAT_CMD_SET_MUTE_RESP, 
					(char *)(&resp), sizeof(aw_cdr_set_mute_resp));
			}
			break;
		case NAT_CMD_SET_REAR_VIEW_MIRROR:
			break;
		case NAT_CMD_SET_TIME: //
			{
				struct tm *time_us = (struct tm *)buf;
				time_us->tm_year -= 1900;
				time_us->tm_mon -= 1;
				db_msg("time is (%d:%d:%d:%d:%d:%d)", time_us->tm_year, time_us->tm_mon, time_us->tm_mday, time_us->tm_hour, time_us->tm_min, time_us->tm_sec);
				ret = setDateTime(time_us);
				aw_cdr_set_time_resp resp;
				resp.result = ret;
				cdrServer->reply(sid, NAT_CMD_SET_TIME_RESP, 
					(char *)(&resp), sizeof(aw_cdr_set_time_resp));
			}
			break;
		case NAT_CMD_SET_LANGUAGE://
			{
				int param_in = ((aw_cdr_set_language*)buf)->index;
				ret = setResIntValue(ID_MENU_LIST_LANG, INTVAL_SUBMENU_INDEX, param_in);
				debug_print("NAT_CMD_SET_LANGUAGE param_in:%d, ret:%d", param_in, ret);
				aw_cdr_set_language_resp resp;
				resp.result = ret;
				cdrServer->reply(sid, NAT_CMD_SET_LANGUAGE_RESP, 
					(char *)(&resp), sizeof(aw_cdr_set_language_resp));
			}
			break;
		case NAT_CMD_SET_SMART_DETECT:
			{
				int param_in = ((aw_cdr_set_smart_algorithm*)buf)->index;
				//ret = setResIntValue(ID_MENU_LIST_SMARTALGORITHM, INTVAL_SUBMENU_INDEX, param_in);
				aw_cdr_set_smart_algorithm_resp resp;
				db_msg("  ***param_in:%d***	",param_in);
				int count = menuDataSmartAlgorithmEnable.count;
				if(param_in >= count) {
					db_msg("NAT_CMD_SET_SMART_DETECT invalid value: %d, count is %d\n", param_in, count);
					ret = -1;
				}else{
					menuDataSmartAlgorithmEnable.current = param_in;
					ret = 0;
				}
				resp.result = ret;
				cdrServer->reply(sid, NAT_CMD_SET_SMART_DETECT_RESP, 
					(char *)(&resp), sizeof(aw_cdr_set_smart_algorithm_resp));
			}
			break;
		case NAT_CMD_SET_FRONTCAR_CRASH_SENSITIVITY:
			{
				int param_in = ((aw_cdr_set_frontcar_crash_sensitivity*)buf)->index;
				ret = setResIntValue(ID_MENU_LIST_FCWSENSITY, INTVAL_SUBMENU_INDEX, param_in);
				aw_cdr_set_frontcar_crash_sensitivity_resp resp;
				resp.result = ret;
				cdrServer->reply(sid, NAT_CMD_SET_FRONTCAR_CRASH_SENSITIVITY_RESP, 
					(char *)(&resp), sizeof(aw_cdr_set_gsensor_level_resp));
			}
			break;
		case NAT_CMD_SET_IMPACT_SENSITIVITY:
			{
				int param_in = ((aw_cdr_set_gsensor_level*)buf)->index;
				ret = setResIntValue(ID_MENU_LIST_IMPACTLEVEL, INTVAL_SUBMENU_INDEX, param_in);
				aw_cdr_set_gsensor_level_resp resp;
				resp.result = ret;
				cdrServer->reply(sid, NAT_CMD_SET_IMPACT_SENSITIVITY_RESP, 
					(char *)(&resp), sizeof(aw_cdr_set_gsensor_level_resp));
			}
			break;
		case NAT_CMD_SET_MOTION_DETECT: //
			{
				int param_in = ((aw_cdr_set_motion_detect*)buf)->index;
				ret = setResBoolValue(ID_MENU_LIST_AWMD, (bool)param_in);
				aw_cdr_set_motion_detect_resp resp;
				resp.result = ret;
				cdrServer->reply(sid, NAT_CMD_SET_MOTION_DETECT_RESP, 
					(char *)(&resp), sizeof(aw_cdr_set_motion_detect_resp));
			}
			break;
		case NAT_CMD_SET_WATERMARK: //
			{
				int param_in = ((aw_cdr_set_watermark*)buf)->index;
				ret = setResBoolValue(ID_MENU_LIST_TWM, (bool)param_in);
				aw_cdr_set_watermark_resp resp;
				resp.result = ret;
				cdrServer->reply(sid, NAT_CMD_SET_WATERMARK_RESP, 
					(char *)(&resp), sizeof(aw_cdr_set_watermark_resp));
			}
			break;
		case NAT_CMD_FORMAT_TF_CARD: //
			{
				ret = SendMessage(mHwnd[WINDOWID_MAIN], MSG_RM_FORMAT_TFCARD, 0, 0);
				debug_print("NAT_CMD_FORMAT_TF_CARD , ret:%d", ret);
				aw_cdr_format_tf_card_resp resp;
				resp.result = ret;
				cdrServer->reply(sid, NAT_CMD_FORMAT_TF_CARD_RESP, 
					(char *)(&resp), sizeof(aw_cdr_format_tf_card_resp));
			}
			break;
		case NAT_CMD_PLAY_RECORDER:
			break;
		case NAT_CMD_GET_FILE_LIST: //
			{
				char *str = NULL;
				int param_in = ((aw_cdr_getfolder_file*)buf)->index;
				ret = SendMessage(mHwnd[WINDOWID_PLAYBACKPREVIEW], MSG_RM_GET_FILELIST, (WPARAM)&str, (LPARAM)param_in);
				char *strIndex = str;
				if (strIndex == NULL) {
					break;
				}
				aw_cdr_get_file_list_resp resp;
				resp.totalCnt = strlen(str) / MAX_PACKET_FILE_LIST_LEN + 1;
				resp.currentIndex = 0;
				while (strIndex != NULL) {
					memset(resp.filelist, 0, MAX_PACKET_FILE_LIST_LEN);
					int cpyLen = (strlen(strIndex) > MAX_PACKET_FILE_LIST_LEN)?
						MAX_PACKET_FILE_LIST_LEN:strlen(strIndex);
					memcpy(resp.filelist, strIndex, cpyLen);
					cdrServer->reply(sid, NAT_CMD_GET_FILE_LIST_RESP, 
						(char *)(&resp), sizeof(aw_cdr_get_file_list_resp));
					resp.currentIndex++;
					if (strlen(strIndex) > MAX_PACKET_FILE_LIST_LEN) {
						strIndex += MAX_PACKET_FILE_LIST_LEN;
					} else {
						strIndex = NULL;
					}
				}
				if (str != NULL) {
					free(str);
				}
			}
			break;
		case NAT_CMD_CHECK_TF_CARD://
			{
				aw_cdr_tf_capacity resp;
				ret = SendMessage(mHwnd[WINDOWID_MAIN], MSG_RM_GET_CAPACITY, (WPARAM)&resp, 0);
				debug_print("NAT_CMD_CHECK_TF_CARD , resp:(total:%llu, remain:%llu)ret:%d, size:%d", 
					resp.total, resp.remain, ret,sizeof(aw_cdr_tf_capacity));
				cdrServer->reply(sid, NAT_CMD_CHECK_TF_CARD_RESP, 
					(char *)(&resp), sizeof(aw_cdr_tf_capacity));
			}
			break;
		case NAT_CMD_DELETE_FILE://
			{
				char *file = ((aw_cdr_handle_file *)buf)->filename;
				aw_cdr_delete_file_resp resp = {0};
				resp.result = SendMessage(mHwnd[WINDOWID_PLAYBACKPREVIEW], MSG_RM_DEL_FILE, (WPARAM)file, 0);
				cdrServer->reply(sid, NAT_CMD_DELETE_FILE_RESP, (char *)&resp, sizeof(resp));
			}
			break;
		case NAT_CMD_RECORD_CTL:
			break;
		case NAT_CMD_SET_SYNC_FILE:
			break;
		case NAT_CMD_GET_CONFIG://
			{
				aw_cdr_net_config config;
				memset(&config, 0, sizeof(aw_cdr_net_config));
				getConfig(&config);
				cdrServer->reply(sid, NAT_CMD_GET_CONFIG_RESP, (char *)(&config), sizeof(aw_cdr_net_config));
			}
			break;
		case NAT_CMD_TAKE_PHOTO://
			{
				ret = SendMessage(mHwnd[WINDOWID_RECORDPREVIEW], MSG_RM_TAKE_PICTURE, 0, 0);
				aw_cdr_take_photo_resp resp;
				resp.result = ret;
				cdrServer->reply(sid, NAT_CMD_TAKE_PHOTO_RESP, 
					(char *)(&resp), sizeof(aw_cdr_take_photo_resp));
			}
			break;
		case NAT_CMD_REQUEST_FILE_FINISH:
			{
				aw_cdr_file_trans* file_name = (aw_cdr_file_trans*) buf;
				db_msg("*******filename(%s)", file_name->filename);
				int flag =strcmp(file_name->filename,"/mnt/extsd/full_img.fex");
				db_msg("*********************flage=%d*******",flag);
				if(flag==0){
				    SendMessage(mHwnd[WINDOWID_MAIN], MSG_OTA_UPDATE, 0, 0);
				}
				
			}
			break;
		case NAT_CMD_TURN_OFF_WIFI: //
			ret = setResBoolValue(ID_MENU_LIST_WIFI, false);
			break;
		case NAT_CMD_DELETE_DATAFILE:
			{
				StorageManager *sm = StorageManager::getInstance();
				aw_cdr_delete_datafile_resp resp;
				ret = sm->deleteFile(GSDataFilePath);
				resp.result = ret;
				cdrServer->reply(sid, NAT_CMD_DELETE_DATAFILE_RESP, 
					(char *)(&resp), sizeof(aw_cdr_delete_datafile_resp));
			}
			break;
		case NAT_CMD_SHOW_RECORD_SWITCH:
			{
				int param_in = ((aw_cdr_set_shotrecord_switch*)buf)->index;
				char val[4] = {0};
				sprintf(val, "%d", param_in);
				ret = cfgDataSetString("persist.sys.shotrecord.switch", val);
				aw_cdr_shot_record_switch resp;
				resp.result = ret;
				cdrServer->reply(sid, NAT_CMD_SHOW_RECORD_SWITCH_RESP, 
						(char *)(&resp), sizeof(aw_cdr_delete_datafile_resp));
			}
			break;
		default:
			ret = -1;
			break;
	}
	return ret;
}

void ResourceManager::syncConfigureToDisk(void)
{
	char buf[3] = {0};
	configTable2 configTableSwitch[] = {
		{CFG_MENU_POR, (void*)&menuDataPOREnable},
		{CFG_MENU_VOLUME,	(void*)&menuDataVolumeEnable},
		{CFG_MENU_TWM,		(void*)&menuDataTWMEnable},
		{CFG_MENU_AWMD,		(void*)&menuDataAWMDEnable},
		{CFG_MENU_WIFI,		(void*)&menuDataWIFIEnable},
		{CFG_MENU_PARK,		(void*)&menuDataPARKEnable},
		{CFG_MENU_ALIGNLINE,		(void*)&menuDataAlignlineEnable}
	};

	configTable2 configTableMenuItem[] = {
		{CFG_MENU_LANG, (void*)&menuDataLang},
		{CFG_MENU_VQ,	(void*)&menuDataVQ},
		{CFG_MENU_PQ,	(void*)&menuDataPQ},
		{CFG_MENU_VTL,	(void*)&menuDataVTL},
		{CFG_MENU_WB,	(void*)&menuDataWB},
		{CFG_MENU_CONTRAST, (void*)&menuDataContrast},
		{CFG_MENU_EXPOSURE, (void*)&menuDataExposure},
		{CFG_MENU_SS,		(void*)&menuDataSS},
		{CFG_MENU_GSENSOR,	(void*)&menuDataGsensor},
		{CFG_MENU_IMPACTLEVEL,	(void*)&menuDataImpactLevel},
		{CFG_MENU_FCWSENSITY,	(void*)&menuDatafcwsensity},
		{CFG_MENU_VOICEVOL,	(void*)&menuDataVoiceVol},
		{CFG_MENU_SHUTDOWN,	(void*)&menuDataShutDown},
#ifdef APP_ADAS
		{CFG_MENU_SMARTALGORITHM,	(void*)&menuDataSmartAlgorithmEnable}
#endif
	};

	if(getCfgMenuHandle() < 0) {
		db_error("get config menu handle failed\n");	
		return;
	}

	db_info("syncConfigureToDisk\n");
	for(unsigned int i = 0; i < TABLESIZE(configTableSwitch); i++) {
		if(*(bool*)configTableSwitch[i].addr == true)
			buf[0] = '1';
		else
			buf[0] = '0';
		db_msg("buf is %s\n", buf);
		SetValueToEtc(mCfgMenuHandle, "switch", configTableSwitch[i].mainkey, buf);
	}
	db_info("syncConfigureToDisk\n");

	for(unsigned int i = 0; i < TABLESIZE(configTableMenuItem); i++) {
		sprintf(buf, "%d", ((menuItem_t*)(configTableMenuItem[i].addr))->current % 10);
		db_msg("buf is %s\n", buf);
		SetValueToEtc(mCfgMenuHandle, configTableMenuItem[i].mainkey, "current", buf);

		sprintf(buf, "%d", ((menuItem_t*)(configTableMenuItem[i].addr))->count % 10);
		db_msg("buf is %s\n", buf);
		SetValueToEtc(mCfgMenuHandle, configTableMenuItem[i].mainkey, "count", buf);
	}
#if 1
	sprintf(buf, "%d", (menuDataLWM.LWMEnable) % 10);
	db_msg("menuDataLWM.LWMEnable buf is %s\n", buf);
	SetValueToEtc(mCfgMenuHandle, CFG_MENU_DEFAULTLICENSE , "current", buf);

//	if(menuDataLWM.LWMEnable && (menuDataLWM.lwaterMark[9] != 0) )
	if(menuDataLWM.LWMEnable  )
		SetValueToEtc(mCfgMenuHandle, CFG_MENU_DEFAULTLICENSE , "position", menuDataLWM.lwaterMark);

	db_msg("menuDataLWM.lwaterMark  is %s\n", menuDataLWM.lwaterMark);
#endif
	SetValueToEtc(mCfgMenuHandle, CFG_VERIFICATION, "current", (char*)"1");
	db_info("syncConfigureToDisk\n");
	SaveEtcToFile(mCfgMenuHandle, configMenu);

	releaseCfgMenuHandle();
	db_info("syncConfigureToDisk finished\n");
}

bool ResourceManager::verifyConfig()
{
	int retval;
	int err_code;

	if((err_code = GetIntValueFromEtc(mCfgMenuHandle, CFG_VERIFICATION, "current", &retval)) != ETC_OK) {
		db_error("get %s->%s failed:%s\n", CFG_VERIFICATION, "current", pEtcError(err_code));
		db_msg("verify %s %s\n", CFG_VERIFICATION, (retval==1)?"correct":"incorrect");
		return false;
	}

	if (retval != 1) {
		db_msg("verify %s %s\n", CFG_VERIFICATION, (retval==1)?"correct":"incorrect");
		return false;
	}
	db_msg("verify %s %s\n", CFG_VERIFICATION, (retval==1)?"correct":"incorrect");
	return true;
}

void ResourceManager::resetResource(void)
{
/*
	releaseCfgFileHandle();
	if(copyFile(defaultConfigFile, configFile) < 0) {
		db_error("reset config file failed\n");	
		return;
	}
*/
	SendMessage(mHwnd[WINDOWID_MAIN], MSG_RESET_WIFI_PWD, 0, 0);
	releaseCfgMenuHandle();
	if(copyFile(defaultConfigMenu, configMenu) < 0) {
		db_error("reset config menu failed\n");	
		return;
	}

	if(initStage1() < 0) {
		db_error("initStage1 failed\n");	
		return;
	}
	if(initStage2() < 0) {
		db_error("initStage2 failed\n");
		return;
	}

	SendMessage(mHwnd[WINDOWID_STATUSBAR], STBM_WIFI, 0, 0);
	SendMessage(mHwnd[WINDOWID_MAIN], MSG_WIFI_SWITCH, 0, 0);

	SendMessage(mHwnd[WINDOWID_RECORDPREVIEW], MSG_AWMD, 0, 0);
	SendMessage(mHwnd[WINDOWID_STATUSBAR], STBM_AWMD, 0, 0);
	
	SendMessage(mHwnd[WINDOWID_RECORDPREVIEW], MSG_ADAS_SCREEN, 0, 0);		
	SendMessage(mHwnd[WINDOWID_RECORDPREVIEW], MSG_DRAW_ALIGN_LINE, 0, 0);
	SendMessage(mHwnd[WINDOWID_STATUSBAR], STBM_PARK, 0, 0);

	struct tm *tblock;
	time_t timer;
	timer = time(NULL);
	tblock = localtime(&timer);
	tblock->tm_year=2015-1900;
	tblock->tm_mon=0;
	tblock->tm_mday=1;
	tblock->tm_hour=0;
	tblock->tm_min=0;
	tblock->tm_sec=0;
	if(setDateTime(tblock)!=0){
	   db_msg("***reset cdr time failed***");
	}

	notifyAll();
	cfgDataSetString("persist.app.showadasfunction", "1");
	cfgDataSetString("persist.sys.shotrecord.switch", "0");

}
