/*************************************************
Copyright (C), 2015, AllwinnerTech. Co., Ltd.
File name: softap_controller.cpp
Author: yinzh@allwinnertech.com
Version: 0.1
Date: 2015-11-23
Description:
    用于实现无线网卡作为softap的控制管理
History:
*************************************************/
#include "softap_controller.h"
#include "debug.h"
#include "ConfigData.h"

#ifdef LOG_TAG
#undef LOG_TAG
#define LOG_TAG "softap_controller.cpp"
#endif

using namespace android;

SoftApController::SoftApController()
    : softap_status_(SOFTAP_DISABLED)
    , enable_thread_(NULL)
    , disable_thread_(NULL)
{
    char password[64] = {0};
    this->GenerateSSID();
    ::setSoftap(const_cast<char*>(ssid_.string()),NULL); // const_cast<char*>(DEFAULT_WIFI_PASSWORD));
    //cfgDataSetString("persist.sys.wifi.pwd", DEFAULT_WIFI_PASSWORD);
}

SoftApController::~SoftApController()
{
}

void SoftApController::EnableSoftAp()
{
    int ret = -1;

	db_msg("Goto EnableSoftAp!!\n");
    if (softap_status_ != SOFTAP_DISABLED) {
        db_msg("softap has been enabled\n");
        return;
    }
    if (enable_thread_ == NULL) {
        enable_thread_ = new SoftApEnableThread(this);
    }
    ret = enable_thread_->start();
    if (ret < 0) {
        db_error("enable softap failed:%d\n", ret);
    } else {
        softap_status_ = SOFTAP_ENABLED;
    }
}

void SoftApController::DisableSoftAp(bool wait_exit)
{
    int ret = -1;
    if (softap_status_ == SOFTAP_DISABLED) {
        db_msg("softap has been disabled\n");
        return;
    }
    if (disable_thread_ == NULL) {
        disable_thread_ = new SoftApDisableThread(this);
    }
    ret = disable_thread_->start();
    if (ret < 0) {
        db_error("disable softap failed:%d\n", ret);
        return;
    }
    if (wait_exit) {
        disable_thread_->join();
    }
    softap_status_ = SOFTAP_DISABLED;
}

void SoftApController::SetSoftApIpAddr(const char *ip)
{

}

void SoftApController::GetSoftApIpAddr(char *ip)
{

}

int SoftApController::ResetSoftAp(const char *ssid, const char *pwd)
{
    if (ssid == NULL || strlen(ssid) == 0) {
        char str[128] = {0};
        cfgDataGetString("persist.sys.wifi.ssid", str, "");
        ssid_ = String8(str);
    } else {
        ssid_ = String8(ssid);
    }
    if (pwd == NULL || strlen(pwd) < 8) {
        db_msg("password length is less than 8, please reset again!\n");
        return -1;
    } else {
        pwd_ = String8(pwd);
    }
	db_msg("new ssid:%s, pwd:%s\n", ssid_.string(), pwd_.string());
	if (softap_status_ == SOFTAP_ENABLED) {
		this->DisableSoftAp();
		::setSoftap(const_cast<char*>(ssid_.string()), const_cast<char*>(pwd_.string()));
		this->EnableSoftAp();
	} else {
		::setSoftap(const_cast<char*>(ssid_.string()), const_cast<char*>(pwd_.string()));
	}
    cfgDataSetString("persist.sys.wifi.ssid", ssid_.string());
    cfgDataSetString("persist.sys.wifi.pwd", pwd_.string());
    return 0;
}

void SoftApController::GenerateSSID()
{
    char ssid[128] = {0};
    cfgDataGetString("persist.sys.wifi.ssid", ssid, "");
    if (strlen(ssid) != 0) {
        ssid_ = String8(ssid);
        return;
    }
    ssid_ = String8("CDR_"); // maybe others
    char line[128];
    FILE *fp = fopen("/proc/cpuinfo", "r");
    if(fp == NULL){
        db_error("open '/proc/cpuinfo' failed.");
    }
    while(fgets(line, sizeof(line), fp)) {
        if (strncmp(line, "Serial", strlen("Serial")) == 0) {
            char *str = strstr(line, ":");
            str += 2;
            if(strlen(str) > 4){
                str +=(strlen(str)-5);
                char* str1 = strstr(str, "\n");
                if (str1 != NULL) {
                    *str1 = '\0';
                }
                ssid_ += String8(str);
            }
        }
    }
    fclose(fp);

    db_msg("ssid:%s", ssid_.string());
    cfgDataSetString("persist.sys.wifi.ssid", ssid_.string());
}
