/*************************************************
Copyright (C), 2015, AllwinnerTech. Co., Ltd.
File name: wifi_manager.cpp
Author: yinzh@allwinnertech.com
Version: 0.1
Date: 2015-11-24
Description:
History:
*************************************************/

#include "wifi_manager.h"
#include "debug.h"

#ifdef LOG_TAG
#undef LOG_TAG
#define LOG_TAG "wifi_manager.cpp"
#endif

using namespace android;

static void WifiEventHandler(event_t event,int arg,void *cookie)
{

}

WifiManager::WifiManager()
    : wifi_status_(WIFI_DISABLED)
{
    scan_list_.clear();
	wifi_setCallBack(WifiEventHandler, (void*)this);
}

WifiManager::~WifiManager()
{
}

void WifiManager::EnableWifi()
{
    int ret = -1;
    if (wifi_status_ != WIFI_DISABLED) {
        db_msg("wifi has been enabled\n");
        return;
    }
    if (enable_thread_.get() == NULL) {
        enable_thread_ = new WifiEnableThread(this);
    }
    ret = enable_thread_->start();
    if (ret < 0) {
        db_error("enable wifi failed:%d\n", ret);
        return;
    }
    wifi_status_ = WIFI_ENABLED;
}

void WifiManager::DisableWifi(bool wait_exit)
{
    int ret = -1;
    if (wifi_status_ == WIFI_DISABLED) {
        db_msg("wifi has been disabled\n");
        return;
    }
    if (disable_thread_.get() == NULL) {
        disable_thread_ = new WifiDisableThread(this);
    }
    ret = disable_thread_->start();
    if (ret < 0) {
        db_error("disable wifi failed:%d\n", ret);
        return;
    }
    if (wait_exit) {
        disable_thread_->join();
    }
    wifi_status_ = WIFI_DISABLED;
}

Vector<ApInfo*> &WifiManager::GetScanResult()
{
    int ret = -1;
    this->ClearScanList();
    if (wifi_status_ != WIFI_DISABLED) {
        // 扫描并获取ap列表
		::android_wifi_scan();
        wifi_status_ = WIFI_SCANNING;
        wifi_scanlist scan_list;
        ret = ::getWifiApList(&scan_list);
        if (ret == 0) {
            db_msg("get scan list failed\n");
        } else {
            // 将扫描列表做相应处理存入Vector<ApInfo*>并返回
            for (int i = 0; i < scan_list.list_count; i++) {
                ApInfo *ap_info = new ApInfo();
                ap_info->ssid = strdup(scan_list.list[i].ssid);
                ap_info->strength = atoi(scan_list.list[i].level);
                if (strstr(scan_list.list[i].flags, "WPA") != NULL) {
                    if (strstr(scan_list.list[i].flags, "WPA2") != NULL) {
                        ap_info->encryp_type = "WPA/WPA2";
                    } else {
                        ap_info->encryp_type = "WPA";
                    }
                } else if (strstr(scan_list.list[i].flags, "WEP") != NULL) {
                    ap_info->encryp_type = "WEP";
                } else {
                    ap_info->encryp_type = "NONE";
                }
                scan_list_.push(ap_info);
            }
        }
    } else {
       db_msg("wifi has not been enabled\n");
    }
    return scan_list_;
}

int WifiManager::ConnectWifiAp(uint8_t scan_idx, const char *ssid, const char *pwd)
{
    int ret = -1;
    if (wifi_status_ == WIFI_DISABLED) {
        db_msg("wifi has not been enabled yet\n");
        return ret;
    } else if (wifi_status_ == WIFI_CONNECTED) {
        this->DisconnectWifiAp();
    }
    if (strcmp(scan_list_[scan_idx]->ssid, ssid) == 0) {
        if (strstr(scan_list_[scan_idx]->encryp_type, "WPA") != NULL) {
            ret = ::connectWifiAp(ssid, pwd, "WPA");
        } else if (strstr(scan_list_[scan_idx]->encryp_type, "WEP") != NULL) {
            ret = ::connectWifiAp(ssid, pwd, "WEP");
        } else {
        }
        wifi_status_ = WIFI_CONNECTING;
        // start connect thread to request ipaddr
        if (connect_thread_.get() == NULL) {
            connect_thread_ = new WifiConnectThread(this);
        }
        ret = connect_thread_->start();
        if (ret < 0) {
            db_error("request ip failed:%d\n", ret);
        }
        wifi_status_ = WIFI_CONNECTED;
    }
    return ret;
}

void WifiManager::DisconnectWifiAp()
{
    if (wifi_status_ == WIFI_CONNECTED)
        android_wifi_disconnect();
}

void WifiManager::ClearScanList()
{
    for(Vector<ApInfo*>::iterator iter = scan_list_.begin();
            iter != scan_list_.end(); iter++) {
        delete (*iter);
    }
    scan_list_.clear();
}

bool WifiManager::WifiEnableThread::threadLoop()
{
    int ret = -1;
    wm_->wifi_lock_.lock();
    ret = ::setWifiEnable(1);
    if(ret == 0) {
        db_error("Enable wifi failed:%d\n", ret);
    } else {
        db_msg("Wifi enabled.");
    }
    wm_->wifi_lock_.unlock();
    return false;
}

bool WifiManager::WifiDisableThread::threadLoop()
{
    int ret = -1;
    wm_->wifi_lock_.lock();
    ret = ::setWifiEnable(0);
    if(ret == 0) {
        db_error("Enable wifi failed:%d\n", ret);
    } else {
        db_msg("Wifi disabled.");
    }
    wm_->wifi_lock_.unlock();
    return false;
}

bool WifiManager::WifiConnectThread::threadLoop()
{
    char ipaddr[32] = {0};
    int ip = 0;
    wm_->wifi_lock_.lock();
    int ret = ::android_wifi_request_ip(&ip);
    ::android_wifi_get_ip(ipaddr);
    if (ret != 0) {
    }
    db_msg("Wifi request ip, ipaddr:%s", ipaddr);
    wm_->wifi_lock_.unlock();
    return false;
}
