#include <cutils/log.h>
#include <CdrServer.h>

#include "debug.h"
#include <hardware_legacy/wifi_api.h>
#include <netutils/ifc.h>
#include <netutils/dhcp.h>
#include "windows.h"

#ifdef LOG_TAG
#undef LOG_TAG
#endif
#define LOG_TAG "CdrServer"

typedef struct __httpHandler {
	FILE *fp;
} httpHandler;

int httpdStatCallback(int stat, void *argv, int argvLen, void *handler)
{
	httpHandler *httpHdr = (httpHandler*)handler;
	ALOGE("httpdStatCallback stat[%d], argvLen[%d]", stat, argvLen);
	switch (stat)
	{
	case POST_FILE_START:
		{
			char absfilename[256] = "";
			snprintf(absfilename, 256, "/mnt/extsd/%s", (char*)argv);
			ALOGE("httpdStatCallback filename[%s]", (char*)argv);
			httpHdr->fp = fopen(absfilename, "wb");
			if (httpHdr->fp == NULL) {
				ALOGE("httpdStatCallback open file error[%d:%s]", errno, strerror(errno));
			}
		}
		break;
	case POST_FILE_ERROR:
		if (httpHdr->fp != NULL) {
			fclose(httpHdr->fp);
			httpHdr->fp = NULL;
		}
		break;		
	case POST_FILE_END:
		if (httpHdr->fp != NULL) {
			fclose(httpHdr->fp);
			httpHdr->fp = NULL;
		}
		break;
	default:
		break;
	}
	return 0;
}

int httpdDataCallback(void *argv, int argvLen, void *handler)
{
	ALOGE("httpdDataCallback  argvLen[%d]",  argvLen);
	httpHandler *httpHdr = (httpHandler*)handler;
	fwrite(argv, 1, argvLen, httpHdr->fp);
	return 0;
}

static void qrDecCallback(void* data, int data_len, void* caller)
{
#if 0
	int rtl = *((int*)data);
	CdrMain* cdrMain = ((CdrServer*)caller)->getCdrMain();
	CdrCamera* camCSI = ((CdrServer*)caller)->getCSICamera();
	char* presult = (char*)data+sizeof(int);
	ALOGE("zbar_dec %d", rtl);
	for (int i = 0; i < rtl; i++) {
		ALOGE("zbar_dec %d:%d result is %s", rtl, i, presult);
		char* wifi_s = strstr(presult, "WIFI:")?strstr(presult, "WIFI:"):strstr(presult, "wifi:");
		if (wifi_s != NULL) {
			wifi_s += strlen("WIFI:");
			char ssid[256] = "", encryption[256] = "", password[256] = "";
			memset(ssid, 0, 256);
			memset(encryption, 0, 256);
			memset(password, 0, 256);
			char* seperate = strchr(wifi_s, ';');
			int getWifiInfo = 0;
			while (strlen(wifi_s) > 2) {
				char type = wifi_s[0];
				int type_len = 0;
				wifi_s += strlen("S:");
				if (seperate == NULL) {
					type_len = strlen(wifi_s);
				} else {	
					type_len = seperate - wifi_s;
				}				
				switch (type)
				{				
				case 's':				
				case 'S':		
					if ((getWifiInfo&0x1) == 1 || type_len >= 256) {
						break;					
					}					
					getWifiInfo|=0x1;	
					memcpy(ssid, wifi_s, type_len);	
					break;				
				case 'T':				
				case 't':					
					if ((getWifiInfo&0x2) == 1 || type_len >= 256) {
						break;					
					}				
					getWifiInfo|=0x2;	
					memcpy(encryption, wifi_s, type_len);	
					break;				
				case 'P':
				case 'p':			
					if ((getWifiInfo&0x4) == 1 || type_len >= 256) {
						break;					
					}			
					getWifiInfo|=0x4;	
					memcpy(password, wifi_s, type_len);	
					break;				
				default:				
					break;				
				}				
				if (seperate == NULL) {	
					break;				
				} else {
					wifi_s = seperate + sizeof(';');	
					seperate = strchr(wifi_s, ';');				
				}			
			}			
			ALOGE("ssid %s encryption %s password %s, getWifiInfo %d", ssid, encryption, password, getWifiInfo);
			if (getWifiInfo != 0) {
				camCSI->setQrDecEnable(false);	
				cdrMain->wifiEnable(NET_CONNECT_MANAGER_ENABLE_CONN_WIFI, ssid, password, encryption);
			}		
		}		
		presult += strlen(presult)+sizeof('\0');	
	}
#endif
}
CdrServer::CdrServer(CdrMain *cdrMain)
{
	db_msg("CdrSever!!!");
	mNetServer = AWNetInterface::getInstance(1);
	if (!mNetServer) {
		db_msg("net init failed");
	}

	//mNetServer->init((char*)"F3J9A9NE4E6RUH6PSFZ1", (char*)"12345", 0x00000);
	mNetServer->init((char*)"F3J9A9NE4E6RUH6PSFZ1", (char*)"12345", 0x10000, (char*)"127.0.0.1", 0);
	mNetCallbackHandler = new AWNetCallbackHandler(this);
	mNetServer->registerNotificationHandler(mNetCallbackHandler);
	
	mCdrMain = cdrMain;
#if 0
	CdrCamera* camCSI = getCSICamera();
	camCSI->setQrDecCaller(this);
	camCSI->setQrDecCallBack(qrDecCallback);
#endif
	mHttpHandler = malloc(sizeof(httpHandler));
	httpdCallback httpdCb = {httpdStatCallback, httpdDataCallback, mHttpHandler};
	startHttpd(&httpdCb);
	mSid = -1;
//	mHttpServer = http_srv_init();
	db_msg("end!!!");
}

CdrServer::~CdrServer()
{
	stopHttpd();
//	http_srv_close(mHttpServer);
	mNetServer->deInit();
//	http_srv_release(mHttpServer);
}
CdrCamera* CdrServer::getCSICamera()
{
	return NULL;//mCdrMain->getCSICamera();
}
CdrMain* CdrServer::getCdrMain()
{
	return mCdrMain;
}

int CdrServer::AWNetCallbackHandler::handleNetCmd(int sid, int cmd, char* buf)
{	
	ALOGD("~~~~~cmd :%d", cmd);
	if(mCdrServer->mSid == -1){
		mCdrServer->mSid = sid;
	}
	return mCdrServer->sessionControl(sid, cmd, buf);
}

int CdrServer::AWNetCallbackHandler::handleRecvVideoData(char *abFrameData, 
	int nFrameDataMaxSize,char *abFrameInfo, 
	int nFrameInfoMaxSize, unsigned int nFrameIdx)
{
	return 0;
}
int CdrServer::AWNetCallbackHandler::handleRecvFile(int SID, AWFileInfo* fileInfo, 
	char* buf, unsigned int buferLen)
{
	return 0;
}
int CdrServer::AWNetCallbackHandler::handleRecvAudioData(char *abAudioData, 
	int nAudioDataMaxSize,char *abFrameInfo, 
	int nFrameInfoMaxSize, unsigned int nFrameIdx)
{
	return 0;
}


int CdrServer::start()
{
//	registerGetDataOrEnd(mHttpServer, (void *)this, handleHttpServerMsg);
	int ret = 0;

//	pthread_t mBroadThread, mTcpSrvThread, iThreadId, iHttpThreadId;
//	int err = pthread_create(&iHttpThreadId, NULL, httpServerThread, this);
	return ret;
}

void* CdrServer::getSession()
{
	db_msg("cdrserver: mNetServer %p", mNetServer);
	return mNetServer;
}
static int encoder_start_flag = 0;
int CdrServer::sessionControl(int sid, int cmd, char* buf)
{
	__android_log_print(ANDROID_LOG_ERROR, "CdrServer", "cmd:%x, param_in:%p", cmd, buf);
	switch (cmd)
	{
		case NAT_CMD_REQUEST_FILE:
//			{
//				aw_cdr_file_trans* in = (aw_cdr_file_trans*)buf;
//				int ret = 0;
//				//in->local_port = 31601;
//				//strcpy(in->local_ip, "192.168.100.1");
//				//if (in->from_to == NAT_CMD_FILE_TRANS_CMD_APK_TO_CDR)
//				//{
//					//db_msg("somebody ")
//				//	return 0;
//				//}
//				//if (in->from_to == NAT_CMD_FILE_TRANS_CMD_CDR_TO_APK)
//				{
//					ret = http_srv_getURL(mHttpServer, in->filename, "192.168.100.1", in->url);
//					db_msg("in->filename(%s), url(%s)", in->filename, in->url);
//					if (access(in->filename, R_OK) != 0)
//					{
//						ret = -1;
//						reply(sid, NAT_CMD_REQUEST_FILE_RESP, 0, 0);
//						return ret;
//					}
//					reply(sid, NAT_CMD_REQUEST_FILE_RESP, (int)in, sizeof(aw_cdr_file_trans));
//					return 0;
//				}
//			}
			break;
		case AW_IOTYPE_USER_IPCAM_START_ENCODER:
			{
				if (encoder_start_flag == 1)
				{
					break;
				}
				encoder_start_flag = 1;
				AWSMsg_AVIOCTRLStartStream *str = (AWSMsg_AVIOCTRLStartStream *)buf;
                		db_msg("str->stream_type=%d",str->stream_type);		
				mCdrMain->sessionChange(true);	
			}
			break;
		case AW_IOTYPE_USER_IPCAM_STOP_ENCODER:
			if (encoder_start_flag == 0)
			{
				break;
			}	
			encoder_start_flag = 0;
			mCdrMain->sessionChange(false);
			break;
		case AW_IOTYPE_USER_IPCAM_START:
			if (encoder_start_flag == 1)
			{
				break;
			}
			mCdrMain->sessionChange(true);
			encoder_start_flag = 1;
			break;
		case AW_IOTYPE_USER_IPCAM_STOP:
			break;
		case AW_IOTYPE_USER_IPCAM_OFF_LINE:
			if (encoder_start_flag == 1)
			{
				mCdrMain->sessionChange(false);
				encoder_start_flag = 0;
			}
			break;
		default:
			break;
	}
	return mCdrMain->sessionControl(sid, cmd, buf);
}

int CdrServer::notify(int cmd,  char *buf, int len, int sid)
{
	int ret = 0;
	switch(cmd)
	{
		case NAT_CMD_EVENT_SOS_FILE:
		case NAT_CMD_INSERT_TF_CARD:
		case NAT_CMD_REMOVE_TF_CARD:
			db_msg("++++++[debug_jaosn]:this is notify cmd 22 ++++++++++\n");
			ret = reply(NAT_BROADCAST_SID, cmd, buf, len);
		break;
	}

	return ret;

}

int CdrServer::reply(int sid, int cmd, char *buf, int buf_len)
{
	db_msg(" CdrServer::reply(sid(%d), cmd(%d), buf(%x), buf_len(%d))", sid, cmd, buf, buf_len);
	return mNetServer->awSendIOCtrl(sid, cmd,(const char *)buf, buf_len);
}

int CdrServer::cdrReplytoApp(int cmd, char* buf, int buf_len)
{
	db_msg(" CdrServer::cdrReplytoApp(sid(%d), cmd(%d), buf(%x), buf_len(%d))", mSid, cmd, buf, buf_len);
	return mNetServer->awSendIOCtrl(mSid, cmd,(const char *)buf, buf_len);	
}
