/*
File description: These interfaces are invoked by web server
*/

#include "platform.h"
#ifdef YOUKU_PLATFORM
#include "YoukuCmdsCallback.h"
#include <youku/ykcamera_log.h>
#include <youku/ykcamera_common.h>
#include "CdrServer.h"
#include "debug.h"
#include "YoukuService.h"
#include "windows.h"
#include "youku/ykcamera_mediastream.h"
#include "youku/ykcamera_error.h"
#include "youku/ykcamera_parameter.h"
#include <string>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <time.h>


#ifdef LOG_TAG
#undef LOG_TAG
#endif
#define LOG_TAG "YoukuCmdsCallback.cpp"

using namespace android;

Mutex gNewSpecialVideoMutex;

YoukuCmdsCallback::YoukuCmdsCallback(CdrMain *cdrMain)
	:mCdrMain(cdrMain)
{
	mRP = cdrMain->mRecordPreview;
	mMenu = cdrMain->mMenu;
	mRM = ResourceManager::getInstance();
	mCameraMode = RECORD_VIDEO;
}

CdrMain *YoukuCmdsCallback::GetCdrMain()
{
	return mCdrMain;
}

void YoukuCmdsCallback::Init(YKStreamSender *sender)
{
	mStreamSender = sender;
}

int YoukuCmdsCallback::SetVideoResolutionFPS(int item, void *data)
{
	int ret = -1;
    switch (item) {
        case YK_VIDEO_1080P_30FPS:
            item = 0;
            break;
        case YK_VIDEO_720P_30FPS:
            item = 1;
            break;
        default:
            return ret;
    }
	db_msg("item:%d\n", item);
	ret = mRM->setResIntValue(ID_MENU_LIST_VQ,
						INTVAL_SUBMENU_INDEX, item);
	setting_status[SETTING_VIDEO_RESOLUTION_FPS].ivalue = item;
	return ret;
}

#if 0
int YoukuCmdsCallback::SetVideoFPS(int item, void *data)
{
	int ret = -1;
	db_msg("item:%d\n", item);
	mRP->setVideoFPS(item);
	ret = item; // ret should not be -1 there.
	setting_status[SETTING_VIDEO_FPS].ivalue = item;
	return ret;
}
#endif

int YoukuCmdsCallback::SetVideoCyclicRecord(int item, void *data)
{
	int ret = -1;
    switch (item) {
        case YK_VIDEO_LOOPING_OFF:
            return ret;
            break;
        case YK_VIDEO_LOOPING_3MIN:
            item = 1;
            break;
        case YK_VIDEO_LOOPING_5MIN:
            item = 2;
            break;
        default:
            return ret;
    }
	db_msg("item:%d\n", item);
	ret = mRM->setResIntValue(ID_MENU_LIST_VTL, INTVAL_SUBMENU_INDEX, item);
	setting_status[SETTING_VIDEO_LOOPING].ivalue = item;
	return ret;
}

int YoukuCmdsCallback::SetVideoWriteBalance(int item, void *data)
{
	int ret = -1;
    switch (item) {
        case YK_WB_AUTO:
            item = 0;
            break;
        case YK_WB_DAYLIGHT:
            item = 1;
            break;
        case YK_WB_CLOUDY:
            item = 2;
            break;
        case YK_WB_TUNGSTEN:
            item = 3;
            break;
        case YK_WB_FLUORESCENT:
            item = 4;
            break;
        default:
            return ret;
    }
	db_msg("item:%d\n", item);
	ret = mRM->setResIntValue(ID_MENU_LIST_WB, INTVAL_SUBMENU_INDEX, item);
	setting_status[SETTING_VIDEO_WHITE_BALANCE].ivalue = item;
	return ret;
}

int YoukuCmdsCallback::SetVideoEV(int item, void *data)
{
	int ret = -1;
    switch (item) {
        case YK_EV_N30:
            item = 0;
            break;
        case YK_EV_N20:
            item = 1;
            break;
        case YK_EV_N10:
            item = 2;
            break;
        case YK_EV_0:
            item = 3;
            break;
        case YK_EV_P10:
            item = 4;
            break;
        case YK_EV_P20:
            item = 5;
            break;
        case YK_EV_P30:
            item = 6;
            break;
        default:
            return ret;
    }
	db_msg("item:%d\n", item);
	ret = mRM->setResIntValue(ID_MENU_LIST_EXPOSURE, INTVAL_SUBMENU_INDEX, item);
	setting_status[SETTING_VIDEO_EV].ivalue = item;
	return ret;
}

int YoukuCmdsCallback::SetVideoDateWaterPrint(int enable, void *data)
{
	int ret = -1;
    switch (enable) {
        case YK_PHOTO_DATETIME_OFF:
            enable = 0;
            break;
        case YK_PHOTO_DATETIME_DATE_TIME:
            enable = 1;
            break;
        default:
            return ret;
    }
	db_msg("item:%d\n", enable);
	ret = mRM->setResBoolValue(ID_MENU_LIST_TWM, (bool)enable);
	setting_status[SETTING_VIDEO_DATETIME_PR].ivalue = enable;
	return ret;
}

int YoukuCmdsCallback::SetAudioRecord(int enable, void *data)
{
	int ret = -1;
	db_msg("item:%d\n", enable);
	ret = mRM->setResBoolValue(ID_MENU_LIST_SILENTMODE, (!(bool)enable));
	setting_status[SETTING_VIDEO_AUDIO_RECORD].ivalue = enable;
	return ret;
}

int YoukuCmdsCallback::SetPhotoResolution(int item, void *data)
{
	int ret = -1;
    switch (item) {
        case YK_PHOTO_RESOLUTION_1M:
            item = 0;
            break;
        case YK_PHOTO_RESOLUTION_2M:
            item = 1;
            break;
        case YK_PHOTO_RESOLUTION_5M:
            item = 2;
            break;
        case YK_PHOTO_RESOLUTION_8M:
            item = 3;
            break;
        default:
            return ret;
    }
	db_msg("item:%d\n", item);
	ret = mRM->setResIntValue(ID_MENU_LIST_PQ,
						INTVAL_SUBMENU_INDEX, item);
	setting_status[SETTING_PHOTO_RESOLUTION].ivalue = item;
	return ret;
}

int YoukuCmdsCallback::SetPhotoWriteBalance(int item, void *data)
{
	int ret = -1;
    switch (item) {
        case YK_WB_AUTO:
            item = 0;
            break;
        case YK_WB_DAYLIGHT:
            item = 1;
            break;
        case YK_WB_CLOUDY:
            item = 2;
            break;
        case YK_WB_TUNGSTEN:
            item = 3;
            break;
        case YK_WB_FLUORESCENT:
            item = 4;
            break;
        default:
            return ret;
    }
	db_msg("item:%d\n", item);
	ret = mRM->setResIntValue(ID_MENU_LIST_WB, INTVAL_SUBMENU_INDEX, item);
	setting_status[SETTING_PHOTO_WHITE_BALANCE].ivalue = item;
	return ret;
}

int YoukuCmdsCallback::SetPhotoEV(int item, void *data)
{
	int ret = -1;
    switch (item) {
        case YK_EV_N30:
            item = 0;
            break;
        case YK_EV_N20:
            item = 1;
            break;
        case YK_EV_N10:
            item = 2;
            break;
        case YK_EV_0:
            item = 3;
            break;
        case YK_EV_P10:
            item = 4;
            break;
        case YK_EV_P20:
            item = 5;
            break;
        case YK_EV_P30:
            item = 6;
            break;
        default:
            return ret;
    }
	db_msg("item:%d\n", item);
	ret = mRM->setResIntValue(ID_MENU_LIST_EXPOSURE, INTVAL_SUBMENU_INDEX, item);
	setting_status[SETTING_PHOTO_EV].ivalue = item;
	return ret;
}

#if 0
int YoukuCmdsCallback::SetPhotoDateWaterPrint(int enable, void *data)
{
	int ret = -1;
	db_msg("item:%d\n", enable);
	ret = mRM->setResBoolValue(ID_MENU_LIST_PHOTO_WM, (bool)enable);
	setting_status[SETTING_PHOTO_DATE_PRINT].ivalue = enable;
	return 0;
}
#endif

int YoukuCmdsCallback::SetStreamResolution(int item, void *data)
{
    switch (item) {
        case YK_STREAM_WINDOW_640X360_16_9:
            item = 0;
            break;
        case YK_STREAM_WINDOW_480X270_16_9:
            item = 1;
            break;
        default:
            return -1;
    }
	db_msg("item:%d\n", item);
	mRP->setSessionSize(item);
	setting_status[SETTING_COMM_STREAM_WINDOW_SIZE].ivalue = item;
	return 0;
}

int YoukuCmdsCallback::SetStreamBitrate(int item, void *data)
{
    switch (item) {
        case YK_STREAM_BIT_RATE_800K:
            item = 0;
            break;
        case YK_STREAM_BIT_RATE_1000K:
            item = 1;
            break;
        case YK_STREAM_BIT_RATE_2000K:
            item = 2;
            break;
        default:
            return -1;
    }
	db_msg("item:%d\n", item);
	mRP->setSessionBitrate(item);
	setting_status[SETTING_COMM_STREAM_BIT_RATE].ivalue = item;
	return 0;
}

int YoukuCmdsCallback::SetLCDSleep(int item, void *data)
{
	int ret = -1;
    switch (item) {
        case YK_LCD_SLEEP_10S:
            item = 0;
            break;
        case YK_LCD_SLEEP_20S:
            item = 1;
            break;
        case YK_LCD_SLEEP_30S:
            item = 2;
            break;
        case YK_LCD_SLEEP_CLOSE:
            item = 3;
            break;
        default:
            return ret;
    }
	db_msg("item:%d\n", item);
	ret = mRM->setResIntValue(ID_MENU_LIST_SS, INTVAL_SUBMENU_INDEX, item);
	setting_status[SETTING_COMM_LCD_SLEEP].ivalue = item;
	return ret;
}

#if 0
int YoukuCmdsCallback::SetBeepVolume(int item, void *data)
{
	float vol = item;
	db_msg("item:%d\n", item);
	mCdrMain->mPlayBack->setPlayerVol(vol/2*100);
	setKeySoundVol(vol/2*100);
	setting_status[SETTING_COMM_BEEP_VOLUME].ivalue = item;
	return 0;
}
#endif

int YoukuCmdsCallback::DoShoot(int enable, void *data)
{
	int ret = enable;
	db_msg("item:%d\n", enable);
	if (mCameraMode == RECORD_VIDEO) {
		if (enable == 0){
			ret = mRP->stopRecord();
		} else {
			ret = mRP->startRecord();
			if(ret == RET_NOT_MOUNT) {
				ret = YOUKU_CMD_CARD_REMOVED;
			} else if(ret == RET_DISK_FULL) {
				ret = YOUKU_CMD_STORAGE_FULL;
			}
		}
	} else if (mCameraMode == TAKE_PHOTO) {
		SendMessage(mCdrMain->getWindowHandle(WINDOWID_RECORDPREVIEW), MSG_RM_TAKE_PICTURE, 0, 0);
		ret = 0;
	}
	return ret;
}

int YoukuCmdsCallback::ChangeCameraMode(int item, void *data)
{
    switch (item) {
        case YK_CAMERA_MODE_PHOTO:
            item = TAKE_PHOTO;
            break;
        case YK_CAMERA_MODE_VIDEO:
             item = RECORD_VIDEO;
            break;
        default:
            return -1;
    }
	db_msg("item:%d\n", item);
	if (item == TAKE_PHOTO || item == MULTI_SHUT) {
		mRP->setRPWindowState(STATUS_PHOTOGRAPH);
	} else if (item == RECORD_VIDEO) {
		if (mRP->getRPWindowState() != STATUS_RECORDPREVIEW) {
			mRP->setRPWindowState(STATUS_RECORDPREVIEW);
			mCdrMain->setCurrentWindowState(STATUS_RECORDPREVIEW);
		}
		mRP->setRPWindowState(STATUS_RECORDPREVIEW);
	} else {
		return YOUKU_CMD_MODE_ERROR;
	}
	mCameraMode = item;
	setting_status[SETTING_COMM_CAMERA_MODE].ivalue = item;
	return 0;
}

int YoukuCmdsCallback::SetAutoPowerOff(int item, void *data)
{
	int ret = -1;
    switch (item) {
        default:
            return ret;
    }
	db_msg("item:%d\n", item);
	ret = mRM->setResIntValue(ID_MENU_LIST_SHUTDOWN, INTVAL_SUBMENU_INDEX, item);
	setting_status[SETTING_COMM_AUTO_POWER_DOWN].ivalue = item;
	return ret;
}

int YoukuCmdsCallback::FactoryMode(int enable, void *data)
{
	db_msg("item:%d\n", enable);
	int curSessionCam = mRP->getCurSessionCam();
	mRP->stopRecord(curSessionCam);
	mMenu->doFactoryReset();
	return 0;
}

int YoukuCmdsCallback::FormatSDCard(int enable, void *data)
{
	db_msg("item:%d\n", enable);
	SendMessage(mMenu->getHwnd(), MSG_RM_FORMAT_TFCARD, 0, 0);
	return 0;
}

#if 0
int YoukuCmdsCallback::SetSSID(int enable, void *data)
{
	int ret = -1;
	db_msg("item:%d\n", enable);
	char pwd[128];
	NetConnectManager *ncm = NetConnectManager::getInstance();
	ncm->getSoftApPwd((char*)pwd, 128);
	// TODO:
	// db_msg("ssid:%s, password :%s\n", (char*)(mDevCtrl.ctrl_data), pwd);
	// mCdrMain->wifiEnable(NET_CONNECT_MANAGER_DISABLE_SOFTAP);
	// ret = mCdrMain->wifiEnable(NET_CONNECT_MANAGER_ENABLE_SOFTAP, (char*)(mDevCtrl.ctrl_data), (char*)data, NULL);
	return ret;
}
#endif

static void *shutdown(void* value)
{
	CdrMain *cdrMain = CdrMain::mYoukuCmdsCallback->GetCdrMain();
	sleep(1);
	cdrMain->shutdown();
	pthread_exit(0);
	return NULL;
}

int YoukuCmdsCallback::SetPowerOff(int enable, void *data)
{
	int ret = -1;
	db_msg("item:%d\n", enable);
	pthread_t t_shutdown = 0;
	ret = pthread_create(&t_shutdown, NULL, shutdown, NULL);
	if (!t_shutdown || ret != 0) {
		db_error("creat shutdown thread failed\n");
	}
	return ret;
}

int YoukuCmdsCallback::DeleteMediaFile(int enable, void *data)
{
	db_msg("item:%d\n", enable);
	StorageManager *sm = StorageManager::getInstance();
	if (mRP->isRecording()) {
		mRP->stopRecord();
	}
	// TODO:
	// sm->deleteFile((char*)data);
	return 0;
}

int YoukuCmdsCallback::DeleteRecentlyMediaFile(int enable, void *data)
{
	db_msg("item:%d\n", enable);
	StorageManager *sm = StorageManager::getInstance();
	if (mRP->isRecording()) {
		mRP->stopRecord();
	}
	sm->deleteRecentlyFile();
	return 0;
}

int YoukuCmdsCallback::CleanSDCard(int enable, void *data)
{
	int ret = -1;
	db_msg("item:%d\n", enable);
	StorageManager *sm = StorageManager::getInstance();
	if (mRP->isRecording()) {
		mRP->stopRecord();
	}
	ret = sm->deleteAllFiles();
	if (ret == RET_IO_OK || ret == RET_IO_NO_RECORD)
		ret = 0;
	else
		ret = -1;
	return ret;
}

int YoukuCmdsCallback::EnableStream(int enable, void *data)
{
	int ret = -1;
	static int state = 0;

	stream_enable_param *param = (stream_enable_param*)data;
	db_msg("enable=%d, remote_ip=%s, stream_port=%d\n",
			param->enable, param->remote_ip, param->stream_port);

	if (param->enable == 1/*  && state == 0 */) {
		SendMessage(mCdrMain->mHwnd, MSG_SESSIONCONNECT, 0, 0);
		mStreamSender->setup(param->remote_ip, param->stream_port);
		mCdrMain->sessionChange(true);
		state = 1;
	} else if (param->enable == 0 && state == 1) {
		SendMessage(mCdrMain->mHwnd, MSG_SESSIONDISCONNECT, 0, 0);
		mStreamSender->stop();
		mCdrMain->sessionChange(false);
		state = 0;
	}
	setting_status[SETTING_COMM_STREAM_ENABLE].ivalue = enable;
	return 0;
}

int YoukuCmdsCallback::GetDeviceInfo(int enable, void *data)
{
	db_msg("item:%d\n", enable);
	devinfo *dinfo = (devinfo*)data;
	FirmwareInfo fmInfo;

	mRM->getConfig(mCdrMain);

	// NetConnectManager *ncm = NetConnectManager::getInstance();
	ResourceManager *rm = ResourceManager::getInstance();

	// TODO: network info
	// ncm->getMacAddress(dinfo->ap_mac, "wlan0");
	// ncm->getSoftApSsid(dinfo->ap_ssid, MAX_WIFI_SSID_LEN);

	dinfo->video_ms_url = strdup("rtsp://192.168.100.1/live");
	dinfo->photo_ms_url = strdup("rtsp://192.168.100.1/live");

	rm->getFirmwareInfo(&fmInfo);
	dinfo->firmware_version = strdup(fmInfo.software_version.string());
	dinfo->yksdk_version = strdup("1.1.2");
	dinfo->serial_number = strdup("000000001111001");

	// new special video flag will be set in another thread
	Mutex::Autolock _l(gNewSpecialVideoMutex);

	return 0;
}

int YoukuCmdsCallback::CleanNewSpecialVideoFlag(int enable, void *data)
{
	Mutex::Autolock _l(gNewSpecialVideoMutex);
	setting_status[SETTING_COMM_CLEAN_NEW_VIDEO_FLAG].ivalue = 0;
	return 0;
}

YoukuDeviceControl &YoukuCmdsCallback::GetYoukuDevCtrl()
{
	return mDevCtrl;
}

int YoukuCmdsCallback::SetSystemlang(int language)
{
	clogd(__func__);
	return 0;
}

int YoukuCmdsCallback::SetLCDDispaly(bool on)
{
	clogd(__func__);
	return 0;
}

int YoukuCmdsCallback::SetSystemDateTime(int item, void *data)
{
	clogd(__func__);
    time_t *time = (time_t*)data;
    struct tm *tm = localtime(time);
    ::setDateTime(tm);
	return 0;
}

#endif
