#include "platform.h"
#ifdef YOUKU_PLATFORM
#include <cutils/log.h>
#include "debug.h"
#include "YoukuService.h"
#include <string.h>
#include <youku/ykcamera.h>
#include <youku/ykcamera_log.h>
#include <CdrServer.h>


#ifdef LOG_TAG
#undef LOG_TAG
#endif
#define LOG_TAG "YoukuService"

YKStreamSender* YoukuService::mStreamSender = NULL;

YoukuService::YoukuService()
	: mStatus(UNINIT)
	, mHandler(NULL)
	, mYKThread(NULL)
{
	db_msg("YoukuService construct\n");
}

YoukuService::~YoukuService()
{
	mYKThread->stopThread();
}

int YoukuService::initCamera()
{
	int ret = -1;
	static hfs_info *hfs = NULL;
	static web_sever_param *wsp = NULL;


	hfs = (hfs_info *)malloc(sizeof(hfs_info));
	if (hfs == NULL) {
		db_error("hfs alloc failed\n");
		return -1;
	}
	hfs->hfs_path = strdup("/mnt/extsd");
	hfs->hfs_preview_path = strdup("/mnt/extsd/video/.preview");
	hfs->hfs_cut_path = strdup("/mnt/extsd/video");
	hfs->hfs_port = strdup("8080");

	wsp = (web_sever_param *)malloc(sizeof(web_sever_param));
	if (wsp == NULL) {
		db_error("wsp alloc failed\n");
		return -1;
	}
	wsp->hfs_info = hfs;
	wsp->user_data = NULL;

	if (mHandler == NULL){
		db_error("mHandler is NULL\n");
		return -1;
	}


	wsp->cam_handler = (youku_camera_request_handler)mHandler;
	wsp->control_port = strdup("8010");

	mCamInterface.wsp = wsp;
	mCamInterface.cam_status = &mCamStatus;
	// mCamInterface.sdk_debug_level = CAM_LOG_ERROR;

	ret = youku_camera_init(&mCamInterface);
	if (ret < 0){
		db_error("Init failed\n");
		return -1;
	}

	// TODO:maybe not use streamsender as a static member
	getStreamSender(TYPE_RTSP);
	// CdrServer *cdrServer = CdrServer::getInstance();
	// CdrMain *cdrMain = cdrServer->getCdrMain();
	CdrMain::mYoukuCmdsCallback->Init(mStreamSender);
	mStatus = INITED;
	return 0;
}

int YoukuService::startCamera()
{
	int ret = -1;
	ret = youku_camera_start();
	if (ret < 0){
		db_error("Start failed\n");
		return -1;
	}
	mStatus = STARTED;
	return 0;
}

void YoukuService::startYoukuThread()
{
	if (mYKThread == NULL) {
		mYKThread = new YoukuServiceThread(this);
	}
	mYKThread->startThread();
}

void YoukuService::stopYoukuThread()
{
	if (mYKThread != NULL) {
		mYKThread->stopThread();
	}
}

int YoukuService::deInitCamera()
{
	int ret = -1;
	free((mCamInterface.wsp)->hfs_info);
	free(mCamInterface.wsp);
	ret = youku_camera_deinitialize();
	if (ret < 0){
		db_error("Deinit failed\n");
		return -1;
	}
	mStatus = UNINIT;
	return 0;
}

void YoukuService::setCamStatus(module_value* device_status, module_value* setting_status)
{
	mCamStatus.device_status = device_status;
	mCamStatus.setting_status = setting_status;
}

camera_status YoukuService::getCamStatus()
{
	return mCamStatus;
}

void YoukuService::setRespCallback(youku_camera_request_handler handler)
{
	mHandler = handler;
}

int YoukuService::getServiceStatus()
{
	return mStatus;
}

YKStreamSender *YoukuService::getStreamSender(int streamtype)
{
	if (mStreamSender == NULL) {
		mStreamSender = YKStreamSender::createNew((enum YKStreamType)streamtype);
	}
	return mStreamSender;
}

#endif
