#include "StorageManager.h"
#include "ResourceManager.h"
#undef LOG_TAG
#define LOG_TAG "StorageManager"
#include "debug.h"
#include <sys/types.h>
#include <sys/wait.h>
#include <stdlib.h>
#include <stdio.h>

#define SYNC_THREAD

static StorageManager* gInstance;
static Mutex gInstanceMutex;

static DBC_TYPE gDBType[] = {
	DB_TEXT,
	DB_UINT32, 
	DB_TEXT,
	DB_TEXT,
	DB_UINT32,
};

static struct sql_tb gSqlTable[] = {
	{"file", "VARCHAR(128)"},
	{"time", "INTEGER"},
	{"type", "VARCHAR(32)"},		/* "video" "pic" */
	{"info", "VARCHAR(32)"},		/* "normal"  */
	{"filesize", "INTEGER"},
};

void StorageManager::init()
{
	mStorageMonitor = new StorageMonitor;
	mStorageMonitor->init();
	mStorageMonitor->setMonitorListener(this);
	if(isMount(MOUNT_POINT)) {
		mMounted = true;
		checkDirs();
	}
	else
		mMounted = false;

	mInitDbTh = new InitDbThread(this);
	mInitDbTh ->start();
/*	mDBInitL.lock();
	dbInit();
	mDBInitL.unlock();*/
	db_info("%s is %s\n", MOUNT_PATH, (mMounted == true) ? "mounted" : "not mount");
}

void StorageManager::exit()
{
	delete mStorageMonitor;
	mStorageMonitor = NULL;
	dbExit();
}

int StorageManager::shareVol()
{
	if (mStorageMonitor) {
		return mStorageMonitor->mountToPC();
	}
	return -1;
}
int StorageManager::unShareVol()
{
	if (mStorageMonitor) {
		return mStorageMonitor->unmountFromPC();
	}
	return -1;
}
StorageManager::StorageManager()
	: mStorageMonitor(NULL)
	, mDBC(NULL)
	, mListener(NULL)
	, mDBInited(false)
	, mSST(NULL)
	, mStopDBUpdate(false)
	, mDbInitStat(false)
{
	init();
}

StorageManager::~StorageManager()
{
	if(mInitDbTh != NULL){
		mInitDbTh.clear();
		mInitDbTh = NULL;
	}
	exit();
}
StorageManager* StorageManager::getInstance(void)
{
	Mutex::Autolock _l(gInstanceMutex);
	if(gInstance != NULL) {
		return gInstance;
	}
	gInstance = new StorageManager();
	return gInstance;
}

void StorageManager::notify(int what, int status, char *msg)
{
	switch(status){
	case STORAGE_STATUS_UNMOUNTING:
	case STORAGE_STATUS_NO_MEDIA:
		{
			db_msg("~~unmounting");
			//dbUpdate();
			if (mMounted == true) {
				mMounted = false;
				if(mListener)
					mListener->notify(EVENT_MOUNT, 0);
			}
            mDbInitStat = false;
			stopUpdate();
			break;
		}
	case STORAGE_STATUS_MOUNTED:
		{
			db_msg("~~mounted");
			mMounted = true;
			checkDirs();
            mDbInitStat = true;
			dbReset();
			if(mListener)
			mListener->notify(EVENT_MOUNT, 1);
			break;
		}
	case STORAGE_STATUS_REMOVE:
		mListener->notify(EVENT_REMOVESD, 1);			
		break;
	case STORAGE_STATUS_PENDING:
		mListener->notify(EVENT_INSERTSD, 1);
		break;
	}
}
int StorageManager::isInsert(const char *checkPath)
{
	FILE *fp;
	db_msg("------------checkPath = %s \n", checkPath);
	if (!(fp = fopen(checkPath, "r"))) {
		db_msg(" isInsert fopen fail, checkPath = %s\n", checkPath);
		return 0;
	}
	if(fp){
		fclose(fp);
	}
	return 1;
}

int StorageManager::isMount(const char *checkPath)
{
	const char *path = "/proc/mounts";
	FILE *fp;
	char line[255];
	const char *delim = " \t";
	ALOGV(" isMount checkPath=%s \n",checkPath);
	if (!(fp = fopen(path, "r"))) {
		ALOGV(" isMount fopen fail,path=%s\n",path);
		return 0;
	}
	memset(line,'\0',sizeof(line));
	while(fgets(line, sizeof(line), fp))
	{
		char *save_ptr = NULL;
		char *p        = NULL;
		//ALOGV(" isMount line=%s \n",line);
		if (line[0] != '/' || strncmp("/dev/block/vold",line,strlen("/dev/block/vold")) != 0)
		{
			memset(line,'\0',sizeof(line));
			continue;
		}       
		if ((p = strtok_r(line, delim, &save_ptr)) != NULL) {		    			
			if ((p = strtok_r(NULL, delim, &save_ptr)) != NULL){
				ALOGV(" isMount p=%s \n",p);
				if(strncmp(checkPath,p,strlen(checkPath) ) == 0){
					fclose(fp);
					ALOGV(" isMount return 1\n");
					return 1;				
				}				
			}					
		}		
	}//end while(fgets(line, sizeof(line), fp))	
	if(fp){
		fclose(fp);
	}
	return 0;
}

bool StorageManager::isInsert(void)
{
	mInsert = mStorageMonitor->isInsert();
	db_msg("------------mInsert = %d \n", mInsert);
	return mInsert;
}
bool StorageManager::isMount(void)
{
	//mMounted = mStorageMonitor->isMount();
//	db_msg("------------mMounted = %d \n", mMounted);
	return mMounted;
}

String8 StorageManager::lockFile(const char *path)
{
	String8 file(path);
	// /mnt/extsd/video/time_A.mp4 -> /mnt/extsd/video/time_A_SOS.mp4
	file = file.getBasePath();
	file += FLAG_SOS;
	file += EXT_VIDEO_MP4;
	file = String8(CON_2STRING(MOUNT_PATH, LOCK_DIR))+file.getPathLeaf();
	db_msg("lockFile:%s --> %s", path, file.string());
	reNameFile(path, file.string());
	
#ifdef USE_TAKEPIC_NETJPD
	String8 netPicPath, newNetPicPath;
	geneNetPicName(String8(path), netPicPath, EXT_PIC_JPG);

	if(FILE_EXSIST(netPicPath.string())){
		geneNetPicName(file, newNetPicPath, EXT_PIC_JPG);
		reNameFile(netPicPath.string(), newNetPicPath.string());
	}
#endif
	
	return file;
}

int StorageManager::reNameFile(const char *oldpath, const char *newpath)
{
	int ret;

	ret = rename(oldpath,newpath);
	if(ret != 0){
		db_error("reNameFile fail oldpath=%s, newpath = %s,ret=%d\n",oldpath,newpath,ret);
	}

	return ret;
}

int needDeleteFiles()
{
	unsigned long long as = availSize(MOUNT_PATH);
	unsigned long long ts = totalSize(MOUNT_PATH);
	db_msg("[Size]:%lluMB/%lluMB, reserved:%lluMB", as, ts, RESERVED_SIZE);

	if (as <= RESERVED_SIZE) {
		db_msg("!Disk FULL");
		return RET_DISK_FULL;
	}
	
	if (as <= LOOP_COVERAGE_SIZE) {
		db_msg("!Loop Coverage");
		return RET_NEED_LOOP_COVERAGE;
	}
	
	return 0;
}

int StorageManager::getCapacity(unsigned long long *as, unsigned long long *ts)
{
	*as = availSize(MOUNT_PATH);
	*ts = totalSize(MOUNT_PATH);
	return 0;
}

int StorageManager::getRemainingVideoTime(bool uvc_used)
{
	uint64_t avail = 0;
	uint64_t  total = 0;
	uint64_t bitRate = 0;
	int remainingTime = 0;

	// ignore audio encode
	if (isMount()) {
		getCapacity(&avail, &total);
		remainingTime = avail - 103;
		if (remainingTime >= 0) {
			bitRate = FRONT_CAM_BITRATE;
			// time(s) = (avail(MB)-reserve)*1024*1024*8 / bitrate(bps)
			remainingTime = (int)(((avail-103) << 23) / bitRate);
			if (uvc_used) {
				bitRate = BACK_CAM_BITRATE;
				remainingTime -= (int)(((avail-103) << 23) / bitRate);
			}
		} else
			remainingTime = 0;
	}
	// return mintues;
	return remainingTime / 60;
}

int StorageManager::getRemainingPictures(void)
{
	uint64_t avail = 0;
	uint64_t total = 0;
	int  idx=0, picResolution = 0,remainingPicTotal=0;
	ResourceManager* rm = ResourceManager::getInstance();
	idx = rm->getResIntValue(ID_MENU_LIST_PQ, INTVAL_SUBMENU_INDEX);
	switch(idx)
	{
		case 0:
			picResolution=242;//1m;
		break;
		case 1:
			picResolution=612;//2m;
		break;
		case 2:
			picResolution=1214;//5m;
		break;
		case 3:
			picResolution=1946;//8m;
		break;
		default:
		break;
	}
	if(isMount()) {
		getCapacity(&avail, &total);
		remainingPicTotal = avail - 103;
		if(remainingPicTotal >=0) {
			remainingPicTotal = (int)((remainingPicTotal << 10) / picResolution);
		}
		else
			remainingPicTotal = 0;

		return remainingPicTotal;
	} else {
		return -1;
	}
	return 0;
}

int StorageManager::deleteFile(const char *file)
{
	int ret = 0;
	String8 path(file);
	if (FILE_EXSIST(file)) {
		if (remove(file) != 0) {
			ret = -1;
		}
#ifdef APP_WIFI
		delNetBmp(file);
#endif
	} else {
		db_msg("file not existed:%s", file);
	}
	/* delete thumbnail */
	if (path.getPathExtension() == EXT_VIDEO_MP4 ||
		path.getPathExtension() == EXT_PIC_JPG) {
		String8 stime;
		stime = path.getBasePath().getPathLeaf();
		path = (path.getPathDir())+"/"+THUMBNAIL_DIR+stime+EXT_PIC_BMP;// /mnt/extst/video/time_A.mp4 ->/mnt/extsd/video->/mnt/extsd/video/.thumb/time_A.bmp
		db_msg("delete pic :%s", path.string());
		return deleteFile(path.string());
	}
	return ret;
}

int StorageManager::deleteFiles(const char *type, const char *info, bool isDeleteAll)
{

	Mutex::Autolock _l(mDBLock);

	void *ptr;

	int count = 0;
	int ret = 0;
	String8 filter;
	String8 filter_type("WHERE TYPE ='");
	String8 tmp;
	String8 time;

	db_debug("delete files in /mnt/extsd/%s", type);
	if (strcmp(type, TYPE_PIC) == 0) {
		filter_type += type;
		filter_type += "'";
	} else {
		filter_type += type;
		filter_type += "' AND info ='";
		filter_type += info;
		filter_type += "'";
	}
	db_debug("%s %d, %s\n", __FUNCTION__, __LINE__, filter_type.string());
	mDBC->setFilter(filter_type);

	count = mDBC->getRecordCnt();
	db_debug("%s recordcnt:%d\n", __FUNCTION__, count);
	if (count <= 0) {
		/* check datebase */
		system("ls -l /data");
		system("df /data");
		return RET_IO_NO_RECORD;
	}

	mDBC->bufPrepare();
	do {
		if ((count--) == 0) {
			return RET_IO_NO_RECORD;
		}
		//system("ls -l /mnt/extsd/video");
		filter = filter_type;
		filter += " ORDER BY time ASC";
		mDBC->setFilter(filter);
		tmp = "file";
		ptr = mDBC->getFilterFirstElement(tmp, 0);
		if (!ptr) {
			db_debug("fail to read db");
			return RET_IO_DB_READ;
		}
		ret = deleteFile((const char*)ptr);
		if (ret < 0) {
			db_debug("fail to delete file %s", (const char *)ptr);
			//return RET_IO_DEL_FILE;
		}
		db_debug("%s has been deleted\n", (char *)ptr);
		tmp = String8::format("DELETE FROM %s WHERE file='%s'", mDBC->getTableName().string(), (char *)ptr);
		db_debug("execute sql %s", tmp.string());
		mDBC->setSQL(tmp);
		mDBC->executeSQL();
		if (!isDeleteAll) {
			unsigned long long as  = availSize(MOUNT_PATH);
			if (as == 0) {
				db_msg("availSize is 0");
				break;
			}
			if (as > LOOP_COVERAGE_SIZE) {
				db_msg("availSize %lluMB > loop size %dMB", as, LOOP_COVERAGE_SIZE);
				break;
			}
		}
	} while(1);
	mDBC->bufRelease();

	return RET_IO_OK;
}

int StorageManager::deleteFiles()
{
	int ret = 0;
	bool isDeleteAll = false;

	ret = deleteFiles(TYPE_VIDEO, INFO_NORMAL, isDeleteAll);
/*	if (ret != RET_IO_OK) {
		ret = deleteFiles(TYPE_VIDEO, INFO_LOCK, isDeleteAll);
	} */
	return ret;
}

int StorageManager::deleteAllFiles()
{
	int ret;
	bool isDeleteAll = true;

	deleteFiles(TYPE_VIDEO, INFO_NORMAL, isDeleteAll);
	deleteFiles(TYPE_PIC, INFO_NORMAL, isDeleteAll);
	ret = deleteFiles(TYPE_VIDEO, INFO_LOCK, isDeleteAll);
#ifdef APP_WIFI
	deleteFiles(TYPE_VIDEO, INFO_WOND, isDeleteAll);
#endif
	return ret;
}

int StorageManager::deleteRecentlyFile()
{
	char *ptr;
	int total, retval;
	retval = dbCount((char*)TYPE_VIDEO);
	if(retval < 0) {
		db_error("get db count failed\n");
		return -1;
	}
	total = retval;
	retval = dbCount((char*)TYPE_PIC);
	if(retval < 0) {
		db_error("get db count failed\n");
		return -1;
	}
	total += retval;
	if(total<0){
		db_msg("no files to delete");
		return -1;
	}
	Mutex::Autolock _l(mDBLock);
	mDBC->bufPrepare();
	ptr = (char *)mDBC->getElement(String8("file"), 0, total-1);
	mDBLock.unlock();
	if(ptr)
	{
		dbDelFile(ptr);
		deleteFile(ptr);
	}
	mDBC->bufRelease();
	return 0;
}

int StorageManager::generateFile(time_t &now, String8 &file, int cam_type, fileType_t type, bool nextFd)
{
	
	int fd;
	int ret;
	struct tm *tm=NULL;
	char buf[128]= {0};
	const char *camType = FILE_FRONT;
	const char *suffix = FLAG_NONE;
	const char *fileType = EXT_VIDEO_MP4;
	const char *path = CON_2STRING(MOUNT_PATH, VIDEO_DIR);
	if(isMount()== false)
		return RET_NOT_MOUNT;
	db_msg("%s %d", __FUNCTION__, __LINE__);
	if((ret = needDeleteFiles())) {
		if((type == PHOTO_TYPE_NORMAL) && (ret == RET_DISK_FULL))
			return RET_DISK_FULL;
		else {
			ret = deleteFiles();

			if(ret != RET_IO_OK ) {
				db_error("deleteFiles ret: RET_IO:%d\n", ret);
				return RET_IO_NO_RECORD;
			}
			
		}
	}

	now = getDateTime(&tm);
	if(nextFd){
		time_t ptime = mktime(tm);
		ptime+=10;
		tm = localtime(&ptime);
		now = ptime;
	}
	
	if (type == VIDEO_TYPE_IMPACT) {
		suffix = FLAG_SOS;
		path = CON_2STRING(MOUNT_PATH, LOCK_DIR);
	}
	if(type == VIDEO_TYPE_WONDERFUL){
		suffix = FLAG_WOND;
		path = CON_2STRING(MOUNT_PATH, WON_VIDEO_DIR);
	}
	if(type == PHOTO_TYPE_WONDERFUL){
		suffix = FLAG_WOND;
		fileType = EXT_PIC_JPG;
		path = CON_2STRING(MOUNT_PATH, WON_VIDEO_DIR);
	}
	if (type == PHOTO_TYPE_NORMAL) {
		fileType = EXT_PIC_JPG;
		path = CON_2STRING(MOUNT_PATH, PIC_DIR);
	}
	if (cam_type == CAM_UVC) {
		camType = FILE_BACK;
	}
	sprintf(buf, "%s%04d%02d%02d_%02d%02d%02d%s%s%s",path,tm->tm_year + 1900, tm->tm_mon + 1,
		tm->tm_mday, tm->tm_hour,tm->tm_min, tm->tm_sec, camType, suffix, fileType);
	file = buf;
//	fd = open(buf, O_RDWR | O_CREAT, 0666);
//	if (fd < 0) {
//		db_msg("failed to open file '%s'!!\n", buf);
//		return RET_IO_OPEN_FILE;
//	}
	db_debug("generate file %s", file.string());
	return RET_IO_OK;
}

int  StorageManager::readCpuInfo(char ret[]){
       char line[128];
       FILE *fp = fopen("/proc/cpuinfo", "r");
       if(fp==NULL){
               db_msg("open file failed.");
               return -1;
       }
       memset(ret,0,sizeof(ret));
       while(fgets(line, sizeof(line), fp)) {
               if (strncmp(line, "Serial", strlen("Serial")) == 0) {
                       char *str = strstr(line, ":");
                       str += 2;
               //      int num = atoi(str);
               //      printf("serial is %d\n", num);
	               if(strlen(str)>4){
	                       str +=(strlen(str)-5);
						   char* str1 = strstr(str, "\n");
						   if (str1 != NULL) {
						       *str1 = '\0';
						   }
	                       strcpy(ret,str);
	                       db_msg("str=%s,ret=%s",str,ret);
	               	}
               }
       }
       fclose(fp);
       return 0;

}

void StorageManager::generateSsid(char ssid[])
{
	char ret[10]={0};
	StorageManager *sm = StorageManager::getInstance();
	sm->readCpuInfo(ret);
	ssid[0]='\0';
	strcpy(ssid, "CDR-");
	strcat(ssid,ret);
	db_msg("ssid:%s", ssid);
	cfgDataSetString("persist.sys.wifi.ssid", ssid);
}

int StorageManager::geneImpactTimeFile(){
	int fd;
	int ret;
	struct tm *tm=NULL;
	char buf[IMPACT_TIME_TEXT_LENGTH]= {0};
	memset(buf,0,sizeof(buf));
	if(isMount()== false)
		return 0;
	db_msg("%s %d", __FUNCTION__, __LINE__);

	if (FILE_EXSIST(GENE_IMPACT_TIME_FILE)){
		db_msg("file in -----------");
	} 

	fd = open(GENE_IMPACT_TIME_FILE, O_RDWR|O_APPEND | O_CREAT, 0666);
	if (fd < 0) {
		db_msg("failed to open file '%s'!!\n", buf);
		return 0;
	}
	getDateTime(&tm);

	sprintf(buf, "%04d/%02d/%02d %02d:%02d",tm->tm_year + 1900, tm->tm_mon + 1,
		tm->tm_mday, tm->tm_hour,tm->tm_min);
	db_msg("%s %d", __FUNCTION__, __LINE__);
	buf[16]='\n';
	ret = write(fd,buf,sizeof(buf));
	close(fd);
	if(ret<0){
		db_msg("write file error!, %s,%d",__FUNCTION__, __LINE__);
		return 0;
	}
	
	return 1;


}

int StorageManager::readImpactTimeFile(	char ret[]){
	int fd,size,count=0;
	char buf[IMPACT_TIME_TEXT_LENGTH];
	db_msg("****--------------------%s,%d***",__FUNCTION__,__LINE__);
	memset(buf,0,sizeof(buf));
	memset(ret,0,sizeof(ret));
    if (!FILE_EXSIST(GENE_IMPACT_TIME_FILE)) {
		db_msg("impct_time_file not found");
		return 0;
	}
	fd = open(GENE_IMPACT_TIME_FILE, O_RDONLY , 0666);
	if (fd < 0) {
		db_msg("open file false'%s'!!\n", buf);
		return 0;
	}
	do{
		if(count<10)
			count++;
		else
			break;
		size=read(fd,buf,sizeof(buf));
		db_msg("******buff=%s----size=%d",buf,size);
		if(size==0)
			break;
		strcat(ret,buf);
		memset(buf,0,sizeof(buf));
	}while(size>1);
	close(fd);
	if(size<0){
		db_msg("read file failed!");
		db_msg("%s,%d",__FUNCTION__,__LINE__);
		return 0;
	}
	return 1;
}

int StorageManager::doMount(const char *fsPath, const char *mountPoint,
		bool ro, bool remount, bool executable,
		int ownerUid, int ownerGid, int permMask, bool createLost) {
	int rc;
	unsigned long flags;
	char mountData[255];
	flags = MS_NODEV | MS_NOSUID | MS_DIRSYNC;
	flags |= (executable ? 0 : MS_NOEXEC);
	flags |= (ro ? MS_RDONLY : 0);
	flags |= (remount ? MS_REMOUNT : 0);

	sprintf(mountData,"utf8,uid=%d,gid=%d,fmask=%o,dmask=%o,shortname=mixed",
			ownerUid, ownerGid, permMask, permMask); 
	rc = mount(fsPath, mountPoint, "vfat", flags, mountData);
	if (rc && errno == EROFS) {
		db_error("%s mount fail try again", fsPath);
		rc = mount(fsPath, mountPoint, "vfat", flags, mountData);
		db_error(" mount rc=%d", rc);
	}

	if (rc == 0 && createLost) {
		char *lost_path;
		asprintf(&lost_path, "%s/LOST.DIR", mountPoint);
		if (access(lost_path, F_OK)) {
			if (mkdir(lost_path, 0755)) {
				db_error("Unable to create LOST.DIR (%s)", strerror(errno));
			}
		}
		free(lost_path);
	}
	return rc;
}

int StorageManager::format(CommonListener *listener)
{

#if 0
	mFT = new FormatThread(this, listener);
	status_t err = mFT->start();
	if (err != OK) {
		mFT.clear();
	}
#else
	format();
	dbReset();
//	listener->finish(0, 0);	
#endif
	return 0;
}

int StorageManager::format()
{
	int ret = -1;
	/*
	db_msg("format start\n");
	int retries = 5;
	if(isMount()== true) {
		while((ret = umount(MOUNT_PATH)) != 0) {
			usleep(500*1000);
			if (!(--retries)) { //retry 5 times
				break;
			}
		}
		if (retries == 0) {
			if((ret = umount(MOUNT_PATH)) != 0) { //the 6th time, total time is 3s
				usleep(500*1000);
				ALOGD("format umount ret=%d,try again\n",ret);
				Process::killProcessesWithOpenFiles(MOUNT_PATH, 2);
				if((ret = umount(MOUNT_PATH)) != 0) {
					ALOGV("format umount again fail ret=%d\n",ret);
					return -1;
				}
			}
		}
	}
	ret = system("newfs_msdos -F 32 -O android -b 65536 -c 128 /dev/block/vold/179:0");
	if(ret < 0) {
		db_error("execute command failed: %s\n", strerror(errno));
		return -1;
	} else {
        if (WIFEXITED(ret)) { //success to exit
            if (0 == WEXITSTATUS(ret)) { //success to execute command
                db_msg("run shell script successfully.\n");
            } else { //fail
                db_error("run shell script fail, script exit code: %d\n", WEXITSTATUS(ret));
				return -1;
            }
        }  else { //exit error
            db_error("exit status = [%d]\n", WEXITSTATUS(ret));
			return -1;
        }
    }
	ret = doMount("/dev/block/vold/179:0", MOUNT_PATH, false, false, true, 0, 0, 0, true);
	if(ret != 0){
		ALOGV("__doMount erro ret=%d,error=%s\n",ret,strerror(errno));
		return -1;
	}
	db_msg("format end ret=%d\n",ret);
	*/
	if (mStorageMonitor) {
		return mStorageMonitor->formatSDcard();
    }
	checkDirs();
	dbReset();
	return ret;
}

long long totalSize(const char *path)
{
	struct statfs diskinfo;
	long long totalSize = 0;
	if(statfs(path, &diskinfo) != -1){
		totalSize = diskinfo.f_blocks * diskinfo.f_bsize;
		return totalSize >> 20; //MB
	}    	
	return 0;
}

long long availSize(const char *path)
{
	struct statfs diskinfo;
	long long availSize = 0;
	if(statfs(path, &diskinfo) != -1){
		availSize = diskinfo.f_bfree * diskinfo.f_bsize;
		return availSize >> 20; //MB
	}    	
	return 0;
}

int getFiles(char *dir, char *filetype, Vector<String8> &files)
{
//	db_msg("%s %d", __FUNCTION__, __LINE__);
	DIR *p_dir;
	String8 file(dir);
	struct stat stat;
	struct dirent *p_dirent;
	p_dir = opendir(dir);
	if (p_dir == NULL) {
		db_error("fail to opendir %s", dir);
		return -1;
	}
	while( (p_dirent = readdir(p_dir)) ) {
		if (strcmp(p_dirent->d_name, ".") == 0 ||
				strcmp(p_dirent->d_name, "..") == 0) {
			continue;
		}
		file = dir;
		file += p_dirent->d_name;
		if(lstat(file.string(), &stat) == -1) {
			db_error("fail to get file stat %s", file.string());
			continue;
		}
		if (!S_ISREG(stat.st_mode)) {
			db_error("%s is not a regular file", file.string());
			continue;
		}
		String8 ext = file.getPathExtension();
		if(ext.isEmpty()) {
			continue;
		}
		if(strcasecmp(ext.string(), filetype) == 0) {
			char* flag = strstr(file.string(), EXT_NET_PIC);
			if(flag == NULL){
				files.add(file);
			}
		} else {
//			db_debug("file is not type:%s", filetype);
		}
	}
	closedir(p_dir);
//	db_msg("%s %d", __FUNCTION__, __LINE__);
	return 0;
}

void StorageManager::dbReset()
{
	mDBInitL.lock();
	if (mDBInited == true) {
		db_debug("%s %d %p\n", __FUNCTION__, __LINE__, mDBC);
		mDBC->deleteTable(String8(CDR_TABLE));
		mDBInited = false;
	}
	//dbInit();
	mDBInitL.unlock();
	if(mInitDbTh != NULL){
		mInitDbTh->start();
	}
}

void StorageManager::dbInit()
{
	if (mDBInited) {
		return ;
	}
	if (mDBC == NULL) {
		system("rm /data/sunxi.db");
		mDBC = new DBCtrl();
		db_debug("%s %d %p\n", __FUNCTION__, __LINE__, mDBC);
	}
	mDBC->createTable(String8(CDR_TABLE), gSqlTable, ARRYSIZE(gSqlTable));	
	mDBC->setColumnType(gDBType, 0);
	dbUpdate();
	mDBInited = true;
}

time_t parase_time(char *ptr1)
{
	char *ptr;
	char chr;
	int value;
	struct tm my_tm;
	time_t my_time_t;
	/*
	   ptr = strchr(filename, '.');
	   if(ptr == NULL) {
	   printf("not found the delim\n");
	   return 0;
	   }
	   db_msg("filename is %s\n", filename);
	   ptr1 = strndup(filename, ptr - filename - 1);
	   */
	memset(&my_tm, 0, sizeof(my_tm));
	if (strlen(ptr1) >= strlen("20141201_121212")) {
	/************ year ************/
	ptr = ptr1;
	chr = ptr[4];
	ptr[4] = '\0';
	value = atoi(ptr);
	my_tm.tm_year = value - 1900;
	ptr[4] = chr;

	/************ month ************/
	ptr = ptr + 4;
	chr = ptr[2]; 
	ptr[2] = '\0';
	value = atoi(ptr);
	my_tm.tm_mon = value - 1;
	ptr[2] = chr;

	/************ day ************/
	ptr = ptr + 2;
	chr = ptr[2]; 
	ptr[2] = '\0';
	value = atoi(ptr);
	my_tm.tm_mday = value;

	/************ hour ************/
	ptr += 3;
	chr = ptr[2]; 
	ptr[2] = '\0';
	value = atoi(ptr);
	my_tm.tm_hour = value;
	ptr[2] = chr;

	/************ minite ************/
	ptr += 2;
	chr = ptr[2]; 
	ptr[2] = '\0';
	value = atoi(ptr);
	my_tm.tm_min = value;
	ptr[2] = chr;

	/************ second ************/
	ptr += 2;
	chr = ptr[2]; 
	ptr[2] = '\0';
	value = atoi(ptr);
	my_tm.tm_sec = value;
	} else {
		my_tm.tm_year = 2014;
		my_tm.tm_mon  = 12;
		my_tm.tm_mday = 1;
		my_tm.tm_hour = 1;
		my_tm.tm_min  = 1;
		my_tm.tm_sec  = 1;
	}
//	db_msg("year: %d, month: %d, day: %d, hour: %d, minute: %d, second: %d\n", 
//			my_tm.tm_year, 
//			my_tm.tm_mon, 
//			my_tm.tm_mday, 
//			my_tm.tm_hour, 
//			my_tm.tm_min, 
//			my_tm.tm_sec);
	my_time_t = mktime(&my_tm);
//	db_msg("time_t is %lu, time str is %s\n", my_time_t, ctime(&my_time_t));
	return my_time_t;
}

int StorageManager::fileSize(String8 path)
{
	struct stat	buf; 
	stat(path.string(),&buf);
	if (buf.st_size == 0) {
		db_msg("%s size is 0", path.string());
	}
	return buf.st_size;
}

int StorageManager::dbUpdate(char *type)
{
	Vector<String8> files_db, files;
	String8 file, file_db, tmp, stime;
	//	Vector<String8>::iterator result;
	Elem elem;
	time_t time;
	int size, size_db, i, ret=0;
	const char *ext = NULL;
	if (0 == strcmp(type, TYPE_VIDEO)) {
		ext = EXT_VIDEO_MP4;
		ret = getFiles((char*)CON_2STRING(MOUNT_PATH, VIDEO_DIR), (char*)ext, files);
		ret = getFiles((char*)CON_2STRING(MOUNT_PATH, LOCK_DIR), (char*)ext, files);
#ifdef APP_WIFI
		ret = getFiles((char*)CON_2STRING(MOUNT_PATH, WON_VIDEO_DIR), (char*)ext, files);
#endif
	} else if (0 == strcmp(type, TYPE_PIC)) {
		ext = EXT_PIC_JPG;
		ret = getFiles((char*)CON_2STRING(MOUNT_PATH, PIC_DIR), (char*)ext, files);
#ifdef APP_WIFI
		ret = getFiles((char*)CON_2STRING(MOUNT_PATH, WON_VIDEO_DIR), (char*)ext, files);
#endif
	}
	/*
	dbGetFiles(type, files_db);
	size_db = files_db.size();
	for(i=0; i<size_db; i++) {
		file_db = files_db.itemAt(i);
		if (!FILE_EXSIST(file_db.string())) {	//File is in the database but does not exist
			db_msg("has not found %s, delete from database", file_db.string());
			dbDelFile(file_db.string());	//delete file record in the database			
			ret = deleteFile(file_db.string());
			if (ret < 0) {
				db_debug("fail to delete file %s", (const char *)file_db.string());
				return RET_IO_DEL_FILE;
			}
		} else {
			db_msg("has found %s in the tfcard", file_db.string());
		}
	}
	*/
	size = files.size();
	for(i=0; i<size; i++)
	{
		if (mStopDBUpdate) {
			return 0;
		}
		//isValidVideoFile(files.itemAt(i));
		file = files.itemAt(i);
		if (!dbIsFileExist(file)) {	//file exsit but there is no record in the database
			if (fileSize(file) == 0) {	//file size is 0
				deleteFile(file.string());
				continue;
			}
			time = parase_time((char*)file.getBasePath().getPathLeaf().string());
			elem.file = (char *)file.string();
			elem.time = (long unsigned int)time;
			elem.type = type;
			if (0 == strcmp(type, TYPE_VIDEO)) {
				elem.info = (char*)INFO_NORMAL;
				if(strstr(file.string(), FLAG_SOS) != NULL) {
					//db_msg("file type is %s\n", FLAG_SOS);
					elem.info = (char*)INFO_LOCK;
				}else if(strstr(file.string(), FLAG_WOND) != NULL){
					elem.info = (char*)INFO_WOND;
				}
			}else if(0 == strcmp(type, TYPE_PIC)){
				elem.info = (char*)"";
			}
			elem.filesize = fileSize(file);
			dbAddFile(&elem);	//new file insert to the database
		}
//		db_msg("file %s exists in the database", files.itemAt(i).string());
	}
	return 0;
}

int StorageManager::dbUpdate()
{
//	db_msg("------dbUpate");
	int ret = 0;
	if(isMount()== false) {
		db_error("not mount\n");
		return -1;
	}
	db_msg(" ");
	mStopDBUpdate = false;
	ret = dbUpdate((char*)TYPE_VIDEO);
	if (ret >= 0) {
		ret = dbUpdate((char*)TYPE_PIC);
	}
	return ret;
}

void StorageManager::dbExit()
{
	if (mDBC) {
		delete mDBC;
		mDBC = NULL;
	}
}

/* create dirs
 * /mnt/extsd/video
 * /mnt/extsd/photo
 * /mnt/extsd/video/.thumb
 * /mnt/extsd/photo/.thumb
 * /mnt/extsd/sos
 * /mnt/estsd/wonderful
 */

int StorageManager::checkDirs()
{
	int retval = 0;

	if(access(CON_2STRING(MOUNT_PATH, VIDEO_DIR), F_OK) != 0) {
		if(mkdir(CON_2STRING(MOUNT_PATH, VIDEO_DIR), 0777) == -1) {
			db_msg("mkdir failed: %s\n", strerror(errno));
			retval = -1;
		}
	}
	if(access(CON_2STRING(MOUNT_PATH, PIC_DIR), F_OK) != 0) {

		if(mkdir(CON_2STRING(MOUNT_PATH, PIC_DIR), 0777) == -1) {
			db_msg("mkdir failed: %s\n", strerror(errno));
			retval = -1;
		}
	}
	if(access(CON_2STRING(MOUNT_PATH, LOCK_DIR), F_OK)!= 0) {
		if(mkdir(CON_2STRING(MOUNT_PATH, LOCK_DIR), 0777) == -1) {
			db_msg("mkdir failed: %s\n", strerror(errno));
			retval = -1;
		}
	}
#ifdef APP_WIFI
	if(access(CON_2STRING(MOUNT_PATH, WON_VIDEO_DIR), F_OK)!= 0) {
		if(mkdir(CON_2STRING(MOUNT_PATH, WON_VIDEO_DIR), 0777) == -1) {
			db_msg("mkdir failed: %s\n", strerror(errno));
			retval = -1;
		}	
	}

	if(access(CON_3STRING(MOUNT_PATH, VIDEO_DIR, APP_THUMBNAIL_DIR), F_OK) != 0) {
		if(mkdir(CON_3STRING(MOUNT_PATH, VIDEO_DIR, APP_THUMBNAIL_DIR), 0777) == -1) {
			db_msg("mkdir failed: %s\n", strerror(errno));
			retval = -1;
		}
	}

	if(access(CON_3STRING(MOUNT_PATH, PIC_DIR, APP_THUMBNAIL_DIR), F_OK) != 0) {
		if(mkdir(CON_3STRING(MOUNT_PATH, PIC_DIR, APP_THUMBNAIL_DIR), 0777) == -1) {
			db_msg("mkdir failed: %s\n", strerror(errno));
			retval = -1;
		}
	}

	if(access(CON_3STRING(MOUNT_PATH, LOCK_DIR, APP_THUMBNAIL_DIR), F_OK) != 0) {
		if(mkdir(CON_3STRING(MOUNT_PATH, LOCK_DIR, APP_THUMBNAIL_DIR), 0777) == -1) {
			db_msg("mkdir failed: %s\n", strerror(errno));
			retval = -1;
		}
	}

	if(access(CON_3STRING(MOUNT_PATH, WON_VIDEO_DIR, APP_THUMBNAIL_DIR), F_OK) != 0) {
		if(mkdir(CON_3STRING(MOUNT_PATH, WON_VIDEO_DIR, APP_THUMBNAIL_DIR), 0777) == -1) {
			db_msg("mkdir failed: %s\n", strerror(errno));
			retval = -1;
		}
	}
#endif
	if(access(CON_3STRING(MOUNT_PATH, VIDEO_DIR, THUMBNAIL_DIR), F_OK) != 0) {

		if(mkdir(CON_3STRING(MOUNT_PATH, VIDEO_DIR, THUMBNAIL_DIR), 0777) == -1) {
			db_msg("mkdir failed: %s\n", strerror(errno));
			retval = -1;
		}
	}
	if(access(CON_3STRING(MOUNT_PATH, PIC_DIR, THUMBNAIL_DIR), F_OK) != 0) {

		if(mkdir(CON_3STRING(MOUNT_PATH, PIC_DIR, THUMBNAIL_DIR), 0777) == -1) {
			db_msg("mkdir failed: %s\n", strerror(errno));
			retval = -1;
		}
	}
	return retval;
}

inline void addSingleQuotes(String8 &str)
{
	String8 tmp("'");
	tmp += str;
	tmp += ("'");
	str = tmp;
}

bool StorageManager::dbIsFileExist(String8 file)
{
	String8 filter("where file=");
	int cnt = 0;
	addSingleQuotes(file);
	filter += file;
	Mutex::Autolock _l(mDBLock);
	mDBC->setFilter(filter);
	cnt = mDBC->getRecordCnt();
	return (cnt>0)?true:false;
}

/* Count item, such as file, time, type, info etc.*/
int StorageManager::dbCount(char *item)
{
	Mutex::Autolock _l(mDBLock);
	String8 str("where type=");
	String8 item_str(item);
	addSingleQuotes(item_str);
	mDBC->setFilter(str+item_str);
	return mDBC->getRecordCnt();
}

void StorageManager::dbGetFiles(char *item, Vector<String8>&file)
{
	int total = dbCount(item);
	int i = 0;
	char *ptr = NULL;
	Mutex::Autolock _l(mDBLock);
	String8 str("where type=");
	String8 item_str(item);
	addSingleQuotes(item_str);
	mDBC->setFilter(str+item_str);
	mDBC->bufPrepare();
	while(i < total) {
		ptr = (char *)mDBC->getElement(String8("file"), 0, i);
		if (ptr) {
			file.add(String8(ptr));
		}
		i++;
	}
	mDBC->bufRelease();
}

void StorageManager::dbGetFile(char* item, String8 &file)
{
	char *ptr = NULL;
	Mutex::Autolock _l(mDBLock);
	String8 str("where type=");
	String8 item_str(item);
	addSingleQuotes(item_str);
	mDBC->setFilter(str+item_str);
	mDBC->bufPrepare();
	ptr = (char *)mDBC->getElement(String8("file"), 0, 0);
	if (ptr) {
		file = String8(ptr);
	}
	mDBC->bufRelease();
}
int StorageManager::dbGetSortByTimeRecord(dbRecord_t &record, int index)
{
	char *ptr = NULL;
	int total, retval;

	retval = dbCount((char*)TYPE_VIDEO);
	if(retval < 0) {
		db_error("get db count failed\n");
		return -1;
	}
	total = retval;

	retval = dbCount((char*)TYPE_PIC);
	if(retval < 0) {
		db_error("get db count failed\n");
		return -1;
	}
	total += retval;

	Mutex::Autolock _l(mDBLock);
	if(index >= total) {
		db_error("invalid index: %d, total is %d\n", index, total);
		return -1;
	}
	mDBC->setFilter(String8("ORDER BY time ASC") );
	mDBC->bufPrepare();
	ptr = (char *)mDBC->getElement(String8("file"), 0, index);
	if (ptr) {
		record.file = String8(ptr);
	}
	ptr = (char *)mDBC->getElement(String8("type"), 0, index);
	if (ptr) {
		record.type = String8(ptr);
	}
	ptr = (char *)mDBC->getElement(String8("info"), 0, index);
	if (ptr) {
		record.info = String8(ptr);
	}
	mDBC->bufRelease();

	return 0;
}
void StorageManager::dbAddFile(Elem *elem)
{
	char str[128];
	int ret;
	sprintf(str, "INSERT INTO %s VALUES('%s',%lu,'%s','%s',%d)",mDBC->getTableName().string(),
			elem->file,elem->time,elem->type,elem->info,elem->filesize);
//	db_msg("sql is %s\n", str);
	Mutex::Autolock _l(mDBLock);
	mDBC->setSQL(String8(str));
	ret = mDBC->executeSQL();
	if (ret != SQLITE_OK) {
		db_msg("error to add file:%d\n", ret);
	}
}

int StorageManager::dbDelFile(char const *file)
{
	Mutex::Autolock _l(mDBLock);
	char str[128];
	int ret = 0;
	
	sprintf(str, "DELETE from %s WHERE file='%s'",mDBC->getTableName().string(), file);
	mDBC->setSQL(String8(str));
	ret = mDBC->executeSQL();
	return ret;
}

void StorageManager::dbUpdateFile(const char* oldname, const char *newname)
{
	Mutex::Autolock _l(mDBLock);
	const String8 sql(String8::format("UPDATE %s SET %s = '%s' WHERE %s = '%s' ",
		mDBC->getTableName().string(), gSqlTable[0].item, oldname, gSqlTable[0].item, newname));
	mDBC->setSQL(sql);
	mDBC->executeSQL();
}

int StorageManager::saveVideoFrameToBmpFile(VideoFrame* videoFrame, const char* bmpFile, char* pic_type)
{
	unsigned int width, height, size;
	unsigned char* data;
	MYBITMAP myBitmap;
	RGB pal[256] = {{0,0,0,0}};
	int Bpp;	/* Bytes per pixel */
	int depth;
	data = (unsigned char*)videoFrame + sizeof(VideoFrame);
	width = videoFrame->mWidth;
	height = videoFrame->mHeight;
	size = videoFrame->mSize;
	Bpp = size / (width * height);
	depth = Bpp * 8;

	if(strcmp(pic_type,EXT_PIC_JPG) == 0){
		return Rgb2Jpg(data, bmpFile, width, height, depth);
	}else{	
		db_msg("Bpp is %d\n", Bpp);
		db_msg("depth is %d\n", depth);
		myBitmap.flags  = MYBMP_TYPE_NORMAL;
		myBitmap.frames = 1;
		myBitmap.depth  = depth;
		myBitmap.w      = width;
		myBitmap.h      = height;
		myBitmap.pitch  = width * Bpp;
		myBitmap.size   = size;
		myBitmap.bits   = data;
		return SaveMyBitmapToFile(&myBitmap, pal, bmpFile);
	}
}

int StorageManager::savePicture(void* data, int size, String8 &file)
{
	int bytes_write = 0;
	int bytes_to_write = size;
	void *ptr = data;

	db_msg("savePciture, size is %d\n", size);
	int fd = open(file.string(), O_RDWR | O_CREAT, 0666);
	if (fd < 0) {
		db_msg("failed to open file '%s'!!\n", file.string());
		return RET_IO_OPEN_FILE;
	}
	while((bytes_write = write(fd, ptr, bytes_to_write)) )
	{
		if((bytes_write == -1) && (errno != EINTR)) {
			db_msg("wirte error: %s\n", strerror(errno));
			break;
		} else if(bytes_write == bytes_to_write) {
			break;
		} else if( bytes_write > 0 ) {
			ptr = (void *)((char *)ptr + bytes_write);
			bytes_to_write -= bytes_write;
		}
	}
	if(bytes_write == -1) {
		db_error("save picture failed: %s\n", strerror(errno));
		return -1;
	}

	fsync(fd);
	close(fd);

	return 0;
}

bool StorageManager::isDiskFull(void)
{
	if(needDeleteFiles() == RET_DISK_FULL)
		return true;
	return false;
}

void StorageManager::setEventListener(EventListener* pListener)
{
	mListener = pListener;
}

StorageManager::StorageSyncThread::StorageSyncThread(StorageManager *sm, bool loop)
{
	mSM = sm;
#if 0
	system("echo 1 > /proc/sys/vm/dirty_ratio");
	system("echo 1 > /proc/sys/vm/dirty_background_ratio");
	system("echo 100 > /proc/sys/vm/dirty_writeback_centisecs");
	system("echo 1500 > /proc/sys/vm/dirty_expire_centisecs");
	system("echo 500 > /proc/sys/vm/vfs_cache_pressure");
#endif
	mLoop = loop;
}

void StorageManager::storageSync()
{
	db_msg(" ");
	system("sync "STORAGE_POINT);
}

void StorageManager::DBInitLock(bool flag)
{
	if(flag){
		mDBInitL.lock();
	}else{
		mDBInitL.unlock();
	}
}

void StorageManager::StorageSyncThread::storageSyncLoop()
{
	while(mLoop) {
		mSM->storageSync();
		usleep(3*1000*1000);
	}
	mSM->storageSync();
}

void StorageManager::startSyncThread(bool loop)
{
#ifdef SYNC_THREAD
	db_msg(" ");
	if (mSST == NULL) {
		mSST = new StorageSyncThread(this, loop);
	}
	mSST->start();
	db_msg("!!mSST is not null");
#endif
}

void StorageManager::stopSyncThread()
{
#ifdef SYNC_THREAD
	db_msg(" ");
	if (mSST != NULL) {
		mSST->stopThread();
		mSST.clear();
		mSST = NULL;
		return ;
	}
	db_msg("!!mSST is null");
#else
	storageSync();
#endif
}

void StorageManager::stopUpdate()
{
	mStopDBUpdate = true;
}

char* StorageManager::getFileType(const char *file)
{
	const char *ptr;
	char buf[8]={0};
	if(file == NULL)
		return NULL;
	ptr = strrchr(file, '.');
	sprintf(buf, "%s", ptr);
	if(strcmp(buf, EXT_VIDEO_MP4) == 0){
		return TYPE_VIDEO;
	}else if(strcmp(buf, EXT_PIC_JPG) == 0){
		return TYPE_PIC;
	}
	return NULL;
}

int StorageManager::delNetBmp(const char* file)   //delete file(mp4, jpg)'s  net bmp eg:file ="/mnt/extsd/video/a.mp4"  delete "/mnt/extsd/video/a_net.bmp"
{
	int ret = 0;
	char* file_type = getFileType(file);
	if(file_type != NULL){
		if((strcmp(file_type, TYPE_VIDEO) == 0) || (strcmp(file_type, TYPE_PIC) == 0)){
			char netFile[64] = {0};
		//	geneNetPicName(String8(file), netFile, EXT_PIC_BMP);
			geneNetPicName(String8(file), netFile, EXT_PIC_JPG);
			if (FILE_EXSIST(netFile)) {
				if (remove(netFile) != 0) {
					ret = -1;
				}
			}
		}
	}else{
		//db_msg("this file is not mp4 or jpg");
	}
	return ret;

}

int StorageManager::judgeFileFolder(const char* file)
{
	if(strstr(file, PIC_DIR)){
		return PHOTO_FOLDER;
	}else if(strstr(file, VIDEO_DIR)){
		return VIDEO_FOLDER;
	}else if(strstr(file, LOCK_DIR)){
		return SOS_FOLDER;
	}else if(strstr(file, WON_VIDEO_DIR)){
		return WONDERFUL_FOLDER;
	}else{
		return -1;
	}
}

int StorageManager::geneNetPicName(String8 file, char netbmp[], char* image_type)
{
	String8 filepath,filePathLeaf;
	char filename[32]={0};

	filepath = file.getPathDir();
	filePathLeaf = file.getPathLeaf();
	if(removeSuffix(filePathLeaf, filename) == NULL) {
		db_error("remove the file suffix failed, file is %s\n", filePathLeaf.string());
		return -1;
	}

	sprintf(netbmp, "%s/%s%s%s%s", filepath.string(),APP_THUMBNAIL_DIR,filename,EXT_NET_PIC,image_type);
	db_msg(" netbmp:%s", netbmp);
	return 0;
}

int StorageManager::geneNetPicName(String8 file, String8 &netFileName, char* image_type)
{
	char netbmp[128] = {0}, filename[32]={0};
	String8 filepath,filePathLeaf;

	filepath = file.getPathDir();
	filePathLeaf = file.getPathLeaf();
	if(removeSuffix(filePathLeaf, filename) == NULL) {
		db_error("remove the file suffix failed, file is %s\n", filePathLeaf.string());
		return -1;
	}

	sprintf(netbmp, "%s/%s%s%s%s", filepath.string(),APP_THUMBNAIL_DIR,filename,EXT_NET_PIC,image_type);
	db_msg(" netbmp:%s", netbmp);
	netFileName = String8(netbmp);
	return 0;
}

void StorageManager::DdInitnotify(int msg)
{
    if(mDbInitStat){
        mListener->notify(msg, 0);
        mDbInitStat = false;
    }
}

