#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/timeb.h>
#include <jpeglib.h>

#include "camera.h"
//#define JPEG_QUALITY 20

void rgb565_2_rgb24(unsigned char *rgb24, unsigned short *rgb565, int size)
{  
    int i = 0;
    unsigned char R, G, B;
    while(i<size) {
        R = *rgb565&0x1f;
        G = (*rgb565>>5)&0x3f;
        B = (*rgb565>>11)&0x1f;
        B = B<<3;
        G = G<<2;
        R = R<<3;
        *rgb24=R;rgb24++;
        *rgb24=G;rgb24++;
        *rgb24=B;rgb24++;
        rgb565++;
        i++;
    }
}

/*
 * source buffer's format must be rgb565 or rgb888.
 * 'jpeg_file' indicates the path to output file.
 * depth must be 16(rgb565) or 24(rgb8888)
 * the memory (1920x1080) consumption = 1920x1080x3x2 = 11M
 *            (480x272) consumption = 480x272x3x2 = 765KB
 */
int Rgb2Jpg(unsigned char *src_buffer, const char *jpeg_file, int width, int height, int depth)
{
    int Bpp = 3;
    JSAMPROW * row_pointer;
    long rgb_index = 0;
	int allocated = 0;
	FILE *outfile;
    struct jpeg_compress_struct cinfo;
    struct jpeg_error_mgr jerr;
	
    int sizeImage = width*height*Bpp;

	unsigned char *dst_buffer;
	if (depth == 16) {
		dst_buffer = (unsigned char*)malloc(sizeImage * sizeof(char));
    	rgb565_2_rgb24(dst_buffer, (unsigned short *)src_buffer, width*height);
		allocated = 1;
	} else {
		dst_buffer = src_buffer;
	}

    cinfo.err = jpeg_std_error(&jerr);
    //* Now we can initialize the JPEG compression object.
    jpeg_create_compress(&cinfo);
 
    if ((outfile = fopen(jpeg_file, "wb")) == NULL)
    {
    	if (allocated) {
			free(dst_buffer);
		}
        fprintf(stderr, "can't open %s\n", jpeg_file);
        return -1;
    }
    jpeg_stdio_dest(&cinfo, outfile);
 
    cinfo.image_width      = width;             //* image width and height, in pixels
    cinfo.image_height     = height;
    cinfo.input_components = Bpp;    //* # of color components per pixel
    cinfo.in_color_space   = JCS_RGB;     //* colorspace of input image
    jpeg_set_defaults(&cinfo);
    
     //Now you can set any non-default parameters you wish to.
     //Here we just illustrate the use of quality (quantization table) scaling:
 
    jpeg_set_quality(&cinfo, NET_JPEG_QUALITY, TRUE ); //* limit to baseline-JPEG values
    jpeg_start_compress(&cinfo, TRUE);
    
    int i = 0, j = 0;
    row_pointer = (JSAMPROW *)malloc(height*width*3);
//    char *line[300];
 
    for(i=0;i<height;i++)
    {    
        unsigned char * lineData = NULL;
        lineData = (unsigned char*)malloc(width*3);
        //line[i]=lineData;
        for(j=0;j<width;j++)
        {
            lineData[j*3+2] = dst_buffer[rgb_index];
            rgb_index ++;
            lineData[j*3+1] = dst_buffer[rgb_index];
            rgb_index ++;
            lineData[j*3+0] = dst_buffer[rgb_index];
            rgb_index ++;
        }
        row_pointer[i] = lineData;
//        row_pointer[height-i-1] = lineData;
    }
    jpeg_write_scanlines(&cinfo, row_pointer, height);
    jpeg_finish_compress(&cinfo);
    jpeg_destroy_compress(&cinfo);
//    for (i=0; i<height; i++)
//    {
//        free(line[i]);
//    }  
    free(row_pointer);
    free(dst_buffer); 
    fclose(outfile);
 
    return 0;
}
/*
int main(int argc, char* argv[])
{
    int w = atoi(argv[3]);
    int h = atoi(argv[4]);

	int fd = open(argv[1], O_RDWR);
	unsigned char *src_buffer = (unsigned char*)malloc(w*h*2);
	read(fd, src_buffer, w*h*2);
	
    Rgb2Jpg(src_buffer, argv[2], w, h, 16);
	
	free(src_buffer);
	close(fd);
    return 0;
}*/

