#include "MainWindow.h"
#ifdef LOG_TAG 
#undef LOG_TAG
#endif
#define LOG_TAG "MainWindow.cpp"
#include "debug.h"

#include "posixTimer.h"
#include <AWNetCmdData.h>
#include <AWNetInterface.h>

#ifdef YOUKU_PLATFORM
#include "YoukuRespCmds.h"
#include "YoukuCmdsCallback.h"
#include <youku/ykcamera_common.h>
#include <youku/ykcamera_error.h>
#endif

using namespace android;

//#define NOT_TO_SHUTDOWN
static timer_set timer_data[4];
int autoShutdownTime[3][3] = {{60, 55},{180, 175},{600, 595}};

static void closeConnectToPCDialog(void)
{
       BroadcastMessage(MSG_CLOSE_CONNECT2PC_DIALOG, 0, 0);
}

void ready2ShutDown(sigval_t sg)
{
#ifdef NOT_TO_SHUTDOWN
	return ;
#endif
	//db_msg("whether to shut down");
	timer_set *data = (timer_set*)sg.sival_ptr;
	int idx = data->idx;
	CdrMain* cdrMain = (CdrMain*)data->win;
	if (cdrMain->getWakeupState()&& cdrMain->getRecordPreview()->isRecording()) {
		db_msg("parking wakeup, not to shutdown");
		return ;
	}
	//db_msg("timer idx is %d", idx);
//	if (cdrMain->isBatteryOnline()) {
		if (idx == TIMER_NOWORK_IDX2) {
			db_info("TIMER_NOWORK_IDX2\n");
			if(cdrMain->isShuttingdown() == false) {
				CloseTipLabel();
				SendNotifyMessage(cdrMain->getHwnd(), MSG_SHOW_TIP_LABEL, LABEL_30S_NOWORK_SHUTDOWN, 0);

			}
			return;
		} else if (idx == TIMER_NOWORK_IDX){
			db_msg("timerid nowork");
			/*
			if (cdrMain->isRecording()) {
				db_msg("isRecording, no need to shutdown");
				goto out;
			}
			if (cdrMain->isPlaying()) {
				db_msg("isStarted, no need to shutdown");
				goto out;
			}
			*/
		}
//   	SendNotifyMessage(cdrMain->getHwnd(), MSG_SHOW_TIP_LABEL, LABEL_SHUTDOWN_NOW, 0);
		cdrMain->shutdown();
		db_msg("now shutdown");
		return;
//	}

out:
	if(idx == TIMER_NOWORK_IDX) {
		//db_msg("setOneShotTimer");
		setOneShotTimer(timer_data[TIMER_NOWORK_IDX2].time, 0, timer_data[TIMER_NOWORK_IDX2].id);
	}
}

void CdrMain::startShutdownTimer(int idx)
{
	int ret;
	db_msg("startShutdownTimer %d", idx);
	if (timer_data[idx].id == 0) {
		ret = createTimer((void*)&timer_data[idx], &timer_data[idx].id, ready2ShutDown);
		db_msg("createTimer, timerId is 0x%lx\n", (unsigned long)timer_data[idx].id);
	}
	if (timer_data[idx].id) {
		if (idx == TIMER_LOWPOWER_IDX || idx == TIMER_NOWORK_IDX2 || idx == TIMER_RECORD_SHUTDOWN_IDX) {
			db_msg("setOneShotTimer");
			setOneShotTimer(timer_data[idx].time, 0, timer_data[idx].id);
		} else if(idx == TIMER_NOWORK_IDX){
			db_msg("setPeriodTimer");
			setPeriodTimer(timer_data[idx].time, 0, timer_data[idx].id);
		}
		db_msg("#Warning: System will be shut down");
	}
}

void CdrMain::initTimerData()
{

	ResourceManager * rm = ResourceManager::getInstance();
	int data = rm->getResIntValue(ID_MENU_LIST_SHUTDOWN, INTVAL_SUBMENU_INDEX);
	if( data == 0 ) {
		timer_data[TIMER_NOWORK_IDX].time = autoShutdownTime[0][0];
		timer_data[TIMER_NOWORK_IDX2].time = autoShutdownTime[0][1];
		isAutoShutdownEnabled = true;
	} else if( data == 1 ) {
		timer_data[TIMER_NOWORK_IDX].time = autoShutdownTime[1][0];
		timer_data[TIMER_NOWORK_IDX2].time = autoShutdownTime[1][1];
		isAutoShutdownEnabled = true;
	} else if( data == 2 ) {
		timer_data[TIMER_NOWORK_IDX].time = autoShutdownTime[2][0];
		timer_data[TIMER_NOWORK_IDX2].time = autoShutdownTime[2][1];
		isAutoShutdownEnabled = true;
	} else {
		isAutoShutdownEnabled = false;
	}

	timer_data[TIMER_LOWPOWER_IDX].win = this;
	timer_data[TIMER_LOWPOWER_IDX].id = 0;
	timer_data[TIMER_LOWPOWER_IDX].time = LOWPOWER_SHUTDOWN_TIME;
	timer_data[TIMER_LOWPOWER_IDX].idx = TIMER_LOWPOWER_IDX;
	timer_data[TIMER_NOWORK_IDX].win = this;
	timer_data[TIMER_NOWORK_IDX].id = 0;
	//timer_data[TIMER_NOWORK_IDX].time = NOWORK_SHUTDOWN_TIME;
	timer_data[TIMER_NOWORK_IDX].idx = TIMER_NOWORK_IDX;
	timer_data[TIMER_NOWORK_IDX2].win = this;
	timer_data[TIMER_NOWORK_IDX2].id = 0;
	//timer_data[TIMER_NOWORK_IDX2].time = NOWORK_SHUTDOWN_TIME2;
	timer_data[TIMER_NOWORK_IDX2].idx = TIMER_NOWORK_IDX2;
	timer_data[TIMER_RECORD_SHUTDOWN_IDX].win = this;
	timer_data[TIMER_RECORD_SHUTDOWN_IDX].id = 0;
	timer_data[TIMER_RECORD_SHUTDOWN_IDX].time = RECORD_PLUGOUTAC_SHUTDOWNTIME;
	timer_data[TIMER_RECORD_SHUTDOWN_IDX].idx = TIMER_RECORD_SHUTDOWN_IDX;	
}

void CdrMain::modifyNoworkTimerInterval(time_t  time, time_t  time2)
{
	timer_data[TIMER_NOWORK_IDX].time = time;
	timer_data[TIMER_NOWORK_IDX2].time = time2;
//	if (isBatteryOnline())
		noWork(true);
}

void CdrMain::noWork(bool idle)
{
	db_msg("****nowork***,idle:%d,shutdown:%d",idle,isAutoShutdownEnabled);
	if (idle && isAutoShutdownEnabled) {
		if(mShutdowning == true) {
			db_msg("shutting down, not set timer\n");
			return;
		}
		db_msg("no work, resume timer");
		startShutdownTimer(TIMER_NOWORK_IDX);
		startShutdownTimer(TIMER_NOWORK_IDX2);
	} else if(idle == false){
		db_msg("working, stop timer");
//		cancelShutdownTimer(TIMER_NOWORK_IDX);
		stopShutdownTimer(TIMER_NOWORK_IDX);
		stopShutdownTimer(TIMER_NOWORK_IDX2);
	//	CloseTipLabel();
	}
}

void CdrMain::cancelShutdownTimer(int idx)
{
	db_msg("cancelShutdownTimer");
	if (timer_data[idx].id != 0) {
		db_msg("deleteTimer");
		deleteTimer(timer_data[idx].id);
		timer_data[idx].id = 0;
	}
}

void CdrMain::stopShutdownTimer(int idx)
{
	db_msg("stopShutdownTimer");
	if (timer_data[idx].id != 0) {
		db_msg("stopShutdownTimer");
		stopTimer(timer_data[idx].id);
	}
}

bool CdrMain::isRecording()
{
	return mRecordPreview->isRecording();
}

bool CdrMain::isPlaying()
{
	if (mPlayBack) {
		return mPlayBack->isStarted();
	}
	return false;
}

#ifdef YOUKU_PLATFORM

YoukuCmdsCallback *CdrMain::mYoukuCmdsCallback;

int CdrMain::ykCamCmdsHandler(int cmd, handler_parameter *param, void *data)
{
	int ret = -1;

	// db_msg("yk handler, cmd:%d\n", cmd);
	ResourceManager *rm = ResourceManager::getInstance();
	YoukuDeviceControl &devCtrl = mYoukuCmdsCallback->GetYoukuDevCtrl();

	if (cmd == CAM_REQ_CAMERA_STATUS) {
		ret = rm->getConfig(mYoukuCmdsCallback->GetCdrMain());
	} else if (s_commands[cmd].respCallback != NULL) {
		if (cmd == CAM_REQ_COMM_RESET_FACTORY
			|| cmd == CAM_REQ_COMM_FORMAT_SD
			|| cmd == CAM_REQ_COMM_POWEROFF
			|| cmd == CAM_REQ_COMM_STORAGE_DELETE_ALL
			|| cmd == CAM_REQ_COMM_STORAGE_DELETE_LAST
			|| cmd == CAM_REQ_COMM_CLEAN_NEW_VIDEO_FLAG
		) {
			devCtrl.ctrl_item = 1;
			devCtrl.ctrl_data = NULL;
		} else {
			if (param == NULL) {
				db_error("param is NULL\n");
				return -1;
			}
			devCtrl.ctrl_cmd = cmd;
			devCtrl.ctrl_item = 0;
			// db_msg("data addr:%p\n", &devCtrl);
			devCtrl.ctrl_data = param;
			devCtrl.called = CdrMain::mYoukuCmdsCallback;

			if (cmd == CAM_REQ_COMM_STREAM_ENABLE || cmd == CAM_REQ_COMM_DATETIME_SYNC) {
				devCtrl.ctrl_data = param->scall_back_param;
			} else if (cmd == CAM_REQ_COMM_DEVINFO_GET) {
				devCtrl.ctrl_data = param->scall_back_return;
			} else {
				devCtrl.ctrl_item = *(int*)param->scall_back_param;
			}
		}
		ret = (mYoukuCmdsCallback->*s_commands[cmd].respCallback)(devCtrl.ctrl_item, devCtrl.ctrl_data);
	} else {
		ret = YOUKU_CMD_NOT_SUPPORTED;
	}

	return ret;
}

YoukuService *CdrMain::getYoukuService(){
	if (mYoukuService == NULL) {
		mYoukuService = new YoukuService();
	}
	mYoukuCmdsCallback = new YoukuCmdsCallback(this);
	return mYoukuService;
}

void CdrMain::initYoukuService()
{
	YoukuService *ys = this->getYoukuService();
	ResourceManager *rm = ResourceManager::getInstance();

	ys->setRespCallback(CdrMain::ykCamCmdsHandler);
	rm->getConfig(this);
	ys->setCamStatus(device_status, setting_status);
	if (ys->getServiceStatus() == UNINIT) {
		ys->initCamera();
		ys->startYoukuThread();
	}
}
#endif

/************* MainWindow Proc ***************/
static int CDRWinProc(HWND hWnd, int message, WPARAM wParam, LPARAM lParam)
{
	CdrMain* mCdrMain = (CdrMain*)GetWindowAdditionalData(hWnd);

	db_msg("message is %s\n", Message2Str(message));
	switch(message) {
	case MSG_FONTCHANGED:
		mCdrMain->updateWindowFont();
		break;
	case MSG_CREATE:
		break;
		/* + + + keyEvent + + + */
	case MSG_PAINT: 
		if(mCdrMain->getIsLowPower()){
			HDC	hdc=BeginPaint(hWnd);
			BITMAP bitImage;
			char *filepath=(char *)"/system/res/others/low_power_notice.png";
			LoadBitmapFromFile(HDC_SCREEN, &bitImage, filepath);
			int screen_w,screen_h;
			getScreenInfo(&screen_w, &screen_h);
			FillBoxWithBitmap(hdc,0,0,screen_w,screen_h,&bitImage);
			EndPaint(hWnd,hdc); 
		}
		break;
	case MSG_KEYDOWN:
		if(mCdrMain->isKeyUp == true) {
			mCdrMain->downKey = wParam;	
			SetTimer(hWnd, ID_TIMER_KEY, LONG_PRESS_TIME);
			mCdrMain->isKeyUp = false;
		}
		break;
	case MSG_KEYUP:
		mCdrMain->isKeyUp = true;
		if(wParam == mCdrMain->downKey)	{
			KillTimer(hWnd, ID_TIMER_KEY);
			db_msg("short press\n");
			mCdrMain->keyProc(wParam, SHORT_PRESS);
		}
		break;
	case MSG_KEYLONGPRESS:
		mCdrMain->downKey = -1;
		db_msg("long press\n");
		mCdrMain->keyProc(wParam, LONG_PRESS);
		break;
	case MSG_TIMER:
		if(wParam == ID_TIMER_KEY) {
			mCdrMain->isKeyUp = true;			
			SendMessage(hWnd, MSG_KEYLONGPRESS, mCdrMain->downKey, 0);
			KillTimer(hWnd, ID_TIMER_KEY);
		}
		break;
		/* - - - keyEvent - - - */
	case MWM_CHANGE_WINDOW:
		mCdrMain->changeWindow(wParam, lParam);
		break;
	case MSG_CLOSE:
		DestroyMainWindow (hWnd);
		PostQuitMessage (hWnd);
		break;
	case MSG_SHOW_TIP_LABEL:
		ShowTipLabel(hWnd, (enum labelIndex)wParam);
		break;
#ifdef APP_WIFI
	case MSG_WIFI_SWITCH:
        if ((bool)wParam)
            mCdrMain->getSoftApController()->EnableSoftAp();
        else
            mCdrMain->getSoftApController()->DisableSoftAp();
		break;
	case MSG_RESET_WIFI_PWD:
		{
            mCdrMain->getSoftApController()->ResetSoftAp(NULL, DEFAULT_WIFI_PASSWORD);
		}
		break;
#endif
#ifdef YOUKU_PLATFORM
	case MSG_YOUKU_SERVICE_SWITCH: {
			if ((bool)wParam) {
				mCdrMain->initYoukuService();
			} else {
				YoukuService *ys = mCdrMain->getYoukuService();
				if (ys->getServiceStatus() == STARTED) {
					ys->stopYoukuThread();
					ys->deInitCamera();
					sleep(2);
				}
			}
		}
		break;
	case MSG_SMALL_VIDEO_DONE: {
			Mutex::Autolock _l(gNewSpecialVideoMutex);
			setting_status[SETTING_COMM_CLEAN_NEW_VIDEO_FLAG].ivalue = 1;
		}
		break;
#endif
	case MSG_OTA_UPDATE:
   		mCdrMain->cdrOtaUpdate();
		break;
	case MSG_KEEP_ADAS_SCREEN_FLAG:
		mCdrMain->keepAdasScreenFlag();
		break;
	case MSG_RECOVER_ADAS_SCREEN_FLAG:
		mCdrMain->recoverAdasScreenFlag();
		break;
	case MSG_DESTROY:
		break;
	default:
		mCdrMain->msgCallback(message, wParam);
	}
	return DefaultWindowProc(hWnd, message, wParam, lParam);
}

static bool inline isKeyEvent(int msg)
{
	if (msg != MSG_KEYDOWN && msg != MSG_KEYUP && msg != MSG_KEYLONGPRESS) {
		return false;	
	}
	return true;
}

int MsgHook(void* context, HWND dst_wnd, int msg, WPARAM wparam, LPARAM lparam)
{
	int ret = HOOK_GOON;
	
	CdrMain *cdrMain = (CdrMain*)context;
	HWND hMainWnd = cdrMain->getHwnd();

    if(cdrMain->isWifiConnecting()){
        SendMessage(hMainWnd, msg, wparam, lparam);
    }

	if(cdrMain->mShutdowning == true || !cdrMain->mFinishConfig) {
		db_msg("shutdowning");
		return !HOOK_GOON;
	}
	if (isKeyEvent(msg)) {
		if(cdrMain->getNoUsbCharge()){
			if(wparam != CDR_KEY_POWER && msg == MSG_KEYDOWN){
				cdrMain->playSounds(CDR_KEY_SOUNDS);
				return 0;
			}
		}else{
			if( !cdrMain->isRecording() && !cdrMain->isPlaying()){
				cdrMain->noWork(true);
			}
		}
		if(msg == MSG_KEYDOWN && (wparam == CDR_KEY_POWER || cdrMain->isScreenOn())){
			if (cdrMain->isPhotoWindow() && wparam == CDR_KEY_OK) {
			//	cdrMain->playSounds(CDR_TAKEPICTURE_SOUNDS);
			} else {
				cdrMain->playSounds(CDR_KEY_SOUNDS);
				playKeySound();
			}
			//playKeySound();
		}
		if (wparam == CDR_KEY_POWER) {  //power key, post to main proc
			SendMessage(hMainWnd, msg, wparam,lparam);
			return !HOOK_GOON; //need not other work
		#ifdef ALLKEY_AWAKE_SCREEN
		} else if (!cdrMain->isScreenOn()) {
			cdrMain->playSounds(CDR_KEY_SOUNDS);
			cdrMain->keyProc(CDR_KEY_POWER, false);
			return !HOOK_GOON;
		#endif
		} else {
			if (cdrMain->isWifiConnecting() && !cdrMain->getIsShowUpdateTips()) {
				return !HOOK_GOON;
			}
			if(cdrMain->isScreenOn()){
				cdrMain->setOffTime();
			}
		}
	}
	/* v3 todo :temporarily cancle */
	if (!cdrMain->isScreenOn()) { //screen off
		return !HOOK_GOON; //nothing to do
	}
	return ret;
}

CdrMain::CdrMain()
	: mShutdowning(false)
	, mSTB(NULL)
	, mRecordPreview(NULL)
	, mPlayBackPreview(NULL)
	, mPlayBack(NULL)
	, mMenu(NULL)
	, mPM(NULL)
	, mEM(NULL)
	, mEnableRecord(FORBID_NORMAL)
	, curWindowState()
	, downKey(0)
	, isKeyUp(true)
	, isLongPress(false)
	, usbStorageConnected(false)
	, isWakenUpOnParking(false)
	, mTime(TIME_INFINITE)
	, mOtherWindowInited(false)
	, mSessionConnect(false)
	, mTinyParkAudioTh(NULL)
	, mGSensorVal(1)
	, mFinishConfig(false)
	, mFormatTFAdasS(false)
	, mNeedCheckSD(true)
	, m2pcConnected(false)
	, mPCCam(false)
#ifdef APP_WIFI
    , mSoftApController(NULL)
#endif
{
	sp<ProcessState> proc(ProcessState::self());
	ProcessState::self()->startThreadPool();

	struct timespec measureTime1;
	clock_gettime(CLOCK_MONOTONIC, &measureTime1);
//	db_msg("Test Time: start cdr, time is %ld secs, %ld nsecs\n", measureTime1.tv_sec, measureTime1.tv_nsec);
	
	mLowPower = EventManager::getBattery();
	
	if (mLowPower < 0) {
		mLowPower = LOW_POWER+1;
		sys_log("===no need to check battery capacity ===");
	} else {
		sys_log("===power capacity: %d%%/%d%%===", mLowPower, 100);
	}
	EventManager::int2set(false);
	int batteryStatus = EventManager::getBatteryStatus();
	if(mLowPower > LOW_POWER || (batteryStatus == BATTERY_CHARGING)){
		mRecordPreview =  new RecordPreview(this);
	}
	mOtherDialogOpened = false;
	mOriginAdasScreen = false;
	mOTAAdasScreen = false;
	mNoUSBCharge = false;
	mUSBcharge = false;
	mShowUpdateTips = false;
	mSessionSubMenu = false;
	mCarBack = false;
    mKeyShutDown = false;
	mShutdownTimer = false;
}



CdrMain::~CdrMain()
{
#ifdef APP_WIFI
	if (mCdrServer) {
		delete mCdrServer;
		mCdrServer = NULL;
	}
#endif
	if(mRecordPreview) {
		delete mRecordPreview;
		mRecordPreview = NULL;
	}
	if (mEM) {
		delete mEM;
		mEM = NULL;
	}
	if (mPM) {
		delete mPM;
		mPM = NULL;
	}
	if(mSTB) {
		delete mSTB;
		mSTB = NULL;
	}
	if(mPlayBackPreview) {
		delete mPlayBackPreview;
		mPlayBackPreview = NULL;
	}
	if(mPlayBack) {
		delete mPlayBack;
		mPlayBack= NULL;
	}
	if(mMenu) {
		delete mMenu;
		mMenu= NULL;
	}
	for (int i=0; i<2; i++) {
		if (timer_data[i].id  != 0) {
			stopTimer(i);
			deleteTimer(i);
			timer_data[i].id = 0;
		}
	}
	UnRegisterCDRWindows();
	UnRegisterCDRControls();
	ExitGUISafely(0);
	MainWindowThreadCleanup(mHwnd);
	if(mTinyParkAudioTh)
		delete mTinyParkAudioTh;
}

bool CdrMain::isPhotoWindow()
{
	return !mOtherDialogOpened && getCurWindowID() == WINDOWID_RECORDPREVIEW  && mRecordPreview->isPhotoWindow();
}

bool CdrMain::isBatteryOnline()
{
	if (mEM) {
		return mEM->isBatteryOnline();
	}
	return false;
}
void CdrMain::initPreview(void* none)
{
	if(mLowPower<=LOW_POWER)
		return ;
	initCdrTime();
	status_t err;	
	// Change by Harry,not create a thread to init the preview
	/*
	mInitPT = new InitPreviewThread(this);
	err = mInitPT->start();
	if (err != OK) {
		mInitPT.clear();
	}
	*/
	initPreview();
}

int CdrMain::initPreview(void)
{
	struct timespec measureTime1;
	db_msg("initPreview\n");
//	clock_gettime(CLOCK_MONOTONIC, &measureTime1);
//	db_msg("Test Time: initPreview , time is %ld secs, %ld nsecs\n", measureTime1.tv_sec, measureTime1.tv_nsec);

	mRecordPreview->init();
//	clock_gettime(CLOCK_MONOTONIC, &measureTime1);
//	db_msg("Test Time: after init time is %ld secs, %ld nsecs\n", measureTime1.tv_sec, measureTime1.tv_nsec);

	mPOR.signal();

	mRecordPreview->startPreview();
	db_msg("startPreview finished\n");
//	clock_gettime(CLOCK_MONOTONIC, &measureTime1);
//	db_msg("Test Time: after startPreview time is %ld secs, %ld nsecs\n", measureTime1.tv_sec, measureTime1.tv_nsec);

	return 0;
}

void CdrMain::initStage2(void* none)
{
	status_t err;

	mInitStage2T = new InitStage2Thread(this);
	err = mInitStage2T->start();
	if (err != OK) {
		db_msg("initStage2 start failed\n");
		mInitPT.clear();
	}
}

#ifdef APP_WIFI
int CdrMain::networkInit()
{
    mSoftApController = new SoftApController();
	ResourceManager *rm = ResourceManager::getInstance();
	bool flag = rm->getResBoolValue(ID_MENU_LIST_WIFI);
	if(flag)
	{
        mSoftApController->EnableSoftAp();
		HWND hwnd=getWindowHandle(WINDOWID_STATUSBAR);
		SendMessage(hwnd, STBM_WIFI, (WPARAM)flag, 0);
	}
	return 0;
}

SoftApController *CdrMain::getSoftApController()
{
    if (mSoftApController == NULL) {
        mSoftApController = new SoftApController();
    }
    return mSoftApController;
}
#endif

int CdrMain::initStage2(void)
{
	StorageManager* sm;
	ResourceManager* rm;
	int result;

	/* wait for the preview is started */
	mLock.lock();
	result  = mPOR.waitRelative(mLock, seconds(1));
	mLock.unlock();

	rm = ResourceManager::getInstance();
	if(rm->initStage2() < 0) {
		db_error("ResourceManager initStage2 failed\n");
		return -1;
	}

	db_msg("%s:%d\n",__func__,__LINE__);
	if(createOtherWindows() < 0) {
		return -1;
	}
	bool hasTVD;
	bool tvdOnline;
	db_msg("%s:%d\n",__func__,__LINE__);
	mPM = new PowerManager(this);
	tvdOnline = mRecordPreview->hasTVD(hasTVD);
	mEM = new EventManager(tvdOnline, hasTVD);
	mEM->setEventListener(this);
	mEM->setCdrCamerCSI(mRecordPreview->getCdrCamera(CAM_CSI));
	mEM->init(hasTVD);
	db_msg("%s:%d\n",__func__,__LINE__);
#ifdef USE_RF_MODULE
	mEM->rf_module_init();
	db_msg("****xxxthis is start 2.4g module!*****");
#endif
	db_msg("%s:%d\n",__func__,__LINE__);
	if(!mEM->isBatteryOnline()) {
		db_msg("battery is charging\n");
		SendMessage(mSTB->getHwnd(), MSG_BATTERY_CHARGING, 0, 0);
	}
	initTimerData();
//	startShutdownTimer(TIMER_NOWORK_IDX);
	if (mEM->isBatteryOnline()) {
		db_msg("AC PLUG OUT");
		int ret;
		ret = mPM->getBatteryLevel();
		if (ret <= BATTERY_LOW_LEVEL) {	
			db_error("LOW LEVEL!!!!");
			SendNotifyMessage(mHwnd, MSG_SHOW_TIP_LABEL, LABEL_LOW_POWER_SHUTDOWN, 0);
			sleep(2);
			shutdown();
		}
		else{
			if(!mRecordPreview->isRecording())
				noWork(true);
			mFiACConnect = false;
		}
	} else {
		db_msg("AC PLUG IN");
		noWork(false);
		mFiACConnect = true;
	}
	sm = StorageManager::getInstance();
	sm->setEventListener(this);
	if(sm->isMount() == true) {
		db_msg("mounted\n");
		SendMessage(mSTB->getHwnd(), STBM_MOUNT_TFCARD, 1, 0);
	} else {
		db_msg("not mount\n");
		SendMessage(mSTB->getHwnd(), STBM_MOUNT_TFCARD, 0, 0);
	}
	db_msg("%s:%d\n",__func__,__LINE__);
	bool isDrawAlign = rm->getResBoolValue(ID_MENU_LIST_ALIGNLINE);
	db_msg("%s:%d\n",__func__,__LINE__);
	SendMessage(getWindowHandle(WINDOWID_RECORDPREVIEW), MSG_DRAW_ALIGN_LINE, isDrawAlign, 0);
	db_msg("%s:%d\n",__func__,__LINE__);
	// Del by Harry
	//bool isParkOn = rm->getResBoolValue(ID_MENU_LIST_PARK);
	//SendMessage(getWindowHandle(WINDOWID_STATUSBAR), STBM_PARK, isParkOn, 0);
	//bool isVoiceOn = rm->getResBoolValue(ID_MENU_LIST_SILENTMODE);
	//SendMessage(getWindowHandle(WINDOWID_STATUSBAR), STBM_VOICE, isVoiceOn, 0);
	// End del by Harry
	if (!mRecordPreview->camExisted(CAM_UVC)) {
		mEM->checkDevMore(CAM_UVC);
	}
	db_msg("%s:%d\n",__func__,__LINE__);
	// Del by Harry
	// int impactlevel = rm->getResIntValue(ID_MENU_LIST_IMPACTLEVEL, INTVAL_SUBMENU_INDEX);
	// mEM->setImpactLevel(impactlevel);
	// Del by Harry
	db_msg("%s:%d\n",__func__,__LINE__);
	configInit();
	db_msg("%s:%d\n",__func__,__LINE__);
#ifdef APP_WIFI
	networkInit();
	sessionServerInit();
#endif
//	mPM->connect2StandbyService();
	// Del by Harry
//	int valAdas = rm->getResIntValue(ID_MENU_LIST_SMARTALGORITHM, INTVAL_SUBMENU_INDEX);
//	if(valAdas == 0 || valAdas == 3){
//		mEM->setAdasIsOpen(false);
//	}else if(valAdas == 1 || valAdas == 2){
//		mEM->setAdasIsOpen(true);
//	}
	//mEM->setAdasIsOpen(rm->getResBoolValue(ID_MENU_LIST_SMARTALGORITHM));
	// End del by Harry
	mFinishConfig = true;

	/*
	if(!(rm->getResBoolValue(ID_MENU_LIST_SMARTALGORITHM))&&!getFormatTFAdasS()){
		mFormatTFAdasS = true;
		if(sm->isInsert()){
			if( !checkSDFs() && !m2pcConnected){
				mNeedCheckSD = true;
				SendNotifyMessage(mMenu->getHwnd(), MSG_FORMAT_TFDIALOG, (WPARAM)0, 0);
			}
		}
	}*/
	
	if(access(UPDATE_FILE_PATH,F_OK) == 0){
		SendMessage(mHwnd, MSG_OTA_UPDATE, 0, 0);
	}
#if 0
	db_msg("  *****m2pcConnected:%d,mNoUSBCharge:%d******* ",m2pcConnected,mNoUSBCharge);
	if(!m2pcConnected ){
		char showadasfunction[4] = {0};
		cfgDataGetString("persist.app.showadasfunction", showadasfunction, "1");
		usleep(2000 * 1000);
		if(strcmp(showadasfunction,"1")==0){
			if(mRecordPreview->getIsAdasScreen()){
				SendMessage(getWindowHandle(WINDOWID_RECORDPREVIEW),MSG_ADAS_SCREEN,0,0);
				m2pcAdasScreen = true;
			}
			SendMessage(mHwnd, MSG_SHOW_ADAS_FUNCTION, 0, 0);
			cfgDataSetString("persist.app.showadasfunction", "0");
			if(m2pcAdasScreen){
				SendMessage(getWindowHandle(WINDOWID_RECORDPREVIEW),MSG_ADAS_SCREEN,1,0);
				m2pcAdasScreen = false;
			}
		}
	}
 #endif
#ifdef USE_RF_MODULE
	// init the thread to start detect the key occur
	mEM->initUinputEvent();
	db_msg("****xxxthis is start detect the key occur!*****");
#endif
#if defined(VIDEO_CONFIG_TEXT_TRACK) && defined(VIDEO_CONFIG_TEXT_GSR)
    mEM->setRecordPreview(mRecordPreview);
#endif
	return 0;
}

void CdrMain::cdrOtaUpdate()
{

	db_msg("****Update*****");
	if(mSessionConnect){
		mShowUpdateTips = true;
		CloseTipLabel();
	}
	int retval;
	const char* ptr;
	MessageBox_t messageBoxData;
	ResourceManager* rm;
	String8 textInfo;

	rm = ResourceManager::getInstance();
	memset(&messageBoxData, 0, sizeof(messageBoxData));
	messageBoxData.dwStyle = MB_OKCANCEL | MB_HAVE_TITLE | MB_HAVE_TEXT;
	messageBoxData.flag_end_key = 0; /* end the dialog when endkey keydown */

	ptr = rm->getLabel(LANG_LABEL_TIPS);
	if(ptr == NULL) {
		db_error("get deletefile title failed\n");
		return;
	}
	messageBoxData.title = ptr;

	ptr = rm->getLabel(LANG_LABEL_OTA_UPDATE);
	if(ptr == NULL) {
		db_error("get deletefile text failed\n");
		return;
	}
	messageBoxData.text = ptr;

	ptr = rm->getLabel(LANG_LABEL_SUBMENU_OK);
	if(ptr == NULL) {
		db_error("get ok string failed\n");
		return;
	}
	messageBoxData.buttonStr[0] = ptr;

	ptr = rm->getLabel(LANG_LABEL_SUBMENU_CANCEL);
	if(ptr == NULL) {
		db_error("get ok string failed\n");
		return;
	}
	messageBoxData.buttonStr[1] = ptr;

	messageBoxData.pLogFont = rm->getLogFont();

	rm->getResRect(ID_MENU_LIST_MB, messageBoxData.rect);
	messageBoxData.fgcWidget = rm->getResColor(ID_MENU_LIST_MB, COLOR_FGC);
	messageBoxData.bgcWidget = rm->getResColor(ID_MENU_LIST_MB, COLOR_BGC);
	messageBoxData.linecTitle = rm->getResColor(ID_MENU_LIST_MB, COLOR_LINEC_TITLE);
	messageBoxData.linecItem = rm->getResColor(ID_MENU_LIST_MB, COLOR_LINEC_ITEM);
	messageBoxData.confirmCallback = NULL;
	messageBoxData.confirmData = (void*)this;

	db_msg("this is %p\n", this);
	db_msg("***mHwdn:%x, curHWND:%x**ID:%d*",mHwnd,getWindowHandle(getCurWindowID()),getCurWindowID());

	if(m2pcConnected){
		return ;
	}
	//mOTAAdasScreen =  rm->getResBoolValue(ID_MENU_LIST_SMARTALGORITHM);
	int valAdas = rm->getResIntValue(ID_MENU_LIST_SMARTALGORITHM, INTVAL_SUBMENU_INDEX);
	if(valAdas == 0 || valAdas == 3){
		mOTAAdasScreen = false;
	}else if(valAdas == 1 || valAdas == 2){
		mOTAAdasScreen = true;
	}
	mRecordPreview->adasScreenSwitch(false);
	
	retval = showMessageBox(getWindowHandle(getCurWindowID()), &messageBoxData);
	db_msg("retval is %d\n", retval);
	if (retval == IDC_BUTTON_OK){
		db_msg("Update ***************");
		otaUpdate();
	}else{
		if(mOTAAdasScreen){
			mOTAAdasScreen = false;
			mRecordPreview->adasScreenSwitch(true);	
		}
		if(mSessionConnect){
			mShowUpdateTips = false;
			SendNotifyMessage(mHwnd, MSG_SHOW_TIP_LABEL, LABEL_WIFI_CONNECT, 0);
		}
	}
}

bool CdrMain::getPC2Connected()
{
	return m2pcConnected;
}

bool CdrMain::getFormatTFAdasS(){
	return mFormatTFAdasS;
}

void CdrMain::setFormatTFAdasS(bool flag)
{
	mFormatTFAdasS = flag;
}

bool CdrMain::getNeedCheckSD()
{
	return mNeedCheckSD;
}

void CdrMain::setNeedCheckSD(bool flag)
{
	mNeedCheckSD = flag;
}

bool CdrMain::getOTAAdasScreen()
{
	return mOTAAdasScreen;
}

bool CdrMain::getUVCCarBackAdasScreen()
{
	return mUVCCarBackAdasScreen;
}

static bool vaild2read()
{
    char *tmp_file = MOUNT_PATH"video/~tmp";
    int fd = open(tmp_file, O_WRONLY|O_CREAT, 666);
    if (fd < 0) {
           return false;
    }
    const char * str = "read the tmp file.";
    int bytes = strlen(str);
    int num = write(fd, str, bytes);
    fsync(fd);
    close(fd);
    if (num != bytes) {
           remove(tmp_file);
           return false;
    }
    fd = open(tmp_file, O_RDONLY|O_CREAT, 666);
    if (fd < 0) {
           remove(tmp_file);
           return false;
    }
    char buffer[32]={0};
    num = read(fd, buffer, 128);
    if (num < 0 || 0 != strcmp(str, buffer)) {
           close(fd);
           remove(tmp_file);
           return false;
    }
    close(fd);
    remove(tmp_file);
    return true;
}

unsigned int getClustSize(char *devname)
{
    int fd;
    unsigned char block[DOSBOOTBLOCKSIZE];
    unsigned int BytesPerSec = 0;
    unsigned int SecPerClust = 0;
    unsigned int BytesPerClust = 0;
	
    fd = open(devname, O_RDONLY, 0);
    if (read(fd, block, sizeof block) < (int)sizeof block) {
        db_msg("Could not read boot block");
        return 0;
    }
	close(fd);
	
    if (block[510] != 0x55 || block[511] != 0xaa) {
        db_msg("Invalid signature in boot block");
        return 0;
    }

    if(!memcmp(&block[3], "EXFAT   ", 8)){
        db_msg("exFAT filesystem is not supported");
        return 0;
    }

    if(!memcmp(&block[3], "NTFS", 4)){
        db_msg("NTFS filesystem is not supported");
        return 0;
    }
	
    if(!memcmp(&block[82], "FAT32   ", 8))
        db_msg("FAT32 filesystem is supported\n");

    BytesPerSec = block[11] + (block[12] << 8);
    if (BytesPerSec != 512 && BytesPerSec != 1024 && BytesPerSec != 2048 && BytesPerSec != 4096) {
        db_msg("Invalid sector in boot block");
		return 0;
    } 

    SecPerClust = block[13];
    if ((SecPerClust&(SecPerClust-1)) != 0) {
        db_msg("Invalid cluster in boot block");
        return 0;
    } 

    BytesPerClust = SecPerClust*BytesPerSec;
	db_msg("BytesPerClust %u\n",BytesPerClust);
	
    return BytesPerClust;
}

bool CdrMain::checkSDFs(){
	char device[256];
	char *path = (char*)STORAGE_POINT;
	char mount_path[256];
	char rest[256];
	FILE *fp;
	int reval=0;
	char line[1024];

	if (!(fp = fopen("/proc/mounts", "r"))) {
		db_msg("Error opening /proc/mounts\n");
		return false;
	}

	while(fgets(line, sizeof(line), fp)) {
		line[strlen(line)-1] = '\0';
		sscanf(line, "%255s %255s %255s\n", device, mount_path, rest);
		if (!strcmp(mount_path, path)) {
			fclose(fp);
			goto checkfat;
		}
	}

	fclose(fp);
	db_msg("Volume is not mounted\n");
	return false;
//	sscanf(&argv[1][0],"%s",path);	 //Ӳ2ȡ·
checkfat:
	reval = getClustSize(device);
	db_msg("getClustSize:reval = %d\n",reval);
	// Change reval==65536 to reval==0 by Harry
	if(reval<=0){
		// Del not format by Harry
        //if (!vaild2read())
        //      SendMessage(mHwnd, MSG_SHOW_TIP_LABEL, LABEL_INVAILD_TFCATD, 3000);
        return false;
	}else
		return true;

}


void CdrMain::initCdrTime(){
   time_t timer;
   struct tm *tblock;
   timer = time(NULL);
   tblock = localtime(&timer);
   if(tblock->tm_year + 1900 >=2015)
   {
       return ;
   }else{
       tblock->tm_year=2015-1900;
       tblock->tm_mon=0;
       tblock->tm_mday=1;
       tblock->tm_hour=0;
       tblock->tm_min=0;
       tblock->tm_sec=0;
       if(setDateTime(tblock)!=0){
          db_msg("***init cdr time failed***");
       }
   }       
}

int CdrMain::sessionServerInit()
{
	int ret = 0;
#ifndef YOUKU_PLATFORM
	mCdrServer = new CdrServer(this);
	ret = mCdrServer->start();
#else
	ResourceManager *rm = ResourceManager::getInstance();
	bool flag = rm->getResBoolValue(ID_MENU_LIST_WIFI);
	if(flag) {
		this->initYoukuService();
	}
#endif
	return ret;
}

void CdrMain::sessionChange(bool connect)
{
	if (connect) {
		db_msg("----------------connect!");
		int curSessionCam = mRecordPreview->getCurSessionCam();
#ifdef YOUKU_PLATFORM
		mRecordPreview->sessionInit(mYoukuService->getStreamSender(-1), SESSION2LOCAL(curSessionCam));
#else
		mRecordPreview->sessionInit(mCdrServer->getSession(), SESSION2LOCAL(curSessionCam));
#endif
		mRecordPreview->startRecord(curSessionCam);
	} else {
		mRecordPreview->setRtspSession(false);
		mRecordPreview->stopRecord(CAM_CSI_NET);
		mRecordPreview->stopRecord(CAM_UVC_NET);
#ifdef PLATFORM_0
		mRecordPreview->stopRecord(CAM_UVC_NET);
#endif
	}
}

void CdrMain::checkShutdown()
{
   db_msg("mShutdowning %d", mShutdowning);
   if (mShutdowning) {
   		showShutdownLogo();
   }
}

void CdrMain::playSounds(CDR_SOUNDS sounds)
{
	char sound_path[64] = {0};
	switch(sounds){
		case CDR_KEY_SOUNDS:
			sprintf(sound_path, "/system/res/others/key.wav");
			break;
		case CDR_SHUTDOWN_SOUNDS:
			sprintf(sound_path, "/system/res/others/shutdown.wav");
			break;
		case CDR_TAKEPICTURE_SOUNDS:
			sprintf(sound_path, "/system/res/others/shutter.wav");
			break;
		default:
			return;
	}
	initKeySound(sound_path);
	playKeySound();
}

void CdrMain::shutdown()
{
	db_msg("shutdown");
	cdrReplyToApp(NAT_CMD_CDR_SHUTDOWN, NULL, 0);
	noWork(false);
	stopShutdownTimer(TIMER_RECORD_SHUTDOWN_IDX);
	playSounds(CDR_SHUTDOWN_SOUNDS);
	SendMessage(getWindowHandle(WINDOWID_RECORDPREVIEW),MSG_ADAS_SCREEN,0,0);
    mRecordPreview->stopRecordSync();
	mShutdowning = true;
	if (!mMenu->getSubMenuFlag()) { /* waiting for the submenu is closed*/
			showShutdownLogo();
	}

	ResourceManager* rm;
	rm = ResourceManager::getInstance();
	mEM->setGsensorSensitivity(mGSensorVal);
	EventManager::int2set(rm->getResBoolValue(ID_MENU_LIST_PARK));
	if(rm->getResBoolValue(ID_MENU_LIST_PARK))
		cfgDataSetString("persist.app.park.config", "1");
	else
		cfgDataSetString("persist.app.park.config", "0");
	rm->syncConfigureToDisk();
	mRecordPreview->stopPreview();
	mPlayBack->stopPlayback(false);
	mPM->powerOff();
}
//#define IGNORE_UVC_STATE
int CdrMain::notify(int message, int val)
{
	int ret = 0;
	StorageManager *smt = StorageManager::getInstance();
	if(mShutdowning == true) {
		db_info("shutting down, ignore events\n");	
		return 0;
	}

	switch(message) {

	case RF_SHORTPRESS_EVENT:
			{
				db_msg("xxxthis is just a test [RF_SHORTPRESS_EVENT]!!!");
				mRecordPreview->startSmallVideoRecord();
			}
		break;
	case RF_LONGPRESS_EVENT:
			{
				db_msg("xxxthis is just a test [RF_LONGPRESS_EVENT]!!!");
				mRecordPreview->startSmallVideoRecord();
			}
		break;
	case EVENT_UVC_PLUG_IN:
		mRecordPreview->setCamExist(CAM_UVC, true);
#ifdef IGNORE_UVC_STATE
		if(curWindowState.curWindowID == WINDOWID_RECORDPREVIEW)
#endif
		{
			SendMessage(mSTB->getHwnd(), STBM_UVC, 1, 0);
		}
		if(mEM){
			mEM->startDetectUVCBackCarState();
		}
		break;
	case EVENT_TVD_PLUG_IN:
		mRecordPreview->setCamExist(CAM_TVD, true);
#ifdef IGNORE_UVC_STATE
		if(curWindowState.curWindowID == WINDOWID_RECORDPREVIEW)
#endif
		{
			SendMessage(mSTB->getHwnd(), STBM_UVC, 1, 0);
		}

		break;
	case EVENT_UVC_PLUG_OUT:
		if(mRecordPreview->getUvcCarModem())
			notify(EVENT_BACK_CAR_OUT, 0);
		if(mEM){
			mEM->stopDetectUVCBackCarState();
		}
		ret = mRecordPreview->setCamExist(CAM_UVC, false);
#ifdef APP_WIFI
		if (ret > 0) {
			int camId = mRecordPreview->getCurSessionCam();
			if(mRecordPreview->getCurSessionCam() == CAM_UVC_NET){
				mRecordPreview->stopRecord(CAM_UVC_NET);
				 camChange(true, 0);
			}
			mCdrServer->reply(NAT_BROADCAST_SID, NAT_CMD_UVC_DISCONNECT, NULL, 0);
		}
#endif
#ifdef IGNORE_UVC_STATE
		if(curWindowState.curWindowID == WINDOWID_RECORDPREVIEW)
#endif
		{
			SendMessage(mSTB->getHwnd(), STBM_UVC, 0, 0);
		}
		break;
	case EVENT_TVD_PLUG_OUT:
		mRecordPreview->setCamExist(CAM_TVD, false);
#ifdef IGNORE_UVC_STATE
		if(curWindowState.curWindowID == WINDOWID_RECORDPREVIEW)
#endif
		{
			SendMessage(mSTB->getHwnd(), STBM_UVC, 0, 0);
		}
		break;
	case EVENT_CONNECT2PC:
		{
			db_msg("EVENT_CONNECT2PC\n");
			noWork(false);
			if(m2pcConnected || mSessionConnect) {
				break;
			}
			m2pcConnected = true;
			mOtherDialogOpened = true;
           	m2pcAdasScreen = false;
			//if (!isRecording() && !isPlaying()) {
			if(mOTAAdasScreen){
				 break;
			}
			SendNotifyMessage(mHwnd, MSG_CONNECT2PC, 0, 0);
			if(mRecordPreview->getIsAdasScreen() && !m2pcAdasScreen){
	           SendMessage(getWindowHandle(WINDOWID_RECORDPREVIEW),MSG_ADAS_SCREEN,0,0);
	           m2pcAdasScreen = true;
            }
			//}
		}
		break;
	case EVENT_DISCONNECFROMPC:
		{
			db_msg("EVENT_DISCONNECFROMPC\n");
			if(!mRecordPreview->isRecording()){
				db_msg("noWork");
				noWork(true);
			}
			int ret;
			if (mPCCam) {
				//String8 val_usb;
				//usleep(800*1000);	//ignore the unstable status
				//if(mEM->usbConnnect() != 0) {
				//	break;
				//}
				stopPCCam();
			}
			if(mSessionConnect)
				break;
			mUSBcharge = false;
			m2pcConnected = false;
			mOtherDialogOpened = false;
			if(m2pcAdasScreen)
				SendMessage(getWindowHandle(WINDOWID_RECORDPREVIEW),MSG_ADAS_SCREEN,1,0);
			if (mEM->isBatteryOnline()) {
				ret = mPM->getBatteryLevel();
				SendMessage(mSTB->getHwnd(), MSG_BATTERY_CHANGED, ret, 0);
			}
			SendNotifyMessage(mHwnd, MSG_DISCONNECT_FROM_PC, 0, 0);
			mNeedCheckSD = true;
		}
		break;
	case EVENT_IMPACT:
		db_msg("****EVENT_IMPACT*******");
		mRecordPreview->impactOccur();
		break;
	case EVENT_BATTERY:
	{
		int status= EventManager::getBatteryStatus();
		ret = mPM->setBatteryLevel();
		if(ret<=BATTERY_LOW_LEVEL){
			if(status == BATTERY_DISCHARGING || status == BATTERY_NOTCHARGING){
				db_msg("**low_power shutdown***");
				SendNotifyMessage(mHwnd, MSG_SHOW_TIP_LABEL, LABEL_LOW_POWER_SHUTDOWN, 0);
				shutdown(); 
			}
		}else{
			if(mEM->isBatteryOnline()) {
				SendMessage(mSTB->getHwnd(), MSG_BATTERY_CHANGED, ret, 0);
			}else{
				SendMessage(mSTB->getHwnd(), MSG_BATTERY_CHARGING, 0, 0);
			}
		}
	}
		break;
	case EVENT_MOUNT:
		SendMessage(mSTB->getHwnd(), STBM_MOUNT_TFCARD, val, 0);
#ifdef APP_WIFI
		{
			aw_cdr_tfcdrd_ismount_resp resp;
			resp.result = val;
			cdrReplyToApp(NAT_CMD_TFCARD_ISMOUNT_RESP, (char *)(&resp), sizeof(aw_cdr_tfcdrd_ismount_resp));
		}
#endif
		if (!val) {
			if (curWindowState.curWindowID == WINDOWID_RECORDPREVIEW) {
				db_msg("-----------umount, stopRecord and setSdcardState(false)");
				mRecordPreview->storagePlug(false);
			} else if (curWindowState.curWindowID == WINDOWID_PLAYBACKPREVIEW) {
				db_msg("current window is playbackview");
#ifdef USE_IPS_SCREEN
                SendMessage(mSTB->getHwnd(), STBM_SHOW_TIMEHWND, 1, 0);
#endif
				SendMessage(mSTB->getHwnd(), STBM_CLEAR_LABEL1_TEXT, WINDOWID_PLAYBACKPREVIEW, 0);
				SendMessage(mHwnd, MWM_CHANGE_WINDOW, WINDOWID_PLAYBACKPREVIEW, WINDOWID_RECORDPREVIEW);
			} else if (curWindowState.curWindowID == WINDOWID_PLAYBACK) {
				db_msg("current window is playback");
#ifdef USE_IPS_SCREEN
                SendMessage(mSTB->getHwnd(), STBM_SHOW_TIMEHWND, 1, 0);
#endif
				SendMessage(mPlayBack->getHwnd(), MSG_PLB_COMPLETE, 0, 0);
				SendMessage(mHwnd, MWM_CHANGE_WINDOW, WINDOWID_PLAYBACK, WINDOWID_RECORDPREVIEW);
			}
		} else {
			if (curWindowState.curWindowID == WINDOWID_PLAYBACKPREVIEW) {
				SendMessage(mPlayBackPreview->getHwnd(), MSG_UPDATE_LIST, 0, 0);
			}
			mRecordPreview->storagePlug(true);
		}
		break;
	case EVENT_DELAY_SHUTDOWN:
		db_msg("AC PLUG OUT");
		mFiACConnect = false;
		if(mEM->isBatteryOnline()) {
			int ret=-1;
			mPM->setBatteryLevel();
			ret = mPM->getBatteryLevel();
			if(ret<=BATTERY_LOW_LEVEL){
				db_msg("**low_power shutdown***");
				SendNotifyMessage(mHwnd, MSG_SHOW_TIP_LABEL, LABEL_LOW_POWER_SHUTDOWN, 0);
				shutdown(); 
			}
			SendMessage(mSTB->getHwnd(), MSG_BATTERY_CHANGED, ret, 0);
		}
		//if (!isRecording()&&!isPlaying()) {
		//	startShutdownTimer(TIMER_LOWPOWER_IDX);
		//	db_msg("xxxxxxxxxx\n");
		//	SendNotifyMessage(mHwnd, MSG_SHOW_TIP_LABEL, LABEL_10S_SHUTDOWN, 0);
		//}
		if( mRecordPreview->isRecording() )
		{
			if (mEM->isBatteryOnline()) {
				startShutdownTimer(TIMER_RECORD_SHUTDOWN_IDX);
				mShutdownTimer = true;
				db_msg("shutdown after 10s\n");
				SendNotifyMessage(mHwnd, MSG_SHOW_TIP_LABEL, LABEL_10S_SHUTDOWN, 0);
			}
		}
		else{
			if(!isPlaying())
				noWork(true);
		}
		break;
	case EVENT_CANCEL_SHUTDOWN:
		db_msg("AC PLUG IN");
		if(!mEM->isBatteryOnline()) {
			SendMessage(mSTB->getHwnd(), MSG_BATTERY_CHARGING, 0, 0);
		}
		if(mShutdownTimer){
			CloseTipLabel();
			stopShutdownTimer(TIMER_RECORD_SHUTDOWN_IDX);
			mShutdownTimer = false;
		}
		//noWork(false);
		//if (!isRecording()&& !isPlaying()) {
		//	cancelShutdownTimer(TIMER_LOWPOWER_IDX);
		//	db_msg("xxxxxxxxxx\n");
		//	CloseTipLabel();
		//	noWork(false);
		//}
		break;	
#ifdef PLATFORM_0
	case EVENT_HDMI_PLUGIN:
		{
			mEM->hdmiAudioOn(true);
			if ( curWindowState.curWindowID == WINDOWID_PLAYBACKPREVIEW ) {
				mPlayBackPreview->beforeOtherScreen();
			}
			mRecordPreview->otherScreen(1);
			if ( curWindowState.curWindowID == WINDOWID_PLAYBACKPREVIEW ) {
				mPlayBackPreview->afterOtherScreen(getHdl(0), 1);
			}
			mPlayBackPreview->otherScreen(1);
		}
		break;
	case EVENT_HDMI_PLUGOUT: 
		{
			mEM->hdmiAudioOn(false);
			if ( curWindowState.curWindowID == WINDOWID_PLAYBACKPREVIEW ) {
				mPlayBackPreview->beforeOtherScreen();
			}
			mRecordPreview->otherScreen(0);
			if ( curWindowState.curWindowID == WINDOWID_PLAYBACKPREVIEW ) {
				mPlayBackPreview->afterOtherScreen(getHdl(0), 0);
			}
			mPlayBackPreview->otherScreen(0);
		}
		break;
#endif
#ifdef APP_TVOUT
	case EVENT_TVOUT_PLUG_IN:
		{
			mRecordPreview->otherScreen(2);
		}
		break;
	case EVENT_TVOUT_PLUG_OUT: 
		{
			mRecordPreview->otherScreen(0);
		}
		break;
#endif
	case EVENT_REMOVESD:
		mNeedCheckSD = true;
		if(!mSessionConnect)
			BroadcastMessage(MSG_CLOSE_TIP_LABEL, 0, 0);
		break;
	case EVENT_INSERTSD:
		break;
	case EVENT_BACK_CAR_IN:
		{
			db_msg("*******EVENT_BACK_CAR_IN*********");
			ResourceManager *rm = ResourceManager::getInstance();
		//	mUVCCarBackAdasScreen = rm->getResBoolValue(ID_MENU_LIST_SMARTALGORITHM);
			int valAdas = rm->getResIntValue(ID_MENU_LIST_SMARTALGORITHM, INTVAL_SUBMENU_INDEX);
			if(valAdas == 0 || valAdas == 3){
				mUVCCarBackAdasScreen = false;
			}else if((valAdas == 1 || valAdas == 2) && !mSessionConnect){
				mUVCCarBackAdasScreen = true;
			}
			if(mUVCCarBackAdasScreen){
				mRecordPreview->adasScreenSwitch(false);
			}
			mRecordPreview->setUvcCarModem(true);
			if(getCurWindowID() != WINDOWID_RECORDPREVIEW){
			//	 if(getCurWindowID() == WINDOWID_MENU){
				if(mMenu->getSubMenuFlag()){
					mIsShowSubMenu = true;
					mCarBack = true;
					BroadcastMessage(MSG_CLOSE_TIP_LABEL, 0, 0);
					break;
				}else{
					BroadcastMessage(MSG_CLOSE_TIP_LABEL, 0, 0);
					if(getCurWindowID() == WINDOWID_PLAYBACK){
						db_msg("  WINDOWID_PLAYBACK  WINDOWID_PLAYBACK ");
					/*	mPlayBack->stopPlayback();
					  	stopPlayback() have KillTimer(mHwnd, PLAYBACK_TIMER) this function 
						should in HWND's pro function can run successed, so use sendMessage to complete this things*/
						SendMessage(mPlayBack->getHwnd(), MSG_STOP_PLAYBACK, 0, 0);
#ifdef USE_IPS_SCREEN
						SendMessage(mSTB->getHwnd(), STBM_SHOW_TIMEHWND, 1, 0);
#endif
						//mRecordPreview->closecarBackCSI();
					}else if(getCurWindowID() == WINDOWID_PLAYBACKPREVIEW){
						SendMessage(mSTB->getHwnd(), STBM_CLEAR_LABEL1_TEXT, WINDOWID_PLAYBACKPREVIEW, 0);
#ifdef USE_IPS_SCREEN
						SendMessage(mSTB->getHwnd(), STBM_SHOW_TIMEHWND, 1, 0);
#endif
					}
					changeWindow(WINDOWID_RECORDPREVIEW);
				}
			}else{
				BroadcastMessage(MSG_CLOSE_TIP_LABEL, 0, 0);
				mRecordPreview->carBackTransPIP(true);
				mRecordPreview->showUvcCarBackLines(true);
			}
		}
		break;
	case EVENT_BACK_CAR_OUT:
		{	/*ToDO:transform back to the previous pip mode*/
			db_msg("*******EVENT_BACK_CAR_OUT*********");

			if(mUVCCarBackAdasScreen){
				mUVCCarBackAdasScreen = false;
				if(!mSessionConnect)
					mRecordPreview->adasScreenSwitch(true);
			}
			mRecordPreview->setUvcCarModem(false);
			mRecordPreview->carBackTransPIP(false);
			mRecordPreview->showUvcCarBackLines(false);
			if(mSessionConnect)
				SendNotifyMessage(mHwnd, MSG_SHOW_TIP_LABEL, LABEL_WIFI_CONNECT, 0);
		}
		break;
     case EVENT_DB_INITEND:
        if(mSessionConnect){
            cdrReplyToApp(NAT_CMD_DBINIT_END, NULL, 0);
        }
        break;
	}
	return ret;
}

static void connectToPCCallback(HWND hWnd, int id, int nc, DWORD context)
{
	CdrMain* cdrMain;
	cdrMain = (CdrMain*)context;
	ResourceManager* rm;
	const char* ptr;
	HWND hDlg;

	int screen_w,screen_h;
	getScreenInfo(&screen_w, &screen_h);
	hDlg = GetParent(hWnd);
	rm = ResourceManager::getInstance();
	if(nc == BN_CLICKED) {
		if(id == ID_CONNECT2PC)	{
			if(cdrMain->getUSBStorageMode() == false) {
				db_msg("open usb storage mode\n");
				cdrMain->setNoUSBCharge(true);
				cdrMain->setUSBStorageMode(true);
				cdrMain->noWork(false);
				ptr = rm->getLabel(LANG_LABEL_CLOSE_USB_STORAGE_DEVICE);
				if(ptr){
				//	SetWindowText(hWnd, ptr);
					ShowWindow(GetDlgItem(hDlg,ID_CONNECT2PC),SW_HIDE);
					ShowWindow(GetDlgItem(hDlg,ID_CONNECT2PC+1),SW_HIDE);
					ShowWindow(GetDlgItem(hDlg,ID_CONNECT2PC+2),SW_HIDE);
					UpdateWindow(hDlg,true);
					HDC hdc = GetDC(hDlg);
					BITMAP bitImage;
					char *filepath=(char *)"/system/res/others/usb_storage_modem.png";
					LoadBitmapFromFile(HDC_SCREEN, &bitImage, filepath);
					FillBoxWithBitmap(hdc,0,0,screen_w,screen_h,&bitImage);
					EndPaint(hDlg,hdc);
				}
				else {
					db_error("get label failed\n");
				}
				
			} else if(cdrMain->getUSBStorageMode() == true) {
				db_msg("close usb storage mode, hDlg is 0x%x\n", hDlg);
				cdrMain->setUSBStorageMode(false);	
				ptr = rm->getLabel(LANG_LABEL_OPEN_USB_STORAGE_DEVICE);
				if(ptr){
					SetWindowText(hWnd, ptr);
					ShowWindow(GetDlgItem(hDlg,ID_CONNECT2PC+1),SW_SHOW);
				}
				else {
					db_error("get label failed\n");
				}				
			}
		} else if(id == ID_CONNECT2PC + 1) {
			db_msg("charge mode, hDlg is 0x%x\n", hDlg);
			SendNotifyMessage(hDlg, MSG_CLOSE_CONNECT2PC_DIALOG, 0, 0);
			if(!cdrMain->getRecordPreview()->isRecording())
				cdrMain->noWork(true);
			if(!cdrMain->isBatteryOnline()) {
				db_msg("battery is charging\n");
				SendMessage(cdrMain->getWindowHandle(WINDOWID_STATUSBAR), MSG_BATTERY_CHARGING, 0, 0);
			}
		}
		else if (id == ID_CONNECT2PC + 2) {
			if(cdrMain->getPCCamMode() == false) {
				db_msg("open PCCamMode\n");
				cdrMain->setNoUSBCharge(true);
				cdrMain->noWork(false);
				ShowWindow(GetDlgItem(hDlg,ID_CONNECT2PC),SW_HIDE);
				ShowWindow(GetDlgItem(hDlg,ID_CONNECT2PC+1),SW_HIDE);
				ShowWindow(GetDlgItem(hDlg,ID_CONNECT2PC+2),SW_HIDE);
				UpdateWindow(hDlg,true);
				HDC hdc = GetDC(hDlg);
				BITMAP bitImage;
				char *filepath=(char *)"/system/res/others/pccam_modem.png";
				LoadBitmapFromFile(HDC_SCREEN, &bitImage, filepath);
				FillBoxWithBitmap(hdc,0,0,screen_w,screen_h,&bitImage);
				EndPaint(hDlg,hdc);
			}
			cdrMain->startPCCam();
		}
	}
}

static int connectToPCDialog(HWND hWnd, void* context)
{
	int retval = 0;
	PopUpMenuInfo_t info;
	ResourceManager* rm;
	static bool hasOpened = false;

	if(hasOpened == true) {
		db_info("dialog has been opened\n");
		return 2;
	}

	rm = ResourceManager::getInstance();

	memset(&info, 0, sizeof(info));
	info.pLogFont = rm->getLogFont();
	retval = rm->getResRect(ID_CONNECT2PC, info.rect);
	if(retval < 0) {
		db_error("get connectToPC rect failed\n");	
		return -1;
	}

	retval = rm->getResIntValue(ID_CONNECT2PC, INTVAL_ITEMWIDTH);
	if(retval < 0) {
		db_error("get connectToPC item_width failed");
		return -1;
	}
	info.item_width = retval;

	retval = rm->getResIntValue(ID_CONNECT2PC, INTVAL_ITEMHEIGHT);
	if(retval < 0) {
		db_error("get connectToPC item_height failed");
		return -1;
	}
	info.item_height = retval;

	info.style = POPMENUS_V | POPMENUS_VCENTER;
	info.item_nrs = 3;
	info.id = ID_CONNECT2PC;
	info.name[0] = rm->getLabel(LANG_LABEL_OPEN_USB_STORAGE_DEVICE);
	if(info.name[0] == NULL) {
		db_error("get the LANG_LABEL_OPEN_USB_STORAGE_DEVICE failed\n");
		return -1;
	}
	info.name[1] = rm->getLabel(LANG_LABEL_CHARGE_MODE);
	if(info.name[1] == NULL) {
		db_error("get the LANG_LABEL_CHARGE_MODE failed\n");
		return -1;
	}
	info.name[2] = rm->getLabel(LANG_LABEL_OPEN_PCCAM);
	if(info.name[2] == NULL) {
		db_error("get the LANG_LABEL_OPEN_PCCAM failed\n");
		return -1;
	}
	info.callback[0] = connectToPCCallback;
	info.callback[1] = connectToPCCallback;
	info.callback[2] = connectToPCCallback;
	info.context = (DWORD)context;

	info.end_key = CDR_KEY_MODE;
	info.flag_end_key = 1;		/* key up to end the dialog */
	info.push_key = CDR_KEY_OK;
	info.table_key = CDR_KEY_RIGHT;

	info.bgc_widget = rm->getResColor(ID_CONNECT2PC, COLOR_BGC);
	info.bgc_item_focus = rm->getResColor(ID_CONNECT2PC, COLOR_BGC_ITEMFOCUS);
	info.bgc_item_normal = rm->getResColor(ID_CONNECT2PC, COLOR_BGC_ITEMNORMAL);
	info.mainc_3dbox = rm->getResColor(ID_CONNECT2PC, COLOR_MAIN3DBOX);

	hasOpened = true;
	retval = PopUpMenu(hWnd, &info);
	db_msg("retval is %d\n", retval);
	hasOpened = false;
	BroadcastMessage(MSG_CLOSE_TIP_LABEL, 0, 0);
	return retval;
}

static int showAdasFunctionDialog(HWND hWnd, void* context)
{
	int retval = 0;
	PopUpMenuInfo_t info;
	ResourceManager* rm;
	static bool hasOpened = false;

	if(hasOpened == true) {
		db_info("dialog has been opened\n");
		return 2;
	}

	rm = ResourceManager::getInstance();

	memset(&info, 0, sizeof(info));
	info.pLogFont = rm->getLogFont();
	info.showImage = 1;
	retval = rm->getResRect(ID_CONNECT2PC, info.rect);
	if(retval < 0) {
		db_error("get connectToPC rect failed\n");	
		return -1;
	}
	info.id = ID_SHOWADAS_FUNCTION;
	info.context = (DWORD)context;

	hasOpened = true;
	retval = PopUpMenu(hWnd, &info);
	db_msg("retval is %d\n", retval);
	hasOpened = false;
	BroadcastMessage(MSG_CLOSE_TIP_LABEL, 0, 0);

	return retval;

}


int CdrMain::msgCallback(int msgType, unsigned int data)
{
	ResourceManager* rm = ResourceManager::getInstance();
	int idx = (int)data;
	int ret = 0;
	switch(msgType) {
	case MSG_RM_LANG_CHANGED:
		{
			PLOGFONT logFont;
			logFont = rm->getLogFont();
			if(logFont == NULL) {
				db_error("invalid log font\n");
				return -1;
			}
			db_msg("type:%s, charset:%s, family:%s\n", logFont->type, logFont->charset, logFont->family);
			SetWindowFont(mHwnd, logFont);
		}
		break;
	case MSG_RM_SCREENSWITCH:
		{
			db_msg("MSG_RM_SCREENSWITCH, data is %d, idx is %d\n", data, idx);
			if (idx < 3) {
				if(mPM) {
					mTime = (idx+1)*10;
					mPM->setOffTime(mTime);
				}
			} else {
				if(mPM) {
					mTime = TIME_INFINITE;
					mPM->setOffTime(mTime);
				}
			}
		}
		break;
	case MSG_CONNECT2PC:
		db_msg("connect2Pc dialog\n");
		if (!isShuttingdown()) {
			noWork(false);
			int ret = connectToPCDialog(mHwnd, this);
			if(ret == 0){
				mUSBcharge = true;
				mOtherDialogOpened = false;
				if(m2pcAdasScreen){
						SendMessage(getWindowHandle(WINDOWID_RECORDPREVIEW),MSG_ADAS_SCREEN,1,0);
						m2pcAdasScreen = false;
				}
				db_msg("****MSG_CONNECT2PC*****");
				mNeedCheckSD = true;
				m2pcConnected = false;
			}
		}
		break;
	case MSG_DISCONNECT_FROM_PC:
		db_msg("close connect2Pc dialog\n");
		closeConnectToPCDialog();
		setUSBStorageMode(false);
		break;
	case MSG_SESSIONCONNECT:
		{
			db_msg("***********MSG_SESSIONCONNECT*************");
			if(!mSessionConnect){
				//close adas
				int valAdas = rm->getResIntValue(ID_MENU_LIST_SMARTALGORITHM, INTVAL_SUBMENU_INDEX);
				if(valAdas == 1 || valAdas == 2){
					mRecordPreview->setADAS(0);
					mRecordPreview->adasScreenSwitch(false);
				}

				//close awmd
				mAWMD = rm->getResBoolValue(ID_MENU_LIST_AWMD);
				if(mAWMD){
				//	SendMessage(getWindowHandle(WINDOWID_RECORDPREVIEW), MSG_AWMD, (WPARAM)false, 0);
                    mRecordPreview->switchAWMD(false);
                    mRecordPreview->setDelayStartAWMD(false);
                    noWork(true);
                    SendMessage(getWindowHandle(WINDOWID_STATUSBAR), STBM_AWMD, (WPARAM)false, 0);

					if(mRecordPreview->isRecording()){
						mRecordPreview->stopRecord();
						cdrReplyToApp(NAT_AWMD_RECORD_STOP, NULL, 0);
					}
				}

				if(mPlayBack->isStarted()){
					mPlayBack->stopPlayback();
				}
				if(getCurWindowID() != WINDOWID_RECORDPREVIEW){
					if(getCurWindowID() == WINDOWID_PLAYBACKPREVIEW){
						SendMessage(mSTB->getHwnd(), STBM_CLEAR_LABEL1_TEXT, WINDOWID_PLAYBACKPREVIEW, 0);
#ifdef USE_IPS_SCREEN
						SendMessage(mSTB->getHwnd(), STBM_SHOW_TIMEHWND, 1, 0);
#endif
					}else if(getCurWindowID() == WINDOWID_PLAYBACK){
#ifdef USE_IPS_SCREEN
						SendMessage(mSTB->getHwnd(), STBM_SHOW_TIMEHWND, 1, 0);
#endif
					}else if(getCurWindowID() == WINDOWID_MENU){
						if(mMenu->getSubMenuFlag()){
							mSessionSubMenu = true;
							BroadcastMessage(MSG_CLOSE_TIP_LABEL, 0, 0);
							break;
						}
					}
					changeWindow(WINDOWID_RECORDPREVIEW);
				}
				if(!mRecordPreview->getUvcCarModem()){
					SendNotifyMessage(mHwnd, MSG_SHOW_TIP_LABEL, LABEL_WIFI_CONNECT, 0);
				}
				mSessionConnect = true;
			}
		}
		break;
	case MSG_SESSIONDISCONNECT:
		{
			db_msg("*********MSG_SESSIONDISCONNECT********************");
			if(!mRecordPreview->getUvcCarModem()){
				BroadcastMessage(MSG_CLOSE_TIP_LABEL, 0, 0);
			}
			mRecordPreview->setCurSessionCam(CAM_CSI);
			//open awmd
			if(mAWMD){
				//SendMessage(getWindowHandle(WINDOWID_RECORDPREVIEW), MSG_AWMD, (WPARAM)true, 0);
                mRecordPreview->switchAWMD(true);
                noWork(false);
                SendMessage(getWindowHandle(WINDOWID_STATUSBAR), STBM_AWMD, (WPARAM)true, 0);
			}
			//open adas
			int valAdas = rm->getResIntValue(ID_MENU_LIST_SMARTALGORITHM, INTVAL_SUBMENU_INDEX);
			if(valAdas == 1 || valAdas == 2){
				mRecordPreview->setADAS(true, valAdas);
				if(!mRecordPreview->getUvcCarModem()){
                    usleep(100 * 1000);
					mRecordPreview->adasScreenSwitch(true);
				}
			}else{
				mRecordPreview->adasScreenSwitch(false);
			}
            sessionChange(false);
			mSessionConnect = false;
		}
		break;
	case MSG_RM_FORMAT_TFCARD:
		{
			StorageManager *sm = StorageManager::getInstance();
			ret = sm->format(NULL);
		}
		break;
	case MSG_RM_GET_CAPACITY:
		{
			StorageManager *sm = StorageManager::getInstance();
			aw_cdr_tf_capacity *capacity = (aw_cdr_tf_capacity *)data;
			ret = sm->getCapacity(&(capacity->remain), &(capacity->total));
		}
		break;
	case MSG_GSENSOR_SENSITIVITY:
			mGSensorVal = data;
		break;
	case MSG_IMPACTLEVEL_VALUE:
		if(mEM){
			mEM->setImpactLevel(data);
		}
		break;
	case MSG_AUTO_SHUTDOWN:
		if( data == 0 ) {
			isAutoShutdownEnabled = true;
			modifyNoworkTimerInterval(autoShutdownTime[0][0],autoShutdownTime[0][1]);
		} else if( data == 1 ) {
			isAutoShutdownEnabled = true;
			modifyNoworkTimerInterval(autoShutdownTime[1][0],autoShutdownTime[1][1]);
		} else if( data == 2 ) {
			isAutoShutdownEnabled = true;
			modifyNoworkTimerInterval(autoShutdownTime[2][0],autoShutdownTime[2][1]);
		} else if( data == 3 ) {
			isAutoShutdownEnabled = false;
			noWork(false);
		}
		break;
	case MSG_SHOW_ADAS_FUNCTION:
		db_msg(" MSG_SHOW_ADAS_FUNCTION in MainWindow ");
		showAdasFunctionDialog(mHwnd, this);
		break;
	case MSG_GET_STATUSBAR_TIME:
		{
			aw_cdr_net_config *config = (aw_cdr_net_config *)data;
			if(mRecordPreview->isRecording()){
				int curtime = SendMessage(getWindowHandle(WINDOWID_STATUSBAR) , STBM_GET_RECORD_TIME, 0, 0);
				config->curRecordTime = curtime;
			}else{
				config->curRecordTime = 0;
			}
			config->videoTime= (mRecordPreview->getDuration()/1000);
		}
		break;
	default:
		break;
	}
	return ret;
}

void CdrMain::forbidRecord(int type)
{
	mEnableRecord = type;
}

int CdrMain::isEnableRecord()
{
	return mEnableRecord;
}

unsigned int CdrMain::getCurWindowID()
{
	return curWindowState.curWindowID;
}

static void *lowPowerShutDown(void *ptx)
{
	db_error("low power shut down");
	sleep(5);
	android_reboot(ANDROID_RB_POWEROFF, 0, 0);
    return NULL;
}

int CdrMain::createMainWindows(void)
{
	RegisterKeyMsgHook((void*)this, MsgHook);
	CDR_RECT rect;
	MAINWINCREATE CreateInfo;
	ResourceManager* rm = ResourceManager::getInstance();
	if(rm->initStage1() < 0) {
		db_error("ResourceManager initStage1 failed\n");
		return -1;
	}

	if(RegisterCDRControls() < 0) {
		db_error("register CDR controls failed\n");
		return -1;
	}
	if(RegisterCDRWindows() < 0) {
		db_error("register CDR windows failed\n");
		return -1;
	}

	CreateInfo.dwStyle = WS_VISIBLE;
	CreateInfo.dwExStyle = WS_EX_NONE | WS_EX_AUTOSECONDARYDC;
	CreateInfo.spCaption = "";
	CreateInfo.hMenu = 0;
	CreateInfo.hCursor = GetSystemCursor(0);
	CreateInfo.hIcon = 0;
	CreateInfo.MainWindowProc = CDRWinProc;

	rm->getResRect(ID_SCREEN, rect);
	CreateInfo.lx = rect.x;
	CreateInfo.ty = rect.y;
	CreateInfo.rx = rect.w;
	CreateInfo.by = rect.h;
	
	CreateInfo.iBkColor = RGBA2Pixel(HDC_SCREEN, 0x00, 0x00, 0x00, 0x00);		/* the color key */
	CreateInfo.dwAddData = (DWORD)this;
	CreateInfo.hHosting = HWND_DESKTOP;

	mHwnd = CreateMainWindow(&CreateInfo);
	if (mHwnd == HWND_INVALID) {
		db_error("create Mainwindow failed\n");
		return -1;
	}
	ShowWindow(mHwnd, SW_SHOWNORMAL);
	rm->setHwnd(WINDOWID_MAIN, mHwnd);

	if(mLowPower<=LOW_POWER ) {
		EventManager::int2set(rm->getResBoolValue(ID_MENU_LIST_PARK));
		int batteryStatus = EventManager::getBatteryStatus();
		db_msg("***batteryStatus:%d***",batteryStatus);
		if(batteryStatus==BATTERY_DISCHARGING || batteryStatus==BATTERY_NOTCHARGING){
			MSG msg;
			pthread_t thread_id = 0;
			char parkconfig[4]={0};
			cfgDataGetString("persist.app.park.config",parkconfig,"0");
			sys_log("************parkconfig:%s",parkconfig);
			if(strcmp(parkconfig,"1")==0)
				EventManager::int2set(true);
			int err = pthread_create(&thread_id, NULL, lowPowerShutDown, NULL);
			if(err || !thread_id) {
				db_msg("Create lowPowerShutDown  thread error.");
				return -1;
			}
			while(GetMessage(&msg,mHwnd))
			{
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
			MainWindowThreadCleanup(mHwnd);	
			return -1;
		}
	}
		
	mSTB = new StatusBar(this);
	if(mSTB->getHwnd() == HWND_INVALID) {
		db_error("create StatusBar failed\n");
		return -1;
	}

	system("dd if=/dev/zero of=dev/graphics/fb0");

	if(mRecordPreview->createWindow() < 0) {
		db_error("create RecordPreview Window failed\n");
		return -1;
	}

	windowState_t windowState;
	windowState.curWindowID = WINDOWID_RECORDPREVIEW;
	windowState.RPWindowState = STATUS_RECORDPREVIEW;
	setCurrentWindowState(windowState);
	ShowWindow(mRecordPreview->getHwnd(), SW_SHOWNORMAL);
	return 0;
}

int CdrMain::createOtherWindows(void)
{


	/* create PlayBackPreview */
	mPlayBackPreview = new PlayBackPreview(this);
	if(mPlayBackPreview->getHwnd() == HWND_INVALID) {
		db_error("create PlayBackPreview Window failed\n");
		return -1;
	}
	db_msg("xxxxxxxxxxxxxxxxxxxxDEBUG\n");

	/* create PlayBack */
	mPlayBack = new PlayBack(this);
	if(mPlayBack->getHwnd() == HWND_INVALID) {
		db_error("create PlayBack Window failed\n");
		return -1;
	}
	db_msg("xxxxxxxxxxxxxxxxxxxxDEBUG\n");

	/* CreateMenu */
	mMenu = new Menu(this);
	if(mMenu->getHwnd() == HWND_INVALID) {
		db_error("create PlayBack Window failed\n");
		return -1;
	}
	db_msg("xxxxxxxxxxxxxxxxxxxxDEBUG\n");
	mOtherWindowInited = true;
	return 0;
}

void CdrMain::msgLoop(void)
{
	MSG Msg;
	while (GetMessage (&Msg, this->mHwnd)) {
		TranslateMessage (&Msg);
		DispatchMessage (&Msg);
	}
}

HWND CdrMain::getWindowHandle(unsigned int windowID)
{
	switch(windowID) {
	case WINDOWID_RECORDPREVIEW:
		return mRecordPreview->getHwnd();
		break;
	case WINDOWID_PLAYBACKPREVIEW:
		return mPlayBackPreview->getHwnd();
		break;
	case WINDOWID_PLAYBACK:
		return mPlayBack->getHwnd();
		break;
	case WINDOWID_MENU:
		return mMenu->getHwnd();
		break;
	case WINDOWID_STATUSBAR:
		return mSTB->getHwnd();
		break;
	default:
		db_error("invalid window id: %d\n", windowID);
	}

	return HWND_INVALID;
}

MainWindow* CdrMain::windowID2Window(unsigned int windowID)
{
	switch(windowID) {
	case WINDOWID_RECORDPREVIEW:
		return mRecordPreview;
		break;
	case WINDOWID_PLAYBACKPREVIEW:
		return mPlayBackPreview;
		break;
	case WINDOWID_PLAYBACK:
		return mPlayBack;
		break;
	case WINDOWID_MENU:
		return mMenu;
		break;
	case WINDOWID_STATUSBAR:
		return mSTB;
		break;
	default:
		db_error("invalid window id: %d\n", windowID);
	}

	return NULL;
}

void CdrMain::setCurrentWindowState(windowState_t windowState)
{
	curWindowState = windowState;
	SendNotifyMessage(mSTB->getHwnd(), MSG_CDRMAIN_CUR_WINDOW_CHANGED, (WPARAM)&curWindowState, 0);
}

void CdrMain::setCurrentWindowState(RPWindowState_t status)
{
	curWindowState.RPWindowState = status;
	SendNotifyMessage(mSTB->getHwnd(), MSG_CDRMAIN_CUR_WINDOW_CHANGED, (WPARAM)&curWindowState, 0);
}

void CdrMain::changeWindow(unsigned int windowID)
{
	HWND toHwnd;
	if (isShuttingdown() || isRecording()) {
		return ;
	}
	if ((curWindowState.curWindowID == WINDOWID_RECORDPREVIEW) && isRecording()) {
		return ;
	}
	toHwnd = getWindowHandle(windowID);
	if(toHwnd == HWND_INVALID) {
		db_error("invalid toHwnd\n");	
		return;
	}
	db_debug("%d %d", toHwnd, windowID);
	if( windowID == WINDOWID_RECORDPREVIEW || windowID == WINDOWID_PLAYBACK) {
		db_msg("SetWindowBkColor:%d", windowID);	
		//SetWindowBkColor(mHwnd, RGBA2Pixel(HDC_SCREEN, 0xFF, 0xFF, 0xFF, 0xFF));//RGBA2Pixel(HDC_SCREEN, 0x00, 0x00, 0x00, 0x00));	/* the color key */
	} else {
		//SetWindowBkColor(mHwnd, COLOR_lightgray);//RGBA2Pixel(HDC_SCREEN, 0x19, 0x19, 0x19, 0xff));	/* the main window color */
	}
	if( windowID == WINDOWID_PLAYBACK ) {
		ShowWindow(getWindowHandle(WINDOWID_STATUSBAR), SW_HIDE);
	} else if(curWindowState.curWindowID == WINDOWID_PLAYBACK ) {
		ShowWindow(getWindowHandle(WINDOWID_STATUSBAR), SW_SHOWNORMAL);
	}
	ShowWindow(getWindowHandle(curWindowState.curWindowID), SW_HIDE);
	if (curWindowState.curWindowID == WINDOWID_RECORDPREVIEW) {
		SendMessage(getWindowHandle(WINDOWID_RECORDPREVIEW),MSG_ADAS_SCREEN,0,0);
	}
	ShowWindow(toHwnd, SW_SHOWNORMAL);
	SendNotifyMessage(toHwnd, MWM_CHANGE_FROM_WINDOW, curWindowState.curWindowID, 0);
	db_debug(" ");
	curWindowState.curWindowID = windowID;
	setCurrentWindowState(curWindowState);	
}

void CdrMain::changeWindow(unsigned int fromID, unsigned int toID)
{
	if(fromID == curWindowState.curWindowID) {
		changeWindow(toID);
	}
}

void CdrMain::keyProc(int keyCode, int isLongPress)
{
	int ret;
	db_msg("key %x %d\n", keyCode, isLongPress);
	MainWindow* curWin;
	if(keyCode == CDR_KEY_POWER) {
		if (isLongPress) {
			CloseTipLabel();
			//ret = CdrDialog(mHwnd, CDR_DIALOG_SHUTDOWN);
			ret = DIALOG_OK;	//to shutdown directly
			if(ret == DIALOG_OK) {
				//				mPM->powerOff();
				mKeyShutDown = true;
				shutdown();
			} else if(ret == DIALOG_CANCEL) {
				db_msg("cancel\n");	
			} else {
				db_msg("ret is %d\n", ret);	
			}
		} else {
			db_msg("xxxxxxxxxx\n");
			if (mPM) {
				mPM->screenSwitch();
			}
		}
		return;
	}
	curWin = windowID2Window(curWindowState.curWindowID);
	db_msg("curWindowID is %d\n", curWindowState.curWindowID);
	if (curWin && (mOtherWindowInited==true)) {
		db_msg("xxxxxxxxxx\n");
		DispatchKeyEvent(curWin, keyCode, isLongPress);
	}
}

CdrDisplay *CdrMain::getCommonDisp(int disp_id)
{
	return mRecordPreview->getCommonDisp(disp_id);
}

#ifdef PLATFORM_0
int CdrMain::getHdl(int disp_id)
{
	return mRecordPreview->getHdl(disp_id);
}
#endif

/*
 * DispatchKeyEvent message to mWin
 * */
void CdrMain::DispatchKeyEvent(MainWindow *mWin, int keyCode, int isLongPress)
{
	HWND hWnd;
	unsigned int windowID;

	hWnd = mWin->getHwnd();
	if( hWnd && (hWnd != HWND_INVALID)) {
        if(!mSessionConnect || (mSessionConnect && keyCode==CDR_KEY_MODE && isLongPress==LONG_PRESS)){
		    windowID = SendMessage(hWnd, MSG_CDR_KEY, keyCode, isLongPress);
            db_msg("windowID is %d\n", windowID);
            if(mSessionConnect && keyCode==CDR_KEY_MODE && isLongPress==LONG_PRESS)   //for if connect wifi close wifi
                return ;

	        if((windowID != curWindowState.curWindowID) ){
				db_msg(" mSessionSubMenu:%d  mCarBack:%d", mSessionSubMenu, mCarBack);
				changeWindow(windowID);
				if(mCarBack)
				mCarBack= false;
			    if(mSessionSubMenu){
					mSessionSubMenu = false;
					mSessionConnect = true;
					SendNotifyMessage(mHwnd, MSG_SHOW_TIP_LABEL, LABEL_WIFI_CONNECT, 0);
		       }
			}
		 }
	}
}

bool CdrMain::isScreenOn()
{
	if (mPM) {
		return mPM->isScreenOn();
	}
	return true;
}

void CdrMain::setOffTime()
{
	if(mPM)
		mPM->setOffTime(mTime);
}

void CdrMain::configInit()
{
	bool val;
	ResourceManager* rm = ResourceManager::getInstance();
	int ret = 0;
	/*TODO*/
	db_msg("%s:%d\n",__func__,__LINE__);
	int time = rm->getResIntValue(ID_MENU_LIST_SS, INTVAL_SUBMENU_INDEX);
	if (time < 3) {
		time = (time+1)*10;
	} else {
		time = TIME_INFINITE;
	}
	db_msg("%s:%d\n",__func__,__LINE__);

	mPM->setOffTime(time);
	db_msg("%s:%d\n",__func__,__LINE__);
	// rm->notifyAll();
	db_msg("%s:%d\n",__func__,__LINE__);
	/* disable int2 initially */
	db_msg("park mode:%d", rm->getResBoolValue(ID_MENU_LIST_PARK));
#if 0
		if((read(fd_wakeup,&val,1)>0) && (val == '1')){ 	//жǷжϻ
#else
	if (rm->getResBoolValue(ID_MENU_LIST_PARK) ) {
#endif
		if( mEM->isBatteryOnline()){
			isWakenUpOnParking = true;
			//ʼ¼15Sػ
			db_msg("PM: parking monitor detect crash!\n");
				if (curWindowState.curWindowID == WINDOWID_RECORDPREVIEW) {
					mRecordPreview->setRecordTime(PARK_MON_VTL_15S);
					mRecordPreview->needCache(false);
					ret = mRecordPreview->startRecord();
					if (ret < 0) {
						//sleep(2);
						//return shutdown();
						return ;
					}
					if(ret ==0 ){
						mSTB->setCountOnce(TRUE);
						mRecordPreview->impactOccur();
						HWND mstb=getWindowHandle(WINDOWID_STATUSBAR);
						StorageManager *sm = StorageManager::getInstance();
						usleep(500000);
						SendMessage(mstb, STBM_SETLABEL1_TEXT, 1, (LPARAM)"");
						sm->geneImpactTimeFile();
					}
				}
				return ;
		}else{
			if (FILE_EXSIST(GENE_IMPACT_TIME_FILE)){
				SendNotifyMessage(mHwnd, MSG_SHOW_TIP_LABEL, LABEL_IMPACT_NUM, 0);
				mTinyParkAudioTh = new TinyParkAudioThread(this,(char*)"/system/res/others/parknotice.wav");
				int ret = mTinyParkAudioTh->start();
				if (ret != OK) {
					db_msg("TinyParkAudioThread start failed\n");
				}
			}	
		}
		
	}
	
	val = rm->getResBoolValue(ID_MENU_LIST_AWMD);
	if(val == true) {
		db_info("AWMD mode\n");
	} else {
		// Force to true by Harry
		val = true; // rm->getResBoolValue(ID_MENU_LIST_POR); //power on record
		
		if (val == true) {
			db_info("POR: power on record\n");
			if (curWindowState.curWindowID == WINDOWID_RECORDPREVIEW) {
				if(mRecordPreview->startRecord()<0){
					noWork(true);
				}
			}
		}
	}
	//mPM->adjustCpuFreq();
#ifdef USE_IPS_SCREEN
        mSTB->startSystemTimeTimer();
#endif
}

void CdrMain::exit()
{
	delete mEM;
	mEM = NULL;
	delete mPM;
	mPM = NULL;
}

void CdrMain::prepareCamera4Playback(bool closeLayer)
{
	if(mRecordPreview)
		mRecordPreview->prepareCamera4Playback(closeLayer);
}

void CdrMain::setUSBStorageMode(bool enable)
{
	if(enable == true) {
		if(usbStorageConnected == false) {
			db_msg("enable USB Storage mode\n");
			if (isRecording()) {
				mRecordPreview->stopRecordSync();
			}
			StorageManager *sm = StorageManager::getInstance();
			sm->stopUpdate();
			mEM->connect2PC();
			usbStorageConnected = true;
		}
	} else {
		if(usbStorageConnected == true)	{
			db_msg("disable USB Storage mode\n");
			mEM->disconnectFromPC();
			StorageManager *sm = StorageManager::getInstance();
			sm->dbReset();
			usbStorageConnected = false;
		}
	}
}

static unsigned char *getShutdownLogo()
{
	int fd;
	unsigned char *buf = NULL;
	int buf_size = 512*1024;

	fd = open("/dev/block/mtdblock5", O_RDWR);
    if( fd < 0) {
    	db_error("open shutlogo fail\n");
    }
	buf  = (unsigned char *)malloc(buf_size);
	read(fd, buf, buf_size);
#ifdef DEBUG_SHUTDOWN_LOGO	
	for(int i = 0; i< 100; i++) {
			db_error("buf[%d]=0x%x\n",i*4,*(unsigned int *)(buf+i*4));
	}
#endif
	return buf;

}

void CdrMain::showShutdownLogo(void)
{
	HWND hStatusBar, hMenu, hPBP;
	CdrDisplay* cd;
	unsigned char *buf = NULL;

	cd = getCommonDisp(CAM_CSI);
	if(cd)
		ck_off(cd->getHandle());
	cd = getCommonDisp(CAM_UVC);
	if(cd)
		ck_off(cd->getHandle());
	hStatusBar = getWindowHandle(WINDOWID_STATUSBAR);
	hMenu = getWindowHandle(WINDOWID_MENU);
	hPBP = getWindowHandle(WINDOWID_PLAYBACKPREVIEW);
#ifdef USE_IPS_SCREEN
    /*IPS screen autoshutdown has a green line in statusbar so if autoshutdown cannot hide statusbar
            but presskey shutdown has no green line  so should hide the statusbar
        */
    if(mKeyShutDown){
        ShowWindow(hStatusBar, SW_HIDE);
	    SendMessage(hStatusBar, MSG_CLOSE, 0, 0);	//cancel statusbar timer
    }else{
		mSTB->stopSystemTimeTimer();
		SendMessage(hStatusBar, STBM_SHOW_TIMEHWND, 2, 0);
    }
#else
    ShowWindow(hStatusBar, SW_HIDE);
	SendMessage(hStatusBar, MSG_CLOSE, 0, 0);	//cancel statusbar timer
#endif
	ShowWindow(hMenu, SW_HIDE);
	ShowWindow(hPBP, SW_HIDE);
//	showImageWidget(mHwnd, IMAGEWIDGET_POWEROFF);
	buf = getShutdownLogo();
	
	CloseTipLabel();
    showImageWidget(mHwnd, IMAGEWIDGET_POWEROFF, buf);
	  free(buf);

}

bool CdrMain::getUSBStorageMode(void)
{
	return usbStorageConnected;
}
bool CdrMain::getWakeupState(void )
{
	return isWakenUpOnParking;
}

void CdrMain::setWakeupState(bool flag)
{
	isWakenUpOnParking = flag;
}

bool CdrMain::isShuttingdown(void)
{
	return mShutdowning;
}

void CdrMain::setSTBRecordVTL(unsigned int vtl)
{
	mSTB->setRecordVTL(vtl);
}

bool CdrMain::getUvcState()
{
#ifdef IGNORE_UVC_STATE
	return false;
#else
	return mRecordPreview->getUvcState();
#endif
}

int CdrMain::camChange(bool session, int param_in)
{
	int ret = 0;

	if (session) {
		if (param_in < CAM_CNT) {
			ret = mRecordPreview->camChange(param_in);
		}
	} else {
		if (param_in) {
			RPWindowState_t mRPWindowState = mRecordPreview->getRPWindowState();
			if((mRPWindowState == STATUS_PHOTOGRAPH || mRPWindowState == STATUS_AWMD)&& !isRecording()) {
				db_msg("change to STATUS_RECORDPREVIEW modem");
				windowState_t windowState;
				windowState.curWindowID = WINDOWID_RECORDPREVIEW;
				windowState.RPWindowState = STATUS_RECORDPREVIEW;
				setCurrentWindowState(windowState);
				mRecordPreview->setRPWindowState(STATUS_RECORDPREVIEW);
			}
			ret = mRecordPreview->startRecord();
		} else {
		    if(mRecordPreview->isRecording(CAM_CSI)){
                ret = mRecordPreview->stopRecord(CAM_CSI);
            }
		    if(mRecordPreview->isRecording(CAM_UVC) && (ret == 0)){
                ret = mRecordPreview->stopRecord(CAM_UVC);
            }
		}
	}
	return ret;
}

int CdrMain::sessionControl(int sid, int cmd, char* buf)
{	
    db_msg(" sid:%d, cmd:%x, buf:%s", sid, cmd, buf);
	ResourceManager* rm = ResourceManager::getInstance();
	int ret;
	switch (cmd) {
		case  NAT_CMD_GET_CONFIG:
			rm->setRecordingStatus(mRecordPreview->isRecording());
			break;
		case NAT_CMD_CHANGE_CAM:
			{
				int param_in = ((aw_cdr_change_cam*)buf)->index;
				aw_cdr_change_cam_resp resp;
				resp.result = camChange(true, param_in);
				return mCdrServer->reply(sid, NAT_CMD_CHANGE_CAM_RESP, 
					(char *)(&resp), sizeof(resp));
			}
		case NAT_CMD_RECORD_ON_OFF:
			{
				int param_in = ((aw_cdr_recorder_onoff*)buf)->index;
				aw_cdr_recorder_onoff_resp resp;
				resp.result = camChange(false, param_in);
				return mCdrServer->reply(sid, NAT_CMD_RECORD_ON_OFF_RESP, 
					(char *)(&resp), sizeof(resp));
			}
		case NAT_CMD_SET_VIDEO_SIZE:
			{
				aw_cdr_set_video_quality_index *p = (aw_cdr_set_video_quality_index*)buf;
				aw_cdr_set_video_quality_index_resp resp;
				resp.result = 0;
				mRecordPreview->changeResolution(p->video_quality_index);
				return mCdrServer->reply(sid, NAT_CMD_SET_VIDEO_SIZE_ACK, 
					(char *)(&resp), sizeof(resp));
			}
		case NAT_CMD_NET_CONNECT_STATE:
			{
				int stat = ((aw_cdr_net_state*)buf)->index;
				if (stat == 1)
				{
					SendMessage(mHwnd, MSG_SESSIONCONNECT, 0, 0);
				}
				else
				{
					SendMessage(mHwnd, MSG_SESSIONDISCONNECT, 0, 0);
				}
			}
			break;
		case AW_IOTYPE_USER_IPCAM_ON_LINE:
			break;
		case AW_IOTYPE_USER_IPCAM_OFF_LINE:
			SendMessage(mHwnd, MSG_SESSIONDISCONNECT, 0, 0);
			break;
		case NAT_CMD_SHOT_RECORD:
			{
				db_msg("  *********NAT_CMD_SHOT_RECORD*********  ");

				char val[4] = {0};
				ret = cfgDataGetString("persist.sys.shotrecord.switch", val, "0");
				if(strcmp(val, "0") == 0){
					ret =  -1;
				}else{
					ret = mRecordPreview->startSmallVideoRecord();
				}
				aw_cdr_set_photo_quality_resp resp;
				resp.result = ret;
				mCdrServer->reply(sid, NAT_CMD_SHOT_RECORD_RESP, 
					(char *)(&resp), sizeof(aw_cdr_set_photo_quality_resp));
			}
			break;
		case NAT_CMD_BEGIN_DOWNLOAD:
			{
				handBeginDownloadBuf(buf);
				aw_cdr_beigin_download_file resp;
				resp.result = 0;
				mCdrServer->reply(sid, NAT_CMD_BEGIN_DOWNLOAD_RESP, 
					(char *)(&resp), sizeof(aw_cdr_beigin_download_file));
			}
			break;
		case NAT_CMD_UPDATE_CDR:
			{
				aw_cdr_getotafile* getOtaFile = (aw_cdr_getotafile*)buf;
				db_msg("ip:%s, port:%d", getOtaFile->ippath, atoi(getOtaFile->port));
				int ret = ((AWNetInterface *)mCdrServer->getSession())->OTAUpdateDownload(getOtaFile->ippath, atoi(getOtaFile->port), UPDATE_FILE_PATH);
			}
			break;
		case NAT_CMD_FACTORY_RESET:
			{
				db_msg("NAT_CMD_FACTORY_RESET " );
				aw_cdr_format_tf_card_resp resp;
				resp.result = 0;
				mCdrServer->reply(sid, NAT_CMD_FACTORY_RESET_RESP, 
					(char *)(&resp), sizeof(aw_cdr_format_tf_card_resp));
				int curSessionCam = mRecordPreview->getCurSessionCam();
				mRecordPreview->stopRecord(curSessionCam);
				mMenu->doFactoryReset();
			}
			break;
		case AW_IOTYPE_USER_IPCAM_OTA_UPDATE_DOWNLOAD_OVER:
			aw_cdr_update_cdr resp;
			resp.result = 1;
			mCdrServer->reply(sid, NAT_CMD_UPDATE_CDR_RESP, 
				(char *)(&resp), sizeof(aw_cdr_update_cdr));
			if(ret == 0){
				SendMessage(mHwnd, MSG_OTA_UPDATE, 0, 0);
			}
			break;
#ifdef APP_WIFI
        case NAT_CMD_TURN_OFF_WIFI:
            mSoftApController->DisableSoftAp();
			break;
		case NAT_CMD_SET_WIFI_PWD:
			{
				aw_cdr_wifi_setting *wifi_info = (aw_cdr_wifi_setting*)buf;
				db_msg("********wifi_info->ssid is %s   wifi_info->new_pwd is %s",wifi_info->ssid,wifi_info->new_pwd);
                mSoftApController->ResetSoftAp(wifi_info->ssid,wifi_info->new_pwd);
			}
			break;
#endif
		default:
			break;
	}
	return rm->sessionControl(sid, cmd, buf, mCdrServer);
}

int CdrMain::lockFile(const char *file)
{
	aw_cdr_sos_filename filename;
	memset(&filename, 0, sizeof(aw_cdr_sos_filename));
	printf("CdrMain::lockFile file: %s\n", file);
	strcpy(filename.filename, file);
	return sessionNotify(NAT_CMD_EVENT_SOS_FILE, (char *)(&filename), 
		sizeof(aw_cdr_sos_filename), 0);
}

int CdrMain::sessionNotify(int cmd, char* param_in, int param_len, int sid)
{
	int ret = 0;
#ifdef APP_WIFI
	if (mCdrServer) {
		ret = mCdrServer->notify(cmd, param_in, param_len, sid);
	}
#endif
	return ret;
}

bool CdrMain::isWifiConnecting()
{
	return mSessionConnect;
}

void CdrMain::otaUpdate()
{
	db_msg("***getpid=%d***",getpid());
	if(mRecordPreview->isRecording()){
		mRecordPreview->stopRecord();
	}
	mRecordPreview->stopPreview();
	int err =cfgDataSetString("ctl.start", "OTAstandby");
	if(err != 0){
		db_msg("start OTAstandby service false");
	}else{
		mPM->setEnterOTAService(false);
		usleep(3000 * 1000);
		mPM->connect2StandbyService();
		mPM->enterStandby(getpid());
	}
}

bool CdrMain::getIsLowPower()
{
	if(mLowPower<=LOW_POWER) {
		return TRUE;
	}else{
		return false;
	}
}

void CdrMain::keepAdasScreenFlag()
{
	if(getCurWindowID()== WINDOWID_RECORDPREVIEW&& mRecordPreview->getIsAdasScreen()){
		SendMessage(getWindowHandle(WINDOWID_RECORDPREVIEW),MSG_ADAS_SCREEN,0,0);
		mOriginAdasScreen = true;
	}
}

void CdrMain::recoverAdasScreenFlag()
{
	if(mShutdowning)
		return;
	if(mOriginAdasScreen){
		SendMessage(getWindowHandle(WINDOWID_RECORDPREVIEW),MSG_ADAS_SCREEN,1,0);
		mOriginAdasScreen = false;
	}
}

void CdrMain::TinyPlayAudio(char *path)
{
    char player[128]="tinyplay ";
    system("tinymix 16 1");
    system("tinymix 1 29");
    strcat(player, path);
    system(player);
}

void CdrMain::setNoUSBCharge(bool val)
{
	mNoUSBCharge = val;
}

bool CdrMain::getNoUsbCharge()
{
	return mNoUSBCharge;
}

void CdrMain::setUsbCharge(bool flag)
{
	mUSBcharge = flag;
	db_msg("******USBcharge:%d",mUSBcharge);
}

bool CdrMain::getUsbcharge()
{
	return mUSBcharge;
}

RecordPreview * CdrMain::getRecordPreview()
{
	return mRecordPreview;
}

void * CdrMain::getObjPreview(unsigned int windowID)
{
	void * ret = NULL;
	switch(windowID) {
	case WINDOWID_RECORDPREVIEW:
		ret = mRecordPreview;
		break;
	case WINDOWID_PLAYBACKPREVIEW:
		ret = mPlayBackPreview;
		break;
	case WINDOWID_PLAYBACK:
		ret =  mPlayBack;
		break;
	case WINDOWID_MENU:
		ret =  mMenu;
		break;
	case WINDOWID_STATUSBAR:
		ret =  mSTB;
		break;
	default:
		db_error("invalid window id: %d\n", windowID);
	}

	return ret;

}

void CdrMain::setFirstACConnect(bool val)
{
	mFiACConnect = val;
}

bool CdrMain::getIfDeleFisrtFile()
{
	if(mFiACConnect && mNoUSBCharge)
		return true;
	else
		return false;
}

void CdrMain::startPCCam()
{
	db_msg(" ");
	mPCCam = true;
	if (isRecording()) {
		mRecordPreview->stopRecordSync();
	}
	mEM->setExtraDetect(true);
	mRecordPreview->startPCCam();
}

void CdrMain::stopPCCam()
{
	db_msg(" ");
	mPCCam = false;
	mNoUSBCharge = false;
	mRecordPreview->stopPCCam();
	mEM->setExtraDetect(false);
}

bool CdrMain::getPCCamMode()
{
	return mPCCam;
}

EventManager * CdrMain::getEventManager()
{
	return mEM;
}


void CdrMain::setm2pcAdasScreen(bool flag)
{
	m2pcAdasScreen = flag;
}

int CdrMain::cdrReplyToApp(int cmd, char *buf, int buf_len)
{
	int ret = -1;
#ifdef APP_WIFI
	if(mCdrServer)
		ret = mCdrServer->cdrReplytoApp(cmd, buf, buf_len);
#endif
	return ret;
}

void CdrMain::handBeginDownloadBuf(char *file_list)
{
	aw_cdr_begin_download_file_list* beginFileList;
	beginFileList= ((aw_cdr_begin_download_file_list*)file_list);

	char *ptr1, *ptr2;
	char val[128] = {0};
	StorageManager* sm = StorageManager::getInstance();
	ptr1= beginFileList->filelist;
	while(ptr1 != NULL){
		memset(val, 0, sizeof(val));
		ptr1 = strstr(ptr1, "/");
		ptr2 = strstr(ptr1, ";");
		if(ptr2 == NULL){
			strcpy(val, ptr1);
		}else if((ptr2-ptr1)>0){
			strncpy(val, ptr1, (ptr2-ptr1));
		}
		ptr1 = ptr2;
		if(val != NULL){
			String8 picFile;
			char* file_type = sm->getFileType(val);
			sm->geneNetPicName(String8(val), picFile, EXT_PIC_JPG);
			if(!FILE_EXSIST(picFile.string())){
				db_msg("picFile.string():%s", picFile.string());
				mPlayBackPreview->getNetPicFileName(picFile, String8(val), file_type);
			}
		}
	}
}

bool CdrMain::getIsShowUpdateTips()
{
	return mShowUpdateTips;
}

bool CdrMain::isToRecordPreview()
{
	if(mSessionSubMenu || mCarBack){
		return true;
	}else{
		return false;
	}
}

int CDRMain(int argc, const char *argv[], CdrMain* cdrMain)
{
	struct timespec measureTime1;
	clock_gettime(CLOCK_MONOTONIC, &measureTime1);
	db_msg("Test Time: int CDRMain time is %ld secs, %ld nsecs\n", measureTime1.tv_sec, measureTime1.tv_nsec);

	if(cdrMain->createMainWindows() < 0) {
		delete cdrMain;
		return -1;
	}
	clock_gettime(CLOCK_MONOTONIC, &measureTime1);
	db_msg("Test Time: after createMainWindows time is %ld secs, %ld nsecs\n", measureTime1.tv_sec, measureTime1.tv_nsec);

	cdrMain->initStage2(NULL);
	clock_gettime(CLOCK_MONOTONIC, &measureTime1);
	db_msg("Test Time: after initStage2 time is %ld secs, %ld nsecs\n", measureTime1.tv_sec, measureTime1.tv_nsec);

	cdrMain->msgLoop();

	delete cdrMain;
	return 0;
}

#ifdef _MGRM_THREADS
#include <minigui/dti.c>
#endif
