
#include "Menu.h"
#undef LOG_TAG
#define	 LOG_TAG "Menu.cpp"
#include "debug.h"
#include "licensePlateWM.h"

#define IDC_MENULIST			400

using namespace android;

int MenuProc(HWND hWnd, int message, WPARAM wParam, LPARAM lParam)
{
	Menu* menu;
	static int formatTf =0,isFormatTimer=0;
	switch(message) {
	case MSG_CREATE:
		menu = (Menu*)GetWindowAdditionalData(hWnd);
		if(menu->createSubWidgets(hWnd) < 0) {
			db_error("create subWidgets failed\n");
			break;
		}
		/* for switch ui Bug: ui from others to RecordPreview will not refresh */
		RECT rect;
		HWND hMainWin;
		HDC secDC, MainWindowDC;

		hMainWin = GetMainWindowHandle(hWnd);
		GetClientRect(hMainWin, &rect);
		secDC = GetSecondaryDC(hMainWin);
		MainWindowDC = GetDC(hMainWin);

		SetMemDCAlpha(secDC, 0, 0);
		BitBlt(secDC, 0, 0, RECTW(rect), RECTH(rect), secDC, 0, 0, 0);

		ReleaseDC(MainWindowDC);
		break;
	case MSG_FONTCHANGED:
		menu = (Menu*)GetWindowAdditionalData(hWnd);
		menu->updateMenuTexts();
		menu->updateWindowFont();
		break;
	case MSG_SHOWWINDOW:
		if(wParam == SW_SHOWNORMAL) {
			/* 因为创建MenuList的时候没有指定WS_VISIBLE的属性(防止启动的时候闪屏)， 所以需要调用ShowWindow */
			ShowWindow(GetDlgItem(hWnd, IDC_MENULIST), SW_SHOWNORMAL);
			SetFocusChild(GetDlgItem(hWnd, IDC_MENULIST));
#ifdef USE_IPS_SCREEN
            UpdateWindow(hWnd, true);
#endif
		} else if (wParam == SW_HIDE) {
			UpdateWindow(hWnd, true);
		}
		break;
	case MWM_CHANGE_FROM_WINDOW:
		SendMessage(GetDlgItem(hWnd, IDC_MENULIST), LB_SETCURSEL, 0, 0);
		menu = (Menu*)GetWindowAdditionalData(hWnd);
		menu->updateMenuTexts();
		menu->updateSwitchIcons();
		menu->updateCheckBoxImage(0);
		break;
	case MLM_NEW_SELECTED:
		menu = (Menu*)GetWindowAdditionalData(hWnd);
		menu->updateNewSelected(wParam, lParam);
		break;
	case MSG_CDR_KEY:
		menu = (Menu*)GetWindowAdditionalData(hWnd);
		return menu->keyProc(wParam, lParam);
	case MSG_TIMER:
		if(wParam == FORMATTF_TIMER){
			formatTf++;
			if(formatTf == 5){
				db_msg("******formatTf:%d*******",formatTf);
				BroadcastMessage(MSG_CLOSE_TIP_LABEL, 0, 0);
				KillTimer(hWnd,FORMATTF_TIMER); 
				isFormatTimer = 0;
			}
		}
		break;
	case MSG_FORMAT_TFDIALOG:
	{
		formatTf = 0;
		if(isFormatTimer == 0){
			SetTimer(hWnd,FORMATTF_TIMER,100);
			isFormatTimer = -1;
		}
		menu = (Menu*)GetWindowAdditionalData(hWnd);
		if((bool)wParam){
			SendMessage(menu->getCdrMain()->getWindowHandle(WINDOWID_RECORDPREVIEW),MSG_ADAS_SCREEN,0,0);
		}
		BroadcastMessage(MSG_CLOSE_TIP_LABEL, 0, 0);
		int retval = menu->ShowMessageBox(MENU_INDEX_FORMAT,100);
		if (retval == IDC_BUTTON_OK){
			menu->ShowFormattingTip(100);	
			menu->getCdrMain()->setNeedCheckSD(false);
		}	
		if((bool)wParam && !menu->getCdrMain()->isShuttingdown()){
			SendMessage(menu->getCdrMain()->getWindowHandle(WINDOWID_RECORDPREVIEW),MSG_ADAS_SCREEN,1,0);
		}	
		break;
	}
	/*
	case MSG_PAINT:
		{
			RECT rect;
			HWND hMainWin;
			HDC secDC, MainWindowDC;

			hMainWin = GetMainWindowHandle(hWnd);
			GetClientRect(hMainWin, &rect);
			secDC = GetSecondaryDC(hMainWin);
			MainWindowDC = GetDC(hMainWin);

			SetMemDCAlpha(secDC, 0, 0);
			BitBlt(secDC, 0, 0, RECTW(rect), RECTH(rect), secDC, 0, 0, 0);

			ReleaseDC(MainWindowDC);
		}
		break;
	*/
	default:
		break;
	}

	return DefaultWindowProc(hWnd, message, wParam, lParam);
}

Menu::Menu(CdrMain* pCdrMain)
	: mCdrMain(pCdrMain)
	, itemImageArray(NULL)
	, value1ImageArray(NULL)
	, unfoldImageLight()
	, unfoldImageDark()
	, mSubMenu(false)
{
	HWND hParent;
	RECT rect;
	CDR_RECT STBRect;
	ResourceManager* rm;

	itemImageArray = new BITMAP[MENU_LIST_COUNT];
	memset(itemImageArray, 0, sizeof(BITMAP) * MENU_LIST_COUNT);

	value1ImageArray = new BITMAP[MENU_LIST_COUNT];
	memset(value1ImageArray, 0, sizeof(BITMAP) * MENU_LIST_COUNT);

	hParent = pCdrMain->getHwnd();
	rm = ResourceManager::getInstance();
	rm->getResRect(ID_STATUSBAR, STBRect);

	GetWindowRect(hParent, &rect);
	rect.top += (STBRect.h + 0); //+1 have a crack

	mHwnd = CreateWindowEx(WINDOW_MENU, "",
			WS_NONE,
			WS_EX_USEPARENTFONT,
			WINDOWID_MENU,
			rect.left, rect.top, RECTW(rect), RECTH(rect),
			hParent, (DWORD)this);
	if(mHwnd == HWND_INVALID) {
		db_error("create menu window failed\n");
		return;
	}

	ShowWindow(mHwnd, SW_HIDE);
	rm->setHwnd(WINDOWID_MENU, mHwnd);

}

Menu::~Menu()
{
	delete []itemImageArray;
	delete []value1ImageArray;
}

int Menu::createSubWidgets(HWND hWnd)
{
	HWND retWnd;
	RECT rect;
	menuListAttr_t menuListAttr;

	GetClientRect(hWnd, &rect);

	getMenuListAttr(menuListAttr);	

	retWnd = CreateWindowEx(CTRL_CDRMenuList, "", 
			WS_CHILD  | WS_VSCROLL | LBS_USEBITMAP | LBS_HAVE_VALUE,
			WS_EX_NONE | WS_EX_USEPARENTFONT,
			IDC_MENULIST,
			rect.left, rect.top, RECTW(rect), RECTH(rect),
			hWnd, (DWORD)&menuListAttr);
	if(retWnd == HWND_INVALID) {
		db_error("create Menu List failed\n");
		return -1;
	}

	createMenuListContents(retWnd);

	return 0;
}

int Menu::getMenuListAttr(menuListAttr_t &attr)
{
	ResourceManager* rm;

	rm = ResourceManager::getInstance();

	attr.normalBgc = rm->getResColor(ID_MENU_LIST, COLOR_BGC);
	attr.normalFgc = rm->getResColor(ID_MENU_LIST, COLOR_FGC);
	attr.linec = rm->getResColor(ID_MENU_LIST, COLOR_LINEC_ITEM);
	attr.normalStringc = rm->getResColor(ID_MENU_LIST, COLOR_STRINGC_NORMAL);
	attr.normalValuec = rm->getResColor(ID_MENU_LIST, COLOR_VALUEC_NORMAL);
	attr.selectedStringc = rm->getResColor(ID_MENU_LIST, COLOR_STRINGC_SELECTED);
	attr.selectedValuec = rm->getResColor(ID_MENU_LIST, COLOR_VALUEC_SELECTED);
	attr.scrollbarc = rm->getResColor(ID_MENU_LIST, COLOR_SCROLLBARC);
	attr.itemHeight = 50;

	return 0;
}

/*
 * ----------------------------------------------------------------------------------
 * |   Image   |     itemString    |      first value			 |  second value    |
 * ----------------------------------------------------------------------------------
 * | itemImage | Example: 录像质量 | (sting or image) (optional) | image (optional) |
 * ----------------------------------------------------------------------------------
 * */
int Menu::createMenuListContents(HWND hMenuList)
{
	MENULISTITEMINFO menuListII[MENU_LIST_COUNT];

	memset(&menuListII, 0, sizeof(menuListII));

	if(getItemImages(menuListII) < 0) {
		db_error("get item images failed\n");
		return -1;
	}
	if(getItemStrings(menuListII) < 0) {
		db_error("get item strings failed\n");
		return -1;
	}

	if(getFirstValueStrings(menuListII, -1) < 0) {
		db_error("get first value strings failed\n");
		return -1;
	}
	if(getFirstValueImages(menuListII, -1) < 0) {
		db_error("get first value images failed\n");
		return -1;
	}

	if(getSecondValueImages(menuListII, mlFlags) < 0) {
		db_error("get second value images failed\n");
		return -1;
	}

	for(unsigned int i = 0; i < MENU_LIST_COUNT; i++) {
		memcpy( &menuListII[i].flagsEX, &mlFlags[i], sizeof(MLFlags) );
	}

	db_msg("xxxxxxxxxxxxxxxxxx\n");
	SendMessage(hMenuList, LB_MULTIADDITEM, MENU_LIST_COUNT, (LPARAM)menuListII);
	db_msg("xxxxxxxxxxxxxxxxxx\n");
	//	SendMessage(hMenuList, LB_SETITEMHEIGHT,0,50);	/* set the height is 50 pixels*/
#ifdef USE_IPS_SCREEN
    SendMessage(hMenuList, LB_SETITEMHEIGHT,0,60);	/* set the height is 60 pixels*/
#endif
	SendMessage(hMenuList, MLM_HILIGHTED_SPACE, 8, 1);
	db_msg("height is %d\n", SendMessage(hMenuList, LB_GETITEMHEIGHT, 0, 0));

	return 0;
}

int Menu::getItemImages(MENULISTITEMINFO *mlii)
{
	int retval;
	ResourceManager* rm;

	rm = ResourceManager::getInstance();

	/* ++++++++++ load the itemImage bmp ++++++++++ */
	for(unsigned int i = 0; i < MENU_LIST_COUNT; i++) {
		retval = rm->getResBmp(menuResourceID[i], BMPTYPE_UNSELECTED, itemImageArray[i]);	
		if(retval < 0) {
			db_error("get bmp failed, i is %d, resID is %d\n", i, menuResourceID[i]);
			return -1;
		}
		mlii[i].hBmpImage = (DWORD)&itemImageArray[i];
	}
	/* ---------- load the itemImageArray bmp ---------- */

	return 0;
}

int Menu::getItemStrings(MENULISTITEMINFO* mlii)
{
	ResourceManager* rm;

	rm = ResourceManager::getInstance();
	for(unsigned int i = 0; i < MENU_LIST_COUNT; i++) {
		mlii[i].string = (char*)rm->getResMenuItemString(menuResourceID[i]);
		if(mlii[i].string == NULL) {
			db_error("get the %d label string failed\n", i);
			return -1;
		}
	}

	return 0;
}

int Menu::getFirstValueStrings(MENULISTITEMINFO* mlii, int menuIndex)
{
	const char* ptr;
	ResourceManager* rm;

	rm = ResourceManager::getInstance();

	if(menuIndex == -1) {
		for(unsigned int i = 0; i < MENU_LIST_COUNT ; i++) {
			if(haveValueString[i] == 1) {
				ptr = rm->getResSubMenuCurString(menuResourceID[i]);
				if(ptr == NULL) {
					db_error("get ResSubMenuString %d failed\n", i);
					return -1;
				}
				mlii[ i ].hValue[0] = (DWORD)ptr;
			}
		}
	} else if(menuIndex >=0 && menuIndex < MENU_LIST_COUNT ){
		if(haveSubMenu[menuIndex]) {
			ptr = rm->getResSubMenuCurString(menuResourceID[menuIndex]);
			if(ptr == NULL) {
				db_error("get ResSubMenuString %d failed\n", menuIndex);
				return -1;
			}
			mlii->hValue[0] = (DWORD)ptr;
		}
	}

	return 0;
}


int Menu::getFirstValueImages(MENULISTITEMINFO* mlii, int menuIndex)
{
	int retval;
	int current;
	ResourceManager* rm;
	HWND hMenuList;

	rm = ResourceManager::getInstance();

	if(menuIndex == -1) {
		for(unsigned int i = 0; i < MENU_LIST_COUNT; i++) {
			if(haveCheckBox[i] == 1) {
				retval = rm->getResBmpSubMenuCheckbox( menuResourceID[i], false, value1ImageArray[i] );
				if(retval < 0) {
					db_error("get first value images failed, i is %d\n", i);
					return -1;
				}
				mlii[i].hValue[0] = (DWORD)&value1ImageArray[i];
			}
		}
	} else if(menuIndex >= 0 && menuIndex < MENU_LIST_COUNT ) {
		if(mHwnd && mHwnd != HWND_INVALID) {
			hMenuList = GetDlgItem(mHwnd, IDC_MENULIST);
			current = SendMessage(hMenuList, LB_GETCURSEL, 0, 0);
			rm->unloadBitMap(value1ImageArray[menuIndex]);
			if(current == menuIndex) {
				retval = rm->getResBmpSubMenuCheckbox( menuResourceID[menuIndex], true, value1ImageArray[menuIndex] );
			} else
				retval = rm->getResBmpSubMenuCheckbox( menuResourceID[menuIndex], false, value1ImageArray[menuIndex] );
			if(retval < 0) {
				db_error("get first value images failed, menuIndex is %d\n", menuIndex);
				return -1;
			}
		}
	}

	return 0;
}

int Menu::getSecondValueImages(MENULISTITEMINFO* mlii, MLFlags* mlFlags)
{
	int retval;
	ResourceManager* rm;

	rm = ResourceManager::getInstance();

	retval = rm->getResBmp(ID_MENU_LIST_UNFOLD_PIC, BMPTYPE_UNSELECTED, unfoldImageLight);
	if(retval < 0) {
		db_error("get secondValue image failed\n");
		return -1;
	}
	retval = rm->getResBmp(ID_MENU_LIST_UNFOLD_PIC, BMPTYPE_SELECTED, unfoldImageDark);
	if(retval < 0) {
		db_error("get secondValue image failed\n");
		return -1;
	}

	for(unsigned int i = 0; i < MENU_LIST_COUNT; i++) {
		if(mlFlags[i].valueCount == 2) {
			mlii[i].hValue[1] = (DWORD)&unfoldImageLight;
		}
	}

	return 0;
}


int Menu::updateNewSelected(int oldSel, int newSel)
{
	HWND hMenuList;
	MENULISTITEMINFO mlii;
	ResourceManager* rm;
	int retval;

	db_msg("xxxxxoldSel is %d, newSel is %d\n", oldSel, newSel);

	if(oldSel >= MENU_LIST_COUNT || newSel >= MENU_LIST_COUNT)
		return -1;

	if(oldSel == newSel)
		return 0;

	if(mHwnd == 0 || mHwnd == HWND_INVALID)
		return -1;

	db_msg("xxxxxxxxxxx\n");

	rm = ResourceManager::getInstance();
	hMenuList = GetDlgItem(mHwnd, IDC_MENULIST);

	if(oldSel >= 0) {
		if(SendMessage(hMenuList, LB_GETITEMDATA, oldSel, (LPARAM)&mlii) != LB_OKAY) {
			db_error("get item info failed\n");
			return -1;
		}
		rm->unloadBitMap(itemImageArray[oldSel]);
		retval = rm->getResBmp(menuResourceID[oldSel] ,BMPTYPE_UNSELECTED, itemImageArray[oldSel]);
		if(retval < 0) {
			db_error("get item image failed, oldSel is %d\n", oldSel);
			return -1;
		}
		mlii.hBmpImage = (DWORD)&itemImageArray[oldSel];

		if(haveCheckBox[oldSel] == 1) {
			rm->unloadBitMap(value1ImageArray[oldSel]);
			retval = rm->getResBmpSubMenuCheckbox(menuResourceID[oldSel], false, value1ImageArray[oldSel]);
			if(retval < 0) {
				db_error("get resBmp SubMenu Check box failed, oldSel is %d\n", oldSel);
				return -1;
			}
			mlii.hValue[0] = (DWORD)&value1ImageArray[oldSel];
		} else if(mlii.flagsEX.valueCount == 2) {
			mlii.hValue[1] = (DWORD)&unfoldImageLight;
		}

		db_msg("xxxxxxxxx\n");
		SendMessage(hMenuList, LB_SETITEMDATA, oldSel, (LPARAM)&mlii);
		db_msg("xxxxxxxxx\n");
	}

	if(newSel >= 0) {
		if(SendMessage(hMenuList, LB_GETITEMDATA, newSel, (LPARAM)&mlii) != LB_OKAY) {
			db_error("get item info failed\n");
			return -1;
		}
		rm->unloadBitMap(itemImageArray[newSel]);
		retval = rm->getResBmp(menuResourceID[newSel],BMPTYPE_SELECTED, itemImageArray[newSel]);
		if(retval < 0) {
			db_error("get item image failed, oldSel is %d\n", oldSel);
			return -1;
		}
		mlii.hBmpImage = (DWORD)&itemImageArray[newSel];

		if(haveCheckBox[newSel] == 1) {
			rm->unloadBitMap(value1ImageArray[newSel]);
			retval = rm->getResBmpSubMenuCheckbox(menuResourceID[newSel], true, value1ImageArray[newSel]);
			if(retval < 0) {
				db_error("get resBmp SubMenu Check box failed, oldSel is %d\n", oldSel);
				return -1;
			}
			mlii.hValue[0] = (DWORD)&value1ImageArray[newSel];
		} else if(mlii.flagsEX.valueCount == 2) {
			mlii.hValue[1] = (DWORD)&unfoldImageDark;
		}

		SendMessage(hMenuList, LB_SETITEMDATA, newSel, (LPARAM)&mlii);
	}

	return 0;
}

int Menu::showLicensePlateWM(void)
{
	int retval;
	const char* ptr;
	int len;

	ResourceManager* rm;
	LicensePlateWM_data_t dlgData;
	BITMAP bmpTitle;
	BITMAP bmpEnableChoice;
	BITMAP bmpDisableChoice;
	BITMAP bmpArrow;

	rm = ResourceManager::getInstance();
	memset(&dlgData, 0, sizeof(dlgData));
	dlgData.pLogFont = GetWindowFont(mHwnd);
	dlgData.dlgRect.x = 20;
	dlgData.dlgRect.y = 20;
	dlgData.dlgRect.w = 280;
	dlgData.dlgRect.h = 200;
//v3s
	dlgData.titleText = rm->getLabel(LANG_LABEL_MENU_LICENSE_PLATE_WM);
	dlgData.enableText = "";
	dlgData.disableText = "";

	LoadBitmapFromFile(HDC_SCREEN, &bmpTitle, "system/res/menu/license_plate_wm.png");
	LoadBitmapFromFile(HDC_SCREEN, &bmpEnableChoice, "system/res/menu/Select.png");
	LoadBitmapFromFile(HDC_SCREEN, &bmpDisableChoice, "system/res/menu/unselect.png");
	LoadBitmapFromFile(HDC_SCREEN, &bmpArrow, "system/res/menu/arrow.png");
	dlgData.pBmpTitle = &bmpTitle;
	dlgData.pBmpEnableChioce = &bmpEnableChoice;
	dlgData.pBmpDisableChioce = &bmpDisableChoice;
	dlgData.pBmpArrow = &bmpArrow;

	ptr = rm->getLabel(LANG_LABEL_LICENSE_PLATE_NUMBERS_EN);
	if(ptr == NULL)
		return -1;
	len = strlen(ptr);
	strncpy(dlgData.candidateEnNumbers, ptr, (len > CANDICATE_BUFFER_LEN) ? CANDICATE_BUFFER_LEN : len );
	ptr = rm->getLabel(LANG_LABEL_LICENSE_PLATE_NUMBERS_CN);
	if(ptr == NULL)
		return -1;
	len = strlen(ptr);
	strncpy(dlgData.candidateCnNumbers, ptr, (len > CANDICATE_BUFFER_LEN) ? CANDICATE_BUFFER_LEN : len );
	dlgData.language = rm->getLanguage();
	showAlphaEffect();
	retval = licensePlateWM_Dialog(mHwnd, &dlgData);
	cancelAlphaEffect();
	UnloadBitmap(&bmpTitle);
	UnloadBitmap(&bmpEnableChoice);
	UnloadBitmap(&bmpDisableChoice);
	UnloadBitmap(&bmpArrow);

	return retval;
}

int Menu::keyProc(int keyCode, int isLongPress)
{
	switch(keyCode) {
	case CDR_KEY_MODE:
		{
			ResourceManager* rm;	
			rm = ResourceManager::getInstance();
			rm->syncConfigureToDisk();
			return WINDOWID_RECORDPREVIEW;
		}
	case CDR_KEY_OK:
		{
			HWND hMenuList;
			unsigned int selectedItem;
			int newSel, retval;
			bool isModified, isChecked;
			StorageManager *sm = StorageManager::getInstance();
			hMenuList = GetDlgItem(mHwnd, IDC_MENULIST);
			if(isLongPress == SHORT_PRESS) {
				selectedItem = SendMessage(hMenuList, LB_GETCURSEL, 0, 0);
				db_msg("selectedItem is %d\n", selectedItem);
				switch(selectedItem) {
				case MENU_INDEX_VQ:	case MENU_INDEX_PQ:			case MENU_INDEX_VTL: 
				case MENU_INDEX_WB:	case MENU_INDEX_CONTRAST:	case MENU_INDEX_EXPOSURE:
				case MENU_INDEX_SS:	case MENU_INDEX_LANG:		case MENU_INDEX_GSENSOR:
				case MENU_INDEX_VOICEVOL: 						case MENU_INDEX_SHUTDOWN:
				case MENU_INDEX_IMPACTLEVEL:	case MENU_INDEX_FCWSENSITY:
#ifdef APP_ADAS
				case MENU_INDEX_SMARTALGORITHM:
#endif
					if( ( newSel = ShowSubMenu(selectedItem, isModified) ) < 0) {
						db_error("show submenu %d failed\n", selectedItem);
						break;
					}
					if(isModified == true)
						HandleSubMenuChange(selectedItem, newSel);
					break;
				case MENU_INDEX_POR:	case MENU_INDEX_SILENTMODE:	
				case MENU_INDEX_TWM:	case MENU_INDEX_AWMD:
				case MENU_INDEX_PARK:   case MENU_INDEX_ALIGNLINE:
#ifdef APP_WIFI
				case MENU_INDEX_WIFI:
#endif
					if(getCheckBoxStatus(selectedItem, isChecked) < 0) {
						db_error("get check box status failed\n");
						break;
					}
					HandleSubMenuChange(selectedItem, isChecked);
					break;
				case MENU_INDEX_DATE:
					ShowDateDialog();
					break;
				case MENU_INDEX_FORMAT:
				case MENU_INDEX_FRESET:
					if(selectedItem == MENU_INDEX_FORMAT) {
						if (!sm->isInsert()) {
							showAlphaEffect();
							ShowTipLabel(mHwnd, LABEL_NO_TFCARD);
							cancelAlphaEffect();
							break;
						}
					}
					retval = ShowMessageBox(selectedItem);
					if(retval < 0) {
						db_error("ShowMessageBox err, retval is %d\n", retval);
						break;
					}
					if(retval == IDC_BUTTON_OK) {
						if(selectedItem == MENU_INDEX_FRESET)
							doFactoryReset();
						else if(selectedItem == MENU_INDEX_FORMAT) {
							StorageManager* sm;
							sm = StorageManager::getInstance();
							if(sm->isInsert() == false) {
								db_msg("tfcard not mount\n");
								ShowTipLabel(mHwnd, LABEL_NO_TFCARD);
								break;
							}
							ShowFormattingTip();
						}
					}
					break;
				#ifdef PLATE_WATERMARK
				case MENU_INDEX_LICENSE_PLATE_WM:
					showLicensePlateWM();
					break;
				#endif
				default:
					break;
				}
			}
		}
		break;
	default:
		break;
	}
	if(mCdrMain->isToRecordPreview()){
		return WINDOWID_RECORDPREVIEW;
	}else{
		return WINDOWID_MENU;
	}
}

int Menu::ShowSubMenu(unsigned int menuIndex, bool &isModified)
{
	int retval;
	int oldSel;
	subMenuData_t subMenuData;

	if(getSubMenuData(menuIndex, subMenuData) < 0) {
		db_error("get submenu data failed\n");
		return -1;
	}
	oldSel = subMenuData.selectedIndex;

	showAlphaEffect();
	retval = showSubMenu(mHwnd, &subMenuData);
	db_msg("retval = %d\n", retval);
	cancelAlphaEffect();

	if(retval >= 0) {
		if(retval != oldSel) {
			db_msg("change from %d to %d\n", oldSel, retval);
			isModified = true;
		}
	}

	return retval;
}

int Menu::getCheckBoxStatus(unsigned int menuIndex, bool &isChecked)
{
	ResourceManager* rm;
	bool curStatus;

	rm = ResourceManager::getInstance();
	if(menuIndex >= MENU_LIST_COUNT )
		return -1;
	if(haveCheckBox[menuIndex] != 1) {
		db_error("menuIndex: %d, don not have checkbox\n", menuIndex);
		return -1;
	}

	curStatus = rm->getResBoolValue(menuResourceID[menuIndex]);
	isChecked = ( (curStatus == true) ? false : true );
	return 0;
}

int Menu::getSubMenuData(unsigned int subMenuIndex, subMenuData_t &subMenuData)
{
	int retval;
	int contentCount;
	const char* ptr;
	ResourceManager* rm;

	if(haveSubMenu[subMenuIndex] != 1) {
		db_error("invalid index %d\n", subMenuIndex);
		return -1;
	}

	rm = ResourceManager::getInstance();

	retval = rm->getResRect(ID_SUBMENU, subMenuData.rect);
	if(retval < 0) {
		db_error("get subMenu rect failed\n");
		return -1;
	}
	retval = rm->getResBmp(ID_SUBMENU_CHOICE_PIC, BMPTYPE_BASE, subMenuData.BmpChoice);
	if(retval < 0) {
		db_error("get subMenu choice bmp failed\n");
		return -1;
	}

	getMenuListAttr(subMenuData.menuListAttr);
	subMenuData.lincTitle = rm->getResColor(ID_SUBMENU, COLOR_LINEC_TITLE);
	subMenuData.pLogFont = rm->getLogFont();

	/* current submenu index */
	retval = rm->getResIntValue(menuResourceID[subMenuIndex], INTVAL_SUBMENU_INDEX );
	if(retval < 0) {
		db_error("get res submenu index failed\n");
		return -1;
	}
	subMenuData.selectedIndex = retval;

	/* submenu count */
	contentCount = rm->getResIntValue(menuResourceID[subMenuIndex], INTVAL_SUBMENU_COUNT);

	/* submenu title */
	ptr = rm->getResSubMenuTitle(menuResourceID[subMenuIndex]);
	if(ptr == NULL) {
		db_error("get video quality title failed\n");
		return -1;
	}
	subMenuData.title = ptr;

	/* submenu contents */
	for(int i = 0; i < contentCount; i++) {
		ptr = rm->getLabel(subMenuContent0Cmd[subMenuIndex] + i);
		if(ptr == NULL) {
			db_error("get video quality content %d failed", i);
			return -1;
		}
		subMenuData.contents.add(String8(ptr));
	}

	return 0;
}

void Menu::showAlphaEffect(void)
{
	RECT rect;
	HWND hMainWin;
	HDC secDC, MainWindowDC;

	hMainWin = GetMainWindowHandle(mHwnd);
	GetClientRect(hMainWin, &rect);
	secDC = GetSecondaryDC(hMainWin);
	MainWindowDC = GetDC(hMainWin);

	db_msg("rect.left is %d\n", rect.left);
	db_msg("rect.top is %d\n", rect.top);
	db_msg("rect.right is %d\n", rect.right);
	db_msg("rect.bottom is %d\n", rect.bottom);

	SetMemDCAlpha(secDC, MEMDC_FLAG_SRCALPHA, 180);
	ShowWindow(mHwnd, SW_HIDE);
	ShowWindow(mCdrMain->getWindowHandle(WINDOWID_STATUSBAR), SW_HIDE);

	BitBlt(secDC, 0, 0, RECTW(rect), RECTH(rect), MainWindowDC, 0, 0, 0);

	ReleaseDC(MainWindowDC);
	setSubMenuFlag(true);
}

void Menu::cancelAlphaEffect(void)
{
	RECT rect;
	HWND hMainWin;
	HDC secDC, MainWindowDC;

	hMainWin = GetMainWindowHandle(mHwnd);
	GetClientRect(hMainWin, &rect);
	secDC = GetSecondaryDC(hMainWin);
	MainWindowDC = GetDC(hMainWin);

	SetMemDCAlpha(secDC, 0, 0);
	BitBlt(secDC, 0, 0, RECTW(rect), RECTH(rect), secDC, 0, 0, 0);
	ShowWindow(mHwnd, SW_SHOWNORMAL);
	ShowWindow(mCdrMain->getWindowHandle(WINDOWID_STATUSBAR), SW_SHOWNORMAL);
	SetSecondaryDC(mHwnd, secDC, ON_UPDSECDC_DEFAULT);

	ReleaseDC(MainWindowDC);
	setSubMenuFlag(false);  //check whether to shutdown
}

int Menu::HandleSubMenuChange(unsigned int menuIndex, int newSel)
{
	ResourceManager* rm;

	rm = ResourceManager::getInstance();

	switch(menuIndex) {
	case MENU_INDEX_LANG:	
	case MENU_INDEX_VQ:	
	case MENU_INDEX_PQ:	
	case MENU_INDEX_VTL:	
	case MENU_INDEX_WB:	
	case MENU_INDEX_CONTRAST:	
	case MENU_INDEX_EXPOSURE:	
	case MENU_INDEX_SS:	
	case MENU_INDEX_GSENSOR:	
	case MENU_INDEX_VOICEVOL:
	case MENU_INDEX_SHUTDOWN:
	case MENU_INDEX_IMPACTLEVEL:
	case MENU_INDEX_FCWSENSITY:
#ifdef APP_ADAS
	case MENU_INDEX_SMARTALGORITHM:
#endif
		if(haveSubMenu[menuIndex] != 1) {
			db_error("invalid menuIndex %d\n", menuIndex);
			return -1;
		}
		if(rm->setResIntValue(menuResourceID[menuIndex], INTVAL_SUBMENU_INDEX, newSel) < 0) {
			db_error("set %d to %d failed\n", menuIndex, newSel);	
			return -1;
		}
		break;
	default:
		db_error("invalid menuIndex %d\n", menuIndex);
		return -1;
	}

	HWND hMenuList;
	MENULISTITEMINFO mlii;
	hMenuList = GetDlgItem(mHwnd, IDC_MENULIST);

	if(menuIndex != MENU_INDEX_LANG) {
		if(SendMessage(hMenuList, LB_GETITEMDATA, menuIndex, (LPARAM)&mlii) != LB_OKAY) {
			db_error("get item info failed, menuIndex is %d\n", menuIndex);
			return -1;
		}

		if(getFirstValueStrings(&mlii, menuIndex) < 0) {
			db_error("get first value strings failed\n");
			return -1;
		}
		db_msg("xxxxxxxx\n");
		SendMessage(hMenuList, LB_SETITEMDATA, menuIndex, (LPARAM)&mlii);
		db_msg("xxxxxxxx\n");
	}

	return 0;
}

int Menu::HandleSubMenuChange(unsigned int menuIndex, bool newValue)
{
	ResourceManager* rm;

	rm = ResourceManager::getInstance();
	switch(menuIndex) {
	case MENU_INDEX_POR:
	case MENU_INDEX_SILENTMODE:
	case MENU_INDEX_TWM:
	case MENU_INDEX_AWMD:
	case MENU_INDEX_PARK:
#ifdef APP_WIFI
	case MENU_INDEX_WIFI:
#endif
	case MENU_INDEX_ALIGNLINE:
		if(haveCheckBox[menuIndex] != 1) {
			db_error("invalid menuIndex %d\n", menuIndex);
			return -1;
		}
		if(rm->setResBoolValue(menuResourceID[menuIndex], newValue) < 0) {
			db_error("set %d to %d failed\n", menuIndex, newValue);
			return -1;
		}
		break;
	default:
		db_error("invalid menuIndex %d\n", menuIndex);
		return -1;
	}

	HWND hMenuList;
	MENULISTITEMINFO mlii;
	hMenuList = GetDlgItem(mHwnd, IDC_MENULIST);
	memset(&mlii, 0, sizeof(mlii));
	if(SendMessage(hMenuList, LB_GETITEMDATA, menuIndex, (LPARAM)&mlii) != LB_OKAY) {
		db_error("get item info failed, menuIndex is %d\n", menuIndex);
		return -1;
	}
	if(getFirstValueImages(&mlii, menuIndex) < 0) {
		db_error("get first value images failed, menuIndex is %d\n", menuIndex);
		return -1;
	}

	db_msg("xxxxxxxx\n");
	bool wififlag = newValue;
	SendMessage(hMenuList, LB_SETITEMDATA, menuIndex, (LPARAM)&mlii);
	db_msg("xxxxxxxx\n");

	return 0;
}


int Menu::updateMenuTexts(void)
{
	int itemCount;
	HWND hMenuList;
	MENULISTITEMINFO menuListII[MENU_LIST_COUNT];

	hMenuList = GetDlgItem(mHwnd, IDC_MENULIST);
	itemCount = SendMessage(hMenuList, LB_GETCOUNT, 0, 0);
	if(itemCount < 0) {
		db_error("get menu counts failed\n");
		return -1;
	}

	memset(menuListII, 0, sizeof(menuListII));
	for(int count = 0; count < itemCount; count++) {
		if( SendMessage(hMenuList, LB_GETITEMDATA, count, (LPARAM)&menuListII[count]) != LB_OKAY) {
			db_msg("get the %d item data failed\n", count);
			return -1;
		}
	}

	if(getItemStrings(menuListII) < 0) {
		db_error("get item strings failed\n");
		return -1;
	}
	if(getFirstValueStrings(menuListII, -1) < 0) {
		db_error("get item value strings failed\n");
		return -1;
	}

	db_msg("xxxxxxxxxx\n");
	for(int count = 0; count < itemCount; count++) {
		SendMessage(hMenuList, LB_SETITEMDATA, count, (LPARAM)&menuListII[count] );
		db_msg("xxxxxxxxxx\n");
	}

	return 0;
}

int Menu::updateSwitchIcons(void)
{
	int itemCount;
	HWND hMenuList;
	MENULISTITEMINFO menuListII[MENU_LIST_COUNT];

	hMenuList = GetDlgItem(mHwnd, IDC_MENULIST);
	itemCount = SendMessage(hMenuList, LB_GETCOUNT, 0, 0);
	if(itemCount < 0) {
		db_error("get menu counts failed\n");
		return -1;
	}

	memset(menuListII, 0, sizeof(menuListII));
	for(int count = 0; count < itemCount; count++) {
		if( SendMessage(hMenuList, LB_GETITEMDATA, count, (LPARAM)&menuListII[count]) != LB_OKAY) {
			db_msg("get the %d item data failed\n", count);
			return -1;
		}
	}

	getFirstValueImages(menuListII, -1);
	db_msg("xxxxxxxxxx\n");
	for(int count = 0; count < itemCount; count++) {
		SendMessage(hMenuList, LB_SETITEMDATA, count, (LPARAM)&menuListII[count] );
		db_msg("xxxxxxxxxx\n");
	}

	return 0;
}


int Menu::getMessageBoxData(unsigned int menuIndex, MessageBox_t &messageBoxData)
{
	const char* ptr;
	unsigned int i;
	ResourceManager* rm;

	unsigned int indexTable [] = {
		MENU_INDEX_FORMAT,
		MENU_INDEX_FRESET
	};

	unsigned int resCmd[][2] = {
		{LANG_LABEL_SUBMENU_FORMAT_TITLE, LANG_LABEL_SUBMENU_FORMAT_TEXT},
		{LANG_LABEL_SUBMENU_FRESET_TITLE, LANG_LABEL_SUBMENU_FRESET_TEXT},
	};


	rm = ResourceManager::getInstance();

	messageBoxData.dwStyle = MB_OKCANCEL | MB_HAVE_TITLE | MB_HAVE_TEXT;
	messageBoxData.flag_end_key = 1; /* end the dialog when endkey keyup */

	for(i = 0; i < TABLESIZE(indexTable); i++) {
		if(indexTable[i] == menuIndex) {
			ptr = rm->getLabel(resCmd[i][0]);
			if(ptr == NULL) {
				db_error("get title failed, menuIndex is %d\n", menuIndex);
				return -1;
			}
			messageBoxData.title = ptr;

			ptr = rm->getLabel(resCmd[i][1]);
			if(ptr == NULL) {
				db_error("get text failed, menuIndex is %d\n", menuIndex);
				return -1;
			}
			messageBoxData.text = ptr;

			ptr = rm->getLabel(LANG_LABEL_SUBMENU_OK);
			if(ptr == NULL) {
				db_error("get ok string failed, menuIndex is %d\n", menuIndex);
				return -1;
			}
			messageBoxData.buttonStr[0] = ptr;

			ptr = rm->getLabel(LANG_LABEL_SUBMENU_CANCEL);
			if(ptr == NULL) {
				db_error("get ok string failed, menuIndex is %d\n", menuIndex);
				return -1;
			}
			messageBoxData.buttonStr[1] = ptr;
		}
		db_msg("xxxxxxxx\n");
	}

	if(i > TABLESIZE(indexTable)) {
		db_error("invalid menuIndex: %d\n", menuIndex);
		return -1;
	}

	messageBoxData.pLogFont = rm->getLogFont();

	rm->getResRect(ID_MENU_LIST_MB, messageBoxData.rect);
	messageBoxData.fgcWidget = rm->getResColor(ID_MENU_LIST_MB, COLOR_FGC);
	messageBoxData.bgcWidget = rm->getResColor(ID_MENU_LIST_MB, COLOR_BGC);
	messageBoxData.linecTitle = rm->getResColor(ID_MENU_LIST_MB, COLOR_LINEC_TITLE);
	messageBoxData.linecItem = rm->getResColor(ID_MENU_LIST_MB, COLOR_LINEC_ITEM);

	return 0;
}

static void formatCallback(HWND hDlg, void* data)
{
	Menu* menu;
	menu = (Menu*)data;
	db_msg("formatMessageBoxCallback\n");

	menu->doFormatTFCard();
	db_msg("formatMessageBoxCallback finish\n");
	EndDialog(hDlg, IDC_BUTTON_OK);
}

int Menu::ShowFormattingTip(int val)
{
	tipLabelData_t tipLabelData;
	ResourceManager* rm;
	int retval;

	rm = ResourceManager::getInstance();
	memset(&tipLabelData, 0, sizeof(tipLabelData));
	if(getTipLabelData(&tipLabelData) < 0) {
		db_error("get TipLabel data failed\n");	
		return -1;
	}

	tipLabelData.title = rm->getLabel(LANG_LABEL_TIPS);
	if(!tipLabelData.title) {
		db_error("get FormattingTip titile failed\n");
		return -1;
	}

	tipLabelData.text = rm->getLabel(LANG_LABEL_SUBMENU_FORMATTING_TEXT);
		if(!tipLabelData.text) {
			db_error("get LANG_LABEL_LOW_POWER_TEXT failed\n");
			return -1;
	}

	tipLabelData.timeoutMs = TIME_INFINITE;
	tipLabelData.disableKeyEvent = true;
	tipLabelData.callback = formatCallback;
	tipLabelData.callbackData = this;
	if(val==0)
		showAlphaEffect();
	retval = showTipLabel(mHwnd, &tipLabelData);
	if(val==0)
		cancelAlphaEffect();

	return retval;
}

int Menu::ShowMessageBox(unsigned int menuIndex,int val)
{
	int retval;
	MessageBox_t messageBoxData;
	ResourceManager* rm;

	rm = ResourceManager::getInstance();
	memset(&messageBoxData, 0, sizeof(messageBoxData));
	if(getMessageBoxData(menuIndex, messageBoxData) < 0) {
		db_error("get messagebox data failed\n");
		return -1;
	}
	if(val==0)
		showAlphaEffect();
	if(val==100){
		ResourceManager* rm = ResourceManager::getInstance();
		messageBoxData.text = rm->getLabel(LANG_LABEL_FORMAT_SDCARD);

	}
	retval = showMessageBox(mHwnd, &messageBoxData);
	if(val==0)
	    cancelAlphaEffect();

	db_msg("retval is %d\n", retval);
	return retval;
}

int Menu::ShowDateDialog(void)
{
	int retval;
	ResourceManager* rm;
	dateSettingData_t configData;

	rm = ResourceManager::getInstance();

	configData.title = rm->getLabel(LANG_LABEL_DATE_TITLE);
	configData.year = rm->getLabel(LANG_LABEL_DATE_YEAR);
	configData.month = rm->getLabel(LANG_LABEL_DATE_MONTH);
	configData.day = rm->getLabel(LANG_LABEL_DATE_DAY);
	configData.hour = rm->getLabel(LANG_LABEL_DATE_HOUR);
	configData.minute = rm->getLabel(LANG_LABEL_DATE_MINUTE);
	configData.second = rm->getLabel(LANG_LABEL_DATE_SECOND);
	if(configData.title == NULL)
		return -1;
	if(configData.year == NULL)
		return -1;
	if(configData.month == NULL)
		return -1;
	if(configData.day == NULL)
		return -1;
	if(configData.hour == NULL)
		return -1;
	if(configData.minute == NULL)
		return -1;
	if(configData.second == NULL)
		return -1;

	retval = rm->getResRect(ID_MENU_LIST_DATE, configData.rect);
	if(retval < 0) {
		db_error("get date rect failed\n");
		return -1;
	}

	configData.titleRectH = rm->getResIntValue(ID_MENU_LIST_DATE, INTVAL_TITLEHEIGHT);
	configData.hBorder	= rm->getResIntValue(ID_MENU_LIST_DATE, INTVAL_HBORDER);
	configData.yearW	= rm->getResIntValue(ID_MENU_LIST_DATE, INTVAL_YEARWIDTH);
	configData.numberW	= rm->getResIntValue(ID_MENU_LIST_DATE, INTVAL_NUMBERWIDTH);
	configData.dateLabelW = rm->getResIntValue(ID_MENU_LIST_DATE, INTVAL_LABELWIDTH);
	configData.boxH	= rm->getResIntValue(ID_MENU_LIST_DATE, INTVAL_BOXHEIGHT);
	if(configData.titleRectH < 0)
		return -1;
	if(configData.hBorder < 0)
		return -1;
	if(configData.yearW < 0)
		return -1;
	if(configData.numberW < 0)
		return -1;
	if(configData.dateLabelW < 0)
		return -1;
	if(configData.boxH < 0)
		return -1;

	configData.bgc_widget = rm->getResColor(ID_MENU_LIST_DATE, COLOR_BGC);
	configData.fgc_label = rm->getResColor(ID_MENU_LIST_DATE, COLOR_FGC_LABEL);
	configData.fgc_number = rm->getResColor(ID_MENU_LIST_DATE, COLOR_FGC_NUMBER);
	configData.linec_title = rm->getResColor(ID_MENU_LIST_DATE, COLOR_LINEC_TITLE);
	configData.borderc_selected = rm->getResColor(ID_MENU_LIST_DATE, COLOR_BORDERC_SELECTED);
	configData.borderc_normal = rm->getResColor(ID_MENU_LIST_DATE, COLOR_BORDERC_NORMAL);
	configData.pLogFont = rm->getLogFont();

	showAlphaEffect();
	retval = dateSettingDialog(mHwnd, &configData);
	cancelAlphaEffect();
	return retval;
}

void Menu::doFactoryReset(void)
{
	db_msg("ido factory reset\n");
	ResourceManager *rm = ResourceManager::getInstance();
	rm->resetResource();
	//resetDateTime();
	updateMenuTexts();
	updateSwitchIcons();
}

void Menu::finish(int what, int extra)
{
	db_error("-----formatfinish!|n");
	mCdrMain->forbidRecord(FORBID_NORMAL);
}

void Menu::doFormatTFCard(void)
{
	StorageManager *sm = StorageManager::getInstance();
	db_msg("do format TF card\n");
	sm->format(this);
	mCdrMain->forbidRecord(FORBID_FORMAT);
	finish(0, 0);
}

CdrMain * Menu::getCdrMain()
{
	if(mCdrMain)
		return mCdrMain;
	return NULL;
}

void Menu::setSubMenuFlag(bool flag)
{
	if (!flag) {
		mCdrMain->checkShutdown();
	}
	mSubMenu = flag;

}

bool Menu::getSubMenuFlag()
{
	return mSubMenu;
}

int Menu::updateCheckBoxImage(int newSel)
{

	HWND hMenuList;
	MENULISTITEMINFO mlii;
	ResourceManager* rm;
	int retval;
	hMenuList = GetDlgItem(mHwnd, IDC_MENULIST);
	rm = ResourceManager::getInstance();
	if(newSel >= 0) {
		if(SendMessage(hMenuList, LB_GETITEMDATA, newSel, (LPARAM)&mlii) != LB_OKAY) {
			db_error("get item info failed\n");
			return -1;
		}
		rm->unloadBitMap(itemImageArray[newSel]);
		retval = rm->getResBmp(menuResourceID[newSel],BMPTYPE_SELECTED, itemImageArray[newSel]);
		if(retval < 0) {
			db_error("get item image failed, oldSel is \n");
			return -1;
		}
		mlii.hBmpImage = (DWORD)&itemImageArray[newSel];

		if(haveCheckBox[newSel] == 1) {
			rm->unloadBitMap(value1ImageArray[newSel]);
			retval = rm->getResBmpSubMenuCheckbox(menuResourceID[newSel], true, value1ImageArray[newSel]);
			if(retval < 0) {
				db_error("get resBmp SubMenu Check box failed, oldSel is \n");
				return -1;
			}
			mlii.hValue[0] = (DWORD)&value1ImageArray[newSel];
		}
		SendMessage(hMenuList, LB_SETITEMDATA, newSel, (LPARAM)&mlii);
	}
	return 0;
}

