#include <minigui/common.h>
#include <minigui/minigui.h>
#include <minigui/gdi.h>
#include <minigui/window.h>
#include <minigui/control.h>

#include <binder/IMemory.h>
#include <include_media/media/HerbMediaMetadataRetriever.h>
#include <private/media/VideoFrame.h>
#include <CedarMediaServerCaller.h>

#include "windows.h"
#include "keyEvent.h"
#include "cdr_message.h"
#include "StorageManager.h"
#include "Dialog.h"
#undef LOG_TAG
#define LOG_TAG "PlayBackPreview.cpp"
#include "debug.h"

#define EMBEDDED_PICTURE_TYPE_ANY		0xFFFF
#define ONT_SHOT_TIMER	100

using namespace android;
int PlayBackPreview::keyProc(int keyCode, int isLongPress)
{
	HWND hStatusBar;
	hStatusBar = mCdrMain->getWindowHandle(WINDOWID_STATUSBAR);
	switch(keyCode) {
	case CDR_KEY_OK:
		if(isLongPress == SHORT_PRESS) {			
			return playBackVideo();
		}
		break;
	case CDR_KEY_MODE:
		if(isLongPress == SHORT_PRESS) {
			db_msg("go to RecordPreview\n");
			SendMessage(hStatusBar, STBM_CLEAR_LABEL1_TEXT, WINDOWID_PLAYBACKPREVIEW, 0);
#ifdef USE_IPS_SCREEN
			ShowWindow(bottomHwnd, SW_HIDE);
            HWND hStatusBar = mCdrMain->getWindowHandle(WINDOWID_STATUSBAR);
            SendMessage(hStatusBar, STBM_SHOW_TIMEHWND, 1, 0);
#endif
			return WINDOWID_RECORDPREVIEW;
		} else if(isLongPress == LONG_PRESS) {
			db_msg("long press");
			StorageManager* sm;
			sm = StorageManager::getInstance();
			if(sm->isMount() == false) {
				ShowTipLabel(mHwnd, LABEL_NO_TFCARD, false);
				break;
			}
			if(fileInfo.fileCounts == 0) {
				db_msg("no file to delete\n");	
				ShowTipLabel(mHwnd, LABEL_FILELIST_EMPTY, false);
			} else
				deleteFileDialog();
		}
		break;
	case CDR_KEY_LEFT:
		if(isLongPress == SHORT_PRESS)
			prevPreviewFile();
		break;
	case CDR_KEY_RIGHT:
		if(isLongPress == SHORT_PRESS)
			nextPreviewFile();
		break;
	default:
		break;
	}
	return WINDOWID_PLAYBACKPREVIEW;
}

int PlayBackPreview::msgCallback(int message, int data)
{
	int ret = 0;
	switch(message) {
	case MSG_RM_DEL_FILE:
		{
			const char *file = (const char *)data;
			StorageManager *sm = StorageManager::getInstance();
			if(!sm->dbIsFileExist(String8(file))) {
				return -1;
			}
			ret = sm->dbDelFile(file);
			if (ret != 0) {
				db_error("fail to delete file %s from db", file);
				return ret;
			}
			ret = sm->deleteFile(file);
			if (ret != 0) {
				db_error("fail to delete file %s", file);
			}
		}
		break;
	default:
		break;
	}
	return ret;
}

int PlayBackPreviewProc(HWND hWnd, int message, WPARAM wParam, LPARAM lParam)
{
	PlayBackPreview* mPLBPreview = (PlayBackPreview*)GetWindowAdditionalData(hWnd);
	int ret = 0;
	db_msg("message code: [%d], %s, wParam=%d, lParam=%lu\n", message, Message2Str(message), wParam, lParam);
	switch(message) {
	case MSG_CREATE:
		mPLBPreview->createSubWidgets(hWnd);
		break;
	case MSG_SHOWWINDOW:
		if(wParam == SW_SHOWNORMAL) {
			mPLBPreview->showPlaybackPreview();
#ifdef USE_IPS_SCREEN
           // UpdateWindow(hWnd, true);
			ShowWindow(mPLBPreview->bottomHwnd, SW_SHOWNORMAL);
#endif
		}
		break;
	case MSG_CDR_KEY:
		db_msg("key code is %d\n", wParam);
		return mPLBPreview->keyProc(wParam, lParam);
	case MSG_UPDATE_LIST:
		{
			int ret = mPLBPreview->updatePreviewFilelist();
			if(ret == 0) {
				mPLBPreview->showThumnail();
			} else if(ret == 1) {
				mPLBPreview->showBlank();
			}
		}
		break;
	case MSG_SET_CURRENT_FILEID:
		mPLBPreview->mCurID = (int)wParam;
		break;
	case MSG_RM_GET_FILELIST:
		{
			*((char**)wParam) = (char*)malloc(1024);
			ret = mPLBPreview->getFileList((char **)wParam, 1024, (enum FOLDER_TYPE)lParam);
		}
		break;
	default:
		ret = mPLBPreview->msgCallback(message, (int)wParam);
		break;
	}
	if (ret != 0) {
		return ret;
	}
	return DefaultWindowProc(hWnd, message, wParam, lParam);
}

	PlayBackPreview::PlayBackPreview(CdrMain* pCdrMain)
	: mCdrMain(pCdrMain)
	, bmpIcon()
	, bmpImage()
	  , fileInfo()
{
	HWND hParent;
	RECT rect;
	CDR_RECT STBRect;
	ResourceManager* mRM;
	hParent = mCdrMain->getHwnd();
	mRM = ResourceManager::getInstance();
	mRM->getResRect(ID_STATUSBAR, STBRect);
	GetWindowRect(hParent, &rect);
	rect.top += (STBRect.h + 0); //+1 will produce gap
	mHwnd = CreateWindowEx(WINDOW_PLAYBACKPREVIEW, "",
			WS_NONE,
#ifdef USE_IPS_SCREEN
			WS_EX_USEPARENTFONT,
#else
            WS_EX_TRANSPARENT | WS_EX_USEPARENTFONT,
#endif
			WINDOWID_PLAYBACKPREVIEW,
#ifdef 	USE_IPS_SCREEN
			rect.left, rect.top, RECTW(rect), RECTH(rect)-4,
#else
			rect.left, rect.top, RECTW(rect), RECTH(rect),
#endif
			hParent, (DWORD)this);
	if(mHwnd == HWND_INVALID) {
		db_error("create PlayBack Preview window failed\n");
		return;
	}
	mRM->setHwnd(WINDOWID_PLAYBACKPREVIEW, mHwnd);
	mCurID = 0;
	ShowWindow(mHwnd, SW_HIDE);

#ifdef  USE_IPS_SCREEN
	/*
		create bottomHwnd for cover the white line in the bootom
		when changwindow PlaybackPreview to RecordPreview
	*/
	int width, height;
	getScreenInfo(&width, &height);	
	bottomHwnd = CreateWindowEx(CTRL_STATIC, "",
			WS_CHILD | WS_VISIBLE ,
		    WS_EX_USEPARENTFONT,
			ID_PLAYBACKPREVIEW_BOTTOMBAR,
			0, height-4, width, 4,
			hParent, 0);
	SetWindowBkColor(bottomHwnd, RGBA2Pixel(HDC_SCREEN, 0x00, 0x00, 0x00, 0xff));
	ShowWindow(bottomHwnd, SW_HIDE);
#endif
}
PlayBackPreview::~PlayBackPreview()
{
}

int PlayBackPreview::createSubWidgets(HWND hParent)
{
	HWND hWnd;
	ResourceManager* mRM;
	CDR_RECT rect;
	int retval;

	mRM = ResourceManager::getInstance();
	mRM->getResRect(ID_PLAYBACKPREVIEW_IMAGE, rect);

	hWnd = CreateWindowEx(CTRL_STATIC, "",
			WS_VISIBLE | WS_CHILD | SS_BITMAP | SS_CENTERIMAGE,
			WS_EX_NONE,
			ID_PLAYBACKPREVIEW_IMAGE,
			rect.x, rect.y, rect.w, rect.h,
			hParent, 0);

	if(hWnd == HWND_INVALID) {
		db_error("create playback previe image label failed\n");
		return -1;
	}

	mRM->getResRect(ID_PLAYBACKPREVIEW_ICON, rect);

	retval = mRM->getResBmp(ID_PLAYBACKPREVIEW_ICON, BMPTYPE_BASE, bmpIcon);
	if(retval < 0) {
		db_error("get RES_BMP_CURRENT_PLBPREVIEW_ICON failed\n");
		return -1;
	}

	hWnd = CreateWindowEx(CTRL_STATIC, "",
			WS_VISIBLE | WS_CHILD | SS_BITMAP | SS_CENTERIMAGE,
			WS_EX_NONE | WS_EX_TRANSPARENT,
			ID_PLAYBACKPREVIEW_ICON,
			rect.x, rect.y, bmpIcon.bmWidth, bmpIcon.bmHeight,
			hParent, (LPARAM)&bmpIcon);
	if(hWnd == HWND_INVALID) {
		db_error("create playback preview image label failed\n");
		return -1;
	}
	//	ShowWindow(hWnd, SW_HIDE);

	return 0;
}

int PlayBackPreview::getFileList(char **file_list, int len, enum FOLDER_TYPE folder_type)
{
	int retval;
	StorageManager* mSTM;
	dbRecord_t dbRecord;
	int cnt = 0;
	int cur_file_list_len = 0;
	int file_len = 0;
	char *str;
	int buf_size = len;
	char *netstr;
	char netPic[128];
	int netPic_len = 0;

	db_msg(" folder_type:%d ", folder_type);
	mSTM = StorageManager::getInstance();
	if (mSTM->isMount() == false) {
		db_msg(" tf card is not mount");
		**file_list = 0;
		return 1;
	}
	retval = mSTM->dbCount((char*)TYPE_VIDEO);
	if(retval < 0) {
		db_error("get db count failed\n");
		return -1;
	}
	cnt += retval;

	retval = mSTM->dbCount((char*)TYPE_PIC);
	if(retval < 0) {
		db_error("get db count failed\n");
		return -1;
	}
	cnt += retval;
	**file_list = 0;
	if (cnt == 0) {
		return 0;
	}
	char sFileSize[32]={0};
	char sNetFileSize[32]={0};
	for(int i=0; i<cnt; i++) {
		if(mSTM->dbGetSortByTimeRecord(dbRecord, i) < 0) {
			db_error("get sort by time record failed\n");
			return -1;
		}
		str = (char*)(dbRecord.file.string());
		if(folder_type != ALL_FOLDER){
			enum FOLDER_TYPE folder = (enum FOLDER_TYPE)mSTM->judgeFileFolder(str);
			if(folder_type != folder)
				continue;
		}

		file_len = strlen(str);
		sprintf(sFileSize, "%d", mSTM->fileSize(String8(str)));

		memset(netPic, 0, sizeof(netPic));
		memset(sNetFileSize, 0, sizeof(sNetFileSize));
		mSTM->geneNetPicName(String8(str), netPic, EXT_PIC_JPG);
		if(FILE_EXSIST(netPic)){
			netPic_len = strlen(netPic);
			sprintf(sNetFileSize, "%d", mSTM->fileSize(String8(netPic)));
		}else{
			netPic_len = 1;
			sprintf(netPic, "0");
			sprintf(sNetFileSize, "0");
			db_msg(" cdr have no %s's netpic, maybe not create ", netPic);
		}
		cur_file_list_len = cur_file_list_len + file_len + strlen(sFileSize) + strlen(":") + strlen(";") +
			netPic_len + strlen(sNetFileSize) + strlen(":")*2;

		if (cur_file_list_len >= buf_size) {

			buf_size += 1024;
			db_error("realloc %d\n", buf_size);
			*file_list = (char*)realloc(*file_list, sizeof(char)*(buf_size));
			if(*file_list == NULL) {
				db_msg("  xxx  xx  ");
				return -1;
			}
		}
		strncat(*file_list, str, file_len);
		strncat(*file_list, ":", 1);
		strncat(*file_list, sFileSize, strlen(sFileSize));

		strncat(*file_list, ":", 1);
		strncat(*file_list, netPic, netPic_len);
		strncat(*file_list, ":", 1);
		strncat(*file_list, sNetFileSize, strlen(sNetFileSize));

		strncat(*file_list, ";", 1);
	}
	*(*file_list + cur_file_list_len) = 0;

	return 0;
}
	
int PlayBackPreview::getFileInfo(void)
{
	int retval;
	StorageManager* mSTM;
	dbRecord_t dbRecord;
	mSTM = StorageManager::getInstance();
	retval = mSTM->dbCount((char*)TYPE_VIDEO);
	if(retval < 0) {
		db_error("get db count failed\n");
		return -1;
	}
	db_msg("video fileCounts is %d\n", retval);
	fileInfo.fileCounts = retval;

	retval = mSTM->dbCount((char*)TYPE_PIC);
	if(retval < 0) {
		db_error("get db count failed\n");
		return -1;
	}
	db_msg("photo fileCounts is %d\n", retval);
	fileInfo.fileCounts += retval;

	if(fileInfo.fileCounts == 0) {
		db_msg("empty database\n");	
		return 0;
	}
	int idx = fileInfo.curIdx;
	bool desc = true;
	if (desc) {
		idx = fileInfo.fileCounts - 1 - fileInfo.curIdx;
	}
	db_msg("curIdx is %d, actual idx %d\n", fileInfo.curIdx, idx);
	if(mSTM->dbGetSortByTimeRecord(dbRecord, (idx)) < 0) {
		db_error("get sort by time record failed\n");
		return -1;
	}
	fileInfo.curFileName = dbRecord.file;
	fileInfo.curTypeString = dbRecord.type;
	fileInfo.curInfoString = dbRecord.info;
	db_msg("cur file is %s\n", fileInfo.curFileName.string());
	db_msg("cur Type is %s\n", fileInfo.curTypeString.string());
	db_msg("cur info is %s\n", fileInfo.curInfoString.string());
	return 0;
}

static void simplifyFileName(char *name)
{
	if (!name) {
		return ;
	}
	char *ptr = NULL;
	ptr = strrchr(name, '.');
	*ptr = 0;
}

int PlayBackPreview::updatePreviewFilelist(void)
{
	HWND hStatusBar;
	char buf[50] = {0};
	char buf2[50] = {0};
	int ret = 0;
	if(getFileInfo() < 0)
		return -1;
	StorageManager *sm = StorageManager::getInstance();
	if(!sm->isMount() || (fileInfo.fileCounts == 0)) {
		sprintf(buf, "(00/00)");
		ret = 1;
	} else {
		if(getBaseName(fileInfo.curFileName.string(), buf2) != NULL)
			simplifyFileName(buf2);
			sprintf(buf, "%s(%02d/%02d)", buf2+strlen("2014"), fileInfo.curIdx + 1, fileInfo.fileCounts);
	}
	hStatusBar = mCdrMain->getWindowHandle(WINDOWID_STATUSBAR);
#ifdef USE_IPS_SCREEN    
	SendMessage(hStatusBar, STBM_SETLABEL2_TEXT, 0, (LPARAM)buf);
#else
    SendMessage(hStatusBar, STBM_SETLABEL1_TEXT, 0, (LPARAM)buf);
#endif
	return ret;
}
int PlayBackPreview::getPicFileName(String8 &bmpFile)
{
	char buf[128] = {0};
	char buf2[128] = {0};
	char picFile[128] = {0};
	int ret = 0;
#ifndef HWDECODE_JPEG
	if(strncmp(fileInfo.curTypeString.string(), TYPE_PIC, 5) == 0) {
		bmpFile = fileInfo.curFileName.string();
	} else {
#else
	if(strncmp(fileInfo.curTypeString.string(), TYPE_PIC, 5) == 0) {	//generate pic thumbnail
		//bmpFile = fileInfo.curFileName.string();
		/* hardware decode jpeg */
		if(getBaseName(fileInfo.curFileName.string(), buf2) == NULL) {
			db_error("get base name failed, file is %s\n", fileInfo.curFileName.string());
			return -1;
		}
		if(removeSuffix(buf2, buf) == NULL) {
			db_error("remove the file suffix failed, file is %s\n", fileInfo.curFileName.string());
			return -1;
		}
		sprintf(picFile, "%s%s.bmp", CON_3STRING(MOUNT_PATH, PIC_DIR, THUMBNAIL_DIR),buf);
		if(access(CON_3STRING(MOUNT_PATH, PIC_DIR, THUMBNAIL_DIR), F_OK) != 0) {
			if(mkdir(CON_3STRING(MOUNT_PATH, PIC_DIR, THUMBNAIL_DIR), 0777) != 0) {
				db_error("create dir %s failed\n", CON_3STRING(MOUNT_PATH, PIC_DIR, THUMBNAIL_DIR));
				return -1;
			}
		}
		if(access(picFile , F_OK) != 0) {
			CedarMediaServerCaller *caller = new CedarMediaServerCaller();
//			int fd = open(fileInfo.curFileName.string(), O_RDWR);
//			if (fd < 0) {
//				db_error("fail to read %s", fileInfo.curFileName.string());
//			}
			if (caller == NULL) {
				//close(fd);
			    ALOGE("Failed to alloc CedarMediaServerCaller");
				return -1;
			}
			sp<IMemory> mem = caller->jpegDecode(fileInfo.curFileName);
			if (mem == NULL) {
			    ALOGE("jpegDecode error!");
			    delete caller;
				//close(fd);
			    return -1;
			}
			//close(fd);
			VideoFrame *videoFrame = NULL;
			videoFrame = static_cast<VideoFrame *>(mem->pointer());
			if (videoFrame == NULL) {
				db_error("getFrameAtTime: videoFrame is a NULL pointer");
				ret = -1;
				return ret;
			}
			StorageManager *sm = StorageManager::getInstance();
			ret = sm->saveVideoFrameToBmpFile(videoFrame, picFile);
		}
		bmpFile = String8(picFile);
	} else { //generate video thumbnail
#endif
		if(getBaseName(fileInfo.curFileName.string(), buf2) == NULL) {
			db_error("get base name failed, file is %s\n", fileInfo.curFileName.string());
			return -1;
		}
		db_msg("basename is %s\n", buf2);
		if(removeSuffix(buf2, buf) == NULL) {
			db_error("remove the file suffix failed, file is %s\n", fileInfo.curFileName.string());
			return -1;
		}
		db_msg("buf is %s\n", buf);

		//		if(strncmp(fileInfo.curInfoString.string(), "normal", 6) == 0) {
		sprintf(picFile, "%s%s.bmp", CON_3STRING(MOUNT_PATH, VIDEO_DIR, THUMBNAIL_DIR), buf);
		db_msg("picFile is %s\n", picFile);
		if(access(CON_3STRING(MOUNT_PATH, VIDEO_DIR, THUMBNAIL_DIR), F_OK) != 0) {
			if(mkdir(CON_3STRING(MOUNT_PATH, VIDEO_DIR, THUMBNAIL_DIR), 0777) != 0) {
				db_error("create dir %s failed\n", CON_3STRING(MOUNT_PATH, VIDEO_DIR, THUMBNAIL_DIR));
				return -1;
			}
		}
		//		} else {
		//			sprintf(picFile, "%s%s.bmp", CON_3STRING(MOUNT_PATH, LOCK_DIR, THUMBNAIL_DIR), buf);
		//			if(access(CON_3STRING(MOUNT_PATH, LOCK_DIR, THUMBNAIL_DIR), F_OK) != 0) {
		//				if(mkdir(CON_3STRING(MOUNT_PATH, LOCK_DIR, THUMBNAIL_DIR), 0777) != 0) {
		//					db_error("create dir %s failed\n", CON_3STRING(MOUNT_PATH, LOCK_DIR, THUMBNAIL_DIR));	
		//					return -1;
		//				}
		//			}
		//		}
		//		db_msg("picFile is %s\n", picFile);	
		if(access(picFile , F_OK) != 0) {
			if(getBmpPicFromVideo(fileInfo.curFileName.string(), picFile, false) != 0) {
				db_error("get picture from video failed\n");
				return -1;
			}
		}
		bmpFile = String8(picFile);
	}
	return 0;
}

int PlayBackPreview::getNetPicFileName(String8 &bmpFile, String8 file_name, char* file_type)
{
	char picFile[128] = {0};
	int ret = 0;
	StorageManager *sm = StorageManager::getInstance();
#ifndef HWDECODE_JPEG
	if(strncmp(file_type, TYPE_PIC, 5) == 0) {
		bmpFile = file_name;
	} else {
#else
	if(strncmp(file_type, TYPE_PIC, 5) == 0) {	//generate pic thumbnail
		//bmpFile = fileInfo.curFileName.string();
		/* hardware decode jpeg */
		sm->geneNetPicName(file_name, picFile, EXT_PIC_JPG);
		if(access(picFile , F_OK) != 0) {
			CedarMediaServerCaller *caller = new CedarMediaServerCaller();
//			int fd = open(fileInfo.curFileName.string(), O_RDWR);
//			if (fd < 0) {
//				db_error("fail to read %s", fileInfo.curFileName.string());
//			}
			if (caller == NULL) {
				//close(fd);
			    ALOGE("Failed to alloc CedarMediaServerCaller");
				return -1;
			}
			sp<IMemory> mem = caller->jpegDecode(file_name);
			if (mem == NULL) {
			    ALOGE("jpegDecode error!");
			    delete caller;
				//close(fd);
			    return -1;
			}
			//close(fd);
			VideoFrame *videoFrame = NULL;
			videoFrame = static_cast<VideoFrame *>(mem->pointer());
			if (videoFrame == NULL) {
				db_error("getFrameAtTime: videoFrame is a NULL pointer");
				ret = -1;
				return ret;
			}
			StorageManager *sm = StorageManager::getInstance();
			ret = sm->saveVideoFrameToBmpFile(videoFrame, picFile, EXT_PIC_JPG);
		}
		bmpFile = String8(picFile);
	} else { //generate video thumbnail
#endif
		sm->geneNetPicName(file_name, picFile, EXT_PIC_JPG);
		if(access(picFile , F_OK) != 0) {
			if(getBmpPicFromVideo(file_name.string(), picFile, true) != 0) {
				db_error("get picture from video failed\n");	
				return -1;
			}
		}
		bmpFile = String8(picFile);
	}
	return 0;
}


void PlayBackPreview::showBlank()
{
 	HWND retWnd;
	retWnd = GetDlgItem(mHwnd, ID_PLAYBACKPREVIEW_IMAGE);

	db_msg("erase PLBPREVIEW_IMAGE background!");

	ResourceManager * rs = ResourceManager::getInstance();
	SendMessage(retWnd, STM_SETIMAGE, (WPARAM)(rs->getNoVideoImage()), 0);

	retWnd = GetDlgItem(mHwnd, ID_PLAYBACKPREVIEW_ICON);
	ShowWindow(retWnd, SW_HIDE);
}


void PlayBackPreview::showThumnail(void)
{
	int ret;
	String8 bmpFile;
	HWND retWnd;
	if(fileInfo.fileCounts == 0) {
		db_msg("no file to preview\n");
		return;
	}
	db_msg("current file type is %s\n", fileInfo.curTypeString.string());
	db_msg("current file name is %s\n", fileInfo.curFileName.string());
	if(getPicFileName(bmpFile) < 0)	{
		db_error("get pic file name failed\n");
		showBlank();
		return ;
	}
	if(bmpImage.bmBits)
		UnloadBitmap(&bmpImage);
	if((ret = LoadBitmapFromFile(HDC_SCREEN, &bmpImage, bmpFile.string())) != ERR_BMP_OK) {
		db_error("load %s failed, ret is %d, load once again\n", bmpFile.string(), ret);
		StorageManager* sm = StorageManager::getInstance();
		sm->deleteFile(bmpFile.string());
		if(getPicFileName(bmpFile) < 0)	{
			db_error("get pic file name failed\n");
			showBlank();
			return ;
		}
		if((ret = LoadBitmapFromFile(HDC_SCREEN, &bmpImage, bmpFile.string())) != ERR_BMP_OK) {
			db_error("load %s failed finally, ret is %d\n", bmpFile.string(), ret);
			return;
		}
	}

	retWnd = GetDlgItem(mHwnd, ID_PLAYBACKPREVIEW_IMAGE);
	SendMessage(retWnd, STM_SETIMAGE, (WPARAM)&bmpImage, 0);
	retWnd = GetDlgItem(mHwnd, ID_PLAYBACKPREVIEW_ICON);
	if(strncmp(fileInfo.curTypeString.string(), TYPE_PIC, 5) == 0) {
		if(IsWindowVisible(retWnd)) {
			db_msg("xxxxxxxxxx\n");	
			ShowWindow(retWnd, SW_HIDE);
		}
	} else {
		if(!IsWindowVisible(retWnd)) {
			ShowWindow(retWnd, SW_SHOWNORMAL);
		}
	}
}

int PlayBackPreview::getBmpPicFromVideo(const char* videoFile, const char* picFile, bool netbmp)
{
	status_t ret;
	int retval = 0;
	sp<IMemory> frameMemory = NULL;
	HerbMediaMetadataRetriever *retriever;
	StorageManager* mSTM;

	mSTM = StorageManager::getInstance();
	retriever = new HerbMediaMetadataRetriever();
	if (retriever == NULL) {
		db_error("Failed to alloc HerbMediaMetadataRetriever");
		return -1;
	}

	ret = retriever->setDataSource(videoFile);
	if (ret != NO_ERROR) {
		db_error("setDataSource failed(%d)", ret);
		retval = -1;
		goto out;
	}

	frameMemory = retriever->getEmbeddedPicture(EMBEDDED_PICTURE_TYPE_ANY);
	if (frameMemory != NULL) {
		MediaAlbumArt* mediaAlbumArt = NULL;
		db_msg("Have EmbeddedPicture");
		mediaAlbumArt = static_cast<MediaAlbumArt *>(frameMemory->pointer());
		if (mediaAlbumArt == NULL) {
			db_error("getEmbeddedPicture: Call to getEmbeddedPicture failed.");
			retval = -1;
			goto out;
		}
		//		unsigned int len = mediaAlbumArt->mSize;
		//		char *data = (char*)mediaAlbumArt + sizeof(MediaAlbumArt);
		/*
		 * TODO: how to get the width and height
		 * */
	} else {
		VideoFrame *videoFrame = NULL;
		db_msg("getFrameAtTime");

		if(netbmp){
			frameMemory = retriever->getFrameAtTime(240, 135);
		}else{
			frameMemory = retriever->getFrameAtTime();
		}
		if (frameMemory == NULL) {
			db_error("Failed to get picture");
			retval = -1;
			goto out;
		}

		videoFrame = static_cast<VideoFrame *>(frameMemory->pointer());
		if (videoFrame == NULL) {
			db_error("getFrameAtTime: videoFrame is a NULL pointer");
			retval = -1;
			goto out;
		}

		db_msg("videoFrame: width=%d, height=%d, disp_width=%d,disp_height=%d, size=%d, RotationAngle=%d",
				videoFrame->mWidth, videoFrame->mHeight, videoFrame->mDisplayWidth, videoFrame->mDisplayHeight, videoFrame->mSize, videoFrame->mRotationAngle);

		if(netbmp){
			ret = mSTM->saveVideoFrameToBmpFile(videoFrame, picFile, EXT_PIC_JPG);
		}else{
			ret = mSTM->saveVideoFrameToBmpFile(videoFrame, picFile);
		}
		if (ret != ERR_BMP_OK) {
			db_msg("Failed to save picture, ret is %d", ret);
			retval = -1;
			goto out;
		}
	}

out:
	if(retriever)
		delete retriever;
	return retval;
}

bool PlayBackPreview::isCurrentVideoFile(void)
{
	if(access(fileInfo.curFileName.string(), F_OK) != 0) {
		db_error("the current file %s not exsist\n", fileInfo.curFileName.string());
		return false;
	}
	if(strncmp(fileInfo.curTypeString.string(), TYPE_VIDEO, 5) == 0)
		return true;
	else 
		return false;
}

void PlayBackPreview::nextPreviewFile(void)
{
	if(fileInfo.fileCounts == 0)
		return;

	fileInfo.curIdx++;
	if(fileInfo.curIdx >= fileInfo.fileCounts)
		fileInfo.curIdx = 0;
	int ret = updatePreviewFilelist();
	if(ret == 0) {
		showThumnail();
	} else if(ret == 1) {
		showBlank();
	}
}

void PlayBackPreview::prevPreviewFile(void)
{
	if(fileInfo.fileCounts == 0)
		return;

	if(fileInfo.curIdx == 0)
		fileInfo.curIdx = fileInfo.fileCounts - 1;
	else
		fileInfo.curIdx--;

	int ret = updatePreviewFilelist();
	if(ret == 0) {
		showThumnail();
	} else if(ret == 1) {
		showBlank();
	}
}

void PlayBackPreview::getCurrentFileName(String8 &file)
{
	file = fileInfo.curFileName;
}

void PlayBackPreview::getCurrentFileInfo(PLBPreviewFileInfo_t &info)
{
	info = fileInfo;
}

void PlayBackPreview::showPlaybackPreview()
{
	fileInfo.curIdx = mCurID;
	mCurID=0;
	int ret = updatePreviewFilelist();
	if(ret == 0) {
		showThumnail();
	} else if(ret == 1) {
		showBlank();
	}

}

void PlayBackPreview::fixIdx()
{
	if(fileInfo.fileCounts == 0)
		return;
	if((fileInfo.curIdx + 1) == fileInfo.fileCounts)
		fileInfo.curIdx--;
}

static void deleteDialogCallback(HWND hDlg, void* data)
{
	PlayBackPreview* playbackPreview;
	StorageManager* sm;
	PLBPreviewFileInfo_t fileInfo;
	int ret = 0;
	playbackPreview = (PlayBackPreview*)data;
	sm = StorageManager::getInstance();

	/* delete file */
	playbackPreview->getCurrentFileInfo(fileInfo);
	db_msg("delete the %drd file %s\n", fileInfo.curIdx, fileInfo.curFileName.string());
	ret = sm->deleteFile(fileInfo.curFileName.string());
	ret = sm->dbDelFile(fileInfo.curFileName.string());
	playbackPreview->fixIdx();
	ret = playbackPreview->updatePreviewFilelist();
	if(ret == 0) {
		playbackPreview->showThumnail();
	} else if(ret == 1) {
		playbackPreview->showBlank();
	}

	EndDialog(hDlg, IDC_BUTTON_OK);
}

void PlayBackPreview::deleteFileDialog(void)
{
	int retval;
	const char* ptr;
	MessageBox_t messageBoxData;
	ResourceManager* rm;
	String8 textInfo;

	rm = ResourceManager::getInstance();
	memset(&messageBoxData, 0, sizeof(messageBoxData));
	messageBoxData.dwStyle = MB_OKCANCEL | MB_HAVE_TITLE | MB_HAVE_TEXT;
	messageBoxData.flag_end_key = 0; /* end the dialog when endkey keydown */

	ptr = rm->getLabel(LANG_LABEL_DELETEFILE_TITLE);
	if(ptr == NULL) {
		db_error("get deletefile title failed\n");
		return;
	}
	messageBoxData.title = ptr;

	ptr = rm->getLabel(LANG_LABEL_DELETEFILE_TEXT);
	if(ptr == NULL) {
		db_error("get deletefile text failed\n");
		return;
	}
	messageBoxData.text = ptr;

	ptr = rm->getLabel(LANG_LABEL_SUBMENU_OK);
	if(ptr == NULL) {
		db_error("get ok string failed\n");
		return;
	}
	messageBoxData.buttonStr[0] = ptr;

	ptr = rm->getLabel(LANG_LABEL_SUBMENU_CANCEL);
	if(ptr == NULL) {
		db_error("get ok string failed\n");
		return;
	}
	messageBoxData.buttonStr[1] = ptr;

	messageBoxData.pLogFont = rm->getLogFont();

	rm->getResRect(ID_MENU_LIST_MB, messageBoxData.rect);
	messageBoxData.fgcWidget = rm->getResColor(ID_MENU_LIST_MB, COLOR_FGC);
	messageBoxData.bgcWidget = rm->getResColor(ID_MENU_LIST_MB, COLOR_BGC);
	messageBoxData.linecTitle = rm->getResColor(ID_MENU_LIST_MB, COLOR_LINEC_TITLE);
	messageBoxData.linecItem = rm->getResColor(ID_MENU_LIST_MB, COLOR_LINEC_ITEM);
	messageBoxData.confirmCallback = deleteDialogCallback;
	messageBoxData.confirmData = (void*)this;

	db_msg("this is %p\n", this);
	retval = showMessageBox(mHwnd, &messageBoxData);
	db_msg("retval is %d\n", retval);
}

int PlayBackPreview::playBackVideo(void)
{
	int retval;
	HWND hPlayBack, hStatusBar;
	StorageManager* sm;

	sm = StorageManager::getInstance();
	if(sm->isMount() == false) {
		ShowTipLabel(mHwnd, LABEL_NO_TFCARD);
		goto out;
	}
	if(fileInfo.fileCounts == 0) {
		ShowTipLabel(mHwnd, LABEL_FILELIST_EMPTY);
		goto out;
	}

	if(isCurrentVideoFile() == true) {
		db_msg("start playback\n");
		hPlayBack = mCdrMain->getWindowHandle(WINDOWID_PLAYBACK);
		retval = SendMessage(hPlayBack, MSG_PREPARE_PLAYBACK, (WPARAM)&(fileInfo.curFileName), fileInfo.curIdx);
		if(retval < 0) {
			db_msg("prepare playback failed\n");	
			ShowTipLabel(mHwnd, LABEL_PLAYBACK_FAIL);
			goto out;
		}

		hStatusBar = mCdrMain->getWindowHandle(WINDOWID_STATUSBAR);
		SendMessage(hStatusBar, STBM_CLEAR_LABEL1_TEXT, WINDOWID_PLAYBACKPREVIEW, 0);
		return WINDOWID_PLAYBACK;
	}

out:
	return WINDOWID_PLAYBACKPREVIEW;;
}


int PlayBackPreview::generateNetPic()
{
	dbRecord_t dbRecord;
	int cnt = 0;
	char *str;
	char buf[128];
	char netPic[128];
	int retval = 0;
	StorageManager *sm = StorageManager::getInstance();

	if (sm->isMount() == false) {
		db_msg(" tf card is not mount");
		return 1;
	}
	retval = sm->dbCount((char*)TYPE_VIDEO);
	if(retval < 0) {
		db_error("get db count failed\n");
		return -1;
	}
	cnt += retval;

	retval = sm->dbCount((char*)TYPE_PIC);
	if(retval < 0) {
		db_error("get db count failed\n");
		return -1;
	}
	cnt += retval;
	if (cnt == 0) {
		return 0;
	}
	for(int i=0; i<cnt; i++) {
		if(sm->dbGetSortByTimeRecord(dbRecord, i) < 0) {
			db_error("get sort by time record failed\n");
			return -1;
		}
		str = (char*)(dbRecord.file.string());
		memset(netPic, 0, sizeof(netPic));	
		sm->geneNetPicName(dbRecord.file, netPic, EXT_PIC_BMP);		
		if(!FILE_EXSIST(netPic)){
			char* file_type = sm->getFileType(dbRecord.file.string());
			if(file_type != NULL){
				String8 bmpFile;
				getNetPicFileName(bmpFile, dbRecord.file, file_type);
			}
		}
	}
	return 0;
}


