#include "RecordPreview.h"
#undef LOG_TAG
#define LOG_TAG "RecordPreview"
#include "debug.h"
#include "EventManager.h"
#include "StorageManager.h"

static const char *WHITE_BALANCE[] = {
	"auto",
	"daylight",
	"cloudy-daylight",
	"incandescent",
	"fluorescent"
};

int RecordPreview::keyProc(int keyCode, int isLongPress)
{
	switch(keyCode) {
	case CDR_KEY_OK:
		if(isLongPress == SHORT_PRESS) {
			if(mRPWindowState == STATUS_RECORDPREVIEW || mRPWindowState == STATUS_AWMD) {
				if (isRecording()) {
					db_debug("===============key_ok to stopRecord\n");
					if(mCdrMain->getWakeupState())
						mKeyStop = true;
					stopRecord();
				} else {
					db_debug("===============key_ok to startRecord\n");
					startRecord();
				}
			} else if(mRPWindowState == STATUS_PHOTOGRAPH) {
				mCdrMain->playSounds(CDR_TAKEPICTURE_SOUNDS);
				if (mCamera[CAM_CSI]){
					mCamera[CAM_CSI]->setJpegQuality(NORMAL_JPEG_QUALITY);
					mCamera[CAM_CSI]->setTakepicType(NORMAL_TAKE_PIC);
					mCamera[CAM_CSI]->setPictureSizeMode(CameraParameters::AWEXTEND_PICTURE_SIZE_MODE::UseParameterPictureSize);
				}
				if (mCamera[CAM_UVC]){
					mCamera[CAM_CSI]->setJpegQuality(NORMAL_JPEG_QUALITY);
					mCamera[CAM_UVC]->setTakepicType(NORMAL_TAKE_PIC);
					mCamera[CAM_UVC]->setPictureSizeMode(CameraParameters::AWEXTEND_PICTURE_SIZE_MODE::UseParameterPictureSize);
				}
				takePicture(1, 1);
			}
		}else if((mRPWindowState == STATUS_RECORDPREVIEW) && (isLongPress == LONG_PRESS)) {
			screenShot(); 
		}
		break;
	case CDR_KEY_MODE:
		if(mUVCCarModem)
			break;
        if(mCdrMain->isWifiConnecting()){
            if(isLongPress == LONG_PRESS){
                ResourceManager *rm = ResourceManager::getInstance();
                rm->setResBoolValue(ID_MENU_LIST_WIFI, false);
                mCdrMain->msgCallback(MSG_SESSIONDISCONNECT, 0);
            }
            break;
        }
		if(isLongPress == LONG_PRESS) {
			if( !isRecording() ) {
				mUvcPlugFlag = false;
				if(mRPWindowState != STATUS_AWMD )
					mOldRPWindowState = mRPWindowState;
				else
					mOldRPWindowState =STATUS_RECORDPREVIEW;
				return WINDOWID_MENU;
			}
		} else if(isLongPress == SHORT_PRESS) {
			if((mRPWindowState == STATUS_RECORDPREVIEW || mRPWindowState == STATUS_AWMD)&& !isRecording()) {
				db_debug("photograph\n");
			//	oldPIPMode = mPIPMode;
			//	transformPIP(CSI_ONLY);
				mRPWindowState = STATUS_PHOTOGRAPH;
				mCdrMain->setCurrentWindowState(mRPWindowState);
			} else if(mRPWindowState == STATUS_PHOTOGRAPH) {
				db_debug("go to PlayBackPreview\n");
				mRPWindowState = STATUS_RECORDPREVIEW;
				mCdrMain->setCurrentWindowState(mRPWindowState);					
				mUvcPlugFlag = false;
				if(mRPWindowState != STATUS_AWMD )
					mOldRPWindowState = mRPWindowState;
				else
					mOldRPWindowState =STATUS_RECORDPREVIEW;
#ifdef USE_IPS_SCREEN
                HWND hStatusBar = mCdrMain->getWindowHandle(WINDOWID_STATUSBAR);
                SendMessage(hStatusBar, STBM_SHOW_TIMEHWND, 0, 0);
#endif
				return WINDOWID_PLAYBACKPREVIEW;
			}
		}
		break;
	case CDR_KEY_LEFT:
		if(isLongPress == SHORT_PRESS) {
			if(mUVCCarModem)
				break;
			if (mCamExist[CAM_CSI] && mCamExist[CAM_UVC]) {
				if(mPIPMode == UVC_ON_CSI)
					transformPIP(CSI_ON_UVC);
				else if(mPIPMode == CSI_ON_UVC)
					transformPIP(UVC_ONLY);
				else if(mPIPMode == UVC_ONLY)
					transformPIP(CSI_ONLY);
				else if(mPIPMode == CSI_ONLY)
					transformPIP(UVC_ON_CSI);
			}
		} else if(isLongPress == LONG_PRESS) {
			switchAWMD();
		}
		break;
	case CDR_KEY_RIGHT:
		if(isLongPress == SHORT_PRESS) {
			if(mRPWindowState == STATUS_RECORDPREVIEW)
				switchVoice();
		} else {
			if(isRecording()){
				if(mCamExist[CAM_CSI])
					lockFile(CAM_CSI);
				if(mCamExist[CAM_UVC])
					lockFile(CAM_UVC);
			}
		}
		break;
    case CDR_KEY_MENU:
       {
#ifdef SMALL_VIDEO
            if(isRecording(CAMERA_CSI_NET_SMALL_RECORD)){
                startSmallVideoRecord();
            }
#endif
        }
        break;
	}
	return WINDOWID_RECORDPREVIEW;
}

bool RecordPreview::isPhotoWindow()
{
	return mRPWindowState == STATUS_PHOTOGRAPH;
}

void RecordPreview::lockFile(int camId)
{
	HWND hStatusBar;
	mLockFlag[camId] = !mLockFlag[camId];
	db_msg("Lockflag: %d", mLockFlag[camId]);
	hStatusBar = mCdrMain->getWindowHandle(WINDOWID_STATUSBAR);
	SendMessage(hStatusBar, STBM_LOCK, mLockFlag[camId], 0);
}

void RecordPreview::switchVoice()
{
	bool voiceStatus;
	HWND hStatusBar;
	db_msg("xxxxxxxxxxxxxxxx");
	MediaRecorderObj *obj = getMediaRecordObj(CAMERA_CSI_RECORD);
	CdrMediaRecorder *cmr = obj->getCdrMediaRecorder();
	if (cmr) {
		voiceStatus = cmr->getVoiceStatus();
		hStatusBar = mCdrMain->getWindowHandle(WINDOWID_STATUSBAR);
		SendMessage(hStatusBar, STBM_VOICE, !voiceStatus, 0);
	}
}

int RecordPreviewProc(HWND hWnd, int message, WPARAM wParam, LPARAM lParam)
{

	//char mWaterMark[22];
	//memset(mWaterMark,0,sizeof(mWaterMark));
	//static int mShowAlignT = 1;

	RecordPreview* recordPreview = (RecordPreview*)GetWindowAdditionalData(hWnd);
	//db_msg("message code: [%d], %s, wParam=%d, lParam=%lu\n", message, Message2Str(message), wParam, lParam);
	switch(message) {
	case MSG_CREATE:
		break;
	case MSG_CDR_KEY:
		db_debug("key code is %d\n", wParam);
		return recordPreview->keyProc(wParam, lParam);
	case MSG_DRAW_ALIGN_LINE:
		recordPreview->setAlignLine((bool)wParam);
		break;
	case MSG_ADAS_SCREEN:
		recordPreview->adasScreenSwitch((bool)wParam);
		break;
	case MWM_CHANGE_FROM_WINDOW:
		if(recordPreview->getUvcCarModem()){
			recordPreview->carBackTransPIP(true);
		}else{
			recordPreview->restorePreviewOutSide(wParam);
		}
		usleep(200 * 1000);		/* 让 layer开始渲染后才允许绘制窗口 */
		{
			ResourceManager *rm = ResourceManager::getInstance();
			int val = rm->getResIntValue(ID_MENU_LIST_SMARTALGORITHM, INTVAL_SUBMENU_INDEX);
			if(val == 1 || val == 2){
				recordPreview->adasScreenSwitch(true);
			}else if( val == 0 ){
				recordPreview->adasScreenSwitch(false);
			}
			db_msg("xxxxxxxx\n");
		}
		if(recordPreview->getUvcCarModem()){
			db_msg("xxxxxxxxclose CSI \n");
			recordPreview->closecarBackCSI();
			recordPreview->showUvcCarBackLines(true);
		}
		break;
	case MSG_SHOWWINDOW:
		db_msg("MSG_SHOWWINDOW\n");
		if(wParam == SW_SHOWNORMAL) {
			db_msg("MSG_SHOWWINDOW\n");
			recordPreview->updateAWMDWindowPic();
		}
		break;
	case MSG_SHOW_TIP_LABEL:
		db_msg("MSG_SHOW_TIP_LABEL,wParam = %d",wParam);
		ShowTipLabel(hWnd, (enum labelIndex)wParam, TRUE, 3000);
		break;
	default:
		recordPreview->msgCallback(message, (int)wParam);
	}

	return DefaultWindowProc(hWnd, message, wParam, lParam);
}

void RecordPreview::takePicFinish(int cam_id)
{
	Mutex::Autolock _l(mLockPic);
	mAllowTakePic[cam_id] = true;
}

void RecordPreview::getAdasCallback()
{
	if(mCdrMain->getCurWindowID() == WINDOWID_RECORDPREVIEW){
		adasScreenSwitch(true);
	}
}
#ifdef APP_ADAS

static void adasCallback(void *data,void *caller)
{
		//db_msg("adasCallback-----");
		RecordPreview *rp;
		if(caller == NULL){
				db_error("caller not initialized\n");
				return;
		}
		rp = (RecordPreview *)caller;
		rp->getAdasCallback();
}

#endif
static void jpegCallback(void *data, int size, void* caller, int id)

{
	RecordPreview* rp;
	db_debug("picture call back %d data %p", id, data);
	if(caller == NULL) {
		db_error("caller not initialized\n");
		return;
	}
	rp = (RecordPreview*)caller;

	rp->savePicture(data, size, id);
	rp->takePicFinish(id);
}

static void errorCallback(int error, void *caller)
{
	RecordPreview* rp;
	db_debug("errorCallback error %d", error);
	if(caller == NULL) {
	       db_error("caller not initialized\n");
	       return;
	}
	rp = (RecordPreview*)caller;
	rp->errorOccur(error);
}

static void awmdTimerCallback(sigval_t val)
{
	db_msg("posix timer data: %p\n", val.sival_ptr);
	RecordPreview *rp = (RecordPreview *)val.sival_ptr;
	rp->stopRecord(CAM_CSI);
	rp->getCdrMain()->cdrReplyToApp(NAT_AWMD_RECORD_STOP, NULL, 0);
}

static void awmdCallback(int value, void *pCaller)
{
	RecordPreview *rp = (RecordPreview *)pCaller;
	CdrRecordParam_t param;
	StorageManager *sm = StorageManager::getInstance();
	bool mounted = sm->isMount();
//	db_msg("awmd : value- %d\n", value);
	if(!IsWindowVisible(rp->getHwnd()) || !mounted) {
		return;
	}

	if(rp->needDelayStartAWMD() == true) {
		db_msg("need to delay awmd\n");
		rp->mAwmdDelayCount++;
		/* skip 100 times , after stop record */
		if(rp->mAwmdDelayCount >= 100) {
			db_msg("clear awmd delay flag\n");
			rp->mAwmdDelayCount = 0;
			rp->setDelayStartAWMD(false);
		}
		return;
	}

	if(value == 1) {
		if((rp->isRecording(CAM_CSI) == false) &&(rp->getRPWindowState()== STATUS_AWMD)) {
			param.fileSize = AWMD_PREALLOC_SIZE(rp->getDuration());
			param.video_type = VIDEO_TYPE_IMPACT;
			param.duration = 60 * 1000;
			int ret = rp->startRecord(CAM_CSI, param);
			if(ret == 0){
				rp->getCdrMain()->cdrReplyToApp(NAT_AWMD_RECORD_START, NULL, 0);
			}else{
				db_msg(" csi start record failed ");
				rp->setDelayStartAWMD(true);
			}
			rp->feedAwmdTimer(5);
		} else if(rp->isRecording(CAM_CSI) == true) {
			time_t sec;
			long nsec;
			if(rp->queryAwmdExpirationTime(&sec, &nsec) < 0) {
				db_error("query awmd timer expiration time failed\n");
				return;
			}
			if( (sec == 4 && nsec <= 500 * 1000 * 1000) || sec < 4)
				rp->feedAwmdTimer(5);
		}
	}
}

int RecordPreview::initCamera(int cam_type, bool needNewCamera)
{
	CDR_RECT rect;
	unsigned int width = 1280, height = 720;	//video size
	unsigned int preview_w  = 640, preview_h = 360; //preview size
	int node = 0;
	int ret = -1;
	if(mCamExist[cam_type] == false)
		return ret;

	if (cam_type == CAM_CSI) {
		rect = rectBig;
		node = CAM_CSI;
		#ifdef CSI_1080P
			width  = 1920;
			height = 1080;
		#endif
		preview_w  = CSI_PREVIEW_WIDTH, preview_h = CSI_PREVIEW_HEIGHT; //preview size
	}
	if (cam_type == CAM_UVC) {
		rect = rectSmall;
		mUVCNode = CAM_UVC;
		node = mUVCNode;
		#ifdef UVC_1080P
			width  = 1920;
			height = 1080;
		#endif
		if(mHasTVD) {
			node = CAM_TVD;
			width = 720;
			height = 480;
		}
#ifdef USE_IPS_SCREEN
        preview_w  = UVC_PREVIEW_WIDTH_IPS, preview_h = UVC_PREVIEW_HEIGHT_IPS;
#else
		preview_w  = UVC_PREVIEW_WIDTH, preview_h = UVC_PREVIEW_HEIGHT; //preview size
#endif
	}
	if(needNewCamera)
		mCamera[cam_type] = new CdrCamera(cam_type, rect, node);
	ret = mCamera[cam_type]->initCamera(width, height, preview_w, preview_h);
	db_msg("initCamera::cam_type = %d,ret = %d,mCamera[%d] = %d\n",cam_type,ret,cam_type,mCamera[ret]);
	if (ret < 0) {
		delete mCamera[cam_type];
		mCamera[cam_type] = NULL;
		mCamExist[cam_type] = false;
		return ret;
	} else {
		mCamExist[cam_type] = true;
	}
	mCamera[cam_type]->setCaller(this);
#ifdef APP_ADAS
	mCamera[cam_type]->setAdasCaller(this);
#endif
	if (cam_type == CAM_CSI) {
		mCamera[cam_type]->setJpegCallback(jpegCallback);
		mCamera[cam_type]->setErrorCallback(errorCallback);
		mCamera[cam_type]->setAwmdCallBack(awmdCallback);
		mCamera[cam_type]->setAwmdTimerCallBack(awmdTimerCallback);
		#ifdef APP_ADAS
		mCamera[cam_type]->setAdasCallback(adasCallback);
		#endif
	}else if(cam_type == CAM_UVC){
		mCamera[cam_type]->setJpegCallback(jpegCallback);
        mCamera[cam_type]->setJpegCallback(jpegCallback);
        mCamera[cam_type]->setErrorCallback(errorCallback);
	}
	return 0;
}

void RecordPreview::startPreview(int cam_type)
{
	if(mCamera[cam_type]) {
		mCamera[cam_type]->startPreview();
		if(cam_type == CAM_UVC){
            ResourceManager* rm;
            rm = ResourceManager::getInstance();    
            bool isFlip = rm->getResBoolValue(ID_MENU_LIST_ALIGNLINE);
            mCamera[CAM_UVC]->setCameraFlip(isFlip);
        }
	}
}

void RecordPreview::startPreview()
{
	Mutex::Autolock _l(mLock);
	if (mCamExist[CAM_CSI]) {
		startPreview(CAM_CSI);
	}
	if (mCamExist[CAM_UVC]) {
		startPreview(CAM_UVC);
	}
}

void RecordPreview::setDeleteFlag(int cam_type, String8 file)
{
	Mutex::Autolock _l(mDeleteLock);
	db_msg("setDeleteFlag %s!", file.string());
	mNeedDelete[cam_type] = file;
}

void RecordPreview::clrDeleteFlag(int cam_type, String8 file)
{
	Mutex::Autolock _l(mDeleteLock);
	if (file == mNeedDelete[cam_type]) {
		db_msg("clrDeleteFlag!");
		mNeedDelete[cam_type] = "";
	} else {
		db_msg("should not be cleared %s", file.string());
	}
}

String8 RecordPreview::getDeleteFlag(int cam_type)
{
	Mutex::Autolock _l(mDeleteLock);
	return mNeedDelete[cam_type];
}

bool RecordPreview::isPreviewing(int cam_type)
{
	if(mCamera[cam_type])
		return mCamera[cam_type]->isPreviewing();

	return false;
}
bool RecordPreview::isPreviewing()
{
	if(isPreviewing(CAM_CSI) == true && isPreviewing(CAM_UVC))
		return true;
	return false;
}

void RecordPreview::errorListener(CdrMediaRecorder* cmr, int what, int extra)
{
	db_msg("_______onError");
	//	stopRecord();
}

void RecordPreview::recordListener(CdrMediaRecorder* cmr, int what, int extra)
{
 	int cameraID;
    int ret;
	CdrRecordParam_t recordParam;
	Elem elem;
	StorageManager* sm = StorageManager::getInstance();
	cameraID = cmr->getCameraID();
	cmr->getRecordParam(recordParam);
	db_msg("cameraID is %d, extra is %d\n", cameraID, extra);
	String8 needDeleteFile;
	switch(what) {
	case MEDIA_RECORDER_INFO_NEED_SET_NEXT_FD: 
		{
			if(mCdrMain->getWakeupState() || cameraID == CAMERA_CSI_NET_SMALL_RECORD) {
				break;
			}
			needDeleteFile = getDeleteFlag(cameraID);
			
			if (!needDeleteFile.isEmpty()) {
				if(sm->fileSize(needDeleteFile) == 0) {
					db_error("MEDIA_RECORDER_INFO_NEED_SET_NEXT_FD %s is empty, will be deleted\n",needDeleteFile.string());
					sm->deleteFile(needDeleteFile.string());
					clrDeleteFlag(cameraID, needDeleteFile );
				}
			}
			db_msg("Please set the next fd.");
			ret = sm->generateFile(mBakNow[cameraID], mBakFileString[cameraID], cameraID, recordParam.video_type, true);
			if(ret < 0) {
				db_error("get fd failed\n");
				clrDeleteFlag(cameraID, mRecordDBInfo[cameraID].fileString);
				stopRecord();
				if (ret == RET_IO_NO_RECORD) {
					ShowTipLabel(mHwnd, LABEL_TFCARD_FULL);
				}
			}
#ifdef USE_TAKEPIC_NETJPD
			sm->geneNetPicName(mBakFileString[cameraID], mTakePicName[cameraID], EXT_PIC_JPG);
#endif
			setDeleteFlag(cameraID, mBakFileString[cameraID]);
			cmr->setNextFile(mBakFileString[cameraID], recordParam.fileSize, extra);
		}
		break; 
	case MEDIA_RECORDER_INFO_RECORD_FILE_DONE: 
		{
			db_msg("xxx  cameraID:%d ",cameraID );
			if(cameraID == CAMERA_CSI_NET_SMALL_RECORD){
				cmr->stopImpactVideo();
				handSmallVideo(elem);
				break;
			}
			if(cmr->getImpactStatus() == true) {
				db_msg("mImpactFlag[%d] is %d\n", cameraID, mImpactFlag[cameraID]);
				if(mImpactFlag[cameraID] <= 0) {
					db_msg("impact file done, file is %s\n", mExtraDBInfo[cameraID].fileString.string());
					if(sm->fileSize(mExtraDBInfo[cameraID].fileString) < (256 * 1024)) {	//file is too small
						sm->deleteFile(mExtraDBInfo[cameraID].fileString.string());
						db_msg("file %s size is 0kb", mExtraDBInfo[cameraID].fileString.string());
					} else {
						elem.info = (char*)INFO_LOCK;
						elem.file = (char*)mExtraDBInfo[cameraID].fileString.string();
						elem.type = (char*)TYPE_VIDEO;
						elem.time = mExtraDBInfo[cameraID].time;
						elem.filesize = sm->fileSize(mExtraDBInfo[cameraID].fileString);
						sm->dbAddFile(&elem);
						db_msg("filename:%s", mExtraDBInfo[cameraID].fileString.string());
					//	String8 netPic;
					//	generateNetPic(mExtraDBInfo[cameraID].fileString, TYPE_VIDEO, netPic);
#ifdef USE_TAKEPIC_NETJPD
						if(mTakePicName[cameraID] != "")
							camTakePicture(cameraID);
#endif
					}
					mImpactFileStart[cameraID] = 0;
					if(mCdrMain->getWakeupState()) {
						cmr->stopImpactVideo();
						if(IsImpactFileStart())
							break;
						handWakeUpRecord(sm);
					}
					cmr->stopImpactVideo();
					mRecordDBInfo[cameraID].fileString = mBakFileString[cameraID];
					mRecordDBInfo[cameraID].time = mBakNow[cameraID];

					mExtraDBInfo[cameraID].fileString.empty();
					if(!IsImpactFileStart())
						mCamera[CAM_CSI]->setAdasMode(0);
					break;
				}
				mImpactFlag[cameraID]--;
				if(mCdrMain->getWakeupState()) {
					mNeedDelFile[cameraID] = mRecordDBInfo[cameraID].fileString;
				}
			}

			String8 newName("");
			db_msg("one file done %s", mRecordDBInfo[cameraID].fileString.string());
			if (mRecordDBInfo[cameraID].fileString.isEmpty()) {
				db_error("filename is empty. it's an accident ");
				break;
			}
			if(mCdrMain->getIfDeleFisrtFile()){
			   db_msg("first open cdr no charge delte file:%s",mRecordDBInfo[cameraID].fileString.string());
			   sm->deleteFile(mRecordDBInfo[cameraID].fileString.string());
			   mCdrMain->setFirstACConnect(false);
			   break;
			}
			if(sm->fileSize(mRecordDBInfo[cameraID].fileString) < (256 * 1024)) { //file is too small
				sm->deleteFile(mRecordDBInfo[cameraID].fileString.string());
				db_msg("file %s size is 0kb", mRecordDBInfo[cameraID].fileString.string());
				break;
			} else {
				elem.file = (char*)mRecordDBInfo[cameraID].fileString.string();
				if (mLockFlag[cameraID]) {
					elem.info = (char*)INFO_LOCK;
					newName   = sm->lockFile(elem.file);
					elem.file = (char*)newName.string();
				} else {
					if(sm->judgeFileFolder(elem.file) == SOS_FOLDER){
						elem.info = (char*)INFO_LOCK;	 //for set awmd file info to INFO_LOCK
					}else{
						elem.info = (char*)mRecordDBInfo[cameraID].infoString.string();
					}
				}
				elem.type = (char*)TYPE_VIDEO;
				elem.time = mRecordDBInfo[cameraID].time;
				elem.filesize = sm->fileSize(mRecordDBInfo[cameraID].fileString);
				sm->dbAddFile(&elem);

#ifdef USE_TAKEPIC_NETJPD
				if(mTakePicName[cameraID] != "")
					camTakePicture(cameraID);
#endif
				if (mLockFlag[cameraID]) {
					mCdrMain->lockFile(newName.string());
					if(cameraID == CAM_UVC && !mCamExist[CAM_UVC]){
						mLockFlag[CAM_UVC] = !mLockFlag[CAM_UVC];
					}else{
						lockFile(cameraID);
					}
				}
			}
			clrDeleteFlag(cameraID, mRecordDBInfo[cameraID].fileString);
			mRecordDBInfo[cameraID].fileString = mBakFileString[cameraID];
			mRecordDBInfo[cameraID].time = mBakNow[cameraID];
			mRecordDBInfo[cameraID].infoString = INFO_NORMAL;
		}
		break;
	}
	
}

int RecordPreview::createWindow(void)
{
	RECT rect;
	CDR_RECT STBRect;
	ResourceManager* rm;
	HWND hParent;
	HWND hStatusBar;

	rm = ResourceManager::getInstance();
	rm->getResRect(ID_STATUSBAR, STBRect);
	hParent = mCdrMain->getHwnd();
	GetWindowRect(hParent, &rect);

	rect.top += STBRect.h;

	mHwnd = CreateWindowEx(WINDOW_RECORDPREVIEW, "",
			WS_CHILD | WS_VISIBLE,
			WS_EX_TRANSPARENT | WS_EX_USEPARENTFONT,
			WINDOWID_RECORDPREVIEW,
			rect.left, rect.top, RECTW(rect), RECTH(rect),
			hParent, (DWORD)this);
	if(mHwnd == HWND_INVALID) {
		db_error("create record Preview window failed\n");
		return -1;
	}
	rm->setHwnd(WINDOWID_RECORDPREVIEW, mHwnd);


	CreateWindowEx(CTRL_STATIC, "",
			WS_CHILD | SS_BITMAP | SS_CENTERIMAGE,
			WS_EX_TRANSPARENT | WS_EX_USEPARENTFONT,
			ID_RECORDPREVIEW_IMAGE,
			rect.left, rect.top, RECTW(rect), RECTH(rect),
			mHwnd, 0);

	hStatusBar = mCdrMain->getWindowHandle(WINDOWID_STATUSBAR);
	if(mCamExist[CAM_UVC] == true)
		SendNotifyMessage(hStatusBar, STBM_UVC, 1, 0);
	else
		SendNotifyMessage(hStatusBar, STBM_UVC, 0, 0);

	return 0;
}

bool checkDev(int cam_type, bool &hasTVD, int &uvcnode)
{

	char dev[32];
	int i = 0;
	struct v4l2_capability cap; 
	int ret;

	if (cam_type == CAM_CSI) {
		sprintf(dev, "/dev/video%d", cam_type);
		return !access(dev, F_OK);
	}

	for (i = 1; i < 8; ++i) {
		sprintf(dev, "/dev/video%d", i);
		if (!access(dev, F_OK)) {
			int fd = open(dev, O_RDWR | O_NONBLOCK, 0);
			if (fd < 0) {
				continue;
			}
			ret = ioctl (fd, VIDIOC_QUERYCAP, &cap);
			if (ret < 0) {
				close(fd);
				continue;
			}
			if (strcmp((char*)cap.driver, "uvcvideo") == 0) {
				db_debug("uvcvideo device node is %s", dev);
				hasTVD = false;	//Tvd could not exist at all.
				close(fd);
				uvcnode = i;
				return true;
			} else if (strcmp((char*)cap.driver, "tvd") == 0) {
				db_debug("TVD device node is %s", dev);
				hasTVD = true;
				if (detectPlugIn(fd) == 0) {
					db_msg("tvd:plug in");
					close(fd);
					return true;
				}
				//else {
					//close(fd);
					//return false;
				//}
				
				//close(fd);	//UVC may be existed.
				//return false;
			}
			close(fd);
			fd = -1;
		}
	}
	db_error("Could not find device for device ID %d!", cam_type);
	return false;
}

RecordPreview::RecordPreview(CdrMain *cdrMain)
	: mCdrMain(cdrMain),
	mRPWindowState(STATUS_RECORDPREVIEW),
	mOldRPWindowState(STATUS_RECORDPREVIEW), 
	mUvcPlugFlag(false),
	mUVCNode(1),
	mAwmdDelayStart(false),
	mStartRtspSession(false),
	mNeedCache(true),
	mSessionBitrate(1<<20)
{
	mVideoSize.width  = 1920;
	mVideoSize.height = 1080;
/*	mRecorder[CAM_CSI] = NULL;
	mRecorder[CAM_UVC] = NULL;
	mRecorder[CAM_CSI_NET] = NULL;
	mRecorder[CAM_UVC_NET] = NULL;*/
	mCamera[CAM_CSI] = NULL;
	mCamera[CAM_UVC] = NULL;

	createMRObjList();
	mDuration = 60*1000;
	int screen_w,screen_h;
	getScreenInfo(&screen_w, &screen_h);
#ifdef USE_IPS_SCREEN
        rectBig.x = BIG_RECT_X;
        rectBig.y = BIG_RECT_Y;
        rectBig.w = screen_h;
        rectBig.h = screen_w;

        rectSmall.w = screen_h*2/5;
        rectSmall.h = screen_w*2/5;
        rectSmall.x = screen_h*3/5-SML_RECT_Y;
        rectSmall.y = screen_w*3/5;
#else
        rectBig.x = BIG_RECT_X;
        rectBig.y = BIG_RECT_Y;
        rectBig.w = screen_w;
        rectBig.h = screen_h;

        rectSmall.w = screen_w*2/5;
        rectSmall.h = screen_h*2/5;
        rectSmall.x = screen_w - rectSmall.w;
        rectSmall.y = SML_RECT_Y;
 #endif
     db_msg("screen-- big: x:%d,y:%d,w:%d,h:%d",rectBig.x,rectBig.y,rectBig.w,rectBig.h);
     db_msg("screen-- small x:%d,y:%d,w:%d,h:%d",rectSmall.x,rectSmall.y,rectSmall.w,rectSmall.h);


	mHasTVD = false;
	mCamExist[CAM_CSI] = checkDev(CAM_CSI, mHasTVD, mUVCNode);
	mCamExist[CAM_UVC] = checkDev(CAM_UVC, mHasTVD, mUVCNode);
	mNeedDelete[CAM_CSI]="";
	mNeedDelete[CAM_UVC]="";
	mLockFlag[CAM_CSI]= false;
	mLockFlag[CAM_UVC]= false;
	mSilent = true;
	if(mCamExist[CAM_CSI] == false && mCamExist[CAM_UVC] == false)
		mPIPMode = NO_PREVIEW;
	else if(mCamExist[CAM_CSI] == true && mCamExist[CAM_UVC] == true)
		mPIPMode = UVC_ON_CSI;
	else if(mCamExist[CAM_CSI] == true)
		mPIPMode = CSI_ONLY;
	else
		mPIPMode = UVC_ONLY;
#ifdef YOUKU_PLATFORM
	mSessionSize.width   = 640;
	mSessionSize.height  = 320;
#else
	mSessionSize.width   = mVideoSize.width / 4;
	mSessionSize.height  = mVideoSize.height / 4;
#endif
	mAllowTakePic[CAM_CSI] = true;
	mAllowTakePic[CAM_UVC] = true;
	db_msg("mPIPMode is %d\n", mPIPMode);
#ifdef PLATFORM_0
	mVirHC = NULL;
	createVirtualCam();
#endif
	mSession = NULL;
	mCurSessionCam = LOCAL2SESSION(CAM_CSI);
	mAlignLine=false;
	mIsAdasScreen=false;
	mKeyStop = false;
	mUVCCarModem = false;
	mImpactFileStart[CAM_CSI] = 0;
	mImpactFileStart[CAM_UVC] = 0;
	mNeedDelFile[CAM_CSI] = "";
	mNeedDelFile[CAM_UVC] = "";
	
	mSmallRecordTakePic = false;
	mTakePicName[CAM_CSI] = "";
	mTakePicName[CAM_UVC] =	"";
#if defined(VIDEO_CONFIG_TEXT_TRACK) && defined(VIDEO_CONFIG_TEXT_GPS)
    mSGpsThread = new SendGpsThread(this);
#endif
}

RecordPreview::~RecordPreview()
{
	/*for(int i=0; i<CAM_CNT+2;  i++)
	{
		if (mRecorder[i]) {
			stopRecord();
			delete mRecorder[i];
			mRecorder[i] = NULL;
		}
	}*/
#if defined(VIDEO_CONFIG_TEXT_TRACK) && defined(VIDEO_CONFIG_TEXT_GPS)
    if(mSGpsThread != NULL){
        mSGpsThread->stopThread();
        mSGpsThread.clear();
        mSGpsThread = NULL;
    }
#endif
	mMRObjList.clear();
	for(int i=0; i<CAM_CNT;  i++)
	{
		if (mCamera[i]) {
			delete mCamera[i];
			mCamera[i] = NULL;
		}
	}
}

bool RecordPreview::hasTVD(bool &hasTVD)
{
	hasTVD = mHasTVD;
	db_msg("hasTVD:%d\n", hasTVD);
	return (mHasTVD && mCamExist[CAM_UVC]);
}

void RecordPreview::init(void)
{	
	CdrDisplay* cd;	
	if(mCamExist[CAM_CSI] == true) {
		initCamera(CAM_CSI);
		startPreview(CAM_CSI);
		CdrMediaRecorder *mCRe = new CdrMediaRecorder(CAM_CSI);
		mCRe->setOnInfoCallback(this);
		mCRe->setOnErrorCallback(this);
		MediaRecorderObj *mrobj = getMediaRecordObj(CAMERA_CSI_RECORD);
		mrobj->setCdrMediaRecord(mCRe);
		//MediaRecorderObj *mMe = new MediaRecorderObj(mCRe, false);
		mMRObjList.add(CAMERA_CSI_RECORD, mrobj);
	}
	struct timespec measureTime1;
	clock_gettime(CLOCK_MONOTONIC, &measureTime1);
	db_msg("Test Time: start uvc , time is %ld secs, %ld nsecs\n", measureTime1.tv_sec, measureTime1.tv_nsec);
	if(mCamExist[CAM_UVC] == true) {
		initCamera(CAM_UVC);
		CdrMediaRecorder *mCRe = new CdrMediaRecorder(CAM_UVC);
		mCRe->setOnInfoCallback(this);
		mCRe->setOnErrorCallback(this);
		MediaRecorderObj *mrobj = getMediaRecordObj(CAMERA_UVC_RECORD);
		mrobj->setCdrMediaRecord(mCRe);
		mMRObjList.add(CAMERA_UVC_RECORD, mrobj);
		
		if(mCamExist[CAM_CSI] && mCamera[CAM_CSI]) {
			cd = mCamera[CAM_CSI]->getDisp();
			cd->setBottom();
		}
	}
}

static void display_setareapercent(CDR_RECT *rect, int percent)
{
	int valid_width  = (percent*rect->w)/100;
	int valid_height = (percent*rect->h)/100;
	
	rect->x = (rect->w - valid_width) / 2;
	rect->w = valid_width;
	rect->y = (rect->h - valid_height) / 2;
	rect->h = valid_height;
}

//rect align with parent_rect on the top-right
static void display_setareapercent(CDR_RECT *rect, const CDR_RECT *parent_rect, int percent)
{
	int valid_width  = (percent*rect->w)/100;
	int valid_height = (percent*rect->h)/100;
	
	rect->x = (parent_rect->x + parent_rect->w) - valid_width;
	rect->w = valid_width;
	rect->y = parent_rect->y;
	rect->h = valid_height;
}

void RecordPreview::otherScreen(int screen)
{
	CdrDisplay *cdCSI, *cdUVC;
	int hlay_uvc = 0;
	cdCSI = getCommonDisp(CAM_CSI);
	cdUVC = getCommonDisp(CAM_UVC);
	if (cdUVC) {
		hlay_uvc = cdUVC->getHandle();
	}
	if (cdCSI) {
		cdCSI->otherScreen(screen, hlay_uvc);
	}
	if (screen == 1) {
		rectBig.x = 0;
		rectBig.y = 0;
		rectBig.w = 1280;
		rectBig.h = 720;
		display_setareapercent(&rectBig, 95);
		rectSmall.x = 920;
		rectSmall.y = 0;
		rectSmall.w = 360;
		rectSmall.h = 240;
		display_setareapercent(&rectSmall, &rectBig, 95);
	} else if (screen == 2) {
		rectBig.x = 0;
		rectBig.y = 0;
		rectBig.w = 720;
		rectBig.h = 480;
		display_setareapercent(&rectBig, 95);
		rectSmall.x = 480;
		rectSmall.y = 0;
		rectSmall.w = 240;
		rectSmall.h = 160;
		display_setareapercent(&rectSmall, &rectBig, 95);
	} else {
		rectBig.x = 0;
		rectBig.y = 0;
		rectBig.w = 480;
		rectBig.h = 272;
		rectSmall.x = 300;
		rectSmall.y = 22;
		rectSmall.w = 180;
		rectSmall.h = 102;
	}
	restoreZOrder();
}

void RecordPreview::storagePlug(bool plugin)
{
	if (plugin) { //tfcar plug in
		setSdcardState(true);
	} else {	//tfcard plug out
		setSdcardState(false);
		stopRecord();
/*
		if(mCamera[CAM_UVC]) {
			stopPreview(CAM_UVC);
			delete mCamera[CAM_UVC];
			mCamera[CAM_UVC] = NULL;

			initCamera(CAM_UVC);
			startPreview(CAM_UVC);
		}
*/
	}
}

void RecordPreview::restoreZOrder()
{
	
	CdrDisplay *cdCSI = getCommonDisp(CAM_CSI);
	CdrDisplay *cdUVC = getCommonDisp(CAM_UVC);
	switch(mPIPMode) {
		case NO_PREVIEW:
		break;
		case CSI_ONLY:
			
			if (cdCSI) {
				cdCSI->setRect(rectBig);
				cdCSI->setBottom();
			}
			if (cdUVC) {
				//db_msg("UVC layer should be close!!!");
				cdUVC->setBottom();
			}
		break;
		case UVC_ONLY:
			
			if (cdUVC) {
				cdUVC->setRect(rectBig);
				cdUVC->setBottom();
			}
			if (cdCSI) {
				//db_msg("CSI layer should be close!!!");
				cdCSI->setBottom();
			}
		break;
		case CSI_ON_UVC:
			if (cdCSI) {
				cdCSI->setRect(rectSmall);
			}
			cdCSI->setBottom();
			if (cdUVC) {
				cdUVC->setRect(rectBig);
			}
			cdUVC->setBottom();
		break;
		case UVC_ON_CSI:
			if (cdUVC) {
				cdUVC->setRect(rectSmall);
			}
			cdUVC->setBottom();
			if (cdCSI) {
				cdCSI->setRect(rectBig);
			}
			cdCSI->setBottom();
		break;
	};
}

void RecordPreview::carBackTransPIP(bool flag)
{
	CdrDisplay *cdCSI, *cdUVC;
	db_msg("mPIPMode is %d\n", mPIPMode);
	if(!mCamExist[CAM_UVC])
		return ;
	ResourceManager * rm = ResourceManager::getInstance();
	bool awmd = rm->getResBoolValue(ID_MENU_LIST_AWMD);
	if(awmd){
		mRPWindowState = STATUS_AWMD;
	}

	if(!mCamera[CAM_UVC]) {
		initCamera(CAM_UVC);
	}

	cdCSI = getCommonDisp(CAM_CSI);
	cdUVC = getCommonDisp(CAM_UVC);
	cdCSI = mCamera[CAM_CSI]->getDisp();
	cdUVC = mCamera[CAM_UVC]->getDisp();
	startPreview(CAM_UVC);
	startPreview(CAM_CSI);

	if(flag){
		db_msg(" xxxxx   mPIPMode:%d", mPIPMode);
		if(mPIPMode==CSI_ON_UVC || mPIPMode==UVC_ONLY){
			cdUVC->setRect(rectBig);
			cdUVC->open();
			cdCSI->close();
		}else if(mPIPMode==UVC_ON_CSI || mPIPMode==CSI_ONLY){
			cdUVC->setRect(rectSmall);
			cdCSI->setRect(rectBig);
			//cdCSI->open();
			cdCSI->setBottom();
			//stopPreview(CAM_CSI);
			cdUVC->open();
			cdCSI->exchange(cdUVC->getHandle(),1);
			cdCSI->close();
		}
		mPIPMode = UVC_ONLY;
	}else{
		cdUVC->setRect(rectBig);
		cdCSI->setRect(rectSmall);
		cdCSI->open();
		cdUVC->open();
		cdUVC->setBottom();
		cdUVC->exchange(cdCSI->getHandle(),1);
		mPIPMode = UVC_ON_CSI;
	}
}

void RecordPreview::closecarBackCSI()
{

	/*if(!mCamExist[CAM_UVC])
		return ;
	if(!mCamera[CAM_UVC]) {
		initCamera(CAM_UVC);
	}*/
	CdrDisplay *cdCSI;
	cdCSI = getCommonDisp(CAM_CSI);
	cdCSI = mCamera[CAM_CSI]->getDisp();
	cdCSI->setBottom();
	cdCSI->close();

}


int RecordPreview::transformPIP(pipMode_t newMode)
{
	CdrDisplay *cdCSI, *cdUVC;
	int ret = 0;
	db_msg("mPIPMode is %d, newMode is %d\n", mPIPMode, newMode);
	switch(mPIPMode) {
	case NO_PREVIEW:	
		{
			if(newMode == UVC_ONLY) {
				if(mCamExist[CAM_UVC] == false)
					break;
				if(!mCamera[CAM_UVC])	{
					initCamera(CAM_UVC);
				}
				cdUVC = getCommonDisp(CAM_UVC);
				cdUVC->setRect(rectBig);
				startPreview(CAM_UVC);
				mPIPMode = UVC_ONLY;
				db_msg("-----------PIP_MODE: NO_PREVIEW ---> UVC_ONLY\n");
			}
		}
		break;
	case CSI_ONLY:
		{
			if(newMode == UVC_ON_CSI) {
				if(mCamExist[CAM_UVC] == false)
					break;
				if(!mCamera[CAM_UVC]){
					ret = initCamera(CAM_UVC);
					if (ret < 0) {
						return ret;
					}
					cdCSI = getCommonDisp(CAM_CSI);
					cdCSI->setBottom();

					startPreview(CAM_UVC);
					mPIPMode = UVC_ON_CSI;
					db_msg("-----------PIP_MODE: CSI_ONLY ---> UVC_ON_CSI\n");
				} else {
					cdUVC = getCommonDisp(CAM_UVC);
					cdUVC->setRect(rectSmall);
					cdUVC->open();

					cdCSI = getCommonDisp(CAM_CSI);
					cdCSI->setBottom();
					startPreview(CAM_UVC);
					mPIPMode = UVC_ON_CSI;
					db_msg("-----------PIP_MODE: CSI_ONLY ---> UVC_ON_CSI\n");
				}
			} else if(newMode == CSI_ONLY) {
				db_msg("-----------PIP_MODE: CSI_ONLY ---> CSI_ONLY\n");
			}
		}
		break;
	case UVC_ONLY:
		{
			if(newMode == CSI_ONLY) {
				if(mCamExist[CAM_CSI] == false)
					break;
				if(!mCamera[CAM_CSI]) {
					initCamera(CAM_CSI);
					startPreview(CAM_CSI);

					cdUVC = getCommonDisp(CAM_UVC);
					cdUVC->close();
					cdUVC->setRect(rectSmall);
					mPIPMode = CSI_ONLY;
					db_msg("-----------PIP_MODE: UVC_ONLY ---> CSI_ONLY\n");
				} else {
					cdUVC = getCommonDisp(CAM_UVC);
					cdCSI = getCommonDisp(CAM_CSI);
					cdCSI->setRect(rectSmall);
					cdUVC->setBottom();

					cdUVC->exchange(cdCSI->getHandle(), 1);
					cdCSI->open();
					cdUVC->close();
					startPreview(CAM_CSI);
					mPIPMode = CSI_ONLY;
					db_msg("-----------PIP_MODE: UVC_ONLY ---> CSI_ONLY\n");
				}
			} else if(newMode == NO_PREVIEW) {
				cdUVC = getCommonDisp(CAM_UVC);
				cdUVC->close();
				mPIPMode = NO_PREVIEW;
				db_msg("-----------PIP_MODE: UVC_ONLY ---> NO_PREVIEW\n");
			}
		}
		break;
	case CSI_ON_UVC:
		if(newMode == UVC_ONLY) {
			cdCSI = getCommonDisp(CAM_CSI);
			cdCSI->setRect(rectSmall);
			cdCSI->close();

			mPIPMode = UVC_ONLY;
			db_msg("-----------PIP_MODE: CSI_ON_UVC ---> UVC_ONLY\n");
		} else if(newMode == CSI_ONLY) {
			cdUVC = getCommonDisp(CAM_UVC);
			cdCSI = getCommonDisp(CAM_CSI);
			cdCSI->setRect(rectSmall);
			cdUVC->setRect(rectBig);
			cdUVC->setBottom();

			cdCSI->exchange(cdUVC->getHandle(), 1);
			cdUVC->close();
			mPIPMode = CSI_ONLY;
			db_msg("-----------PIP_MODE: CSI_ON_UVC ---> CSI_ONLY\n");
		}
		break;
	case UVC_ON_CSI:
		cdUVC = getCommonDisp(CAM_UVC);	
		cdCSI = getCommonDisp(CAM_CSI);
		cdUVC->setRect(rectSmall);
		cdCSI->setRect(rectBig);
		cdCSI->setBottom();
		if(newMode == CSI_ONLY) {
			cdUVC->close();
			mPIPMode = CSI_ONLY;
			db_msg("-----------PIP_MODE: UVC_ON_CSI ---> CSI_ONLY\n");
		} else if(newMode == CSI_ON_UVC) {
			cdCSI->exchange(cdUVC->getHandle(), 1);
			mPIPMode = CSI_ON_UVC;
			db_msg("-----------PIP_MODE: UVC_ON_CSI ---> CSI_ON_UVC\n");
		}
		break;
	}
	return ret;
}

int RecordPreview::setCamExist(int cam_type, bool bExist)
{
	db_msg("-----------cam_type: %d %d\n", cam_type, bExist);
	unsigned int curWindowID;
	int hasTVD = cam_type;
	int ret = 0;

	if (cam_type == CAM_TVD) {
		mHasTVD = bExist;
		cam_type = CAM_UVC;
	} else if (cam_type == CAM_UVC) {
		mHasTVD = !bExist;
	}

	if (mCamExist[cam_type] == bExist) {
		return -1;
	}

	Mutex::Autolock _l(mLock);
	mCamExist[cam_type] = bExist;
	curWindowID = mCdrMain->getCurWindowID();
	if(curWindowID != WINDOWID_PLAYBACKPREVIEW && curWindowID != WINDOWID_PLAYBACK) {
		/* RecordPreview and Menu */
		if(bExist == true) {
			/* plug in */	
			if(mPIPMode == NO_PREVIEW)
				transformPIP(UVC_ONLY);
			else if(mPIPMode == CSI_ONLY) {
				usleep(5 * 1000);
				ret = transformPIP(UVC_ON_CSI);
				if (ret < 0) {
					return ret;
				}
				if(mCdrMain->mFinishConfig){
					ResourceManager *rm = ResourceManager::getInstance();
					int wmFlag = 0;
					if (rm->menuDataTWMEnable) {
						wmFlag |= WATERMARK_TWM;
					}
					if (rm->menuDataLWM.LWMEnable) {
						wmFlag |= WATERMARK_LWM;
					}
					mCamera[CAM_UVC]->setWaterMarkPos();
					mCamera[CAM_UVC]->setLWaterMark(wmFlag&WATERMARK_TWM, wmFlag&WATERMARK_LWM, rm->menuDataLWM.lwaterMark);
				}
			} 
		} else {
			/* plug out */
			stopRecord(CAM_UVC_NET);
			stopRecord(CAM_UVC);	
			if(mCamExist[CAM_CSI])
				transformPIP(CSI_ONLY);	
			else
				transformPIP(NO_PREVIEW);
			if (cam_type == CAM_UVC) {
				stopPreview(CAM_UVC);
			}
			if(mCamera[CAM_UVC]) {
				if (hasTVD != CAM_TVD) {
					delete mCamera[CAM_UVC];
					mCamera[CAM_UVC] = NULL;
				}
			}
		}
	} else {
		/* PlayBackPreview and PlayBack */
		mUvcPlugFlag = true;
		if(bExist == false) {
			if(mCamera[CAM_UVC]) {
				stopPreview(CAM_UVC);
				if (hasTVD != CAM_TVD) {
					delete mCamera[CAM_UVC];
					mCamera[CAM_UVC] = NULL;
				}
			}
		}
	}
	mLock.unlock();
	if (cam_type == CAM_UVC && bExist == false && mCurSessionCam == CAM_UVC_NET) {
		camChange(CAM_CSI, false);
		return 1;
	}
	return 0;
}
#ifdef PLATFORM_0
void RecordPreview::createVirtualCam()
{
	if (!mVirHC) {
		mVirHC = new VirtualCamera();
		int w = mSessionSize.width;
		int h = mSessionSize.height;
		ALOGD("(f:%s, l:%d) wxh[%dx%d]", __FUNCTION__, __LINE__, w, h);
		mVirHC->initCamera(w, h, w, h);
	}
}

void RecordPreview::destroyVirtualCam()
{
	if (mVirHC) {
		delete mVirHC;
		mVirHC = NULL;
	}
}

void RecordPreview::setVirtualCam(CdrMediaRecorder *cmr)
{
	ALOGE("%s %d", __FUNCTION__, __LINE__);
	if (cmr && mVirHC) {
		cmr->setRecorderOutputThumb(mVirHC->getCamera());
	}
}
#endif

int RecordPreview::prepareRecord(int recorder_idx, CdrRecordParam_t param, Size *resolution)
{
	int ret = -1;
	int retval;
	HerbCamera *mHC, *virHC=NULL;
	Size size(1280, 720);
	int framerate = BACK_CAM_FRAMERATE;
	uint32_t sessionBitrate = 0;
	int fileSize = 0;

	MediaRecorderObj *mr = getMediaRecordObj((enum CameraIdRecord)recorder_idx);
	CdrMediaRecorder *mcr = mr->getCdrMediaRecorder();
	bool hasSession = mr->getSession();
	int cam_idx = mr->getUseCam();

	db_msg(" MediaRecorderObj:%p, CdrMediaRecorder:%p", mr,mcr);
	db_msg("  cam_idx:%d ,recorder_idx:%d, hasSession:%d ", cam_idx,recorder_idx,hasSession);	
	
	if (!hasSession && recorder_idx != CAMERA_CSI_NET_SMALL_RECORD)
     {
		time_t now;
		String8 file;
		StorageManager* sm = StorageManager::getInstance();
		ret = sm->generateFile(now, file, cam_idx, param.video_type);
#ifdef USE_TAKEPIC_NETJPD
		sm->geneNetPicName(file, mTakePicName[recorder_idx], EXT_PIC_JPG);
#endif
		if (ret < 0) {
			db_error("~~fail to generate file, retval is %d\n", ret);
			return ret;
		}
		setDeleteFlag(cam_idx, file);

		mRecordDBInfo[cam_idx].fileString = file;
		mRecordDBInfo[cam_idx].time = now;
		mRecordDBInfo[cam_idx].infoString = INFO_NORMAL;
		mcr->setRecordParam(param);
		mcr->setDuration(param.duration);
		fileSize = param.fileSize;
		
	}
	Mutex::Autolock _l(mLock);
	mHC = mCamera[cam_idx]->getCamera();

	if (cam_idx == CAM_CSI) {
		size = mVideoSize;
		framerate = FRONT_CAM_FRAMERATE;
	} else if (cam_idx == CAM_UVC) {
		size = mVideoSize;
		if (mHasTVD) {
			size.width = 720;
			size.height = 480;
		}
	}
	if (resolution != NULL) {
		mSessionSize.width  = resolution->width;
		mSessionSize.height = resolution->height;
	}
	if (hasSession) {
		size.width  = mSessionSize.width;
		size.height = mSessionSize.height;
		sessionBitrate = mSessionBitrate;
#ifdef PLATFORM_0
		if(mcr->getRecorderState() == RECORDER_RECORDING)
        {
            ALOGD("(f:%s, l:%d) isRecording is true!", __FUNCTION__, __LINE__);
            mHC = mVirHC->getCamera();
        }
#endif
        ALOGD("(f:%s, l:%d) record video size[%dx%d]!", __FUNCTION__, __LINE__, size.width, size.height);
	}
	if (mNeedCache) {
		mcr->setCacheTime();
	} else {
		mcr->setCacheTime(0);
	}
	//retval = mRecorder[recorder_idx]->initRecorder(mHC, size, framerate, fd, fileSize);
	if(recorder_idx == CAMERA_CSI_NET_SMALL_RECORD){
		size.width = SMALL_VIDEO_WIDTH;
		size.height = SMALL_VIDEO_HEIGHT;
		String8 files;
	//	mcr->setRecordParam(param);
	//	mcr->setDuration(param.duration);
		//retval = mcr->initRecorder(mHC, size, framerate, "", 0,false);
		framerate = SMALL_VIDEO_FRAMERATE;
		retval = mcr->initRecorder(mHC, size, framerate, files, 0,false);
		mcr->setSilent(mSilent);
		return retval;
	}
	if (recorder_idx == CAMERA_UVC_RECORD){
		size.width = 1280;
		size.height = 720;
	}
	if(mCdrMain->getWakeupState()){
		retval = mcr->initRecorder(mHC, size, framerate, mRecordDBInfo[cam_idx].fileString, fileSize,true);
		if(retval != 0){
			mcr->resetMediaRecord();
			usleep(200*1000);
			retval = mcr->initRecorder(mHC, size, framerate, mRecordDBInfo[cam_idx].fileString, fileSize,true);
		}
	}else{
		retval = mcr->initRecorder(mHC, size, framerate, mRecordDBInfo[cam_idx].fileString, fileSize, false, sessionBitrate);
		if(retval != 0){
			mcr->resetMediaRecord();
			usleep(200*1000);
			retval = mcr->initRecorder(mHC, size, framerate, mRecordDBInfo[cam_idx].fileString, fileSize, false, sessionBitrate);
		}
	}
	mcr->setSilent(mSilent);
	return retval;
}


int RecordPreview::startRecord(int recorder_idx, CdrRecordParam_t param)
{
	int retval;
	HWND hStatusBar;
	int isEnableRecord;
	
	int cam_idx = recorder_idx;
	bool hasSession = false;

	MediaRecorderObj *mr = getMediaRecordObj((enum CameraIdRecord)recorder_idx);
	CdrMediaRecorder *mcr = mr->getCdrMediaRecorder();
	hasSession = mr->getSession();

	db_msg("startRecord hasSession = %d",hasSession);
	
	{
		Mutex::Autolock _l(mRecordLock);
	//	if (!hasSession) {
			if(mcr == NULL) {
				db_msg("  recorder_idx:%d,hasSession:%d",recorder_idx,hasSession);
				if (hasSession) {
					mcr = new CdrMediaRecorder(recorder_idx, mSession);
				} else {
					mcr = new CdrMediaRecorder(recorder_idx);
				}
				mr->setCdrMediaRecord(mcr);
				mcr->setOnInfoCallback(this);
				mcr->setOnErrorCallback(this);
			}
			if(mcr->getRecorderState() == RECORDER_RECORDING) {
				db_error("cam_type: %d, recording is started\n", recorder_idx);
				return -1;
			}
			isEnableRecord = mCdrMain->isEnableRecord();
			if (isEnableRecord != FORBID_NORMAL) {
				db_msg("xxxxxxxx\n");
				return -1;
			}
	//	}
		retval = prepareRecord(recorder_idx, param);
		if (!hasSession) {
			if (retval < 0 ) {
				db_error("fail to prepare");
				CdrMediaRecorder *csi_mrecord =getMediaRecordObj(CAMERA_CSI_RECORD)->getCdrMediaRecorder();
				if (csi_mrecord->getRecorderState() != RECORDER_RECORDING) {  //use CSI record to commder
					if(retval == RET_NOT_MOUNT && !mCdrMain->getPC2Connected()) {
					//ShowTipLabel(mHwnd, LABEL_NO_TFCARD);
					SendNotifyMessage(mHwnd, MSG_SHOW_TIP_LABEL, LABEL_NO_TFCARD, 0);
					} else if(retval == RET_IO_NO_RECORD) {
					//ShowTipLabel(mHwnd, LABEL_TFCARD_FULL);
					CloseTipLabel();
					SendNotifyMessage(mHwnd, MSG_SHOW_TIP_LABEL, LABEL_TFCARD_FULL, 0);
					}
				}
				return retval;
			}
			if(recorder_idx == CAM_CSI) {
				mCdrMain->setSTBRecordVTL(param.duration);
				hStatusBar = mCdrMain->getWindowHandle(WINDOWID_STATUSBAR);
				SendNotifyMessage(hStatusBar, STBM_START_RECORD_TIMER, 0, 0);
			}
			mBakFileString[cam_idx].empty();
			mBakNow[cam_idx] = 0;
		}
		
#ifdef USE_TAKEPIC_NETJPD
		if(recorder_idx == CAM_CSI){
			mCamera[recorder_idx]->setImageQuality(CSIImageQualityNet);
			mCamera[recorder_idx]->setJpegQuality(NET_JPEG_QUALITY);
			mCamera[recorder_idx]->setTakepicType(SMALL_TAKE_PIC);
			camTakePicture(recorder_idx);
		}else if(recorder_idx == CAM_UVC){
			mCamera[recorder_idx]->setImageQuality(UVCImageQualityNet);
			mCamera[recorder_idx]->setJpegQuality(NET_JPEG_QUALITY);
			mCamera[recorder_idx]->setTakepicType(SMALL_TAKE_PIC);
			camTakePicture(recorder_idx);
		}
#endif
		retval = mcr->start();
#if defined(VIDEO_CONFIG_TEXT_TRACK) && defined(VIDEO_CONFIG_TEXT_GPS)
        if (mcr->getCameraID() == CAM_CSI) {
            if((mSGpsThread != NULL) && (mSGpsThread->isRunning() == false)){
                db_msg("start SendGpsThread");
                mSGpsThread->setPCMR(mcr);
                mSGpsThread->startThread();                     // SendGpsThread
            }
            mcr->sendDriverIdData(String8("AllWinnerC1234"));   // plate num
        }
#endif
	}
	return 0;
}

bool RecordPreview::checkSDIsFAT()
{
	if(mCdrMain->getNeedCheckSD()){
		StorageManager *smt = StorageManager::getInstance();
		if(smt->isInsert() && !mCdrMain->checkSDFs()){
			ResourceManager *rm = ResourceManager::getInstance();
			//bool isAdasScreen = rm->getResBoolValue(ID_MENU_LIST_SMARTALGORITHM);
			bool isAdasScreen;
			int valAdas = rm->getResIntValue(ID_MENU_LIST_SMARTALGORITHM, INTVAL_SUBMENU_INDEX);
			if(valAdas == 0 || valAdas == 3){
				isAdasScreen = false;
			}else if(valAdas == 1 || valAdas == 2){
				isAdasScreen = true;
			}

			
			if(!mCdrMain->getPC2Connected())
				SendNotifyMessage(mCdrMain->getWindowHandle(WINDOWID_MENU), MSG_FORMAT_TFDIALOG, (WPARAM)isAdasScreen, 0);
			return  false;
		}
	}
	return true;
}

bool RecordPreview::IsImpactFileStart()
{
	if( mImpactFileStart[CAM_CSI] || mImpactFileStart[CAM_UVC])
		return true;
	else
		return false;
}

int RecordPreview::startRecord(int cam_type)
{
	db_msg("startRecord cam_type = %d",cam_type);
	
	if(!checkSDIsFAT()) {
		db_msg("startRecord:checkSDIsFAT error!!\n");
		return -1;
	}
	CdrRecordParam_t param;
	param.video_type = VIDEO_TYPE_NORMAL;
	if(cam_type == CAM_CSI)
		param.fileSize = CSI_PREALLOC_SIZE(mDuration);
	else
		param.fileSize = UVC_PREALLOC_SIZE(mDuration);
	param.duration = mDuration;
	db_msg("mDuration = %d",mDuration);
	return startRecord(cam_type, param);
}
int RecordPreview::startRecord()
{
	int ret;

	db_msg("startRecord,CAM_CSI = %d,CAM_UVC = %d",mCamera[CAM_CSI],mCamera[CAM_UVC]);
	if(mCamera[CAM_CSI]) {
		if((ret = startRecord(CAM_CSI)) < 0) {
			db_error("startRecord CAM_CSI failed, ret is %d\n", ret);
			return ret;
		}
	}
	if(mCamera[CAM_UVC]) {
		if((ret = startRecord(CAM_UVC)) < 0) {
			db_error("startRecord CAM_UVC failed, ret is %d\n", ret);
			return ret;
		}
	}

#ifdef SMALL_VIDEO
	db_msg(" **** startRecord(CAMERA_CSI_NET_SMALL_RECORD)) *************  ");
    if(mCamera[CAM_CSI]){
        if((ret == startRecord(CAMERA_CSI_NET_SMALL_RECORD)) < 0){
            db_error(" startRecord  CAMERA_CSI_NET_SMALL_RECORD failed.");
        }
    }
#endif

	mCdrMain->noWork(false);
	//StorageManager *sm = StorageManager::getInstance();
	//sm->startSyncThread();
	return 0;
}


int RecordPreview::stopRecord(int recorder_idx)
{
	int ret = 0;
	StorageManager* sm = StorageManager::getInstance();
	HWND hStatusBar;
	String8 needDeleteFile;
	/*int cam_idx = recorder_idx;
	bool hasSession = false;
	if (recorder_idx >= CAM_CNT) {
		cam_idx = recorder_idx - CAM_CNT;
		hasSession = true;
	}*/
#if defined(VIDEO_CONFIG_TEXT_TRACK) && defined(VIDEO_CONFIG_TEXT_GPS)
    if(mSGpsThread != NULL && mSGpsThread->isRunning()){
        mSGpsThread->stopThread();
    }
#endif

#ifdef USE_TAKEPIC_NETJPD
	if((recorder_idx == CAM_CSI) || (recorder_idx == CAM_UVC)){
		mTakePicName[recorder_idx]="";
	}
#endif
	MediaRecorderObj *mr = getMediaRecordObj((enum CameraIdRecord)recorder_idx);
	CdrMediaRecorder *mcr = mr->getCdrMediaRecorder();
	int cam_idx = mr->getUseCam();
	bool hasSession = mr->getSession();

	{
		Mutex::Autolock _l(mRecordLock);		
		db_msg("mCamExist recorder_idx :%d", recorder_idx);
		if(mcr == NULL) {
			db_msg("mRecorder[%d] is NULL\n", recorder_idx);
			return 0;
		}
		if(mcr->getRecorderState() != RECORDER_RECORDING) {
			db_error("!!!!!cam_type: %d, recording is stopped\n", recorder_idx);
			return -1;
		}
		if (!hasSession) {
			if(cam_idx == CAM_CSI) {
				hStatusBar = mCdrMain->getWindowHandle(WINDOWID_STATUSBAR);
				SendNotifyMessage(hStatusBar, STBM_STOP_RECORD_TIMER, 0, 0);
			}
			/*if(mLockFlag == true) {
				lockFile();
			}*/
		}
		db_msg("mcr->stop()   ----");
		ret = mcr->stop();
		db_msg("mcr->stop()  ");
		db_error("mRecorder[recorder_idx]->stop() ret:%d\n", ret);
		if (ret < 0) {
			return ret;
		}
		mcr->stopImpactVideo();
		//if (hasSession) {
			needDeleteFile = getDeleteFlag(cam_idx);
			db_msg("~~~~~~~~~~STOP record ,check need delete file:%d %s\n", !needDeleteFile.isEmpty(), needDeleteFile.string());
			if (!needDeleteFile.isEmpty()) {
				if(sm->fileSize(needDeleteFile) == 0) {
					sm->deleteFile(needDeleteFile.string());
					clrDeleteFlag(cam_idx, needDeleteFile );
				}
			}
		//}
	}
	db_msg("stopRecord finished\n");

#ifdef USE_TAKEPIC_NETJPD
	if((recorder_idx == CAM_CSI) || (recorder_idx == CAM_UVC)){
		ResourceManager *rm = ResourceManager::getInstance();
		int val =  rm->getResIntValue(ID_MENU_LIST_PQ, INTVAL_SUBMENU_INDEX);
		mCamera[recorder_idx]->setImageQuality((ImageQuality_t)val);
		mCamera[recorder_idx]->resetJpegQuality();
	}
#endif
	return ret;
}

int RecordPreview::stopRecord()
{
	int ret = 0;

	if(mRPWindowState == STATUS_AWMD) {
		if(isRecording(CAM_CSI)) {
			db_msg("need delay start the awmd record\n");
			setDelayStartAWMD(true);
		}
	}
	ret = stopRecord(CAM_CSI);
	if (ret < 0) {
		db_msg("fail to stop recorder csi");
	}

	ret = stopRecord(CAM_UVC);
	if (ret < 0) {
		db_msg("fail to stop recorder uvc");
	}

#ifdef SMALL_VIDEO
    if(isRecording(CAMERA_CSI_NET_SMALL_RECORD)){
        ret = stopRecord(CAMERA_CSI_NET_SMALL_RECORD);
    }
    if(ret < 0 ){
        db_msg("fail to stop recorder camera_csi_net_small_record.");
    }
#endif

	StorageManager *sm = StorageManager::getInstance();
	sm->startSyncThread();
	if(!mCdrMain->getNoUsbCharge()){
		mCdrMain->noWork(true);
	}
	return ret;
}


int RecordPreview::stopRecordSync()
{
	int ret = 0;

	if(mRPWindowState == STATUS_AWMD) {
		if(isRecording(CAM_CSI)) {
			db_msg("need delay start the awmd record\n");
			setDelayStartAWMD(true);
		}
	}
	ret = stopRecord(CAM_CSI);
	if (ret < 0) {
		db_msg("fail to stop recorder csi");
	}

	ret = stopRecord(CAM_UVC);
	if (ret < 0) {
		db_msg("fail to stop recorder uvc");
	}
	StorageManager *sm = StorageManager::getInstance();
	sm->storageSync();
	return ret;

}

void RecordPreview::recordRelease(int cam_type)
{
	MediaRecorderObj *mobj = getMediaRecordObj((enum CameraIdRecord)cam_type);
	CdrMediaRecorder *mcr = mobj->getCdrMediaRecorder();
	if(mcr){
		mcr->release();
	}
}

void RecordPreview::recordRelease()
{
	recordRelease(CAM_CSI);
}

void RecordPreview::previewRelease(int cam_type)
{
	if(mCamera[cam_type])
		mCamera[cam_type]->release();
}

void RecordPreview::previewRelease()
{
	previewRelease(CAM_CSI);
}

void RecordPreview::stopPreview(int cam_type)
{
	if(mCamera[cam_type]) {
		mCamera[cam_type]->stopPreview();
	}
}

void RecordPreview::stopPreview()
{
	stopPreview(CAM_CSI);
	stopPreview(CAM_UVC);
}

CdrDisplay *RecordPreview::getCommonDisp(int disp_id)
{
	if(mCamera[disp_id])
		return mCamera[disp_id]->getDisp();
	else
		return NULL;
}

#ifdef PLATFORM_0
int RecordPreview::getHdl(int disp_id)
{
       CdrDisplay *cd = getCommonDisp(disp_id);
       return cd->getHandle();
}
#endif
void RecordPreview::setSilent(bool mode)
{
	db_msg("-------------------setSilent:%d\n", mode);
	/*if (mRecorder[CAM_CSI]) {
		mRecorder[CAM_CSI]->setSilent(mode);
	}
	if (mRecorder[CAM_UVC]) {
		mRecorder[CAM_UVC]->setSilent(mode);
	}*/
	MediaRecorderObj* obj = NULL;
	for(int i=0 ; i<CAM_CNT; i++){
		 obj = getMediaRecordObj((enum CameraIdRecord)i);
		 if(obj->getCdrMediaRecorder()){
			obj->getCdrMediaRecorder()->setSilent(mode);
		 }
	}

	obj = getMediaRecordObj(CAMERA_CSI_NET_SMALL_RECORD);
	if(obj->getCdrMediaRecorder()){
	   obj->getCdrMediaRecorder()->setSilent(mode);
	}
	
	mSilent = mode;
}

void RecordPreview::setVideoQuality(int idx)
{
	Mutex::Autolock _l(mLock);
	db_msg("----------------setVideoQuality, %d", idx);
	if (idx == 0) {
		mVideoSize = Size(1920, 1080);
	} else if (idx == 1) {
		mVideoSize = Size(1280, 720);
	}
#ifndef YOUKU_PLATFORM
	mSessionSize.width   = mVideoSize.width  / 4;
	mSessionSize.height  = mVideoSize.height / 4;
#endif

	if (mCamera[CAM_CSI]) {
		mCamera[CAM_CSI]->setWaterMarkPos(mVideoSize.height);
	}
	if (mCamera[CAM_UVC]) {
		mCamera[CAM_UVC]->setWaterMarkPos(mVideoSize.height);
	}
}

// set net stream resolution
void RecordPreview::setSessionSize(int idx)
{
	switch (idx) {
		case 0:
			mSessionSize.width   = 640;
			mSessionSize.height  = 360;
			break;
		case 1:
			mSessionSize.width   = 480;
			mSessionSize.height  = 270;
			break;
		default:
			break;
	}
}

// set net stream bitrate
void RecordPreview::setSessionBitrate(int idx)
{
	switch (idx) {
		case 0:
			mSessionBitrate = 800 << 10; // 8kbps
			break;
		case 1:
			mSessionBitrate = 1 << 20; // 1Mbps
			break;
		case 2:
			mSessionBitrate = 1 << 21; // 2Mbps
			break;
		default:
			break;
	}
}

void RecordPreview::setPicQuality(int idx)
{
	Mutex::Autolock _l(mLock);
	db_msg("----------------setPicQuality %d", idx);
	if (mCamera[CAM_CSI]) {
		mCamera[CAM_CSI]->setImageQuality((ImageQuality_t)idx);
	}
	if (mCamera[CAM_UVC]) {
		mCamera[CAM_UVC]->setImageQuality((ImageQuality_t)idx);
	}
}

void RecordPreview::setRecordTime(int idx)
{
	Mutex::Autolock _l(mLock);
	db_msg("----------------setRecordTime %d!", idx);
	if(idx < 0 || idx >= 3) {
		idx = 0;
	}
	int time ;
	switch(idx)
	{
		case 0:
			time = 1*60*1000;
			break;
		case 1:
			time = 3*60*1000;
			break;
		case 2:
			time = 5*60*1000;
			break;
		default:
			break;
	}
	mDuration = time;
#if 0
	MediaRecorderObj *mr = getMediaRecordObj(CAMERA_CSI_RECORD);
	CdrMediaRecorder *mcr = mr->getCdrMediaRecorder();
	if (mcr) {
		mcr->setDuration(time);
	}
#endif
}

void RecordPreview::setRecordTime(PARK_MON_VTL_t value)
{
	Mutex::Autolock _l(mLock);
	db_msg("----------------setRecordTime %ds!", value);
	if(value < 0 || value > 30)
		return;
	int time = (value*1000);
	mDuration = time;
#if 0
	MediaRecorderObj *mr = getMediaRecordObj(CAMERA_CSI_RECORD);
	CdrMediaRecorder *mcr = mr->getCdrMediaRecorder();
	if (mcr) {
		mcr->setDuration(time);
	}

#endif
}


void RecordPreview::setAWMD(bool val)
{
	db_msg("----------------setAWMD %d!", val);
	if (mCamera[CAM_CSI]) {
		mCamera[CAM_CSI]->setAWMD(val);
	}
}

void RecordPreview::setADAS(bool val, int tips)
{
	db_msg("-----------mADAS-----setADAS %d!", val);
	if (mCamera[CAM_CSI]) {
		mCamera[CAM_CSI]->setAdas(val);
		EventManager *em = mCdrMain->getEventManager();
		if(em){
			em->setAdasIsOpen(val);
		}
		if(val){
			ResourceManager * rm = ResourceManager::getInstance();
			int fcwSensity = rm->getResIntValue(ID_MENU_LIST_FCWSENSITY, INTVAL_SUBMENU_INDEX);
			if(mCamera[CAM_CSI]){
				mCamera[CAM_CSI]->setAdasFCWSensity(fcwSensity);
			}
			db_msg(" ****tips:%d***  ",tips);
			if(tips !=0 ){
				mCamera[CAM_CSI]->setAdasTipsMode(tips);
			}
		}
	}
}


void RecordPreview::setWatermark(int flag)
{
	ResourceManager * rm = ResourceManager::getInstance();
	if(mCamera[CAM_CSI])
		mCamera[CAM_CSI]->setLWaterMark(flag&WATERMARK_TWM, flag&WATERMARK_LWM, rm->menuDataLWM.lwaterMark);
	if(mCamera[CAM_UVC])
		mCamera[CAM_UVC]->setLWaterMark(flag&WATERMARK_TWM, flag&WATERMARK_LWM, rm->menuDataLWM.lwaterMark);
}

int RecordPreview::msgCallback(int msgType, int idx)
{
	ResourceManager* rm = ResourceManager::getInstance();
	switch(msgType) {
	case MSG_REFRESH:
		{
			idx = rm->getResIntValue(ID_MENU_LIST_VQ, INTVAL_SUBMENU_INDEX);
			setVideoQuality(idx);

			idx = rm->getResIntValue(ID_MENU_LIST_PQ, INTVAL_SUBMENU_INDEX);
			setPicQuality(idx);

			idx = rm->getResIntValue(ID_MENU_LIST_VTL, INTVAL_SUBMENU_INDEX);
			setRecordTime(idx);

			idx = rm->getResIntValue(ID_MENU_LIST_WB, INTVAL_SUBMENU_INDEX);
			if (mCamera[CAM_CSI]) {
				mCamera[CAM_CSI]->setWhiteBalance(WHITE_BALANCE[idx]);
			}
			idx = rm->getResIntValue(ID_MENU_LIST_CONTRAST, INTVAL_SUBMENU_INDEX);
			if (mCamera[CAM_CSI]) {
				mCamera[CAM_CSI]->setContrast(idx);
			}
			idx = rm->getResIntValue(ID_MENU_LIST_EXPOSURE, INTVAL_SUBMENU_INDEX);
			if (mCamera[CAM_CSI]) {
				mCamera[CAM_CSI]->setExposure(idx-3);
			}
			bool val;
			val = rm->getResBoolValue(ID_MENU_LIST_SILENTMODE);
			setSilent(val);

			val = rm->getResBoolValue(ID_MENU_LIST_AWMD);
			setAWMD(val);
#ifdef APP_ADAS
			idx = rm->getResIntValue(ID_MENU_LIST_SMARTALGORITHM, INTVAL_SUBMENU_INDEX);
			db_msg("******mADAS idx:%d******",idx);
			if(idx == 0){
				setADAS(false);
			}else if(idx == 1){
				setADAS(true, idx);
			}else if(idx == 2){
				setADAS(true, idx);
			}
#endif
			updateAWMDWindowPic();
		}
		break;
	case MSG_RM_VIDEO_QUALITY:
		{
			setVideoQuality(idx);
		}
		break;
	case MSG_RM_PIC_QUALITY:
		{
			setPicQuality(idx);
		}
		break;
	case MSG_RM_VIDEO_TIME_LENGTH:
		{
			setRecordTime(idx);
		}
		break;
	case MSG_RM_WHITEBALANCE:
		{
			if (mCamera[CAM_CSI]) {
				mCamera[CAM_CSI]->setWhiteBalance(WHITE_BALANCE[idx]);
			}
		}
		break;
	case MSG_RM_CONTRAST:
		{
			if (mCamera[CAM_CSI]) {
				mCamera[CAM_CSI]->setContrast(idx);
			}
		}
		break;
	case MSG_RM_EXPOSURE:
		{
			if (mCamera[CAM_CSI]) {
				mCamera[CAM_CSI]->setExposure(idx-3);
			}
		}
		break;
	case MSG_AUDIO_SILENT:
		{
			setSilent((bool)idx);
		}
		break;
	case MSG_AWMD:
		setAWMD((bool)idx);
		break;
	#ifdef APP_ADAS
	case MSG_ADAS_SWITCH:
		db_msg("******mADAS idx:%d******",idx);
		if(idx==0){
			setADAS(false);
		}else if(idx == 1){
			setADAS(true, idx);
		}else if(idx == 2){
			setADAS(true, idx);
		}
		break;
	#endif
	case MSG_WATER_MARK:
		setWatermark((int)idx);
		break;
	case MSG_RM_TAKE_PICTURE:
		{
			if((mRPWindowState == STATUS_RECORDPREVIEW || mRPWindowState == STATUS_AWMD)&& !isRecording()) {
				db_msg("change to photograph modem\n");
				mRPWindowState = STATUS_PHOTOGRAPH;
				mCdrMain->setCurrentWindowState(mRPWindowState);
			}
			if(!isRecording()){
				if (mCamera[CAM_CSI])
					mCamera[CAM_CSI]->setPictureSizeMode(CameraParameters::AWEXTEND_PICTURE_SIZE_MODE::UseParameterPictureSize);
				if (mCamera[CAM_UVC])
					mCamera[CAM_UVC]->setPictureSizeMode(CameraParameters::AWEXTEND_PICTURE_SIZE_MODE::UseParameterPictureSize);
				takePicture(1, 1);
			}
		}
		break;
	case MSG_ADAS_AUDIO_VOL:
		if(mCamera[CAM_CSI]){
			mCamera[CAM_CSI]->setAdasAudioVol(idx);
		}
		break;
	case MSG_ADAS_FCW_SENSITY:
		if(mCamera[CAM_CSI]){
			mCamera[CAM_CSI]->setAdasFCWSensity(idx);
		}
	default:
		break;
	}
	return 0;
}

void RecordPreview::feedAwmdTimer(int seconds)
{
	if(mCamera[CAM_CSI])
		mCamera[CAM_CSI]->feedAwmdTimer(seconds);
}

int RecordPreview::queryAwmdExpirationTime(time_t* sec, long* nsec)
{
	if(mCamera[CAM_CSI])
		return mCamera[CAM_CSI]->queryAwmdExpirationTime(sec, nsec);
	else
		return -1;
}


void RecordPreview::updateAWMDWindowPic()
{
	bool awmd;
	ResourceManager* rm;

	rm = ResourceManager::getInstance();
	awmd = rm->getResBoolValue(ID_MENU_LIST_AWMD);
	if(awmd == true) {
		db_msg("updateAWMDWindowPic true\n");
		mRPWindowState = STATUS_AWMD;
		mCdrMain->setCurrentWindowState(mRPWindowState);					
	} else {
		db_msg("updateAWMDWindowPic false\n");
		mRPWindowState = mOldRPWindowState;
		//	mRPWindowState = STATUS_RECORDPREVIEW;	
		mCdrMain->setCurrentWindowState(mRPWindowState);					
	}
}

void RecordPreview::switchAWMD()
{
	if(mRPWindowState != STATUS_RECORDPREVIEW && mRPWindowState != STATUS_AWMD)
		return;
	if(mRPWindowState == STATUS_AWMD) {
		db_msg("disable awmd\n");	
		switchAWMD(false);
		setDelayStartAWMD(false);
		mCdrMain->noWork(true);
	} else {
		db_msg("enable awmd\n");	
		switchAWMD(true);
		mCdrMain->noWork(false);
	}
}

void RecordPreview::switchAWMD(bool enable)
{
	ResourceManager* rm;
	time_t sec = 0;
	long nsec = 0;
	CdrRecordParam_t param;

	rm = ResourceManager::getInstance();
	if(enable == true) {
		stopRecord(CAM_CSI);
		rm->setResBoolValue(ID_MENU_LIST_AWMD, true);
		mRPWindowState = STATUS_AWMD;
		mCdrMain->setCurrentWindowState(mRPWindowState);					
		usleep(500);
		setAWMD(true);
	} else {
		if(queryAwmdExpirationTime(&sec, &nsec) < 0) {
			db_error("query awmd timer expiration time failed\n");
		}
		setAWMD(false);
		if(isRecording(CAM_CSI) == true) {
			if(sec > 0 || (sec == 0 && nsec > 500 * 1000 * 1000)) { 
				param.video_type = VIDEO_TYPE_NORMAL;
				param.fileSize = CSI_PREALLOC_SIZE(mDuration);
				param.duration = mDuration;
				db_msg("set duration %d ms\n", mDuration);

				MediaRecorderObj *mr = getMediaRecordObj(CAMERA_CSI_RECORD);
				CdrMediaRecorder *mcr = mr->getCdrMediaRecorder();
				mcr->setDuration(mDuration);
				mcr->setRecordParam(param);
			}
		}
		rm->setResBoolValue(ID_MENU_LIST_AWMD, false);
		mRPWindowState = STATUS_RECORDPREVIEW;
		mCdrMain->setCurrentWindowState(mRPWindowState);					
	}
}

/*flag: 1 take picture, 2 screen shot ; mode: 1(default value) CSI and UVC take pic or screen shot, 2 csi take pic or screen shot, 3 uvc take pic or screen shot */
int RecordPreview::takePicture(int flag, int mode) 
{
	if(!checkSDIsFAT())
		return -1;
	HWND hStatusBar;
	const char* ptr;
	StorageManager* sm;
	ResourceManager* rm;
	int ret = 0;
	if(mCamera[CAM_CSI] || mCamera[CAM_UVC]) {
		sm = StorageManager::getInstance();
		if(sm->isMount() == false) {
			if (!mStartRtspSession)
				ShowTipLabel(mHwnd, LABEL_NO_TFCARD);
			return -1;
		}
		if(sm->isDiskFull() == true) {
			if (!mStartRtspSession)
				ShowTipLabel(mHwnd, LABEL_TFCARD_FULL);
			return -1;
		}
		rm = ResourceManager::getInstance();
		hStatusBar = mCdrMain->getWindowHandle(WINDOWID_STATUSBAR);
		{
			Mutex::Autolock _l(mLockPic);
			if(mode == 1){
				camTakePicture(CAM_CSI);
				camTakePicture(CAM_UVC);
			}else if(mode == 2){
				camTakePicture(CAM_CSI);
			}else if(mode == 3){
				camTakePicture(CAM_UVC);
			}
		}

		if (!mStartRtspSession) {
			if(flag==1){
				ptr = rm->getLabel(LANG_LABEL_TAKE_PICTURE);
				if(ptr){
					SendMessage(hStatusBar, STBM_SETLABEL1_TEXT, 0, (LPARAM)ptr);
				}
			}else if(flag==2){
				ptr = rm->getLabel(LANG_LABEL_SCREEN_SHOT);
				if(ptr){
					SendMessage(hStatusBar, STBM_SETLABEL1_TEXT, 2, (LPARAM)ptr);
				}
			}
		}
	}
	return ret;
}

void RecordPreview::savePicture(void*data, int size, int id)
{
	int ret = 0;
	time_t now;
	HWND hStatusBar;
	StorageManager* sm;
	String8 file;
	Elem elem;

	sm = StorageManager::getInstance();
	hStatusBar = mCdrMain->getWindowHandle(WINDOWID_STATUSBAR);

	db_msg(" TakePicType:%d ", mCamera[id]->getTakepicType());

	if(mSmallRecordTakePic){
		ret = sm->generateFile(now, file, id, PHOTO_TYPE_WONDERFUL);
		mSmallRecordDBInfo.picString = file;
	}else{
#ifdef USE_TAKEPIC_NETJPD
		if(mCamera[id]->getTakepicType() == SMALL_TAKE_PIC){
			if(mTakePicName[id] != ""){
				file = mTakePicName[id];
			}else{
				db_msg("mTakePicName NULL so cannot generate netPic");
				return ;
			}
		}else if(mCamera[id]->getTakepicType() == GSENSOR_TAKE_PIC){
			sm->geneNetPicName(mExtraDBInfo[id].fileString, file, EXT_PIC_JPG);
			mCamera[id]->setTakepicType(SMALL_TAKE_PIC);
		}else{
			ret = sm->generateFile(now, file, id, PHOTO_TYPE_NORMAL);
		}
#else
		ret = sm->generateFile(now, file, id, PHOTO_TYPE_NORMAL);
#endif
	}
	if(ret < 0) {
		db_error("generate picture file failed, retval is %d\n", ret);
		SendMessage(hStatusBar, STBM_CLEAR_LABEL1_TEXT, WINDOWID_RECORDPREVIEW, 0);
		if((ret==RET_DISK_FULL) || (ret == RET_IO_NO_RECORD)){
			CloseTipLabel();
			SendNotifyMessage(mHwnd, MSG_SHOW_TIP_LABEL, LABEL_TFCARD_FULL, 0);
		}
		return;	
	}
	db_msg("picture is %s\n", file.string());
	sm->savePicture(data, size, file);
	if((mCamera[id]->getTakepicType() == NORMAL_TAKE_PIC) 
		||(mCamera[id]->getTakepicType() == SCREENSHOT_TAKE_PIC)){
		elem.file = (char*)file.string();
		elem.time = now;
		elem.type = (char*)TYPE_PIC;
		elem.info = (char*)"";
		elem.filesize = sm->fileSize(file);
		sm->dbAddFile(&elem);
		if(mSmallRecordTakePic){
			generateNetPic(file, TYPE_PIC, mSmallRecordDBInfo.netPic);
			mSmallRecordTakePic = false;
		}else{
			String8 netPic;
			generateNetPic(file, TYPE_PIC, netPic); 
		}
	}
	if(mCamera[id]->getTakepicType() == SCREENSHOT_TAKE_PIC){
		mCamera[id]->setJpegQuality(NET_JPEG_QUALITY);
		mCamera[id]->setTakepicType(SMALL_TAKE_PIC);
		mCamera[id]->setPictureSizeMode(CameraParameters::AWEXTEND_PICTURE_SIZE_MODE::UseParameterPictureSize);	
	}
	if(!isRecording(id)){
		usleep(500 * 1000);
		SendMessage(hStatusBar, STBM_CLEAR_LABEL1_TEXT, WINDOWID_RECORDPREVIEW, 0);
	}
}

bool RecordPreview::isRecording(int cam_type)
{
	Mutex::Autolock _l(mRecordLock);
	MediaRecorderObj *mr = getMediaRecordObj((enum CameraIdRecord)cam_type);
	CdrMediaRecorder *mcr = mr->getCdrMediaRecorder();
	if(mcr) {
		if(mcr->getRecorderState() == RECORDER_RECORDING)	
			return true;
	}
	return false;
}
bool RecordPreview::isRecording()
{
	if(isRecording(CAM_CSI) == true || isRecording(CAM_UVC) == true)
		return true;
	return false;
}

int RecordPreview::impactOccur()
{
	db_msg("impact Occur");
	int ret1=-1,ret2 = -1;
	time_t now;
	StorageManager* sm;
	String8 file;
	HWND hStatusBar;
	unsigned int flag = 0;

	if(!isRecording()) {
		db_error("record is not started\n");
		return -1;
	}

	hStatusBar = mCdrMain->getWindowHandle(WINDOWID_STATUSBAR);
	//if(SendMessage(hStatusBar, STBM_GET_RECORD_TIME, 0, 0) <= (BFTIMEMS / 1000) ) {
	//	db_error("record time less than %d ms\n", BFTIMEMS);
	//	return -1;
	//}

	Mutex::Autolock _l(mImpactLock);

/*	if(mCamExist[CAM_CSI] && mRecorder[CAM_CSI] && (mRecorder[CAM_CSI]->getImpactStatus() == true) ) {
		db_msg("CSI impact video has not finished\n");
		return -1;
	}
	if(mCamExist[CAM_UVC] && mRecorder[CAM_UVC] && (mRecorder[CAM_UVC]->getImpactStatus() == true) ) {
		db_msg("UVC impact video has not finished\n");
		return -1;
	}*/
	mCamera[CAM_CSI]->setAdasMode(1);
	sm = StorageManager::getInstance();
	if ( isRecording(CAM_CSI) ) {
		MediaRecorderObj *mr = getMediaRecordObj((enum CameraIdRecord)CAM_CSI);
		CdrMediaRecorder *mcr = mr->getCdrMediaRecorder();
		if(!mcr->getImpactStatus()){
			ret1 = sm->generateFile(now, file, CAM_CSI, VIDEO_TYPE_IMPACT);
			if(mCdrMain->getWakeupState()) {
				mImpactFileStart[CAM_CSI]=1;
			}
			if(ret1 == RET_IO_OK) {
				mExtraDBInfo[CAM_CSI].fileString = file;
				mExtraDBInfo[CAM_CSI].infoString = INFO_LOCK;
				mExtraDBInfo[CAM_CSI].time = now;
				mImpactFlag[CAM_CSI] = 1;
				flag = 1;
#ifdef APP_WIFI
				mCamera[CAM_CSI]->setTakepicType(GSENSOR_TAKE_PIC);
				camTakePicture(CAM_CSI);
				mCdrMain->cdrReplyToApp(NAT_SOS_IMPACT_OCCUR, NULL, 0);
#endif
				mcr->startImpactVideo(file, SZ_1M * 30);
				SendMessage(hStatusBar, STBM_START_RECORD_TIMER, 1, 0);

			} else {
				db_msg("generate CSI impact file failed\n");
			}
		}else{
			mcr->startImpactVideo(mExtraDBInfo[CAM_CSI].fileString, SZ_1M * 30);
			SendMessage(hStatusBar, STBM_START_RECORD_TIMER, 1, 0);
#ifdef APP_WIFI
			mCdrMain->cdrReplyToApp(NAT_SOS_IMPACT_OCCUR, NULL, 0);
#endif
		}
	}

	if ( isRecording(CAM_UVC) ) {
		MediaRecorderObj *mr = getMediaRecordObj((enum CameraIdRecord)CAM_UVC);
		CdrMediaRecorder *mcr = mr->getCdrMediaRecorder();
		if(!mcr->getImpactStatus()){
			ret2 = sm->generateFile(now, file, CAM_UVC, VIDEO_TYPE_IMPACT);
			if(mCdrMain->getWakeupState()) {
				mImpactFileStart[CAM_UVC]=1;
			}
			if(ret2 == RET_IO_OK) {
				mExtraDBInfo[CAM_UVC].fileString = file;
				mExtraDBInfo[CAM_UVC].infoString = INFO_LOCK;
				mExtraDBInfo[CAM_UVC].time = now;
				mImpactFlag[CAM_UVC] = 1;
				flag = 1;
#ifdef APP_WIFI
				mCamera[CAM_UVC]->setTakepicType(GSENSOR_TAKE_PIC);
				camTakePicture(CAM_UVC);
#endif
				mcr->startImpactVideo(file, SZ_1M * 30);
			} else {
				db_msg("generate UVC impact file failed\n");
			}
		}else{
			mcr->startImpactVideo(mExtraDBInfo[CAM_UVC].fileString, SZ_1M * 30);
		//	SendMessage(hStatusBar, STBM_START_RECORD_TIMER, 1, 0);
		}
	}

#if 0
	if(flag == 1)
		lockFile();
#endif

	return 0;
}

/*
 * camera CSI should stop preview, keep layer open, and keep layer rectBig
 * camera UVC should stop preview, keep layer close
 * */
void RecordPreview::prepareCamera4Playback(bool closeLayer)
{
	CdrDisplay *cdCSI = NULL, *cdUVC = NULL;

	if(mCamera[CAM_CSI]) {
		cdCSI = mCamera[CAM_CSI]->getDisp();
		cdCSI->open();
		cdCSI->setRect(rectBig);
		stopPreview(CAM_CSI);
		if (cdCSI && closeLayer) {
			cdCSI->clean();
		}
	}

	if(mCamera[CAM_UVC]) {
		cdUVC = mCamera[CAM_UVC]->getDisp();
		cdUVC->close();
		stopPreview(CAM_UVC);
		if(cdUVC && closeLayer) {
			cdUVC->clean();
		}
	}
}

void RecordPreview::restorePreviewOutSide(unsigned int windowID)
{
	CdrDisplay *cdCSI = NULL, *cdUVC = NULL;//, *top = NULL, *bot=NULL;
	pipMode_t newPipMode;

	if(windowID == WINDOWID_MENU) {
		db_info("return from menu\n");
		return;
	}
	ResourceManager * rm = ResourceManager::getInstance();
	bool awmd = rm->getResBoolValue(ID_MENU_LIST_AWMD);
	if(awmd){
		mRPWindowState = STATUS_AWMD;
	}
	if(!mCamera[CAM_CSI])
		return;

	if(!mCamExist[CAM_UVC]) {
		newPipMode = CSI_ONLY;
	} else {
		if(!mCamera[CAM_UVC]) {
			initCamera(CAM_UVC);
		}
		newPipMode = UVC_ON_CSI;
	}
	db_msg("pipmode :%d", newPipMode);

	if((mPIPMode==CSI_ON_UVC || mPIPMode==UVC_ONLY)&& mCamera[CAM_UVC]){
		cdUVC = mCamera[CAM_UVC]->getDisp();
		startPreview(CAM_UVC);
		cdCSI = mCamera[CAM_CSI]->getDisp();
		startPreview(CAM_CSI);
		cdUVC->setRect(rectBig);
		cdCSI->setRect(rectSmall);
		cdUVC->open();
		cdUVC->setBottom();
		cdCSI->open();
		cdUVC->exchange(cdCSI->getHandle(),1);
		mPIPMode = UVC_ON_CSI;
		return ;
	}	
	
	if (newPipMode == UVC_ON_CSI) {
		cdUVC = mCamera[CAM_UVC]->getDisp();
	//	cdUVC->clean();
		startPreview(CAM_UVC);
		cdUVC->setBottom();
		cdUVC->setRect(rectSmall);
		cdUVC->open();
	}
	cdCSI = mCamera[CAM_CSI]->getDisp();
	//cdCSI->clean();
	startPreview(CAM_CSI);
	cdCSI->setBottom();
	cdCSI->setRect(rectBig);
	cdCSI->open();
	mPIPMode = newPipMode;
}

void RecordPreview::setSdcardState(bool bExist)
{
	MediaRecorderObj *mr_CSI = getMediaRecordObj((enum CameraIdRecord)CAM_CSI);
	CdrMediaRecorder *mcr_CSI = mr_CSI->getCdrMediaRecorder();

	MediaRecorderObj *mr_UVS = getMediaRecordObj((enum CameraIdRecord)CAM_UVC);
	CdrMediaRecorder *mcr_UVS = mr_UVS->getCdrMediaRecorder();

	if (mcr_CSI) {
		mcr_CSI->setSdcardState(bExist);
	}
	if (mcr_UVS) {
		mcr_UVS->setSdcardState(bExist);
	}
}
bool RecordPreview::needDelayStartAWMD(void)
{
	Mutex::Autolock _l(mAwmdLock);
	return mAwmdDelayStart;
}

void RecordPreview::setDelayStartAWMD(bool value)
{
	Mutex::Autolock _l(mAwmdLock);
	mAwmdDelayStart = value;
	if(value == true)
		mAwmdDelayCount = 0;
}

bool RecordPreview::getUvcState()
{
	return mCamExist[CAM_UVC];
}

void RecordPreview::releaseRecorder(int cam_id)
{

	MediaRecorderObj *mr = getMediaRecordObj((enum CameraIdRecord)cam_id);
	CdrMediaRecorder *mcr = mr->getCdrMediaRecorder();

	if(mcr) {
		delete mcr;
		mcr = NULL;
	}
}
void RecordPreview::sessionInit(void *rtspSession, int camId)
{
	mStartRtspSession = true;
#ifndef YOUKU_PLATFORM
	mSessionSize.width   = mVideoSize.width  / 4;
	mSessionSize.height  = mVideoSize.height / 4;
#endif

	if (mCamExist[CAM_CSI]) {
		MediaRecorderObj *mr = getMediaRecordObj(CAMERA_CSI_NET_RECORD);
		CdrMediaRecorder *mcr = mr->getCdrMediaRecorder();

		if (!mcr) {
			mcr = new CdrMediaRecorder(CAM_CSI_NET, rtspSession);
			mcr->setOnInfoCallback(this);
			mcr->setOnErrorCallback(this);
#ifdef PLATFORM_0
			if(mcr->getRecorderState() == RECORDER_RECORDING){	
				CdrMediaRecorder *mcr_CSI = mr->getCdrMediaRecorder();
				setVirtualCam(mcr_CSI);
			}
#endif
		}
	}
#ifdef PLATFORM_1
	if (mCamExist[CAM_UVC])  {
		MediaRecorderObj *mr = getMediaRecordObj(CAMERA_UVC_NET_RECORD);
		CdrMediaRecorder *mcr = mr->getCdrMediaRecorder();
		
		if (!mcr) {
			mcr = new CdrMediaRecorder(CAM_UVC_NET, rtspSession);
			mcr->setOnInfoCallback(this);
			mcr->setOnErrorCallback(this);
			
		}
	}
#endif
	mSession = rtspSession;
	mCurSessionCam = LOCAL2SESSION(camId);
}

int RecordPreview::camChange(int cam_to, bool bStop)
{
	int ret = -1;
	int cam_from = SESSION2LOCAL(mCurSessionCam);
	cam_to = CAM_OTHER(cam_from);
	db_msg(" change from cam%d to cam%d", cam_from, cam_to);
	if(mCamExist[cam_to] && mCamExist[cam_from] && bStop) {
		if(ret = stopRecord(mCurSessionCam) < 0) {
			db_error("stopRecord cam%d failed, ret is %d\n", cam_from, ret);
			return ret;
		}
	}
	ret = -1;
	if(mCamExist[cam_to]) {
		if(ret = startRecord(LOCAL2SESSION(cam_to)) < 0) {
			db_error("startRecord cam%d failed, ret is %d\n", cam_to, ret);
			return -1;
		}
		mCurSessionCam = LOCAL2SESSION(cam_to);
	}
	return ret;
}

void RecordPreview::changeResolution(int level)
{
	int max_h = mVideoSize.height;
	int max_w = mVideoSize.width;
	int w;
	int h;
	void *session = NULL;
	db_msg("level : %d", level);
	if (level == 0) {
		w = max_w / 2;
		h = max_h / 2;
	} else {
		w = max_w / 4;
		h = max_h / 4;
	}
	db_msg("final_w : %d, final_h : %d", w, h);
	stopRecord(mCurSessionCam);
	CdrRecordParam_t param;
	param.duration = 30*1000;
	param.fileSize = 10*1024;
	param.video_type = VIDEO_TYPE_NORMAL;
	Size size;
	size.width  = w;
	size.height = h;
	prepareRecord(mCurSessionCam, param, &size);

	MediaRecorderObj *mr = getMediaRecordObj((enum CameraIdRecord)mCurSessionCam);
	CdrMediaRecorder *mcr = mr->getCdrMediaRecorder();
	mcr->start();
}

unsigned int RecordPreview::getWindowsID()
{
	return mCdrMain->getCurWindowID();
}

bool RecordPreview::getAlignLine()
{
	if(mAlignLine)
		return mAlignLine;
	return FALSE;
}

void RecordPreview::setAlignLine(bool flag)
{
	if(mCamera[CAM_UVC]) {
		mCamera[CAM_UVC]->setCameraFlip(flag);
	}
}

void RecordPreview::adasScreenSwitch(bool flag)
{
	db_msg("<**RecordPreview::adasScreenSwitch**flag:%d**mIsAdasScreen:%d*>",flag,mIsAdasScreen);
	if (mIsAdasScreen == flag || mCdrMain->getCurWindowID() != WINDOWID_RECORDPREVIEW || (mCdrMain->isShuttingdown() && flag)) {
		return;
	}
	CdrDisplay *cdCSI;
	cdCSI = getCommonDisp(CAM_CSI);
	if(flag){
		if(mCdrMain->getOTAAdasScreen() || mCdrMain->getUVCCarBackAdasScreen()){
			return ;
		}
		if( !mCdrMain->getPC2Connected() || mCdrMain->getUsbcharge()){
			BroadcastMessage(MSG_CLOSE_TIP_LABEL, 0, 0);
			cdCSI->openAdasScreen();
		}else{
			mCdrMain->setm2pcAdasScreen(true);
			return ;
		}
	} else {
		cdCSI->closeAdasScreen();
	}
	mIsAdasScreen = flag;
   /* if(!mCdrMain->getFormatTFAdasS()&& mIsAdasScreen){
        StorageManager* smt;
        smt = StorageManager::getInstance();
        if(smt->isInsert()){
                if( !mCdrMain->checkSDFs()){    
                        mCdrMain->setFormatTFAdasS(true);
                        SendNotifyMessage(mCdrMain->getWindowHandle(WINDOWID_MENU), MSG_FORMAT_TFDIALOG, (WPARAM)mIsAdasScreen, 0);
                }
        }
    }*/
	
}

bool RecordPreview::getIsAdasScreen()
{
	return mIsAdasScreen;
}


bool RecordPreview::camExisted(int idx)
{
	return mCamExist[idx];
}

int RecordPreview::getDuration()
{
	return mDuration;
}

void RecordPreview::needCache(bool val)
{
	mNeedCache = val;
}

void RecordPreview::startPCCam()
{
	//HerbCamera::Parameters params;
	//mCamera[CAM_CSI]->getParameters(&params);
	//params.setVideoSize(1920, 1080);
    //params.setPreviewSize(640, 360);
    //params.setPreviewFrameRate(30);
    //mCamera[CAM_CSI]->setParameters(&params);
    cfgDataSetString("sys.usb.config", "webcam,mass_storage");
	usleep(200*1000);
	mCamera[CAM_CSI]->setUvcMode(1);
}

void RecordPreview::stopPCCam()
{
	mCamera[CAM_CSI]->setUvcMode(0);
	cfgDataSetString("sys.usb.config", "mass_storage,adb");
}

RPWindowState_t RecordPreview::getRPWindowState()
{
	return mRPWindowState;
}

void RecordPreview::setRPWindowState(RPWindowState_t rWindowStat)
{
	mRPWindowState = rWindowStat;
}


CdrCamera * RecordPreview::getCdrCamera(int camId)
{
       if(camId == CAM_CSI){
               return mCamera[CAM_CSI];
       }else if(camId == CAM_UVC){
               return mCamera[CAM_UVC];
       }
       return NULL;
}

void RecordPreview::errorOccur(int error)
{
	db_msg("****ESD errorOccur******");
	if (error == CAMERA_ERROR_SELECT_TIMEOUT) {
	       db_msg("****select time out**********");
		   suspend();
		   resume();
		   mCdrMain->configInit();
	}
}

void RecordPreview::suspend()
{
	//stopRecordSync();
	storagePlug(false);
	stopPreview();
	CdrDisplay *cdCSI = NULL;
	cdCSI = getCommonDisp(CAM_CSI);
	cdCSI->clean();
	cdCSI->close();
	if (mCamera[CAM_CSI]) {
		mCamera[CAM_CSI]->suspend();
	}
}

void RecordPreview::resume()
{
	if (mCamera[CAM_CSI]) {
		mCamera[CAM_CSI]->resume();
	}
	initCamera(CAM_CSI, false);
	startPreview();
	CdrDisplay *cdCSI = NULL;
	cdCSI = getCommonDisp(CAM_CSI);
	cdCSI->open();
}

void RecordPreview::createMRObjList()   //create record list
{
	for(int i=0; i<CameraIdRecordNum; i++){
		MediaRecorderObj *mrobj = NULL;
		switch(i){
			case 0:
				mrobj = new MediaRecorderObj(NULL, false);
				mrobj->setUseCam(0);
				mMRObjList.add(CAMERA_CSI_RECORD, mrobj);
				break;
			case 1:
				mrobj = new MediaRecorderObj(NULL, false);
				mrobj->setUseCam(1);
				mMRObjList.add(CAMERA_UVC_RECORD, mrobj);
				break;
			case 2:
				mrobj = new MediaRecorderObj(NULL, true);
				mrobj->setUseCam(0);
				mMRObjList.add(CAMERA_CSI_NET_RECORD, mrobj);
				break;
			case 3:
				mrobj = new MediaRecorderObj(NULL, true);
				mrobj->setUseCam(1);
				mMRObjList.add(CAMERA_UVC_NET_RECORD, mrobj);
				break;
			case 4:
				mrobj = new MediaRecorderObj(NULL, false);
				mrobj->setUseCam(0);
				mMRObjList.add(CAMERA_CSI_NET_SMALL_RECORD, mrobj);
				break;
			default:
				break;
		}
	}

	db_msg(" ************************************  ");
	for(int kk =0 ;kk<mMRObjList.size(); kk++){
		MediaRecorderObj *mr1 = getMediaRecordObj((enum CameraIdRecord)kk);
		CdrMediaRecorder *mcr1 = mr1->getCdrMediaRecorder();
		db_msg("mr1:%p,mcr1:%p ,",mr1,mcr1);
		db_msg("fuck cam_idx:%d, hassession:%d", mr1->getUseCam(),mr1->getSession());
	}
	
}

MediaRecorderObj* RecordPreview::getMediaRecordObj(enum CameraIdRecord id)
{
	MediaRecorderObj* obj = NULL;
	
	ssize_t index = mMRObjList.indexOfKey(id);
	if((index >= 0) && (index < mMRObjList.size())) {
		obj = mMRObjList.valueAt(index);
	}

//	db_msg("index is %ld, current MediaRecorderObj is 0x%p\n", index, obj);
	return obj;

}

void RecordPreview::handWakeUpRecord(StorageManager* sm)
{	
	//plung in uvc, cam_csi file(no sos),may > 256kb,but you need delete
	if(strcmp(mNeedDelFile[CAM_CSI].string(), "") != 0){
		db_msg("mNeedDelFile[CAM_CSI].string():%s",mNeedDelFile[CAM_CSI].string());
		sm->deleteFile(mNeedDelFile[CAM_CSI].string());
		sm->dbDelFile(mNeedDelFile[CAM_CSI].string());
		mNeedDelFile[CAM_CSI]= "";
	}
	if(strcmp(mNeedDelFile[CAM_UVC].string(), "") != 0){
		sm->deleteFile(mNeedDelFile[CAM_UVC].string());
		sm->dbDelFile(mNeedDelFile[CAM_UVC].string());
		mNeedDelFile[CAM_UVC]= "";
	}
	mCdrMain->setWakeupState(false);
	if(!mCdrMain->isBatteryOnline()){
		stopRecord();
		HWND hStatusBar = mCdrMain->getWindowHandle(WINDOWID_STATUSBAR);
		SendNotifyMessage(hStatusBar, STBM_COUNT_ONCE, 0, 0);
		needCache(true);
		ResourceManager* rm = ResourceManager::getInstance();
		int rTime = rm->getResIntValue(ID_MENU_LIST_VTL, INTVAL_SUBMENU_INDEX);
		setRecordTime(rTime);
		if(!mKeyStop)
			startRecord();
		mCamera[CAM_CSI]->setAdasMode(0);
		return ;
	}else{
		mCdrMain->shutdown();
	}
}

int RecordPreview::startSmallVideoRecord()
{
	int ret = 0;
	db_msg(" startSmallVideoRecord ");
	//startRecord(CAMERA_CSI_NET_SMALL_RECORD);
	MediaRecorderObj *mobj = getMediaRecordObj(CAMERA_CSI_NET_SMALL_RECORD);
	CdrMediaRecorder *mcr = mobj->getCdrMediaRecorder();
	time_t now;
	String8 file;

	StorageManager *sm = StorageManager::getInstance();
	if(!sm->isMount()){
		return -1;
	}
//	memset(&mSmallRecordDBInfo, 0, sizeof(mSmallRecordDBInfo));
	db_msg(" mcr->getImpactStatus():%d  ",mcr->getImpactStatus());
	if(!mcr->getImpactStatus()){
        HWND hStatusBar = mCdrMain->getWindowHandle(WINDOWID_STATUSBAR);
        SendMessage(hStatusBar, STBM_WOND, 1, 0);

        mSmallRecordDBInfo.fileString = "";
        mSmallRecordDBInfo.picString = "";
		mSmallRecordTakePic = true;
		ret = CSIScreenShot();
		ret = sm->generateFile(now, file, CAMERA_CSI_NET_SMALL_RECORD, VIDEO_TYPE_WONDERFUL);
		if(ret == RET_IO_OK) {
			mSmallRecordDBInfo.fileString = file;
			mSmallRecordDBInfo.infoString = INFO_WOND;
			mSmallRecordDBInfo.time = now;
			db_msg("xxx   small_video_name:%s ", mSmallRecordDBInfo.fileString.string());
			mcr->startImpactVideo(file, SZ_1M * 2);
			ret = 0;
		} else {
			db_msg("generate small video file failed\n");
			ret = -1;
		}
	}
	
	return -1;
}

int RecordPreview::CSIScreenShot()
{
	int ret = 0;
	if (mCamera[CAM_CSI]){
		mCamera[CAM_CSI]->setPictureSizeMode(CameraParameters::AWEXTEND_PICTURE_SIZE_MODE::KeepSourceSize);
		mCamera[CAM_CSI]->setTakepicType(SCREENSHOT_TAKE_PIC);
	}
	if(isRecording()){
		mCamera[CAM_CSI]->resetJpegQuality();
	}
	ret = takePicture(2, 2); 
	return ret;
}

int RecordPreview::handSmallVideo(Elem elem)
{
	int ret = 0;
	StorageManager *sm = StorageManager::getInstance();
	elem.info = (char*)INFO_WOND;
	elem.file = (char*)mSmallRecordDBInfo.fileString.string();
	elem.type = (char*)TYPE_VIDEO;
	elem.time = mSmallRecordDBInfo.time;
	elem.filesize = sm->fileSize(mSmallRecordDBInfo.fileString);
	sm->dbAddFile(&elem);
/*	use screen shot picture to generate net picture
	String8 netPic;	
	sm->geneNetPicName(mSmallRecordDBInfo.fileString, netPic, EXT_PIC_JPG);
	generateNetPic(mSmallRecordDBInfo.fileString, TYPE_VIDEO, netPic);*/
	
	aw_cdr_get_smallvideo resp;
	if(mSmallRecordDBInfo.fileString != ""){
		sprintf(resp.videopath, "%s;", mSmallRecordDBInfo.fileString.string());
	}else{
		db_msg("create small video failed");
	}
	if(mSmallRecordDBInfo.picString != ""){
		sprintf(resp.picpath,"%s;" ,mSmallRecordDBInfo.picString.string());
	}else{
		db_msg("create small_video's picture failed");
	}
	
	sprintf(resp.netPic, "%s;", mSmallRecordDBInfo.netPic.string());
	db_msg(" xxxxresp.videopath:%s, resp.pic:%s ,resp.netPic:%s",resp.videopath, resp.picpath, resp.netPic);
	ret = mCdrMain->cdrReplyToApp(NAT_CMD_SHOT_RECORD_FINISH_RESP,(char *)(&resp), sizeof(aw_cdr_get_smallvideo));
	mSmallRecordDBInfo.fileString = "";
	mSmallRecordDBInfo.picString = "";

#ifdef YOUKU_PLATFORM
	HWND hMainWin = mCdrMain->getWindowHandle(WINDOWID_MAIN);
	SendMessage(hMainWin, MSG_SMALL_VIDEO_DONE, 0, 0);
#endif

    HWND hStatusBar = mCdrMain->getWindowHandle(WINDOWID_STATUSBAR);
    SendMessage(hStatusBar, STBM_WOND, 0, 0);

	return ret;
}

int RecordPreview::generateNetPic(String8 filename, char* file_type, String8 &picFile)
{
//	usleep(500 * 1000);
	int ret = 0;
#ifdef APP_WIFI
	if (FILE_EXSIST(filename)){
		PlayBackPreview *mPB = (PlayBackPreview *)mCdrMain->getObjPreview(WINDOWID_PLAYBACKPREVIEW);
		//String8 bmpFile;
		ret = mPB->getNetPicFileName(picFile, filename, file_type);
		db_msg(" generateNetPic :%s ", picFile.string());
	}else{
		return -1;
	}
#endif

	return ret;
}

void RecordPreview::camTakePicture(int cam)
{
	if (mCamera[cam]) {
		if (mAllowTakePic[cam]) {
			db_msg("camera:%d take pic", cam);
			mAllowTakePic[cam] = false;
			mCamera[cam]->takePicture();
		} else {
			db_msg("camera:%d last action hasn't finished", cam);
		}
	}
}

int RecordPreview::getCurSessionCam()
{
	return mCurSessionCam;
}

void RecordPreview::setCurSessionCam(int camId)
{
	mCurSessionCam = LOCAL2SESSION(camId);
}

void RecordPreview::screenShot()
{
	if (mCamera[CAM_CSI]){
		mCamera[CAM_CSI]->resetJpegQuality();
		mCamera[CAM_CSI]->setTakepicType(SCREENSHOT_TAKE_PIC);
		mCamera[CAM_CSI]->setPictureSizeMode(CameraParameters::AWEXTEND_PICTURE_SIZE_MODE::KeepSourceSize);
	}
	if (mCamera[CAM_UVC]){
		mCamera[CAM_UVC]->resetJpegQuality();
		mCamera[CAM_UVC]->setTakepicType(SCREENSHOT_TAKE_PIC);
		mCamera[CAM_UVC]->setPictureSizeMode(CameraParameters::AWEXTEND_PICTURE_SIZE_MODE::KeepSourceSize);
	}
	takePicture(2, 1);
}

CdrMain* RecordPreview::getCdrMain()
{
	return mCdrMain;
}

void RecordPreview::setUvcCarModem(bool val)
{
	mUVCCarModem = val;
}

bool RecordPreview::getUvcCarModem()
{
	return mUVCCarModem;
}

void RecordPreview::showUvcCarBackLines(bool val)
{
	int screen_w,screen_h;
	getScreenInfo(&screen_w, &screen_h);
	if(val){
		LoadBitmapFromFile(HDC_SCREEN, &backCarBmp, "/system/res/others/back_car.png");
		HWND retWnd = GetDlgItem(mHwnd, ID_RECORDPREVIEW_IMAGE);
		SendMessage(retWnd, STM_SETIMAGE, (WPARAM)&backCarBmp, 0);
		ShowWindow(retWnd, SW_SHOWNORMAL);
	//	FillBoxWithBitmap(HDC_SCREEN,0,SML_RECT_Y,screen_w,screen_h-SML_RECT_Y,&backCarBmp);
	}else{
		HWND retWnd = GetDlgItem(mHwnd, ID_RECORDPREVIEW_IMAGE);
		ShowWindow(retWnd, SW_HIDE);
		UnloadBitmap(&backCarBmp);
		backCarBmp.bmBits = NULL;
		SetBrushColor(HDC_SCREEN,RGBA2Pixel(HDC_SCREEN, 0x00, 0x00, 0x00, 0x00));
		FillBox(HDC_SCREEN,0,SML_RECT_Y,screen_w,screen_h-SML_RECT_Y);
	}
}

void RecordPreview::setRtspSession(bool flag)
{
	mStartRtspSession = flag;
}

#if defined(VIDEO_CONFIG_TEXT_TRACK) && defined(VIDEO_CONFIG_TEXT_GSR)
void RecordPreview::addGsensorData(float xa, float ya, float za)
{
    CdrMediaRecorder *pCMR;
    float data[3];
    data[0] = xa;
    data[1] = ya;
    data[2] = za;

    if(isRecording(CAM_CSI)){
        MediaRecorderObj *mobj = getMediaRecordObj(CAMERA_CSI_RECORD);
        if(mobj != NULL){
            pCMR = mobj->getCdrMediaRecorder();
            if (pCMR) {
                pCMR->sendGsensorData(data);
            }
        }
    }
}
#endif

