
#include "StatusBar.h"
#undef LOG_TAG
#define LOG_TAG "StatusBar.cpp"
#include "debug.h"

using namespace android;

static pic_t pic_sets[] = {
	{ID_STATUSBAR_ICON_WINDOWPIC,	"WindowPic"},
	{ID_STATUSBAR_ICON_AWMD,		"AWMD"},
	{ID_STATUSBAR_ICON_UVC,			"UVC"},
	{ID_STATUSBAR_ICON_WIFI,		"Wifi"},
	{ID_STATUSBAR_ICON_LOCK,		"Lock"},
	{ID_STATUSBAR_ICON_AUDIO,		"Voice"},
	{ID_STATUSBAR_ICON_TFCARD,		"TFCard"},
	{ID_STATUSBAR_ICON_BAT,			"Bat"},
	{ID_STATUSBAR_ICON_PARK,		"Park"},
	{ID_STATUSBAR_ICON_WOND,		"Wond"},
};

enum iconIndex{
	ICON_INDEX_WINDOWPIC = 0,
	ICON_INDEX_AWMD,
	ICON_INDEX_UVC,
	ICON_INDEX_WIFI,
	ICON_INDEX_LOCK,
	ICON_INDEX_AUDIO,
	ICON_INDEX_TFCARD,
	ICON_INDEX_BAT,
	ICON_INDEX_PARK,
	ICON_INDEX_WOND,
};

#define PIC_CNT		(sizeof(pic_sets)/sizeof(pic_sets[0]))

int StatusBarProc(HWND hWnd, int message, WPARAM wParam, LPARAM lParam)
{
	StatusBar* stb = (StatusBar*)GetWindowAdditionalData(hWnd);
 	switch(message) {
	case MSG_FONTCHANGED:
		stb->updateWindowFont();
		break;
	case MSG_CREATE:
		{
			ResourceManager *rm;
			gal_pixel color;

			rm = ResourceManager::getInstance();
			color = rm->getResColor(ID_STATUSBAR, COLOR_BGC);
			SetWindowBkColor(hWnd, color);

			if(stb->createSTBWidgets(hWnd) < 0) {
				db_error("create sub widgets failed\n");	
			}

			color = rm->getResColor(ID_STATUSBAR, COLOR_FGC);
			SetWindowElementAttr(stb->getLabelHwnd(), WE_FGC_WINDOW, Pixel2DWORD(HDC_SCREEN, color));
#ifdef USE_IPS_SCREEN
            SetWindowElementAttr(stb->getFileNameHwnd(), WE_FGC_WINDOW, Pixel2DWORD(HDC_SCREEN, color));
            SetWindowElementAttr(stb->getTimeHwnd(), WE_FGC_WINDOW,  Pixel2DWORD(HDC_SCREEN, color));
          //  SetTimer(hWnd, STB_SHOW_TIME, 100);
#endif
		}
		break;
	case MSG_TIMER:
		{
			if(wParam == STB_TIMER_SOS){
				if(STB_SOS>0){
					ResourceManager *rmx = ResourceManager::getInstance();
					const char *text = rmx->getLabel(LANG_LABEL_SOS_WARNING);
					char dis_buf[40];
					sprintf(dis_buf,"%s",text);
					STB_SOS--;
				    SetWindowText(stb->getLabelHwnd(), dis_buf);
				}else{
				    SetWindowText(stb->getLabelHwnd(), "");
					 KillTimer(hWnd, STB_TIMER_SOS);
				}
			}
		}
		break;
	case STBM_START_RECORD_TIMER:
	case STBM_RESUME_RECORD_TIMER:
	case STBM_STOP_RECORD_TIMER:
	case STBM_GET_RECORD_TIME:
	case STBM_UPDATE_TIME:
		return stb->handleRecordTimerMsg(message, wParam);
		break;
	case STBM_SETLABEL1_TEXT:
		/* lParam: msg str */
		{
			if(wParam == 1) {
				db_msg("*88888888888888******lParam=%s ***",(char*)lParam);
				STB_SOS=5;
				SetTimer(hWnd, STB_TIMER_SOS, 100);
				ResourceManager *rmx = ResourceManager::getInstance();
				const char *text = rmx->getLabel(LANG_LABEL_SOS_WARNING);
				char dis_buf[40];
				sprintf(dis_buf,"%s",text);
				SetWindowText(stb->getLabelHwnd(), dis_buf);
			}else if(wParam == 2){
				 stb->mScreenShotText = true;
				 SetWindowText(stb->getLabelHwnd(), (char*)lParam);
			}else{
				SetWindowText(stb->getLabelHwnd(), (char*)lParam);
			}
		}
		break;
    case STBM_SETLABEL2_TEXT:
        SetWindowText(stb->getFileNameHwnd(), (char*)lParam);
        break;
	case STBM_TIMELABEL_TEXT:
        SetWindowText(stb->getTimeHwnd(), (char*)lParam);
        break;
	case STBM_CLEAR_LABEL1_TEXT:
		{
			if (stb->getCurrenWindowId() != wParam) {
				break;
			}
			if(stb->mScreenShotText){
				stb->mScreenShotText = false;
			}
			RECT rect;
			GetClientRect(stb->getLabelHwnd(), &rect);
			SetWindowText(stb->getLabelHwnd(), "");
			InvalidateRect(stb->getLabelHwnd(), &rect, TRUE);
		}
		break;
	case MSG_DESTROY:
#ifdef USE_IPS_SCREEN
		stb->stopSystemTimeTimer();
#endif
		break;
    case STBM_SHOW_TIMEHWND:
        if(wParam == 0){
            ShowWindow(stb->getLabelHwnd(), SW_HIDE);
            ShowWindow(stb->getTimeHwnd(), SW_HIDE);
            ShowWindow(stb->getFileNameHwnd(), SW_SHOWNORMAL);
        }else if(wParam == 1){
            ShowWindow(stb->getFileNameHwnd(), SW_HIDE);
            ShowWindow(stb->getTimeHwnd(), SW_SHOWNORMAL);
            ShowWindow(stb->getLabelHwnd(), SW_SHOWNORMAL);
        }else if(wParam == 2){
            ShowWindow(stb->getLabelHwnd(), SW_HIDE);
            ShowWindow(stb->getTimeHwnd(), SW_HIDE);
        }
        break;
	default:
		stb->msgCallback(message, wParam);
	}

	return DefaultWindowProc(hWnd, message, wParam, lParam);
}


static void record_timer_call(sigval_t sg)
{
	timer_set *data = (timer_set*)sg.sival_ptr;
	StatusBar* winSB = (StatusBar *)data->win;
	SendMessage(winSB->getHwnd(), STBM_UPDATE_TIME, 0, 0);
	winSB->needStop();
}

static void system_time_timer_call(sigval_t sg)
{
	static char mWaterMark[32];
	StatusBar* winSB = NULL;
	timer_set *data = NULL;
	struct tm *tm = NULL;

	memset(mWaterMark,0,sizeof(mWaterMark));
	data = (timer_set*)sg.sival_ptr;
	winSB = (StatusBar*)data->win;

	getDateTime(&tm);
	sprintf(mWaterMark, "%04d-%02d-%02d  %02d:%02d:%02d", tm->tm_year + 1900, tm->tm_mon + 1,
	tm->tm_mday, tm->tm_hour,tm->tm_min, tm->tm_sec);
	// db_msg("update system time:%s\n", mWaterMark);

	/*	UpdateWindow some time may not erase clean in IPS screen so use window's pro function   */
	SendMessage(winSB->getHwnd(), STBM_TIMELABEL_TEXT, 0, (LPARAM)mWaterMark);
	//SetWindowText(winSB->getTimeHwnd(), mWaterMark);
	//UpdateWindow(winSB->getTimeHwnd(), true);
}

StatusBar::StatusBar(CdrMain *cdrMain)
: mCdrMain(cdrMain),
	mRecordTime(0),
	mCountOnce(false),
	mLabelDisp(true),
	mSystemTimeTimer(NULL),
	mScreenShotText(false)
{
	CDR_RECT rect;
	HWND hParent;
	ResourceManager *rm;
	PicObj* pic[PIC_CNT];

	rm = ResourceManager::getInstance();
	hParent = mCdrMain->getHwnd();
	rm->getResRect(ID_STATUSBAR, rect);
	for(unsigned int i = 0; i < PIC_CNT; i++ ) {
		pic[i] = new PicObj(String8(pic_sets[i].name));
		pic[i]->setId(pic_sets[i].id);
		mPic.add(pic[i]);
		if(i == ICON_INDEX_AUDIO){
			pic[i]->set(1);
		}
	}

	mHwnd = CreateWindowEx(WINDOW_STATUSBAR, "",
			WS_VISIBLE | WS_CHILD,
			WS_EX_NONE | WS_EX_USEPARENTFONT,
			WINDOWID_STATUSBAR,		
			rect.x, rect.y, rect.w, rect.h,
			hParent, (DWORD)this);
	if(mHwnd == HWND_INVALID) {
		db_error("create status bar failed\n");
		return;
	}

	rm->setHwnd(WINDOWID_STATUSBAR, mHwnd);
	mCurWindowID = WINDOWID_RECORDPREVIEW;
	mRecordTimer = NULL;
}

StatusBar::~StatusBar()
{
	db_msg("StatusBar Destructor\n");
	for(unsigned int i = 0; i < mPic.size(); i++) {
		delete mPic.itemAt(i);
	}
	if (mSystemTimeTimer != NULL) {
		delete mSystemTimeTimer;
	}
	db_msg("StatusBar Destructor\n");
}

HWND StatusBar::getHwnd()
{
	return mHwnd;
}

int StatusBar::initPic(PicObj *po)
{
	int retval;
	CDR_RECT rect;
	HWND retWnd;
	ResourceManager* rm;

//	db_msg("initPic %p", po);
	rm = ResourceManager::getInstance();
	retval = rm->getResRect(po->getId(), rect);
	if(retval < 0) {
		db_error("get %s failed\n", po->mName.string());
		return -1;
	}
//	db_msg("name: %s, ID[%d], %d %d %d %d", po->mName.string(), po->getId(), rect.x, rect.y, rect.w, rect.h);

	retval = rm->getResBmp(po->getId(), BMPTYPE_BASE, *po->getImage());
	if(retval < 0) {
		db_error("loadb bitmap of %s failed\n", po->mName.string());
	}

	retWnd = CreateWindowEx(CTRL_STATIC, "",
			WS_CHILD | WS_VISIBLE | SS_BITMAP | SS_CENTERIMAGE ,
			WS_EX_TRANSPARENT,
			po->getId(),
			rect.x, rect.y, rect.w, rect.h,
			po->mParent, (DWORD)(po->getImage()));
	if(retWnd == HWND_INVALID) {
		db_error("create status bar volume failed\n");
		return -1;
	}
	return 0;
}

HWND StatusBar::getFileNameHwnd()
{
	return mFileNameHwnd;
}

HWND StatusBar::getLabelHwnd()
{
	return mLabelHwnd;
}

void StatusBar::setLabelHwnd(HWND hwnd)
{
	mLabelHwnd = hwnd;
}

HWND StatusBar::getTimeHwnd()
{
    return mTimeHwnd;
}

void StatusBar::setTimeHwnd(HWND hwnd)
{
    mTimeHwnd = hwnd;
}

int StatusBar::createSTBWidgets(HWND hParent)
{
	CDR_RECT rect;
	int retval = 0;
	HWND retWnd;
	ResourceManager *rm;

	rm = ResourceManager::getInstance();

	/* Label1 */
	retval = rm->getResRect(ID_STATUSBAR_LABEL1, rect);
	if(retval < 0) {
		db_error("get ID_STATUSBAR_LABEL1 failed\n");
		return -1;
	}

	retWnd = CreateWindowEx(CTRL_STATIC, "",
			WS_CHILD | WS_VISIBLE | SS_SIMPLE,
			WS_EX_TRANSPARENT | WS_EX_USEPARENTFONT,
			ID_STATUSBAR_LABEL1,
			rect.x, rect.y, rect.w, rect.h,
			hParent, 0);
	if(retWnd == HWND_INVALID) {
		db_error("create status bar window name failed\n");
		return -1;
	}
	setLabelHwnd(retWnd);


	/* Label2 */
 #ifdef USE_IPS_SCREEN
	retval = rm->getResRect(ID_STATUSBAR_LABEL2, rect);
	if(retval < 0) {
		db_error("get ID_STATUSBAR_LABEL1 failed\n");
		return -1;
	}

    db_msg("label2 x:%d,y:%d,w:%d,h:%d", rect.x, rect.y, rect.w, rect.h);
	retWnd = CreateWindowEx(CTRL_STATIC, "",
			WS_CHILD | WS_VISIBLE | SS_SIMPLE,
			WS_EX_TRANSPARENT | WS_EX_USEPARENTFONT,
			ID_STATUSBAR_LABEL2,
			rect.x, rect.y, rect.w, rect.h,
			hParent, 0);
	if(retWnd == HWND_INVALID) {
		db_error("create status bar window name failed\n");
		return -1;
	}
	mFileNameHwnd = retWnd;
    ShowWindow(mFileNameHwnd, SW_HIDE);
#endif

	retval = rm->getResRect(ID_STATUSBAR_LABEL_RESERVE, rect);
	if(retval < 0) {
		db_error("get ID_STATUSBAR_LABEL_RESERVE failed\n");
		return -1;
	}

	retWnd = CreateWindowEx(CTRL_STATIC, "",
			WS_CHILD | WS_VISIBLE | SS_SIMPLE ,
			WS_EX_TRANSPARENT | WS_EX_USEPARENTFONT,
			ID_STATUSBAR_LABEL_RESERVE,
			rect.x, rect.y, rect.w, rect.h,
			hParent, 0);

	if(retWnd == HWND_INVALID) {
		db_error("create status bar reserved label failed\n");
		return -1;
	}

#ifdef USE_IPS_SCREEN
    setTimeHwnd(retWnd);
#endif

	for( unsigned int i = 0; i < mPic.size(); i++ ) {
		mPic.itemAt(i)->mParent = hParent;
		if(initPic(mPic.itemAt(i)) < 0) {
			db_msg("init pic mId: %d failed\n", mPic.itemAt(i)->getId());
			return -1;
		}
	}

	return 0;
}

int StatusBar::updateRecordTime(bool setLabelBlank)
{
	char timeStr[30] = "";
	int ret = increaseRecordTime();
	if (ret>=0) {
		sprintf(timeStr, "%02d:%02d", (unsigned int)mRecordTime/60, (unsigned int)mRecordTime%60);
	}
	mTimerLock.lock();
	if (mLabelDisp == true) {
		if (mCdrMain->getCurWindowID() == WINDOWID_RECORDPREVIEW) {
			if(mScreenShotText){
				mScreenShotText = false;
			}else{
				SetWindowText(getLabelHwnd(), timeStr);
				UpdateWindow(getLabelHwnd(), false);
			}
		}
		ret = 0;
	} else {
		if (mCdrMain->getCurWindowID() == WINDOWID_RECORDPREVIEW) {
			SetWindowText(getLabelHwnd(), "");
			UpdateWindow(getLabelHwnd(), false);
			mRecordTime = 0;
		}
		ret = -1;
	}
	mTimerLock.unlock();
	return ret;
}


void StatusBar::setLabelDisp(bool val)
{
	Mutex::Autolock _l(mTimerLock);
	mLabelDisp = val;
}
bool StatusBar::getLabelDisp()
{
	Mutex::Autolock _l(mTimerLock);
	return mLabelDisp;
}
	
int StatusBar::increaseRecordTime()
{
	mRecordTime++;

	if(mRecordTime >= mCurRecordVTL / 1000) {
		if (mCountOnce) {
			SendMessage(mHwnd, STBM_STOP_RECORD_TIMER, 0, 0);
		}
		mRecordTime = 0;
		SendMessage(mHwnd,STBM_LOCK,0,0);
		if(mCurRecordVTL == AFTIMEMS || mCurRecordVTL == PARK_RECORD_TIME) {
			mCurRecordVTL = mRecordVTL;
			db_msg("reset the mCurRecordVTL to %ld ms\n", mCurRecordVTL);
			SendMessage(mHwnd, STBM_LOCK, 0, 0);
#ifdef APP_WIFI
			mCdrMain->cdrReplyToApp(NAT_SOS_IMPACT_FILE_DOWN, NULL, 0);
#endif
		}
	}
	return 0;
}

void StatusBar::msgCallback(int msgType, unsigned int data)
{
	ALOGV("recived msg %x, data is 0x%x\n", msgType, data);
	switch(msgType) {
	case MSG_CDRMAIN_CUR_WINDOW_CHANGED:
		windowState_t *curWindowState;
		curWindowState = (windowState_t*)data;
		if(curWindowState == NULL) {
			db_error("invalid windowState\n");
			break;
		}
		handleWindowPicEvent(curWindowState);
		break;
	case MSG_BATTERY_CHANGED:
		mPic.itemAt(ICON_INDEX_BAT)->set(data);
		mPic.itemAt(ICON_INDEX_BAT)->update();
		break;
	case MSG_BATTERY_CHARGING:
		mPic.itemAt(ICON_INDEX_BAT)->set(6);
		mPic.itemAt(ICON_INDEX_BAT)->update();
		break;
	case STBM_MOUNT_TFCARD:
		mPic.itemAt(ICON_INDEX_TFCARD)->set(data);
		mPic.itemAt(ICON_INDEX_TFCARD)->update();
		break;
	case STBM_VOICE:
		if(mPic.itemAt(ICON_INDEX_AUDIO)->get() != (int)data) {
			mPic.itemAt(ICON_INDEX_AUDIO)->set(data);
			mPic.itemAt(ICON_INDEX_AUDIO)->update();
		}
		break;
	case STBM_LOCK:
		db_msg("*****STBM_LOCK,  data:%d*****",data);
		mPic.itemAt(ICON_INDEX_LOCK)->set(data);
		mPic.itemAt(ICON_INDEX_LOCK)->update();
		break;
	case STBM_UVC:
		mPic.itemAt(ICON_INDEX_UVC)->set(data);
		mPic.itemAt(ICON_INDEX_UVC)->update();
		break;
	case STBM_WIFI:
		mPic.itemAt(ICON_INDEX_WIFI)->set(data);
		mPic.itemAt(ICON_INDEX_WIFI)->update();
		break;
	case STBM_PARK:
		mPic.itemAt(ICON_INDEX_PARK)->set(data);
		mPic.itemAt(ICON_INDEX_PARK)->update();
		break;
	case STBM_AWMD:
		mPic.itemAt(ICON_INDEX_AWMD)->set(data);
		mPic.itemAt(ICON_INDEX_AWMD)->update();
		break;
	case STBM_COUNT_ONCE:
		setCountOnce((bool)data);
		break;
	case STBM_WOND:
		mPic.itemAt(ICON_INDEX_WOND)->set(data);
		mPic.itemAt(ICON_INDEX_WOND)->update();
		break;
	default:
		break;
	}
}

void StatusBar::handleWindowPicEvent(windowState_t *windowState)
{
	mCurWindowID = windowState->curWindowID;
	switch(windowState->curWindowID) {
	case WINDOWID_RECORDPREVIEW:
		{
			RPWindowState_t state;
			state = windowState->RPWindowState;
			if(state == STATUS_RECORDPREVIEW) {
				if(mCdrMain->getUvcState()){
					mPic.itemAt(ICON_INDEX_UVC)->set(1);
					mPic.itemAt(ICON_INDEX_UVC)->update();
				} else {
					mPic.itemAt(ICON_INDEX_UVC)->set(0);
					mPic.itemAt(ICON_INDEX_UVC)->update();
				}
			} else if(state == STATUS_PHOTOGRAPH){
				if(mCdrMain->getUvcState()){
					mPic.itemAt(ICON_INDEX_UVC)->set(1);
					mPic.itemAt(ICON_INDEX_UVC)->update();
				} else {
					mPic.itemAt(ICON_INDEX_UVC)->set(0);
					mPic.itemAt(ICON_INDEX_UVC)->update();
				}
			}
			if(state == STATUS_AWMD) {
				mPic.itemAt(ICON_INDEX_AWMD)->set(1);
				mPic.itemAt(ICON_INDEX_AWMD)->update();
			} else {
				mPic.itemAt(ICON_INDEX_AWMD)->set(0);
				mPic.itemAt(ICON_INDEX_AWMD)->update();
			}

			if(state == STATUS_RECORDPREVIEW || state == STATUS_AWMD) {
				mPic.itemAt(ICON_INDEX_WINDOWPIC)->update(BMPTYPE_WINDOWPIC_RECORDPREVIEW);
			} else if(state == STATUS_PHOTOGRAPH) {
				mPic.itemAt(ICON_INDEX_WINDOWPIC)->update(BMPTYPE_WINDOWPIC_PHOTOGRAPH);
			}
		}
		break;
	case WINDOWID_PLAYBACKPREVIEW:
		mPic.itemAt(ICON_INDEX_WINDOWPIC)->update(BMPTYPE_WINDOWPIC_PLAYBACKPREVIEW);
		if(mCdrMain->getUvcState()){
			mPic.itemAt(ICON_INDEX_UVC)->set(1);
			mPic.itemAt(ICON_INDEX_UVC)->update();
		} else {
			mPic.itemAt(ICON_INDEX_UVC)->set(0);
			mPic.itemAt(ICON_INDEX_UVC)->update();
		}
		break;
	case WINDOWID_MENU:
		mPic.itemAt(ICON_INDEX_WINDOWPIC)->update(BMPTYPE_WINDOWPIC_MENU);
		if(mCdrMain->getUvcState()){
			mPic.itemAt(ICON_INDEX_UVC)->set(1);
			mPic.itemAt(ICON_INDEX_UVC)->update();
		} else {
			mPic.itemAt(ICON_INDEX_UVC)->set(0);
			mPic.itemAt(ICON_INDEX_UVC)->update();
		}
		break;
	default:
		db_error("invalide windowID: %d\n", windowState->curWindowID);
		break;
	}
}

int StatusBar::handleRecordTimerMsg(unsigned int message, unsigned int recordType)
{
	switch(message) {
	case STBM_START_RECORD_TIMER:
		{
			db_msg("recieved STBM_START_RECORD_TIMER, recordType is %d, mRecordVTL is %ld ms, mCurRecordVTL is %ld ms\n", recordType, mRecordVTL, mCurRecordVTL);
			if(mRecordVTL == 0) {
				db_msg("start record timer failed, record vtl not initialized\n");
				break;
			}
			if (!mRecordTimer) {
				mRecordTimer = new Timer(this);
				mRecordTimer->init(record_timer_call, TIMER_RECORD_IDX);
			}
			mTimerLock.lock();
			mRecordTime = 0;
			SetWindowText(getLabelHwnd(), "00:00");
			UpdateWindow(getLabelHwnd(), false);
			mRecordTimer->start(1);
			mLabelDisp = true;
			mTimerLock.unlock();
			if(recordType == 1) {
				/* if recordType is impact record, set the mCurRecordVTL with AFTIMEMS
				 * when time expired, compare the mCurRecordVTL with mRecordVTL, 
				 * if not equal, then reset mCurRecordVTL with mRecordVTL
				 * */
				if(mCdrMain->getWakeupState())
					mCurRecordVTL = PARK_RECORD_TIME;
				else
					mCurRecordVTL = AFTIMEMS;
				SendMessage(mHwnd, STBM_LOCK, 1, 0);
			}
		}
		break;
	case STBM_STOP_RECORD_TIMER:
		{
			db_msg("recieved STBM_STOP_RECORD!\n");
			mTimerLock.lock();
			mLabelDisp = false;
			if (mCdrMain->getCurWindowID() == WINDOWID_RECORDPREVIEW) {
				SetWindowText(getLabelHwnd(), "");
				UpdateWindow(getLabelHwnd(), false);
			}
			mTimerLock.unlock();
 			SendMessage(mHwnd, STBM_LOCK, 0, 0);
		}
		break;
	case STBM_GET_RECORD_TIME:
		return mRecordTime;
		break;
	case STBM_UPDATE_TIME:
		updateRecordTime();
		break;
	}

	return 0;
}

/*
 * set the record video time length, unit(ms)
 * */
void StatusBar::setRecordVTL(unsigned int vtl)
{
	mRecordVTL = vtl;
	mCurRecordVTL = vtl;

}

unsigned int StatusBar::getCurrenWindowId()
{
	return mCurWindowID;
}

void StatusBar::setCountOnce(bool value)
{
	mCountOnce = value;
}

void StatusBar::stopRecordTimer()
{
	if (mRecordTimer != NULL) {
		mRecordTimer->stop();
		delete mRecordTimer;
		mRecordTimer = NULL;
	}
}

void StatusBar::startSystemTimeTimer()
{
	mSystemTimeTimer = new Timer(this);
	mSystemTimeTimer->init(system_time_timer_call, TIMER_SYSTEM_TIME_IDX);
	mSystemTimeTimer->start(1);
}

void StatusBar::stopSystemTimeTimer()
{
	if (mSystemTimeTimer != NULL) {
		mSystemTimeTimer->stop();
		delete mSystemTimeTimer;
		mSystemTimeTimer = NULL;
	}
}

StatusBar::Timer::Timer(MainWindow *win)
	: mTimerStarted(false)
{
	mWin = win;
}

void StatusBar::Timer::init(void(*callback)(sigval_t), int timer_idx)
{
	mTimerdata.win = mWin;
	mTimerdata.id = 0;
	mTimerdata.time = 1;
	mTimerdata.idx = timer_idx;
	createTimer((void*)&mTimerdata, &mTimerdata.id, callback);
}

void StatusBar::Timer::start(unsigned int time_sec, bool isOneShot)
{
	if (isOneShot) {
		setOneShotTimer(time_sec, 0, mTimerdata.id);
	} else {
		setPeriodTimer(time_sec, 0, mTimerdata.id);
	}
	 mTimerStarted = true;
}

void StatusBar::Timer::stop()
{
	if (mTimerStarted) {
		stopTimer(mTimerdata.id);
		mTimerStarted = false;
	}
}

StatusBar::Timer::~Timer()
{
	deleteTimer(mTimerdata.id);
}

bool StatusBar::Timer::timerStoped()
{
	time_t sec = 0;
	long int nsec = 0;
	getExpirationTime(&sec, &nsec, mTimerdata.id);
	db_msg("sec %d, nsec %ld", sec, nsec);
	if (sec == 0 && nsec == 0) {
		return true;
	}
	return false;
}

void StatusBar::needStop()
{
	mTimerLock.lock();
	if (!mLabelDisp) {
		stopRecordTimer();
	}
	mTimerLock.unlock();
}
